.\"
.\" Copyright (c) Erik Kunze 1995 - 2007
.\"
.\" This file is a part of the Spectrum emulator package 'XZX-Pro', and may not
.\" be distributed.  The copyright holder makes no representations about the
.\" suitability of this software for any purpose.  It is provided "as is" without
.\" express or implied warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT
.\" PERMISSION!
.\"
.\" THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
.\" INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
.\" EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR
.\" CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
.\" DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
.\" TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
.\" OF THIS SOFTWARE.
.\"
.\" Author: Erik Kunze
.\"
.\" $Id: xzx.man,v 4.45 2006/12/22 14:15:53 erik Rel $
.\"
.TH "XZX\-Pro" 1
.\"
.SH NAME
.\"
XZX\-Pro \- a Sinclair ZX Spectrum emulator
.\"
.SH SYNOPSIS
.\"
.B xzx
.B [options]
.B [\fIsnaphot-file\fB]
.\"
.SH DESCRIPTION
.\"
\fBXZX-Pro\fR is built on top of a Z-80 emulator, with added routines to
support keyboard and video I/O through an X interface.  The hardware emulation
can operate as a Spectrum 48K, 128K, +2, +3, Pentagon 128 or Scorpion 256,
Didaktik Kompakt.

\fBXZX-Pro\fP supports up to 256K of RAM in Scorpion mode.  Floppy disks are
emulated using files to store the data.  A printer is emulated by sending its
output to a file or a command (pipe).  There is also an emulation of cassette
I/O that uses files for cassette tapes. Sound support is available under most
UNIX systems.
.SH OPTIONS
\fBXZX-Pro\fR accepts the following command line options.  Options may also be
specified as X Resources using the same spelling.  Note that the shortest
unique spelling of a resource is acceptable, e.g. "\fIxzx -ma 128\fR" is
equivalent to "\fIxzx -machine 128\fR".
.\"
.SS General options
.\"
.TP
.BI \-version
Print the version and compile-time options of this \fBXZX-Pro\fR and exit.
.TP
.BI \-libDir " \fI<path>\fR"
Use \fIpath\fR as the colon-separated directory list to check when loading the
Spectrum ROM and any snapshots specified.  Directories are guaranteed to be
searched in the order they are listed.  Defaults to
"\fI.:@prefix@/lib/@PACKAGE@\fR".
.TP
.\".BI \-infoDir " \fI<path>\fR"
.\"Use \fIpath\fR as the colon-separated directory list to check when loading the
.\"game instructions.  Directories are guaranteed to be searched in the order
.\"they are listed.  Defaults to
.\""\fI.:@prefix@/lib/@PACKAGE@\fR".
.\".TP
.\".BI \-pokeDir " \fI<path>\fR"
.\"Use \fIpath\fR as the colon-separated directory list to check when loading the
.\"poke instructions.  Directories are guaranteed to be searched in the order
.\"they are listed.  Defaults to
.\""\fI.:@prefix@/lib/@PACKAGE@\fR".
.\".TP
.BI \-quiet " \fI<boolean>\fR"
Suppresses all but error messages. Defaults to \fIfalse\fR.
.TP
.BI \-display " \fI<displayname>\fR"
Set the X display to \fIdisplayname\fR. The default is to use the DISPLAY
environment variable.
.TP
.BI \-monocrome " \fI<boolean>\fR"
Use only Black and White.  In this case, the foreground and background will
contrast, e.g. blue on black will appear as white on black, and green on white
will appear as black on white.
.\"The default for monochrome X servers is half toning if the scaling factor is
.\"increased.
Defaults to \fIfalse\fR.
.TP
.BI \-private " \fI<boolean>\fR"
Use a private colourmap at startup.  This is the default if \fBXZX-Pro\fR fails
to get all the colours it needs.  Defaults to \fIfalse\fR.
.TP
.BI \-xsync " \fI<boolean>\fR"
Causes \fBXZX-Pro\fR to call the X11 function XSync() before updating the
emulation window: this might be necessary on low-end systems to prevent it
from stealing so many system resources that it would become impossible for the
user to interact with it.  Defaults to \fIfalse\fR.
.TP
.BI \-speed " \fI<n>\fR"
Specify the emulation speed.  The default setting is \fI100\fR, which causes
the emulation to never run faster than the real machine.  A higher value makes
the emulator faster, a lower one makes it slower.
.TP
.BI \-fastmode " \fI<boolean>\fR"
Enables special speed mode, which disables any speed limit, turns sound
emulation off and uses a 1/10 refresh rate, so that it will run at the maximum
possible speed.  Defaults to \fIfalse\fR.
.TP
.BI \-scale " \fI<1|2|3>\fR"
Specify the Spectrum to X11 pixel magnification. Defaults to \fI1\fR.
.TP
.BI \-refresh " \fI<n>\fR"
Specify how often the Spectrum screen is refreshed.  It is refreshed every
\fIn\fR interrupts, so a lower number means more frequent refreshes.
Specifying it too low, however, could result in more time being spent in
screen painting than in CPU emulation!  Defaults to every \fI3\fR interrupts,
you might get away with every interrupt if you are lucky.
.TP
.BI \-cache " \fI<boolean>\fR"
Enables a video cache that can speed up the emulation when little graphics
activity is going on; it is especially useful when you run \fBXZX-Pro\fR on a
networked X terminal, as it can reduce the amount of bandwidth required.
Defaults to \fItrue\fR.
.\"
.SS Hardware options
.\"
.TP
.BI \-machine " \fI<48-2|48|128|3|Pentagon|Scorpion|Didaktik>\fR"
Specifies the memory model to emulate.  Defaults to \fI48K\fR operation (48).

\fI48-2\fR selects a Spectrum 48K with issue 2, \fI48\fR selects a Spectrum
48K with issue 3.  Issue 3 is by far the most common.  Occasionally a program
will need this option to be set to \fI48-2\fR (usually an old program for
instance Spinads) otherwise the keys will not respond.  Note that all 128K
Spectrums were issue 3 and some Spectrum 48K were issue 2, but not that many.
.TP
.BI \-warn48 " \fI<boolean>\fR"
Warn if a 48k program accesses ports from the 128K machinery.  Useful to
detect if a programm has been written for the Spectrum 128K/+3.  Defaults to
\fIfalse\fR.
.TP
.BI \-smart " \fI<boolean>\fR"
Controls the automatic attachment of storage media. Defaults to \fItrue\fR.
.PP
In Spectrum mode the following options apply.
.TP
.BI \-sp48Rom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR in Spectrum 48K mode. The romfile
has to be a raw binary dump, 16K long.  Defaults to \fIspectrum.rom\fR.
.TP
.BI \-sp128Rom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR in Spectrum 128K mode. The romfile
has to be a raw binary dump, 32K long.  \fBXZX-Pro\fR will emulate a Spectrum
+2 if the right romfile is provided.  Defaults to \fI128.rom\fR.
.TP
.BI \-sp3Rom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR in Spectrum +3 mode. The romfile
has to be a raw binary dump, 64K long.  Defaults to \fIplus3.rom\fR.
.TP
.BI \-sp3Drive\fI<A|B>\fR " \fI<filename>\fR"
Load disk file \fIfilename\fR into the emulated disk drive \fIA:\fR or
\fIB:\fR.  Defaults to \fIdisk<a|b>.dsk\fR.
.PP
In Pentagon mode the following options apply.
.TP
.BI \-pentagonRom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR in Pentagon mode. The romfile
has to be a raw binary dump, 48K long.  Defaults to \fIpentagon.rom\fR.  The
romfile is not distributed with \fBXZX-Pro\fR.
.PP
In Scorpion mode the following options apply.
.TP
.BI \-scorpionRom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR in Scorpion mode. The romfile has
to be a raw binary dump, 64K long.  Defaults to \fIscorpion.rom\fR.  The
romfile is not distributed with \fBXZX-Pro\fR.
.PP
In Didaktik mode the following options apply.
.TP
.BI \-didaktikRom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR in Didaktik mode. The romfile has
to be a raw binary dump, 32K long.  Defaults to \fIdidaktik.rom\fR.  The
romfile is not distributed with \fBXZX-Pro\fR.
.TP
.BI \-didaktikDrive\fI<A|B>\fR " \fI<filename>\fR"
Load disk file \fIfilename\fR into the emulated disk drive \fIA:\fR or
\fIB:\fR.  Defaults to \fIdisk<a|b>.d80\fR.
.PP
The ZX Interface I and Microdrive emulation uses .MDR files as Microdrive
cartridges.  The following options apply.
.TP
.BI \-if1 " \fI<boolean>\fR"
Enable or disable the Interface I.  \fIEnabled\fR by default.  In that case,
an image of the Interface I ROM must be present.  The ROM switching is done in
lower 16K.

With the emulator, a new version of the 8K ZX Interface ROM is supported
(there were two different versions of ROMs).  Some ill-based programs will not
work with new the version, but there are also programs which work only with
new version.  Some programs (for example, Beta Basic 3.0.) have two versions,
one for the old ROM and one for the new one.

Turn it off if you want to work without extended basic / microdrive support.
.TP
.BI \-if1Rom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR as Interface I ROM. The romfile
has to be a raw binary dump, 8 or 16K long.  Defaults to \fIif1-v2.rom\fR.
The romfiles of the Interface I versions 1 and 2 are distributed with
\fBXZX-Pro\fR.
.TP
.BI \-if1Drive\fI<n>\fR " \fI<filename>\fR"
Load cartridge file \fIfilename\fR into the emulated Microdrive \fIn\fR.
Defaults to \fIcart<n>.mdr\fR.

A blank, unformatted cartridge file can be created by a command such as
"\fIdd if=/dev/zero of=test.mdr bs=137923 count=1\fR".  A shell script
\fBmkcart\fR has also been provided to do this.  You can then format it with
the Sinclair BASIC \fBFORMAT\fR command as normal.
.TP
.BI \-if1Rs232Mode " \fI<0|1|2>\fR"
Select the RS232 emulation mode.  \fI0\fR disables emulation.  \fI1\fR
connects Interface I to serial port of the host machine.  \fI2\fR sends all
output to the printer.  Defaults to \fI0\fR.
.TP
.BI \-rs232Dev " \fI<device>\fR"
Specifies the serial device to use. Defaults to \fI@RS232DEV@\fR.
.TP
.BI \-rs232Baud " \fI<n>\fR"
Specifies \fIn\fR as baudrate for the serial device.  Defaults to \fI9600\fR
Baud.
.PP
For Multiface emulation the following options apply.
.TP
.BI \-mf " \fI<boolean>\fR"
Enable or disable the Multiface.  \fIDisabled\fR by default.
.TP
.BI \-mf1Rom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR as Multiface 1 ROM. The romfile
has to be a raw binary dump, 8K long.  Defaults to \fImf1.rom\fR.  The romfile
is not distributed with \fBXZX-Pro\fR.
.TP
.BI \-mf128Rom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR as Multiface 128 ROM. The romfile
has to be a raw binary dump, 8K long.  Defaults to \fImf128.rom\fR.  The
romfile is not distributed with \fBXZX-Pro\fR.
.TP
.BI \-mf3Rom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR as Multiface 3 ROM. The romfile
has to be a raw binary dump, 8K long.  Defaults to \fImf3.rom\fR.  The romfile
is not distributed with \fBXZX-Pro\fR.
.PP
For Beta 128 disk interface the following options apply.
.TP
.BI \-trdos " \fI<boolean>\fR"
Enable or disable Beta 128. \fIEnabled\fR by default.
.TP
.BI \-trdosRom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR as TR-DOS ROM mode. The romfile
has to be a raw binary dump, 16K long.  Defaults to \fItrdos.rom\fR.  The
romfile is not distributed with \fBXZX-Pro\fR.
.TP
.BI \-trdosDrive\fI<A-D>\fR " \fI<filename>\fR"
Load disk file \fIfilename\fR into the emulated disk drive \fIA:\fR, \fIB:\fR,
\fIC:\fR or \fID:\fR.  Defaults to \fIdisk<a-d>.trd\fR.

This option is also valid in Pentagon and Scorpion mode.
.PP
For +D disk interface emulation the following options apply.
.TP
.BI \-plusd " \fI<boolean>\fR"
Enable or disable +D. \fIDisabled\fR by default.
.TP
.BI \-plusdRom " \fI<filename>\fR"
Use the romfile specified by \fIfilename\fR as +D ROM mode. The romfile has to
be a raw binary dump, either 8 or 16K long.  Defaults to \fIplusd.rom\fR.  The
romfile is not distributed with \fBXZX-Pro\fR.
.TP
.BI \-plusdDrive\fI<A|B>\fR " \fI<filename>\fR"
Load disk file \fIfilename\fR into the emulated disk drive \fIA:\fR or
\fIB:\fR.  Defaults to \fIdisk<a|b>.mgt\fR.
.PP
For Kempston mouse emulation the following options apply.
.TP
.BI \-kmouse " \fI<boolean>\fR"
Enable or disable Kempston mouse. \fIDisabled\fR by default.
.\"
.SS Sound options
.\"
If \fBXZX-Pro\fR runs on a system with native audio support, such as AIX, HP-UX,
IRIX and SunOS/Solaris, or support for either Network Audio System (NAS) or Open
Sound System (OSS) it is possible to get Speaker and AY sound emulation.
.TP
.BI \-sound " \fI<boolean>\fR"
Enable or disable sound emulation. \fIDisabled\fR by default.
.TP
.BI \-soundDev " \fI<path>\fR"
Specifies the audio device to use.  Defaults to \fI/dev/audio\fR.
.TP
.BI \-soundServer " \fI<server>\fR"
Specifies the NAS audio server to use.
.TP
.BI \-soundNonblocking " \fI<boolean>\fR"
Enable or disable nonblocking I/O. Use with care. \fIDisabled\fR by default.
.TP
.BI \-soundSample " \fI<n>\fR"
Specify the sampling frequency, ranging from 8000 to 48000 Hz (not all the
sound cards and/or sound drivers can support all the frequencies, so actually
the nearmost candidate will be picked up).  Defaults to \fI8000\fR Hz.
.TP
.BI \-soundBits " \fI<8|16>\fR"
Specify the sampling precision, either 8 or 16 Bit. Defaults to \fI8\fR Bit.
.TP
.BI \-soundStereo " \fI<0|1|2>\fR"
Specifies the AY mode to emulate (0=Mono, 1=ACB Stereo, 2=ABC Stereo).
Provided that the audio device could be initialized for stereo output.
Defaults to \fI0\fR.
.TP
.BI \-soundVolumeSp " \fI<n>\fR"
Sets volume for speaker emulation as a percentage.  Defaults to \fI50\fR.
.TP
.BI \-soundVolumeAy " \fI<n>\fR"
Sets volume for AY emulation as a percentage.  Defaults to \fI100\fR.
.\"
.SS Printer options
.\"
\fBXZX-Pro\fR can emulate printers connected to either the Spectrum +3 parallel
or Interface I serial port. Emulation can be achieved by redirecting the
printer output to a file or by piping it through an external process.
.TP
.BI \-printerMode " \fI<0|1|2>\fR"
Specify destination of characters written to the printer (0=None, 1=File,
2=Command).  Defaults to \fI0\fR.
.TP
.BI \-printerFile " \fI<filename>\fR"
Write printer output to \fIfilename\fR.  Default is \fIdump.prt\fR.
.TP
.BI \-printerCmd " \fI<command>\fR"
Send printer output to \fIcommand\fR.  Default is \fIlpr\fR.
.\"
.SS Joystick options
.\"
.TP
.BI \-joyType " \fI<0|1|2|3>\fR"
Specifies the joystick type to emulate (0=Sinclair 1, 1=Sinclair 2,
2=Cursor, 3=Kempston). Defaults to \fI3\fR.
.TP
It is possible to redefine keys to emulate a joystick.
.TP
.BI \-joyUp " \fI<keysym>\fR"
The key to emulate moving the joystick forwards.  Default is \fIq\fR.
.TP
.BI \-joyDown " \fI<keysym>\fR"
The key to emulate moving the joystick backwards.  Default is \fIa\fR.
.TP
.BI \-joyLeft " \fI<keysym>\fR"
The key to emulate moving the joystick to the left.  Default is \fIo\fR.
.TP
.BI \-joyRight " \fI<keysym>\fR"
The key to emulate moving the joystick to the right.  Default is \fIp\fR.
.TP
.BI \-joyFire " \fI<keysym>\fR"
The key to emulate pressing the joystick Fire button.  Default is
\fIspace\fR.
.PP
A full list of keysyms can be obtained by viewing the file
\fI<X11/keysymdef.h>\fR, and removing the \fIXK_\fR prefix from the
definitions listed there.
.PP
Note that it is possible for a key to emulate both a keyboard keypress and a
joystick movement.  No checks are made to ensure that the same key is not
mapped to two or more joystick directions.  It is also possible to fool
\fBXZX-Pro\fR into thinking that the joystick is being simultaneously moved in
opposite directions!
.PP
If \fBXZX-Pro\fR is running on Linux or FreeBSD with installed joystick kernel
patches, then it is possible to use a PC analogue or digital joystick to
emulate a joystick.  In this case, the following options also apply
in addition.
.TP
.BI \-joy " \fI<boolean>\fR"
Enables the usage of the joystick.  The joystick must have been calibrated
before.  Defaults to \fIfalse\fR.
.TP
.BI \-joyDev " \fI<path>\fR"
Use the device specified by \fIpath\fR to read the joystick status,
\fI@JOYDEV@\fR by default.  You may want to use \fI/dev/js1\fR, depending on
how your system is setup.
.TP
.BI \-joyCalibration " \fI<filename>\fR"
Load joystick calibration data from file \fI<filename>\fR. Defaults to
\fI.joystick\fR.
.TP
.BI \-joyAxisX " \fI<int>\fR"
The axis number the joystick driver returns for X axis. Defaults to \fI0\fR.
.TP
.BI \-joyAxisY " \fI<int>\fR"
The axis number the joystick driver returns for Y axis. Defaults to \fI1\fR.
.TP
.BI \-joyButtonFire " \fI<int>\fR"
The button number the joystick driver returns for fire button. Defaults to
\fI0\fR.
.TP
.BI \-joyButtonAutofire " \fI<int>\fR"
The button number the joystick driver returns for autofire button. Defaults to
\fI1\fR.
.TP
.BI \-joyButtonJuggle " \fI<int>\fR"
The button number the joystick driver returns for L-R-Juggle button. Defaults
to \fI2\fR.
.TP
.BI \-joyTolerance " \fI<int>\fR"
This specifies how far from center the joystick must be displaced before it
registers with \fBXZX-Pro\fR as being moved.  The default displacement
tolerance is 20.  You may have to experiment with this to get the best
response for your joystick.
.TP
.BI \-joyAutofire " \fI<int>\fR"
This specifies how many times in a second the pressing of the fire button
should be simulated.  Defaults to \fI25\fR.
.TP
.BI \-joyJuggle " \fI<int>\fR"
This specifies how many times in a second the pressing of the left and right
direction button should be simulated.  Defaults to \fI25\fR.
.\"
.SS Tape options
.\"
.TP
.BI \-tapeQuickload " \fI<boolean>\fR"
If \fItrue\fR, the emulator loads tape blocks in one block, rather than bit
by bit, if the LD_BYTES routine in the ROM is used.  Defaults to \fItrue\fR.
.TP
.BI \-tapeFastload " \fI<boolean>\fR"
Enable special speed mode while tape is loading. Defaults to \fIfalse\fR.
.TP
.BI \-tapeAutostop " \fI<boolean>\fR"
Stop tape playback after each block. Defaults to \fIfalse\fR.
.TP
.BI \-tapeNoise " \fI<boolean>\fR"
If \fIfalse\fR, then no sound while loading, otherwise you get an
approximation of the sound depending on how many reads from ULA are happening.
Having it switched on will require more CPU time.  Defaults to \fIfalse\fR.
.TP
.BI \-tapeQuicksave " \fI<boolean>\fR"
If \fItrue\fR, the emulator saves tape blocks in one block, rather than bit
by bit, if the SA_BYTES routine in the ROM is used.  Defaults to \fItrue\fR.
Defaults to \fItrue\fR.
.\"
.SS Debug options
.\"
.TP
.BI \-debug " \fI<n>\fR"
If debug code has been compiled in, specifies the level for debugging.  You
should not have any reason to mess with this one.  It just allows developers
to set the debug level and display debug information depending on what the
debug level is.  Defaults to \fI0\fR.
.TP
.BI \-core " \fI<boolean>\fR"
Enables the generation of core dumps.  Defaults to \fIfalse\fR.
.\"
.SH RUNNING THE EMULATOR
.\"
If \fBXZX-Pro\fR is started with no snapshot file as a command-line argument,
it will reset the machine and start either Sinclair BASIC (48K mode), or
display the start-up menu (128K and +3 modes).  If exactly one argument is
passed (not counting options), \fBXZX-Pro\fR will search each directory in
turn on its \fIlibDir\fR resource.  \fBXZX-Pro\fR is able to read and write
.SNA format snapshots and any .Z80 format snapshots (including .SLT format),
although only 3.05+ format .Z80 snapshots will be written.
.PP
Support for the original Spectrum keyboard is emulated as follows:

Left Shift emulates Caps Shift and this is guaranteed to work (only) for the
basic Spectrum keys, that is letters, numbers and Space.  Alt/Meta emulates
Symbol Shift, Control emulates Extend mode.  Also guaranteed for basic
Spectrum keys.  Return emulates Enter, Backspace and Delete emulate
CapsShift-0 (Rubout).  The cursor keys emulate CapsShift-5 to 8, or plain 5 to
8 (Cursor Joystick).  The Tab key toggles between the two.  Escape emulates
CapsShift-1 (Edit).  Right Shift and other local modifier keys(*) work
locally: The character printed on your key is emulated wherever possible.
This actually goes for all "non-Spectrum" keys with or without local shift.

The keyboard emulation is not perfect and it can get confused at times.  If
you think this has happened, try pressing Bar or Backslash which clears all
Spectrum input ports.

The emulator turns keyboard autorepeat off when the mouse pointer is in the
\fBXZX-Pro\fR window.

(*) like some "Alt" keys that are used to get extra characters.  On both DEC
and Sun (when I tried), this key is not decoded as Alt or Meta but as
Mode_switch.
.\"
.SH EXTENSIONS AND ALTERATIONS TO THE BASIC
.\"
.PP
The BASIC LOAD and SAVE commands will behave differently.  Instead of trying
to read or write data to a tape, SAVE will immediately dump data to a tape
file on disk, and LOAD will try and read data from a tape file on disk.
.PP
If emulation of the Interface I has been compiled in, the emulator behaves as
though an Interface I is attached in terms of paging the shadow ROM in and
out, allowing the use of extended basic commands and emulating the Microdrive
hardware.
.PP
The microdrive cartridge file format used is the .MDR format used by several
other Spectrum emulators.
.PP
The +3 floppy disk format used is identical to that of Marco Vieth's Amstrad
CPC emulator.  Disk images should therefore be compatible between the two
emulators, though I have not tested this personally.
.\"
.SH BUGS AND DEFICIENCIES
.\"
See latest TODO list.
.\"
.SH AUTHOR
.\"
Erik Kunze <xzx-pro@zx-spectrum.net>.
