/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: ddrive.c,v 4.20 2006/12/22 14:28:54 erik Rel $";
#endif
#include <stdlib.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "ui/sbar.h"
#include "fileio/dsk.h"
#ifdef XZX_TRDOS
#include "fileio/trd.h"
#include "fileio/fdi.h"
#include "fileio/scl.h"
#ifdef VIRTUAL_DISK_IMAGES
#include "fileio/hobeta.h"
#endif
#endif
#ifdef XZX_PLUSD
#include "fileio/mgt.h"
#include "fileio/img.h"
#endif
#ifdef XZX_DIDAKTIK
#include "fileio/d80.h"
#endif
#ifdef REAL_DISKS
#include "fileio/floppy.h"
#endif
#include "ddrive.h"
#define DDRIVE_LED_ON()	SbarSetLed(LED_ON)
#define DDRIVE_LED_OFF()	SbarSetLed(LED_OFF)
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_FDC) != 0) { x } }
#else
#define DEB(x)
#endif
static DiskVtable dummyVtable;Ddrive*DdriveNew(DdriveType type,int heads,int tracks){Ddrive*ddp=Malloc(sizeof(Ddrive),"DdriveNew");ddp->type=type;ddp->heads=heads;ddp->tracks=tracks;ddp->vtable=&dummyVtable;return ddp;}int DdriveInsertDisk(Ddrive*ddp,const char*filename){static struct{FileType type;int(*func)(Ddrive*,const char*);}type2func[]={
#ifdef XZX_TRDOS
{FILE_TYPE_TRD,TrdOpen},{FILE_TYPE_FDI,FdiOpen},{FILE_TYPE_SCL,SclOpen},
#ifdef VIRTUAL_DISK_IMAGES
{FILE_TYPE_HOB,HobetaOpen},
#endif
#endif
#ifdef XZX_PLUSD
{FILE_TYPE_MGT,MgtOpen},{FILE_TYPE_IMG,ImgOpen},
#endif
#ifdef XZX_DIDAKTIK
{FILE_TYPE_D80,D80Open},
#endif
#ifdef REAL_DISKS
{FILE_TYPE_UNKNOWN,FloppyOpen},
#endif
{FILE_TYPE_DSK,DskOpen}};FileType type;int i;DdriveEjectDisk(ddp);type=GetFileType(filename);for(i=0;i<NELEM(type2func);i++){if(type==type2func[i].type){return(type2func[i].func(ddp,filename));}}return-1;}void DdriveEjectDisk(Ddrive*ddp){if(ddp->vtable!=&dummyVtable){if(ddp->vtable->close){ddp->vtable->close(ddp);}ddp->vtable=&dummyVtable;}}void DdriveMotor(Ddrive*ddp,unsigned int state){if(state!=ddp->motor){DEB(Msg(M_DEBUG,"DD: drive %d motor %s",ddp->id,state==0?"off":"on"););if(state==0){DDRIVE_LED_OFF();}else{}ddp->motor=state;}}DdriveStatus DdriveSeek(Ddrive*ddp,int track){DEB(Msg(M_DEBUG,"DD: drive %d seek %d->%d",ddp->id,ddp->track,track););DDRIVE_LED_ON();ddp->sector=0;if(ddp->vtable->seek){return(ddp->vtable->seek(ddp,track));}ddp->track=track;if(ddp->track>=ddp->tracks){ddp->track=ddp->tracks-1;return DDRIVE_STATUS_SEEKFAIL;}else if(ddp->track<0){ddp->track=0;return DDRIVE_STATUS_SEEKFAIL;}return DDRIVE_STATUS_OK;}DdriveStatus DdriveReadId(Ddrive*ddp,DiskSectorInfo*sip){DEB(Msg(M_DEBUG,"DD: drive %d read id %d:%d",ddp->id,ddp->track,ddp->head););DDRIVE_LED_ON();ddp->sector++;if(ddp->vtable->readId){return(ddp->vtable->readId(ddp,ddp->track,ddp->head,ddp->sector,sip));}return DDRIVE_STATUS_NOTREADY;}uns8*DdriveReadSector(Ddrive*ddp,int sector){uns8*sectorp=(uns8*)0;DEB(Msg(M_DEBUG,"DD: drive %d read sector %d:%d:%d",ddp->id,ddp->track,ddp->head,sector););DDRIVE_LED_ON();if(ddp->vtable->readSector){sectorp=ddp->vtable->readSector(ddp,ddp->track,ddp->head,sector);}return sectorp;}uns8*DdriveWriteSector(Ddrive*ddp,int sector){uns8*sectorp=(uns8*)0;DEB(Msg(M_DEBUG,"DD: drive %d write sector %d:%d:%d",ddp->id,ddp->track,ddp->head,sector););DDRIVE_LED_ON();if(ddp->vtable->writeSector){sectorp=ddp->vtable->writeSector(ddp,ddp->track,ddp->head,sector);}return sectorp;}int DdriveIsReady(Ddrive*ddp){DEB(Msg(M_DEBUG,"DD: drive %d is ready",ddp->id););if(ddp->vtable==&dummyVtable&&CFGGET(smartAttach)!=0){assert(ddp->insert!=(int(*)(int))0);ddp->insert(ddp->id);}if(ddp->vtable->isReady){return(ddp->vtable->isReady(ddp));}return 0;}
