/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: emul.c,v 4.65 2006/12/22 14:28:54 erik Rel $";
#endif
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/time.h>
#include <assert.h>
#include "z80/z80.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "event.h"
#include "ui/sbar.h"
#include "ui/screen.h"
#include "sound/sound.h"
#include "ui/ui.h"
#ifdef HAVE_VNCSERVER
#include "netplay/vncserver.h"
#endif
#include "ui/xkey.h"
#include "ui/xwin.h"
#include "emul.h"
#define TIMER_INTERVAL	100	
#define UPDATE_INTERVAL	2000	
#ifdef DEBUG_EVENTS
#define DEB(x)	{ x }
#else
#define DEB(x)
#endif
static void emulTimer(int);static void emulTimerHandler(unsigned int);static void emulInterruptHandler(unsigned int);static void emulHsyncHandler(unsigned int);static void emulVsyncHandler(unsigned int);static unsigned int videoLine;static volatile unsigned int timerInterrupt;static volatile unsigned int tstatesPerTimer;static volatile unsigned int tstatesPerUpdate;static unsigned int fps;static sigset_t smask;void EmulLoop(void){struct sigaction sigact;struct itimerval itv;unsigned int tstates;(void)sigfillset(&smask);(void)sigdelset(&smask,SIGALRM);sigact.sa_handler=emulTimer;(void)sigemptyset(&sigact.sa_mask);sigact.sa_flags=SA_RESTART;(void)sigaction(SIGALRM,&sigact,(struct sigaction*)0);itv.it_interval.tv_sec=0;itv.it_interval.tv_usec=TIMER_INTERVAL*1000;itv.it_value.tv_sec=0;itv.it_value.tv_usec=TIMER_INTERVAL*1000;(void)setitimer(ITIMER_REAL,&itv,(struct itimerval*)0);tstates=Machine->tstatesPerFrame;EventAdd((EventFunc)emulVsyncHandler,(void*)tstates,tstates);tstates=tstatesPerTimer;EventAdd((EventFunc)emulTimerHandler,(void*)0,tstates);tstates=Machine->intLine*Machine->tstatesPerLine;EventAdd((EventFunc)emulInterruptHandler,(void*)tstates,tstates);for(;;){DEB(Msg(M_DEBUG,"z80 exec : tstates %5u",EventNextTstates););(void)Z80Execute(&EventNextTstates);EventSchedule();}}static void emulTimer(int signo){USED(signo);if(UiIsActive()==0){timerInterrupt++;}}static void emulTimerHandler(unsigned int arg){static int ipu=UPDATE_INTERVAL/TIMER_INTERVAL;static unsigned int tpu=0;USED(arg);tpu+=tstatesPerTimer;if(CFGGET(fastmode)==0){if(timerInterrupt==0){(void)sigsuspend(&smask);}}if((ipu-=timerInterrupt)<=0){SbarSetSpeed(tpu/tstatesPerUpdate,fps/(UPDATE_INTERVAL/1000));ipu+=UPDATE_INTERVAL/TIMER_INTERVAL;tpu=0;fps=0;}timerInterrupt=0;EventAdd((EventFunc)emulTimerHandler,(void*)0,TSTATES+tstatesPerTimer);}static void emulInterruptHandler(unsigned int arg){DEB(Msg(M_DEBUG,"inter: %6u (%6u)",arg,TSTATES););Z80Int(0xff,arg);}static void emulHsyncHandler(unsigned int arg){DEB(Msg(M_DEBUG,"hsync: l%6u, %6u (%6u)",videoLine,arg,TSTATES););ScreenUpdateLine(videoLine);if(++videoLine<(BORDER_LINES+Y_PIXELS+BORDER_LINES)){unsigned int tstates=(int)arg+Machine->tstatesPerLine;EventAdd((EventFunc)emulHsyncHandler,(void*)tstates,tstates);}}static void emulVsyncHandler(unsigned int arg){static int flashCount=16,refreshCount=1;unsigned int tstates;DEB(Msg(M_DEBUG,"vsync: %6u (%6u)",arg,TSTATES););videoLine=0;SoundPlayBuffer();assert(arg==Machine->tstatesPerFrame&&TSTATES>=arg);TSTATES-=arg;EventAdjust(arg);XwinProcessEvents();
#ifdef HAVE_VNCSERVER
VncServerProcessEvents();
#endif
XkeyRefresh();if(--flashCount==0){flashCount=16;ScreenDoFlashing();}if(--refreshCount==0){refreshCount=CFGGET(fastmode)==0?CFGGET(refresh):10;ScreenRefresh();fps++;}tstates=Machine->tstatesPerFrame;EventAdd((EventFunc)emulVsyncHandler,(void*)tstates,tstates);if(refreshCount==1){tstates=Machine->tstatesPerLine*(Machine->topBorder-BORDER_LINES);EventAdd((EventFunc)emulHsyncHandler,(void*)tstates,tstates);}tstates=Machine->intLine*Machine->tstatesPerLine;EventAdd((EventFunc)emulInterruptHandler,(void*)tstates,tstates);}void EmulSetSpeed(unsigned int speed){double d;d=(double)Machine->tstatesPerFrame*(double)speed/2.0;d/=(double)(1000/TIMER_INTERVAL);tstatesPerTimer=(unsigned int)(d+0.5);d=(double)Machine->tstatesPerFrame*50.0;d*=(double)(UPDATE_INTERVAL/1000);d/=100.0;tstatesPerUpdate=(unsigned int)(d+0.5);SoundControl(SOUND_ACTION_INIT);}
