/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: dsk.c,v 4.22 2006/12/22 14:28:57 erik Rel $";
#endif
#include <stdlib.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "ddrive.h"
#include "fileio/dsk.h"
typedef struct _dskSectorInfo{uns8 track;uns8 head;uns8 sector;uns8 bps;uns8 st1;uns8 st2;uns8 sectorsize[2];}dskSectorInfo;typedef struct _dskTrackInfo{uns8 magic[13];uns8 unused[3];uns8 track;uns8 head;uns8 unused2[2];uns8 bps;uns8 spt;uns8 gap3;uns8 fill;dskSectorInfo si[29];}dskTrackInfo;typedef struct _dskDiskInfo{uns8 magic[34];uns8 creator[14];uns8 tracks;uns8 sides;uns8 tracksize[2];uns8 tracksizes[204];}dskDiskInfo;
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_DISK) != 0) { x } }
#else
#define DEB(x)
#endif
static void dskClose(Ddrive*);static DdriveStatus dskReadId(Ddrive*,int,int,int,DiskSectorInfo*);static uns8*dskReadSector(Ddrive*,int,int,int);static uns8*dskWriteSector(Ddrive*,int,int,int);static int dskIsReady(Ddrive*);static DdriveStatus dskLookupTrack(Ddrive*,int,int);static long dskLookupSector(Ddrive*,int);static DiskVtable dskVtable={dskClose,0,dskReadId,dskReadSector,dskWriteSector,dskIsReady};int DskOpen(Ddrive*ddp,const char*filename){int readonly;size_t size;uns8*mmp;readonly=0;if((mmp=Mmap(filename,1,&readonly,&size))!=(uns8*)-1){if(size>(sizeof(dskDiskInfo)+sizeof(dskTrackInfo))){dskDiskInfo*dip=(dskDiskInfo*)mmp;Disk*dp=&(ddp->disk);DiskFileDsk*dfp=&(dp->File.dsk);if(memcmp(dip->magic,"MV - CPC",8)==0){dfp->extended=0;}else if(memcmp(dip->magic,"EXTENDED",8)==0){dfp->extended=1;readonly=1;}else{goto error;}dp->filename=filename;dp->readonly=readonly;dfp->mmp=mmp;dfp->mmsize=size;ddp->vtable=&dskVtable;DEB(dskTrackInfo*tip=(dskTrackInfo*)(mmp+sizeof(dskDiskInfo));Msg(M_DEBUG,"DSK: %s format",dfp->extended==0?"standard":"extended");Msg(M_DEBUG,"     %d sides, %d tracks, %d sectors, %d bytes, read%s",dip->sides,dip->tracks,tip->spt,128<<tip->bps,readonly==0?"write":"only"););return 0;}error:Msg(M_WARN,"disk image is not in DSK format");Munmap(mmp,size);}return-1;}static void dskClose(Ddrive*ddp){Disk*dp=&(ddp->disk);DiskFileDsk*dfp=&(dp->File.dsk);Munmap(dfp->mmp,dfp->mmsize);free((void*)dp->filename);dp->filename=(const char*)0;}static DdriveStatus dskReadId(Ddrive*ddp,int track,int head,int sector,DiskSectorInfo*sip){DdriveStatus status=DDRIVE_STATUS_OK;DEB(Msg(M_DEBUG,"DSK: read id %d:%d:%d",track,head,sector););switch(dskLookupTrack(ddp,track,head)){case DDRIVE_STATUS_OK:{dskTrackInfo*tip=(dskTrackInfo*)ddp->disk.File.dsk.trackInfo;dskSectorInfo*dsip;sector%=tip->spt;dsip=(dskSectorInfo*)&(tip->si[sector]);sip->track=dsip->track;sip->head=dsip->head;sip->sector=dsip->sector;sip->bps=dsip->bps;sip->crc1=dsip->st1;sip->crc2=dsip->st2;}break;case DDRIVE_STATUS_SEEKFAIL:status=DDRIVE_STATUS_SEEKFAIL;sector%=9;break;case DDRIVE_STATUS_NODATA:status=DDRIVE_STATUS_NOADDR;sector%=9;break;default:assert(0);break;}ddp->sector=sector;return status;}static uns8*dskReadSector(Ddrive*ddp,int track,int head,int sector){uns8*sectorp=(uns8*)0;long offs;USED(track);USED(head);DEB(Msg(M_DEBUG,"DSK: read sector %d:%d:%d",track,head,sector););if((offs=dskLookupSector(ddp,sector))>=0){sectorp=ddp->disk.File.dsk.trackInfo+offs;}return sectorp;}static uns8*dskWriteSector(Ddrive*ddp,int track,int head,int sector){uns8*sectorp=(uns8*)0;long offs;USED(track);USED(head);DEB(Msg(M_DEBUG,"DSK: write sector %d:%d:%d",track,head,sector););if((offs=dskLookupSector(ddp,sector))>=0){sectorp=ddp->disk.File.dsk.trackInfo+offs;}return sectorp;}static int dskIsReady(Ddrive*ddp){int ready=ddp->motor!=0?1:0;DEB(Msg(M_DEBUG,"DSK: is ready: %d",ready););return ready;}static DdriveStatus dskLookupTrack(Ddrive*ddp,int track,int head){DiskFileDsk*dfp=&(ddp->disk.File.dsk);dskDiskInfo*dip=(dskDiskInfo*)dfp->mmp;dskTrackInfo*tip;long offs=0;int t=track;if(track>=ddp->tracks||head>=ddp->heads){return DDRIVE_STATUS_SEEKFAIL;}
#ifdef DDRIVE_DOUBLE_SPACING
if(dip->tracks<44&&ddp->tracks>=80){t/=2;}
#endif
if(dip->sides>1){t*=2;}t+=head;if(dfp->extended==1){int i;for(i=0;i<t;i++){offs+=dip->tracksizes[i]<<8;}}else{offs=(long)Le2Cpu(dip->tracksize,2);offs*=t;}offs+=sizeof(dskDiskInfo);if((size_t)offs+sizeof(dskTrackInfo)>=dfp->mmsize){DEB(Msg(M_DEBUG,"DSK: truncated input file"););return DDRIVE_STATUS_NODATA;}tip=(dskTrackInfo*)(dfp->mmp+offs);if(memcmp(tip->magic,"Track-Info\r\n",sizeof(tip->magic)-1)!=0){DEB(Msg(M_DEBUG,"DSK: track %d header not found",track););return DDRIVE_STATUS_NODATA;}if(tip->track!=track||tip->head!=head){DEB(Msg(M_DEBUG,"DSK: track %d not formatted",track););return DDRIVE_STATUS_NODATA;}dfp->trackInfo=(uns8*)tip;return DDRIVE_STATUS_OK;}static long dskLookupSector(Ddrive*ddp,int sector){DiskFileDsk*dfp=&(ddp->disk.File.dsk);dskTrackInfo*tip=(dskTrackInfo*)dfp->trackInfo;dskSectorInfo*sip=tip->si;long offs=sizeof(dskTrackInfo),seclen;int i;if(dfp->extended==1){for(i=0;i<tip->spt;i++){seclen=(long)Le2Cpu(sip->sectorsize,2);if(sip->sector==sector){return offs;}offs+=seclen;sip++;}}else{seclen=128<<tip->bps;for(i=0;i<tip->spt;i++){if(sip->sector==sector){return offs;}offs+=seclen;sip++;}}return-1;}
