/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: slt.c,v 4.15 2006/12/22 14:28:58 erik Rel $";
#endif
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "fileio/z80.h"
#include "fileio/slt.h"
typedef enum{SLT_BLOCK_EOT=0x00,SLT_BLOCK_LEVEL,SLT_BLOCK_INSTRUCTIONS,SLT_BLOCK_SCREEN,SLT_BLOCK_PICTURE,SLT_BLOCK_POKES}SltBlockType;
#define SLT_SIGNATURE	"\0\0\0SLT"
#define SLT_SIGNATURE_SIZE	6
#define SLT_ENTRY_SIZE	8
typedef struct _sltInfo{SltBlockType type;unsigned int identifier;unsigned int length;}sltInfo;typedef struct _sltData{off_t offsHeader,offsData;sltInfo*info;}sltData;
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_SNAP) != 0) { x } }
#else
#define DEB(x)
#endif
static sltData sltDatas;int SltRead(const char*filename){int ret=-1;if(Z80Read(filename)==0){int readonly;size_t size;void*mmp;readonly=1;if((mmp=Mmap(filename,1,&readonly,&size))!=(void*)-1){uns8*scanBegin,*scanEnd,*scanPtr;scanBegin=(uns8*)mmp;scanEnd=(uns8*)mmp+size;for(scanPtr=scanBegin;scanPtr<scanEnd;scanPtr++){if(scanPtr[0]==SLT_SIGNATURE[0]&&scanPtr[1]==SLT_SIGNATURE[1]&&scanPtr[2]==SLT_SIGNATURE[2]&&scanPtr[3]==SLT_SIGNATURE[3]&&scanPtr[4]==SLT_SIGNATURE[4]&&scanPtr[5]==SLT_SIGNATURE[5]){break;}}if(scanPtr<scanEnd){sltData*sdp=&sltDatas;int blockcnt,blocknum;scanPtr+=SLT_SIGNATURE_SIZE;sdp->offsHeader=scanPtr-scanBegin;for(blockcnt=1;;blockcnt++){if(scanPtr+SLT_ENTRY_SIZE>scanEnd){Msg(M_ERR,"premature end on SLT snapshot");goto error;}if(scanPtr[0]==SLT_BLOCK_EOT&&scanPtr[1]==0x00){break;}scanPtr+=SLT_ENTRY_SIZE;}if(sdp->info!=(sltInfo*)0){free(sdp->info);}sdp->info=Malloc(sizeof(sltInfo)*(size_t)blockcnt,"SltRead");for(blocknum=0,scanPtr=scanBegin+sdp->offsHeader;blocknum<blockcnt;blocknum++,scanPtr+=SLT_ENTRY_SIZE){sdp->info[blocknum].type=(SltBlockType)((int)Le2Cpu(scanPtr,2));sdp->info[blocknum].identifier=Le2Cpu(scanPtr+2,2);sdp->info[blocknum].length=Le2Cpu(scanPtr+4,4);DEB(Msg(M_DEBUG,"block %2d, type %1d, id %2u, length %5u",blocknum,sdp->info[blocknum].type,sdp->info[blocknum].identifier,sdp->info[blocknum].length););}sdp->offsData=scanPtr-scanBegin;ret=0;}else{Msg(M_ERR,"couldn't find SLT signature");}error:Munmap(mmp,size);}}else{Msg(M_ERR,"SLT file does not contain a valid Z80 snapshot");}return ret;}int SltReadLevel(const char*filename,unsigned int level,unsigned int addr){sltData*sdp=&sltDatas;off_t disp;unsigned int which,remain;int ret=-1;assert(sdp->info!=(sltInfo*)0);for(which=0,disp=0;sdp->info[which].type!=SLT_BLOCK_EOT;which++){if(sdp->info[which].type==SLT_BLOCK_LEVEL&&sdp->info[which].identifier==level){break;}disp+=(off_t)(sdp->info[which].length);}if(sdp->info[which].identifier==level){int readonly;size_t size;void*mmp;readonly=1;if((mmp=Mmap(filename,1,&readonly,&size))!=(void*)-1){uns8*scanBegin,*scanEnd,*scanPtr;scanBegin=(uns8*)mmp;scanEnd=(uns8*)mmp+size;remain=sdp->info[which].length;scanPtr=scanBegin+sdp->offsData+disp;if(scanPtr+remain<=scanEnd){DEB(Msg(M_DEBUG,"reading level %2u of length %5u",sdp->info[which].identifier,sdp->info[which].length););while(scanPtr<scanEnd&&remain>0){if(*scanPtr==0xed &&*(scanPtr+1)==0xed){unsigned int count;for(count=*(scanPtr+2);count>0&&addr<65536;count--){MEM(addr)=*(scanPtr+3);addr++;}remain-=4;scanPtr+=4;}else{MEM(addr)=*scanPtr;addr++;remain--;scanPtr++;}}if(remain==0){ret=0;}else{Msg(M_PWARN,"error while loading level %u",level);}}else{Msg(M_ERR,"couldn't load level %u",level);}Munmap(mmp,size);}}else{Msg(M_WARN,"requested level %u doesn't exist",level);}return ret;}int SltWrite(const char*filename,const char*sltfilename){int ret=-1;if(Z80Write(filename)==0){FILE*ifp;if((ifp=Fopen(sltfilename,"rb"))!=(FILE*)0){FILE*ofp;if((ofp=Fopen(filename,"ab"))!=(FILE*)0){(void)putc(0,ofp);(void)putc(0,ofp);(void)putc(0,ofp);(void)putc('S',ofp);(void)putc('L',ofp);(void)putc('T',ofp);if(fseek(ifp,sltDatas.offsHeader,SEEK_SET)==0){int byte;while((byte=fgetc(ifp))!=EOF){if(putc(byte,ofp)==EOF){goto write_error;}}ret=0;}else{write_error:Msg(M_PERR,"couldn't write SLT levels");}(void)fclose(ofp);}(void)fclose(ifp);}}return ret;}
