/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: machine.c,v 4.46 2006/12/22 14:29:01 erik Rel $";
#endif
#include <string.h>
#include <assert.h>
#include "z80/z80.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#ifdef XZX_DIDAKTIK
#include "machine/didaktik.h"
#endif
#include "emul.h"
#ifdef XZX_IF1
#include "machine/if1.h"
#endif
#ifdef XZX_KMOUSE
#include "machine/kmouse.h"
#endif
#include "ui/menu.h"
#ifdef XZX_MF
#include "machine/multiface.h"
#endif
#ifdef XZX_LIGHTGUN
#include "machine/lightgun.h"
#endif
#ifdef XZX_OPUS
#include "machine/opus.h"
#endif
#ifdef XZX_PLUSD
#include "machine/plusd.h"
#endif
#ifdef XZX_PENTAGON
#include "machine/pentagon.h"
#endif
#include "machine/plus3.h"
#include "ui/sbar.h"
#ifdef XZX_SCORPION
#include "machine/scorpion.h"
#endif
#include "machine/spec48.h"
#include "machine/spec128.h"
#ifdef XZX_TRDOS
#include "machine/trdos.h"
#endif
MachineSettings Machines[NMACHINES];MachineSettings*Machine;int MachinePageNumbers[8];uns8*MachineMemory[8];uns8*MachineAllMemory[NPAGES];void MachineInit(void){uns8*mp;int i;assert(CFGGET(machine)<NMACHINES);Machine=&Machines[CFGGET(machine)];Z80Init();mp=Malloc(sizeof(uns8)*(size_t)(XZX_PAGESIZE*NPAGES),"MemoryInit");for(i=0;i<NPAGES;i++,mp+=XZX_PAGESIZE){MachineAllMemory[i]=mp;}Spec48Init();Spec128Init();Plus3Init();
#ifdef XZX_PENTAGON
PentagonInit();
#endif
#ifdef XZX_SCORPION
ScorpionInit();
#endif
#ifdef XZX_DIDAKTIK
DidaktikInit();
#endif
#ifdef XZX_IF1
If1Init();
#endif
#ifdef XZX_OPUS
OpusInit();
#endif
#ifdef XZX_TRDOS
TrdosInit();
#endif
#ifdef XZX_PLUSD
PlusdInit();
#endif
#ifdef XZX_MF
MultifaceInit();
#endif
#ifdef XZX_LIGHTGUN
LightgunInit();
#endif
#ifdef XZX_KMOUSE
KmouseInit();
#endif
}void MachineSelect(int machId,int doReset){static const char*warnNotSupported="%s not supported by the %s, disabling";static const char*warnIfConflict="%s and %s not supported, disabling";MachineSettings*oldMachine=Machine;USED(warnNotSupported);USED(warnIfConflict);assert(machId<NMACHINES);CFGSET(machine,machId);Machine=&Machines[machId];SbarSetModel(Machine->name);EmulSetSpeed(CFGGET(speed));
#ifdef XZX_IF1
if(CFGGET(if1)>0&&machId>SP_128){Msg(M_WARN,warnNotSupported,"Interface I",Machine->name);CFGSET(if1,0);MenuFileCartridgeUpdate();}
#endif 
#ifdef XZX_OPUS
if(CFGGET(opus)>0&&machId>SP_128){Msg(M_WARN,warnNotSupported,"Opus Discovery",Machine->name);CFGSET(opus,0);}
#ifdef XZX_IF1
if(CFGGET(opus)>0&&CFGGET(if1)>0){Msg(M_WARN,warnIfConflict,"Opus Discovery","Interface");CFGSET(opus,0);}
#endif
#endif 
#ifdef XZX_TRDOS
if(CFGGET(trdos)>0&&machId>SP_128){Msg(M_WARN,warnNotSupported,"Beta 128",Machine->name);CFGSET(trdos,0);}
#ifdef XZX_OPUS
if(CFGGET(trdos)>0&&CFGGET(opus)){Msg(M_WARN,warnIfConflict,"Beta 128","Opus Discovery");CFGSET(trdos,0);}
#endif
#ifdef XZX_MF
if(CFGGET(trdos)>0&&(machId<=SP_48_2?CFGGET(mf1):CFGGET(mf128))>0){Msg(M_WARN,warnIfConflict,"Beta 128","Multiface");CFGSET(mf1,0);}
#endif
#endif 
#ifdef XZX_PLUSD
if(CFGGET(plusd)>0&&machId>SP_128){Msg(M_WARN,warnNotSupported,"+D",Machine->name);CFGSET(plusd,0);}
#ifdef XZX_IF1
if(CFGGET(plusd)>0&&CFGGET(if1)>0){Msg(M_WARN,warnIfConflict,"+D","Interface I");CFGSET(plusd,0);}
#endif
#ifdef XZX_OPUS
if(CFGGET(plusd)>0&&CFGGET(opus)>0){Msg(M_WARN,warnIfConflict,"+D","Opus Discovery");CFGSET(plusd,0);}
#endif
#ifdef XZX_MF
if(CFGGET(trdos)>0&&(machId<=SP_48_2?CFGGET(mf1):CFGGET(mf128))>0){Msg(M_WARN,warnIfConflict,"+D","Multiface");CFGSET(plusd,0);}
#endif
#endif 
#ifdef XZX_LIGHTGUN
if(CFGGET(lightgun)>0&&machId!=SP_128&&machId!=SP_3){Msg(M_WARN,warnNotSupported,"Lightgun",Machine->name);LightgunOff();}
#endif
#ifdef XZX_KMOUSE
if(CFGGET(kmouse)>0&&!PENTAGON_OR_SCORPION){Msg(M_WARN,warnNotSupported,"Kempston Mouse",Machine->name);KmouseOff();}
#endif
MenuFileDiskUpdate();if(doReset==0){Machine->initPages();Machine->screen=oldMachine->screen;Machine->lastUla=oldMachine->lastUla;Machine->lastMem128=oldMachine->lastMem128;Machine->lastMem3=oldMachine->lastMem3;Machine->ay.current=oldMachine->ay.current;(void)memcpy(Machine->ay.registers,oldMachine->ay.registers,sizeof(Machine->ay.registers));}else{Machine->reset();}}int MachineLoadRom(const char*image,int page,int num){FILE*ifp;size_t nread;if((ifp=Fopen(image,"rb"))==(FILE*)0){Msg(M_PERR,"couldn't open ROM image <%s> for reading",image);return-1;}while(num-->0){nread=fread(MachineAllMemory[page++],1,XZX_PAGESIZE,ifp);if(nread!=XZX_PAGESIZE&&nread!=XZX_PAGESIZE/2){Msg(M_WARN,"ROM image <%s> is too short",image);break;}}(void)fclose(ifp);return 0;}int MachineChecksumRom(int rom,int length){int checksum=0,tmp,i;for(i=0;i<length;i++){tmp=checksum&0x01;checksum>>=1;if(tmp){checksum|=0x8000;}checksum^=MachineAllMemory[rom][i];}return checksum;}uns8 MachineDummyInByte(void){return 0xff;}void MachineDummyOutByte(uns8 val){USED(val);}
