/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifndef _XZX_MACHINE_MACHINE_H
#define _XZX_MACHINE_MACHINE_H
#include "machine/ay.h"
#define P_ULA	0xfe	
#define B_ULA	0x01	
#define B_BORDER	0x07	
#define B_MIC	0x08	
#define B_SPEAKER	0x10	
#define B_KEYBOARD	0x1f	
#define B_EAR	0x40	
#define P_128	0x00fd	
#define B_128	0x0002	
#define P_SP128_BANK	0x7ffd	
#define B_SP128_BANK	0x8002	
#define B_SP128_SELRAM	0x07	
#define B_SP128_SELSCREEN	0x08	
#define B_SP128_SELROM	0x10	
#define B_SP128_PAGING	0x20	
#define PAGING_SP128_DISABLED(port)	(((port) & B_SP128_PAGING) != 0)
#define P_SND_CONTROL	0xfffd	
#define B_SND_CONTROL	0xc002	
#define P_SND_DATA	0xbffd	
#define B_SND_DATA	0xc002	
#define P_SP3_BANK	0x1ffd	
#define B_SP3_BANK	0xc002	
#define P_SP3_CENTRONICS	0x0ffd	
#define B_SP3_CENTRONICS	0xf002	
#define P_SP3_FDCCONTROL	0x2ffd	
#define B_SP3_FDCCONTROL	0xf002	
#define P_SP3_FDCDATA	0x3ffd	
#define B_SP3_FDCDATA	0xf002	
#define P_ZXPRINTER	0xfb	
#define B_ZXPRINTER	0x04	
#define P_IF1_MDDATA	0xe7	
#define B_IF1_MDDATA	0x18	
#define P_IF1_MDCONTROL	0xef	
#define B_IF1_MDCONTROL	0x18	
#define P_IF1_NETWORK	0xf7	
#define B_IF1_NETWORK	0x18	
#define P_MF1_IN	0x9f	
#define P_MF1_OUT	0x1f	
#define P_MF128_IN	0xbf	
#define P_MF128_IN_V2	0x9f	
#define P_MF128_OUT	0x3f	
#define P_MF3_IN	0x3f	
#define P_MF3_OUT	0xbf	
#define P_MF3_BUTTON	0x3f	
#define P_MF3_P7FFD	0x7f3f	
#define P_MF3_P1FFD	0x1f3f	
#define P_SINCLAIR1	0xeffe	
#define B_SINCLAIR1	0x1001	
#define P_SINCLAIR2	0xf7fe	
#define B_SINCLAIR2	0x0801	
#define P_KEMPSTON	0x1f	
#define B_KEMPSTON	0xe0	
#define P_KMOUSE_BUTTONS	0xfadf	
#define B_KMOUSE_BUTTONS	0xffff	
#define P_KMOUSE_X	0xfbdf	
#define B_KMOUSE_X	0xffff	
#define P_KMOUSE_Y	0xffdf	
#define B_KMOUSE_Y	0xffff	
#define P_FULLER_CONTROL	0x3f	
#define P_FULLER_DATA	0x5f	
#define P_FULLER_JOY	0x7f	
#define P_TRDOS_CMD	0x1f	
#define P_TRDOS_STATE	0x1f	
#define P_TRDOS_TRACK	0x3f	
#define P_TRDOS_SECTOR	0x5f	
#define P_TRDOS_DATA	0x7f	
#define P_TRDOS_SYSTEM	0xff	
#define P_PLUSD_CMD	0xe3	
#define P_PLUSD_STATE	0xe3	
#define P_PLUSD_PAGE	0xe7	
#define P_PLUSD_TRACK	0xeb	
#define P_PLUSD_SYSTEM	0xef	
#define P_PLUSD_SECTOR	0xf3	
#define P_PLUSD_PRINTER	0xf7	
#define P_PLUSD_DATA	0xfb	
#define P_D80_CMD	0x81	
#define P_D80_STATE	0x81	
#define P_D80_TRACK	0x83	
#define P_D80_SECTOR	0x85	
#define P_D80_DATA	0x87	
#define P_D80_SYSTEM	0x89	
#define P_LPRINT_IN	0xfb	
#define B_LPRINT_IN	0x84	
#define P_LPRINT_OUT	0x7b	
#define B_LPRINT_OUT	0x84	
#define P_GRAFPAD_PEN	0xff3f	
#define P_GRAFPAD_X	0xffbf	
#define P_GRAFPAD_Y	0xff7f	
typedef enum{SP48ROM,SP128ROM0,SP128ROM1,SP3ROM0,SP3ROM1,SP3ROM2,SP3ROM3,
#ifdef XZX_PENTAGON
PENTROM0,PENTROM1,PENTROM2,
#endif
#ifdef XZX_SCORPION
SCORROM0,SCORROM1,SCORROM2,SCORROM3,
#endif
#ifdef XZX_DIDAKTIK
DIDROM,D80ROM,
#endif
#ifdef XZX_IF1
IF1ROM,
#endif
#ifdef XZX_MF
MF1ROM,MF128ROM,MF3ROM,
#endif
#ifdef XZX_TRDOS
TRDOSROM,
#endif
#ifdef XZX_PLUSD
PLUSDROM,
#endif
RAM0,RAM1,RAM2,RAM3,RAM4,RAM5,RAM6,RAM7,
#ifdef XZX_SCORPION
RAM8,RAM9,RAMA,RAMB,RAMC,RAMD,RAME,RAMF,
#endif
NPAGES}MachinePage;
#define MFRAM	RAM0
#define PLUSDRAM	RAM0
#define XZX_PAGESIZE	16384	
#define VPAGE(addr)	((addr) >> 13)
#define RPAGE(addr)	(MachinePageNumbers[VPAGE(addr)])
#define MEM(addr)	(MachineMemory[VPAGE(addr)][(addr) & 0x1fff])
#define READ_ONLY(addr)	(RPAGE(addr) < RAM0)
typedef struct _machineTrap{int page;unsigned int addr;}MachineTrap;typedef struct _machineSettings{const char*name;int basicRom;unsigned int tvLines;unsigned int tstatesPerLine;unsigned int tstatesPerFrame;unsigned int intLine;unsigned int topBorder;unsigned char*ulaDelay;struct{MachineTrap load;MachineTrap save;}traps;void(*reset)(void);void(*nmi)(void);uns8(*readOpcode)(uns16);uns8(*readMem)(uns16);void(*writeMem)(uns16,uns8);uns8(*inPort)(uns16);void(*outPort)(uns16,uns8);void(*initPages)(void);void(*pageIn)(int,int);uns8*screen;uns8 lastUla;uns8 lastMem128;uns8 lastMem3;Ay ay;}MachineSettings;typedef enum{SP_48_2,SP_48_3,SP_128,SP_3,
#ifdef XZX_PENTAGON
SP_PENTAGON,
#endif
#ifdef XZX_SCORPION
SP_SCORPION,
#endif
#ifdef XZX_DIDAKTIK
SP_DIDAKTIK,
#endif
NMACHINES}MachineId;extern void MachineInit(void);extern void MachineSelect(int,int);extern int MachineLoadRom(const char*,int,int);extern int MachineChecksumRom(int,int);extern uns8 MachineDummyInByte(void);extern void MachineDummyOutByte(uns8);extern MachineSettings Machines[NMACHINES],*Machine;extern uns8*MachineMemory[8];extern int MachinePageNumbers[8];extern uns8*MachineAllMemory[NPAGES];
#endif 

