/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef XZX_PENTAGON
#ifndef lint
static char rcsid[]="$Id: pentagon.c,v 4.58 2006/12/22 14:29:01 erik Rel $";
#endif
#include <assert.h>
#include "z80/z80.h"
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "keyboard.h"
#include "machine/ay.h"
#include "machine/kjoy.h"
#ifdef XZX_KMOUSE
#include "machine/kmouse.h"
#endif
#include "ui/screen.h"
#include "sound/sound.h"
#include "tables.h"
#include "tape.h"
#include "machine/trdos.h"
#include "machine/pentagon.h"
#define Y_INTERRUPT	304
#define Y_TOP_BORDER	64
#define Y_SCREEN	192
#define Y_BOTTOM_BORDER	48
#define Y_RETRACE	16
#define X_LEFT_BORDER	36
#define X_SCREEN	128
#define X_RIGHT	28
#define X_RETRACE	32
#define ULA_DRAWS_SCREEN	0x00
#define ULA_DRAWS_BORDER	0x20
#define ULA_HORIZONTAL_RETRACE	0x40
#define ULA_VERTICAL_RETRACE	0x80
#define P_BANK128	P_SP128_BANK
#define B_BANK128	B_SP128_BANK
#define B_SELROM	B_SP128_SELROM
#define B_SELRAM	B_SP128_SELRAM
#define B_SELSCREEN	B_SP128_SELSCREEN
#define PAGING_DISABLED	PAGING_SP128_DISABLED
enum{ROM_UNKNOWN,ROM_PENTAGON};
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_IO) != 0) { x } }
#else
#define DEB(x)
#endif
static void pentagonReset(void);static void pentagonNmi(void);static uns8 pentagonReadOpcode(uns16);static uns8 pentagonReadMem(uns16);static void pentagonWriteMem(uns16,uns8);static void pentagonInitPages(void);static void pentagonPageIn(int,int);static inline void pentagonOutBankm128(uns8);static void pentagonOutPort(uns16,uns8);static uns8 pentagonInPort(uns16);void PentagonInit(void){MachineSettings*msp;int checksum,romtype;unsigned int i,x,y;msp=&Machines[SP_PENTAGON];msp->name="Pentagon 128";msp->basicRom=PENTROM1;msp->tvLines=Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER+Y_RETRACE;msp->tstatesPerLine=X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;msp->tstatesPerFrame=msp->tvLines*msp->tstatesPerLine;msp->intLine=Y_INTERRUPT;msp->topBorder=Y_TOP_BORDER;msp->ulaDelay=Malloc(sizeof(unsigned char)*(size_t)msp->tstatesPerFrame,"PentagonInit");msp->reset=pentagonReset;msp->nmi=pentagonNmi;msp->readOpcode=pentagonReadOpcode;msp->readMem=pentagonReadMem;msp->writeMem=pentagonWriteMem;msp->inPort=pentagonInPort;msp->outPort=pentagonOutPort;msp->initPages=pentagonInitPages;msp->pageIn=pentagonPageIn;msp->screen=MachineAllMemory[RAM5];msp->ay.type=AY_TYPE_YM2149F;if(MachineLoadRom(CFGGET(pentagonRom),PENTROM0,3)==0){checksum=MachineChecksumRom(msp->basicRom,XZX_PAGESIZE);switch(checksum){case 44809:romtype=ROM_PENTAGON;break;default:Msg(M_WARN,"unknown Pentagon ROM image - checksum %u",checksum);romtype=ROM_UNKNOWN;break;}if(romtype!=ROM_UNKNOWN){msp->traps.load.page=msp->basicRom;msp->traps.load.addr=0x563;msp->traps.save.page=msp->basicRom;msp->traps.save.addr=0x4c7;}}for(i=0,y=0;y<Y_TOP_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_TOP_BORDER+Y_SCREEN;y++){for(x=0;x<X_LEFT_BORDER;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN;x++){msp->ulaDelay[i++]=ULA_DRAWS_SCREEN;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER+Y_RETRACE;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_VERTICAL_RETRACE;}}assert(i==msp->tstatesPerFrame);}static void pentagonReset(void){Z80Reset();Machine->initPages();Machine->lastMem128=0;Machine->screen=MachineAllMemory[RAM5];AyReset();}static void pentagonNmi(void){Z80Nmi((NmiCallbackFunc)0);Machine->pageIn(0,PENTROM2);}static uns8 pentagonReadOpcode(uns16 addr){TRDOS_CHECK2(PENTROM2,addr);return MEM(addr);}static uns8 pentagonReadMem(uns16 addr){return MEM(addr);}static void pentagonWriteMem(uns16 addr,uns8 byte){if(!READ_ONLY(addr)){MEM(addr)=byte;if(MachineMemory[VPAGE(addr)]==Machine->screen&&(addr&=0x3fff)<(uns16)(PIXEL_LENGTH+ATTR_LENGTH)){ScreenMarkDirty(addr);}}}static void pentagonInitPages(void){Machine->pageIn(0,PENTROM0);Machine->pageIn(1,RAM5);Machine->pageIn(2,RAM2);Machine->pageIn(3,RAM0);}static void pentagonPageIn(int z80page,int vpage){int i=z80page<<1;MachineMemory[i]=MachineAllMemory[vpage];MachinePageNumbers[i]=vpage;i++;MachineMemory[i]=MachineAllMemory[vpage]+XZX_PAGESIZE/2;MachinePageNumbers[i]=vpage;}static inline void pentagonOutBankm128(uns8 val){uns8 changed=val^Machine->lastMem128;DEB(Msg(M_DEBUG,"bankm: RAM %d, ROM %d, screen %d, paging %sabled",val&B_SELRAM,(val&B_SELROM)>>4,(val&B_SELSCREEN)>>3,PAGING_DISABLED(Machine->lastMem128)?"dis":"en"););if(changed==0||PAGING_DISABLED(Machine->lastMem128)){return;}if((changed&B_SELSCREEN)!=0){Machine->screen=MachineAllMemory[val&B_SELSCREEN?RAM7:RAM5];ScreenMarkDirtyAll();}Machine->pageIn(3,RAM0+(val&B_SELRAM));Machine->pageIn(0,PENTROM0+((val&B_SELROM)>>4));Machine->lastMem128=val;}static void pentagonOutPort(uns16 port,uns8 val){DEB(Msg(M_DEBUG,"writing <%02x> to port <%04x>",val,port););if((port&B_ULA)==(P_ULA&B_ULA)){uns8 changes=val^Machine->lastUla;if((changes&B_BORDER)!=0){ScreenSetBorder(val&B_BORDER);}if((changes&(B_SPEAKER|B_MIC))!=0){SpeakerOutByte(val);}Machine->lastUla=val;}if((port&B_SND_CONTROL)==(P_SND_CONTROL&B_SND_CONTROL)){AyRegisterOutByte(val);}else if((port&B_SND_DATA)==(P_SND_DATA&B_SND_DATA)){AyDataOutByte(val);}else if((port&B_BANK128)==(P_BANK128&B_BANK128)){pentagonOutBankm128(val);}TRDOS_OUTBYTE(PENTROM2,port,val);}static uns8 pentagonInPort(uns16 port){uns8 res=0xff;int attached=0;if((port&B_ULA)==(P_ULA&B_ULA)){uns8 res2=0xbf;KEYBOARD_INBYTE(port,res2);res2^=TapeInGetBit();res&=res2;attached++;}if((port&B_SND_CONTROL)==(P_SND_CONTROL&B_SND_CONTROL)){res&=AyRegisterInByte();attached++;}else if((port&B_SND_DATA)==(P_SND_DATA&B_SND_DATA)){res&=AyDataInByte();attached++;}else if((port&B_BANK128)==(P_BANK128&B_BANK128)){attached++;}if(RPAGE(0)!=PENTROM2){KJOY_INBYTE(port,res,attached);
#ifdef XZX_KMOUSE
KMOUSE_INBYTE(port,res,attached);
#endif
}else{TRDOS_INBYTE2(port,res,attached);}if(attached==0&&Machine->ulaDelay[TSTATES%Machine->tstatesPerFrame]==ULA_DRAWS_SCREEN){unsigned int tstatesInLine=TSTATES%Machine->tstatesPerLine;unsigned int line=(TSTATES/Machine->tstatesPerLine)-Machine->topBorder;res=Machine->screen[(tstatesInLine&0x02 ?Line2Attr[line]:Line2Pix[line])+(tstatesInLine>>2)];}DEB(Msg(M_DEBUG,"reading <%02x> from port <%04x>",res,port););return res;}
#endif 

