/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef XZX_SCORPION
#ifndef lint
static char rcsid[]="$Id: scorpion.c,v 4.51 2006/12/22 14:29:01 erik Rel $";
#endif
#include <assert.h>
#include "z80/z80.h"
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#ifdef XZX_SCORPION_PRINTER
#include "printer/printer.h"
#endif
#include "keyboard.h"
#include "machine/ay.h"
#include "machine/kjoy.h"
#ifdef XZX_KMOUSE
#include "machine/kmouse.h"
#endif
#include "ui/screen.h"
#include "sound/sound.h"
#include "tables.h"
#include "tape.h"
#include "machine/trdos.h"
#include "machine/scorpion.h"
#define Y_INTERRUPT	0
#define Y_TOP_BORDER	64
#define Y_SCREEN	192
#define Y_BOTTOM_BORDER	40
#define Y_RETRACE	16
#define X_LEFT_BORDER	24
#define X_SCREEN	128
#define X_RIGHT	32
#define X_RETRACE	40
#define ULA_DRAWS_SCREEN	0x00
#define ULA_DRAWS_BORDER	0x20
#define ULA_HORIZONTAL_RETRACE	0x40
#define ULA_VERTICAL_RETRACE	0x80
#undef B_ULA
#define B_ULA	0x27	
#define B_RS232IN	0x20	
#define B_BUSY	0x80	
#undef B_SND_CONTROL
#define B_SND_CONTROL	0xefff	
#undef B_SND_DATA
#define B_SND_DATA	0xefff	
#define P_BANK128	P_SP128_BANK
#define B_BANK128	0xd027	
#define B_SELROM	B_SP128_SELROM
#define B_SELRAM	B_SP128_SELRAM
#define B_SELSCREEN	B_SP128_SELSCREEN
#define PAGING_DISABLED	PAGING_SP128_DISABLED
#define P_BANKX	0x1ffd	
#define B_BANKX	0xd027	
#define B_SPECIAL	0x01	
#define B_SELROMX	0x02	
#define B_RS232OUT	0x08	
#define B_SELRAMX	0x10	
#define B_STROBE	0x20	
#define EXTENDED_RAM(port)	((port) & (B_SPECIAL | B_SELROMX))
#define lastMemX	lastMem3
#define P_CENTRONICS	0xffdd	
#define B_CENTRONICS	0x27	
#define P_ATTRIBUTE	0xffff	
#define B_ATTRIBUTE	0x27	
enum{ROM_UNKNOWN,ROM_SCORPION};
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_IO) != 0) { x } }
#else
#define DEB(x)
#endif
static void scorpionReset(void);static void scorpionNmi(void);static uns8 scorpionReadOpcode(uns16);static uns8 scorpionReadMem(uns16);static void scorpionWriteMem(uns16,uns8);static void scorpionInitPages(void);static void scorpionPageIn(int,int);static inline void scorpionOutBankm128(uns8);static inline void scorpionOutBankmX(uns8);static void scorpionOutPort(uns16,uns8);static uns8 scorpionInPort(uns16);void ScorpionInit(void){MachineSettings*msp;int checksum,romtype;unsigned int i,x,y;msp=&Machines[SP_SCORPION];msp->name="Scorpion 256";msp->basicRom=SCORROM1;msp->tvLines=Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER+Y_RETRACE;msp->tstatesPerLine=X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;msp->tstatesPerFrame=msp->tvLines*msp->tstatesPerLine;msp->intLine=Y_INTERRUPT;msp->topBorder=Y_TOP_BORDER;msp->ulaDelay=Malloc(sizeof(unsigned char)*(size_t)msp->tstatesPerFrame,"ScorpionInit");msp->reset=scorpionReset;msp->nmi=scorpionNmi;msp->readOpcode=scorpionReadOpcode;msp->readMem=scorpionReadMem;msp->writeMem=scorpionWriteMem;msp->inPort=scorpionInPort;msp->outPort=scorpionOutPort;msp->initPages=scorpionInitPages;msp->pageIn=scorpionPageIn;msp->screen=MachineAllMemory[RAM5];msp->ay.type=AY_TYPE_YM2149F;if(MachineLoadRom(CFGGET(scorpionRom),SCORROM0,4)==0){checksum=MachineChecksumRom(msp->basicRom,XZX_PAGESIZE);switch(checksum){case 22821:romtype=ROM_SCORPION;break;default:Msg(M_WARN,"unknown Scorpion ROM image - checksum %u",checksum);romtype=ROM_UNKNOWN;break;}if(romtype!=ROM_UNKNOWN){msp->traps.load.page=msp->basicRom;msp->traps.load.addr=0x563;msp->traps.save.page=msp->basicRom;msp->traps.save.addr=0x4c7;}}for(i=0,y=0;y<Y_TOP_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_TOP_BORDER+Y_SCREEN;y++){for(x=0;x<X_LEFT_BORDER;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN;x++){msp->ulaDelay[i++]=ULA_DRAWS_SCREEN;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER+Y_RETRACE;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_VERTICAL_RETRACE;}}assert(i==msp->tstatesPerFrame);}static void scorpionReset(void){Z80Reset();Machine->initPages();Machine->lastMem128=0;Machine->screen=MachineAllMemory[RAM5];Machine->lastMemX=0;AyReset();}static void scorpionNmi(void){if(PC>=0x4000){Z80Nmi((NmiCallbackFunc)0);(void)Machine->pageIn(0,SCORROM2);}}static uns8 scorpionReadOpcode(uns16 addr){if((TSTATES&1)!=0){TSTATES++;}if(addr>=0x4000){if(RPAGE(0)==SCORROM3){Machine->pageIn(0,Machine->basicRom);}}else if((addr&0xff00)==0x3d00){if(RPAGE(0)==Machine->basicRom){Machine->pageIn(0,SCORROM3);}}return MEM(addr);}static uns8 scorpionReadMem(uns16 addr){return MEM(addr);}static void scorpionWriteMem(uns16 addr,uns8 byte){if(!READ_ONLY(addr)){MEM(addr)=byte;if(MachineMemory[VPAGE(addr)]==Machine->screen&&(addr&=0x3fff)<(uns16)(PIXEL_LENGTH+ATTR_LENGTH)){ScreenMarkDirty(addr);}}}static void scorpionInitPages(void){Machine->pageIn(0,SCORROM0);Machine->pageIn(1,RAM5);Machine->pageIn(2,RAM2);Machine->pageIn(3,RAM0);}static void scorpionPageIn(int z80page,int vpage){int i=z80page<<1;MachineMemory[i]=MachineAllMemory[vpage];MachinePageNumbers[i]=vpage;i++;MachineMemory[i]=MachineAllMemory[vpage]+XZX_PAGESIZE/2;MachinePageNumbers[i]=vpage;}static inline void scorpionOutBankm128(uns8 val){uns8 changed=val^Machine->lastMem128;DEB(Msg(M_DEBUG,"bankm: RAM %X, ROM %d, screen %d, paging %sabled",((Machine->lastMemX&B_SELRAMX)>>1)|(val&B_SELRAM),(val&B_SELROM)>>4,(val&B_SELSCREEN)>>3,PAGING_DISABLED(Machine->lastMem128)?"dis":"en"););if(changed==0||PAGING_DISABLED(Machine->lastMem128)){return;}if((changed&B_SELSCREEN)!=0){Machine->screen=MachineAllMemory[val&B_SELSCREEN?RAM7:RAM5];ScreenMarkDirtyAll();}Machine->pageIn(3,RAM0+(((Machine->lastMemX&B_SELRAMX)>>1)|(val&B_SELRAM)));if(!EXTENDED_RAM(Machine->lastMemX)){Machine->pageIn(0,SCORROM0+((val&B_SELROM)>>4));}Machine->lastMem128=val;}static inline void scorpionOutBankmX(uns8 val){uns8 changed=val^Machine->lastMemX;int page0;DEB(Msg(M_DEBUG,"bankx: RAM %X, RAM8 %d, ROM2 %d",((val&B_SELRAMX)>>1)|(Machine->lastMem128&B_SELRAM),val&B_SPECIAL,(val&B_SELROMX)>>1););if(changed==0||PAGING_DISABLED(Machine->lastMem128)){return;}switch(EXTENDED_RAM(val)){default:page0=SCORROM0+((Machine->lastMem128&B_SELROM)>>4);break;case B_SPECIAL:page0=RAM8;break;case B_SPECIAL|B_SELROMX:Msg(M_WARN,"RAM and ROM together at page 0 is impossible");case B_SELROMX:page0=SCORROM2;break;}Machine->pageIn(0,page0);Machine->pageIn(3,RAM0+(((val&B_SELRAMX)>>1)|(Machine->lastMem128&B_SELRAM)));
#ifdef XZX_SCORPION_PRINTER
if((changed&B_STROBE)!=0){CentronicsStrobe(val&B_STROBE);}
#endif
Machine->lastMemX=val;}static void scorpionOutPort(uns16 port,uns8 val){int trdosActive=RPAGE(0)>=SCORROM2?1:0;DEB(Msg(M_DEBUG,"writing <%02x> to port <%04x>",val,port););if((port&B_ULA)==(P_ULA&B_ULA)&&trdosActive==0){uns8 changes=val^Machine->lastUla;if((changes&B_BORDER)!=0){ScreenSetBorder(val&B_BORDER);}if((changes&(B_SPEAKER|B_MIC))!=0){SpeakerOutByte(val);}Machine->lastUla=val;}if((port&B_SND_CONTROL)==(P_SND_CONTROL&B_SND_CONTROL)){AyRegisterOutByte(val);}if((port&B_SND_DATA)==(P_SND_DATA&B_SND_DATA)){AyDataOutByte(val);}if((port&B_BANK128)==(P_BANK128&B_BANK128)){scorpionOutBankm128(val);}if((port&B_BANKX)==(P_BANKX&B_BANKX)){scorpionOutBankmX(val);}
#ifdef XZX_SCORPION_PRINTER
if((port&B_CENTRONICS)==(P_CENTRONICS&B_CENTRONICS)){CentronicsOutByte(val);}
#endif
if(trdosActive!=0){TRDOS_OUTBYTE2(port,val);}}static uns8 scorpionInPort(uns16 port){uns8 res=0xff;int attached=0;int trdosActive=RPAGE(0)>=SCORROM2?1:0;if((port&B_ULA)==(P_ULA&B_ULA)&&trdosActive==0){uns8 res2=0x7f;KEYBOARD_INBYTE(port,res2);res2^=TapeInGetBit();
#ifdef XZX_SCORPION_PRINTER
res2|=CentronicsStatus()!=0?0:B_BUSY;
#endif
res&=res2;}if((port&B_SND_CONTROL)==(P_SND_CONTROL&B_SND_CONTROL)){res&=AyRegisterInByte();}if((port&B_SND_DATA)==(P_SND_DATA&B_SND_DATA)){res&=AyDataInByte();}if((port&B_BANK128)==(P_BANK128&B_BANK128)){Msg(M_WARN,"turbo mode on - not supported by XZX");}if((port&B_BANKX)==(P_BANKX&B_BANKX)){Msg(M_WARN,"turbo mode off");}if((port&B_CENTRONICS)==(P_CENTRONICS&B_CENTRONICS)){}if(trdosActive==0){if((port&B_ATTRIBUTE)==(P_ATTRIBUTE&B_ATTRIBUTE)){if(Machine->ulaDelay[TSTATES%Machine->tstatesPerFrame]==ULA_DRAWS_SCREEN){unsigned int tstatesInLine=TSTATES%Machine->tstatesPerLine;unsigned int line=(TSTATES/Machine->tstatesPerLine)-Machine->topBorder;res&=Machine->screen[(tstatesInLine&0x02 ?Line2Attr[line]:Line2Pix[line])+(tstatesInLine>>2)];}}KJOY_INBYTE(port,res,attached);
#ifdef XZX_KMOUSE
KMOUSE_INBYTE(port,res,attached);
#endif
}else{TRDOS_INBYTE2(port,res,attached);}DEB(Msg(M_DEBUG,"reading <%02x> from port <%04x>",res,port););return res;}
#endif 

