/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: spec128.c,v 4.55 2006/12/22 14:29:01 erik Rel $";
#endif
#include <assert.h>
#include "z80/z80.h"
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "machine/ay.h"
#include "machine/cjoy.h"
#ifdef XZX_IF1
#include "machine/if1.h"
#endif
#include "keyboard.h"
#include "machine/kjoy.h"
#ifdef XZX_MF
#include "machine/multiface.h"
#endif
#ifdef XZX_PLUSD
#include "machine/plusd.h"
#endif
#include "ui/screen.h"
#include "machine/sjoy.h"
#include "sound/sound.h"
#include "tables.h"
#include "tape.h"
#ifdef XZX_TRDOS
#include "machine/trdos.h"
#endif
#include "machine/spec128.h"
#define Y_INTERRUPT	0
#define Y_RETRACE	15
#define Y_TOP_BORDER	48
#define Y_SCREEN	192
#define Y_BOTTOM_BORDER	56
#define X_LEFT_BORDER	26
#define X_SCREEN	128
#define X_RIGHT	26
#define X_RETRACE	48
#define ULA_DRAWS_SCREEN	0x04
#define ULA_DRAWS_BORDER	0x20
#define ULA_HORIZONTAL_RETRACE	0x40
#define ULA_VERTICAL_RETRACE	0x80
#define ULA_DELAY_MASK	0x0f
#define T_STATES_OVERFLOW	23 + 19 
#define IS_CONTENDED(addr)	((RPAGE((addr)) & 1) != 0)
#define P_BANK128	P_SP128_BANK
#define B_BANK128	B_SP128_BANK
#define B_SELROM	B_SP128_SELROM
#define B_SELRAM	B_SP128_SELRAM
#define B_SELSCREEN	B_SP128_SELSCREEN
#define PAGING_DISABLED	PAGING_SP128_DISABLED
enum{ROM_UNKNOWN,ROM_128,ROM_PLUS2};
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_IO) != 0) { x } }
#else
#define DEB(x)
#endif
static void spec128Reset(void);static void spec128Nmi(void);static uns8 spec128ReadOpcode(uns16);static uns8 spec128ReadMem(uns16);static void spec128WriteMem(uns16,uns8);static void spec128InitPages(void);static void spec128PageIn(int,int);static inline void spec128OutBankm128(uns8);static void spec128OutPort(uns16,uns8);static uns8 spec128InPort(uns16);void Spec128Init(void){MachineSettings*msp;int checksum,romtype;unsigned i,x,y;msp=&Machines[SP_128];msp->name="Spectrum 128K";msp->basicRom=SP128ROM1;msp->tvLines=Y_RETRACE+Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER;msp->tstatesPerLine=X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;msp->tstatesPerFrame=msp->tvLines*msp->tstatesPerLine;msp->intLine=Y_INTERRUPT;msp->topBorder=Y_RETRACE+Y_TOP_BORDER;msp->ulaDelay=Malloc(sizeof(unsigned char)*(size_t)(msp->tstatesPerFrame+T_STATES_OVERFLOW),"Spec128Init");msp->reset=spec128Reset;msp->nmi=spec128Nmi;msp->readOpcode=spec128ReadOpcode;msp->readMem=spec128ReadMem;msp->writeMem=spec128WriteMem;msp->inPort=spec128InPort;msp->outPort=spec128OutPort;msp->initPages=spec128InitPages;msp->pageIn=spec128PageIn;msp->screen=MachineAllMemory[RAM5];msp->ay.type=AY_TYPE_AY8912;if(MachineLoadRom(CFGGET(sp128Rom),SP128ROM0,2)==0){checksum=MachineChecksumRom(msp->basicRom,XZX_PAGESIZE);switch(checksum){case 44809:romtype=ROM_128;break;case 32991:msp->name="Spectrum +2";romtype=ROM_PLUS2;break;default:Msg(M_WARN,"unknown 128K ROM image - checksum %u",checksum);romtype=ROM_UNKNOWN;break;}if(romtype!=ROM_UNKNOWN){msp->traps.load.page=msp->basicRom;msp->traps.load.addr=0x563;msp->traps.save.page=msp->basicRom;msp->traps.save.addr=0x4c7;}}for(i=0,y=0;y<Y_RETRACE;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_VERTICAL_RETRACE;}}for(;y<Y_RETRACE+Y_TOP_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_RETRACE+Y_TOP_BORDER+Y_SCREEN;y++){for(x=0;x<X_LEFT_BORDER;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN;x++){msp->ulaDelay[i++]=ULA_DRAWS_SCREEN;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_RETRACE+Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}while(i<msp->tstatesPerFrame+T_STATES_OVERFLOW){msp->ulaDelay[i++]=ULA_VERTICAL_RETRACE;}assert(i==msp->tstatesPerFrame+T_STATES_OVERFLOW);}static void spec128Reset(void){Z80Reset();Machine->initPages();Machine->lastMem128=0;Machine->screen=MachineAllMemory[RAM5];AyReset();
#ifdef XZX_TRDOS
if(CFGGET(trdos)>0){TrdosReset();}
#endif
#ifdef XZX_PLUSD
if(CFGGET(plusd)>0){PlusdReset();}
#endif
}static void spec128Nmi(void){Z80Nmi((NmiCallbackFunc)0);
#ifdef XZX_MF
(void)Machine->inPort(P_MF128_IN);
#endif
#ifdef XZX_TRDOS
if(CFGGET(trdos)>0){Machine->pageIn(0,TRDOSROM);}
#endif
#ifdef XZX_PLUSD
if(CFGGET(plusd)>0){Machine->pageIn(0,PLUSDROM);}
#endif
}static uns8 spec128ReadOpcode(uns16 addr){uns8 op;
#ifdef XZX_IF1
IF1_CHECK_AND_PAGEIN(addr);
#endif
#ifdef XZX_TRDOS
TRDOS_CHECK(TRDOSROM,addr);
#endif
#ifdef XZX_PLUSD
PLUSD_CHECK(addr);
#endif
op=MEM(addr);if(IS_CONTENDED(addr)){TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}
#ifdef XZX_IF1
IF1_CHECK_AND_PAGEOUT(addr);
#endif
return op;}static uns8 spec128ReadMem(uns16 addr){uns8 byte=MEM(addr);if(IS_CONTENDED(addr)){TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}return byte;}static void spec128WriteMem(uns16 addr,uns8 byte){if(!READ_ONLY(addr)){MEM(addr)=byte;if(IS_CONTENDED(addr)){TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}if(MachineMemory[VPAGE(addr)]==Machine->screen&&(addr&=0x3fff)<(uns16)(PIXEL_LENGTH+ATTR_LENGTH)){ScreenMarkDirty(addr);}}}static void spec128InitPages(void){Machine->pageIn(0,SP128ROM0);Machine->pageIn(1,RAM5);Machine->pageIn(2,RAM2);Machine->pageIn(3,RAM0);}static void spec128PageIn(int z80page,int vpage){int i=z80page<<1;MachineMemory[i]=MachineAllMemory[vpage];MachinePageNumbers[i]=vpage;i++;MachineMemory[i]=MachineAllMemory[vpage]+XZX_PAGESIZE/2;switch(vpage){
#ifdef XZX_MF
case MF128ROM:MachinePageNumbers[i]=MFRAM;break;
#endif
#ifdef XZX_PLUSD
case PLUSDROM:MachinePageNumbers[i]=PLUSDRAM;break;
#endif
default:MachinePageNumbers[i]=vpage;break;}}static inline void spec128OutBankm128(uns8 val){uns8 changed=val^Machine->lastMem128;DEB(Msg(M_DEBUG,"bankm: RAM %d, ROM %d, screen %d, paging %sabled",val&B_SELRAM,(val&B_SELROM)>>4,(val&B_SELSCREEN)>>3,PAGING_DISABLED(Machine->lastMem128)?"dis":"en"););if(changed==0||PAGING_DISABLED(Machine->lastMem128)){return;}if((changed&B_SELSCREEN)!=0){Machine->screen=MachineAllMemory[val&B_SELSCREEN?RAM7:RAM5];ScreenMarkDirtyAll();}Machine->pageIn(3,RAM0+(val&B_SELRAM));
#ifdef XZX_MF
if(RPAGE(0)!=MF128ROM)
#endif
{Machine->pageIn(0,SP128ROM0+((val&B_SELROM)>>4));}Machine->lastMem128=val;}static void spec128OutPort(uns16 port,uns8 val){DEB(Msg(M_DEBUG,"writing <%02x> to port <%04x>",val,port););if((port&B_ULA)==(P_ULA&B_ULA)){uns8 changes=val^Machine->lastUla;if((changes&B_BORDER)!=0){ScreenSetBorder(val&B_BORDER);}if((changes&(B_SPEAKER|B_MIC))!=0){SpeakerOutByte(val);}Machine->lastUla=val;TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}if((port&B_SND_CONTROL)==(P_SND_CONTROL&B_SND_CONTROL)){AyRegisterOutByte(val);}else if((port&B_SND_DATA)==(P_SND_DATA&B_SND_DATA)){AyDataOutByte(val);}else if((port&B_BANK128)==(P_BANK128&B_BANK128)){spec128OutBankm128(val);}
#ifdef XZX_IF1
IF1_OUTBYTE(port,val);
#endif
#ifdef XZX_TRDOS
TRDOS_OUTBYTE(TRDOSROM,port,val);
#endif
#ifdef XZX_PLUSD
PLUSD_OUTBYTE(port,val);
#endif
}static uns8 spec128InPort(uns16 port){uns8 res=0xff;int attached=0;if((port&B_ULA)==(P_ULA&B_ULA)){uns8 res2=(Machine->lastUla&B_SPEAKER)!=0?0xff :0xbf;KEYBOARD_INBYTE(port,res2);SJOY_INBYTE(port,res2);CJOY_INBYTE(port,res2);res2^=TapeInGetBit();TSTATES+=1U;res&=res2;attached++;}if((port&B_SND_CONTROL)==(P_SND_CONTROL&B_SND_CONTROL)){res&=AyRegisterInByte();attached++;}else if((port&B_SND_DATA)==(P_SND_DATA&B_SND_DATA)){res&=AyDataInByte();attached++;}else if((port&B_BANK128)==(P_BANK128&B_BANK128)){attached++;}
#ifdef XZX_TRDOS
if(RPAGE(0)!=TRDOSROM)
#endif
{KJOY_INBYTE(port,res,attached);}
#ifdef XZX_IF1
IF1_INBYTE(port,res,attached);
#endif
#ifdef XZX_MF
MF128_INBYTE(port,res,attached);
#endif
#ifdef XZX_TRDOS
TRDOS_INBYTE(TRDOSROM,port,res,attached);
#endif
#ifdef XZX_PLUSD
PLUSD_INBYTE(port,res,attached);
#endif
if(attached==0&&(Machine->ulaDelay[TSTATES%Machine->tstatesPerFrame]&ULA_DELAY_MASK)==ULA_DRAWS_SCREEN){unsigned int tstatesInLine=TSTATES%Machine->tstatesPerLine;unsigned int line=(TSTATES/Machine->tstatesPerLine)-Machine->topBorder;res=Machine->screen[(tstatesInLine&0x02 ?Line2Attr[line]:Line2Pix[line])+(tstatesInLine>>2)];}DEB(Msg(M_DEBUG,"reading <%02x> from port <%04x>",res,port););return res;}
