/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: spec48.c,v 4.51 2006/12/22 14:29:01 erik Rel $";
#endif
#include <assert.h>
#include "z80/z80.h"
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "machine/cjoy.h"
#ifdef XZX_FULLER
#include "machine/fuller.h"
#endif
#ifdef XZX_IF1
#include "machine/if1.h"
#endif
#include "keyboard.h"
#include "machine/kjoy.h"
#ifdef XZX_MF
#include "machine/multiface.h"
#endif
#ifdef XZX_PLUSD
#include "machine/plusd.h"
#endif
#ifdef XZX_PRINTER
#include "printer/printer.h"
#endif
#include "ui/screen.h"
#include "machine/sjoy.h"
#include "sound/sound.h"
#include "tables.h"
#include "tape.h"
#ifdef XZX_TRDOS
#include "machine/trdos.h"
#endif
#include "machine/spec48.h"
#define Y_INTERRUPT	0
#define Y_RETRACE	16
#define Y_TOP_BORDER	48
#define Y_SCREEN	192
#define Y_BOTTOM_BORDER	56
#define X_LEFT_BORDER	24
#define X_SCREEN	128
#define X_RIGHT	24
#define X_RETRACE	48
#define ULA_DRAWS_SCREEN	0x04
#define ULA_DRAWS_BORDER	0x20
#define ULA_HORIZONTAL_RETRACE	0x40
#define ULA_VERTICAL_RETRACE	0x80
#define ULA_DELAY_MASK	0x0f
#define T_STATES_OVERFLOW	23 + 19 
#define IS_CONTENDED(addr)	(RPAGE((addr)) == RAM5)
enum{ROM_UNKNOWN,ROM_STANDARD,ROM_TK90X,ROM_TK95,ROM_GROOT,ROM_IMC};
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_IO) != 0) { x } }
#else
#define DEB(x)
#endif
static void spec48Reset(void);static void spec48Nmi(void);static uns8 spec48ReadOpcode(uns16);static uns8 spec48ReadMem(uns16);static void spec48WriteMem(uns16,uns8);static void spec48InitPages(void);static void spec48PageIn(int,int);static void spec48OutPort(uns16,uns8);static uns8 spec48InPort(uns16);void Spec48Init(void){MachineSettings*msp,*msp2;int checksum,romtype;unsigned int i,x,y;msp=&Machines[SP_48_2];msp2=&Machines[SP_48_3];msp->basicRom=SP48ROM;msp->tvLines=Y_RETRACE+Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER;msp->tstatesPerLine=X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;msp->tstatesPerFrame=msp->tvLines*msp->tstatesPerLine;msp->intLine=Y_INTERRUPT;msp->topBorder=Y_RETRACE+Y_TOP_BORDER;msp->ulaDelay=Malloc(sizeof(unsigned char)*(size_t)(msp->tstatesPerFrame+T_STATES_OVERFLOW),"Spec48Init");msp->reset=spec48Reset;msp->nmi=spec48Nmi;msp->readOpcode=spec48ReadOpcode;msp->readMem=spec48ReadMem;msp->writeMem=spec48WriteMem;msp->inPort=spec48InPort;msp->outPort=spec48OutPort;msp->initPages=spec48InitPages;msp->pageIn=spec48PageIn;msp->screen=MachineAllMemory[RAM5];
#ifdef XZX_FULLER
msp->ay.type=AY_TYPE_AY8912;
#else
msp->ay.type=AY_TYPE_NONE;
#endif
(void)memcpy(msp2,msp,sizeof(MachineSettings));msp->name="Spectrum 48K Issue 2";msp2->name="Spectrum 48K";if(MachineLoadRom(CFGGET(sp48Rom),SP48ROM,1)==0){checksum=MachineChecksumRom(msp->basicRom,XZX_PAGESIZE);switch(checksum){case 8686:romtype=ROM_STANDARD;break;case 56154:msp->name="TK 90X";msp2->name="TK 90X";romtype=ROM_TK90X;break;case 44175:msp->name="TK 95";msp2->name="TK 95";romtype=ROM_TK95;break;case 58561:romtype=ROM_GROOT;break;case 53858:romtype=ROM_IMC;break;default:Msg(M_WARN,"unknown 48K ROM image - checksum %u",checksum);romtype=ROM_UNKNOWN;break;}if(romtype==ROM_STANDARD){msp->traps.load.page=msp2->traps.load.page=msp->basicRom;msp->traps.load.addr=msp2->traps.load.addr=0x563;msp->traps.save.page=msp2->traps.save.page=msp->basicRom;msp->traps.save.addr=msp2->traps.save.addr=0x4c7;}}for(i=0,y=0;y<Y_RETRACE;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_VERTICAL_RETRACE;}}for(;y<Y_RETRACE+Y_TOP_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_RETRACE+Y_TOP_BORDER+Y_SCREEN;y++){for(x=0;x<X_LEFT_BORDER;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN;x++){msp->ulaDelay[i++]=ULA_DRAWS_SCREEN;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_RETRACE+Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}while(i<msp->tstatesPerFrame+T_STATES_OVERFLOW){msp->ulaDelay[i++]=ULA_VERTICAL_RETRACE;}assert(i==msp->tstatesPerFrame+T_STATES_OVERFLOW);}static void spec48Reset(void){Z80Reset();Machine->initPages();Machine->screen=MachineAllMemory[RAM5];
#ifdef XZX_FULLER
FullerReset();
#endif
#ifdef XZX_TRDOS
if(CFGGET(trdos)>0){TrdosReset();}
#endif
#ifdef XZX_PLUSD
if(CFGGET(plusd)>0){PlusdReset();}
#endif
}static void spec48Nmi(void){Z80Nmi((NmiCallbackFunc)0);
#ifdef XZX_MF
(void)Machine->inPort(P_MF1_IN);
#endif
#ifdef XZX_TRDOS
if(CFGGET(trdos)>0){Machine->pageIn(0,TRDOSROM);}
#endif
#ifdef XZX_PLUSD
if(CFGGET(plusd)>0){Machine->pageIn(0,PLUSDROM);}
#endif
}static uns8 spec48ReadOpcode(uns16 addr){uns8 op;
#ifdef XZX_IF1
IF1_CHECK_AND_PAGEIN(addr);
#endif
#ifdef XZX_TRDOS
TRDOS_CHECK(TRDOSROM,addr);
#endif
#ifdef XZX_PLUSD
PLUSD_CHECK(addr);
#endif
op=MEM(addr);if(IS_CONTENDED(addr)){TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}
#ifdef XZX_IF1
IF1_CHECK_AND_PAGEOUT(addr);
#endif
return op;}static uns8 spec48ReadMem(uns16 addr){uns8 byte=MEM(addr);if(IS_CONTENDED(addr)){TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}return byte;}static void spec48WriteMem(uns16 addr,uns8 byte){if(!READ_ONLY(addr)){MEM(addr)=byte;if(RPAGE(addr)==RAM5&&(addr&=0x3fff)<(uns16)(PIXEL_LENGTH+ATTR_LENGTH)){ScreenMarkDirty(addr);TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}}}static void spec48InitPages(void){Machine->pageIn(0,SP48ROM);Machine->pageIn(1,RAM5);Machine->pageIn(2,RAM2);Machine->pageIn(3,RAM0);}static void spec48PageIn(int z80page,int vpage){int i=z80page<<1;MachineMemory[i]=MachineAllMemory[vpage];MachinePageNumbers[i]=vpage;i++;MachineMemory[i]=MachineAllMemory[vpage]+XZX_PAGESIZE/2;switch(vpage){
#ifdef XZX_MF
case MF1ROM:MachinePageNumbers[i]=MFRAM;break;
#endif
#ifdef XZX_PLUSD
case PLUSDROM:MachinePageNumbers[i]=PLUSDRAM;break;
#endif
default:MachinePageNumbers[i]=vpage;break;}}static void spec48OutPort(uns16 port,uns8 val){DEB(Msg(M_DEBUG,"writing <%02x> to port <%04x>",val,port););if((port&B_ULA)==(P_ULA&B_ULA)){uns8 changes=val^Machine->lastUla;if((changes&B_BORDER)!=0){ScreenSetBorder(val&B_BORDER);}if((changes&(B_SPEAKER|B_MIC))!=0){SpeakerOutByte(val);}Machine->lastUla=val;TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}if((port&B_128)==(P_128&B_128)){if(CFGGET(warn48)!=0){Msg(M_WARN,"output to 128K/+3 port <%04x> in 48K mode",port);}}
#ifdef XZX_ZX_PRINTER
ZXPRINTER_OUTBYTE(port,val);
#endif
#ifdef XZX_IF1
IF1_OUTBYTE(port,val);
#endif
#ifdef XZX_FULLER
FULLER_OUTBYTE(port,val);
#endif
#ifdef XZX_TRDOS
TRDOS_OUTBYTE(TRDOSROM,port,val);
#endif
#ifdef XZX_PLUSD
PLUSD_OUTBYTE(port,val);
#endif
}static uns8 spec48InPort(uns16 port){uns8 res=0xff;int attached=0;if((port&B_ULA)==(P_ULA&B_ULA)){uns8 res2=Machine->lastUla&(B_SPEAKER|(CFGGET(machine)==SP_48_2?B_MIC:0))?0xff :0xbf;KEYBOARD_INBYTE(port,res2);SJOY_INBYTE(port,res2);CJOY_INBYTE(port,res2);res2^=TapeInGetBit();TSTATES+=1U;res&=res2;attached++;}if((port&B_128)==(P_128&B_128)){if(CFGGET(warn48)!=0){Msg(M_WARN,"input from 128K/+3 port <%04x> in 48K mode",port);}}
#ifdef XZX_TRDOS
if(RPAGE(0)!=TRDOSROM)
#endif
{KJOY_INBYTE(port,res,attached);}
#ifdef XZX_ZX_PRINTER
ZXPRINTER_INBYTE(port,res,attached);
#endif
#ifdef XZX_IF1
IF1_INBYTE(port,res,attached);
#endif
#ifdef XZX_MF
MF1_INBYTE(port,res,attached);
#endif
#ifdef XZX_FULLER
FULLER_INBYTE(port,res,attached);
#endif
#ifdef XZX_TRDOS
TRDOS_INBYTE(TRDOSROM,port,res,attached);
#endif
#ifdef XZX_PLUSD
PLUSD_INBYTE(port,res,attached);
#endif
if(attached==0&&(Machine->ulaDelay[TSTATES%Machine->tstatesPerFrame]&ULA_DELAY_MASK)==ULA_DRAWS_SCREEN){unsigned int tstatesInLine=TSTATES%Machine->tstatesPerLine;unsigned int line=(TSTATES/Machine->tstatesPerLine)-Machine->topBorder;res=MachineAllMemory[RAM5][(tstatesInLine&0x02 ?Line2Attr[line]:Line2Pix[line])+(tstatesInLine>>2)];}DEB(Msg(M_DEBUG,"reading <%02x> from port <%04x>",res,port););return res;}
