/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: upd765a.c,v 4.53 2006/12/22 14:29:01 erik Rel $";
#endif
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "ddrive.h"
#include "machine/upd765a.h"
#define STM_RDY	0x80	
#define STM_DIR	0x40	
#define STM_EXEC	0x20	
#define STM_BUSY	0x10	
#define STM_BUSY3	0x08	
#define STM_BUSY2	0x04	
#define STM_BUSY1	0x02	
#define STM_BUSY0	0x01	
#define ST0_ERRCODE	0xc0	
#define ST0_OK	0x00	
#define ST0_NOTFINISH	0x40	
#define ST0_INVALCMD	0x80	
#define ST0_STOPPED	0xc0	
#define ST0_SEEKEND	0x20	
#define ST0_EQFAULT	0x10	
#define ST0_NOTRDY	0x08	
#define ST0_SIDE	0x04	
#define ST0_DRIVE	0x03	
#define ST1_ENDCYL	0x80	
#define ST1_DATAERR	0x20	
#define ST1_OVERRUN	0x10	
#define ST1_NODATA	0x04	
#define ST1_WRPROT	0x02	
#define ST1_ADDRMARK	0x01	
#define ST2_CTRLMARK	0x40	
#define ST2_DATAERR	0x20	
#define ST2_ERRCYLTRK	0x10	
#define ST2_SCANOK	0x08	
#define ST2_ERRSCAN	0x04	
#define ST2_BADCYL	0x02	
#define ST2_ADDRMARK	0x01	
#define ST3_FAULT	0x80	
#define ST3_WRPROT	0x40	
#define ST3_RDY	0x20	
#define ST3_TRK0	0x10	
#define ST3_DS	0x08	
#define ST3_SIDE	0x04	
#define ST3_DRIVE	0x03	
#define si_st1	crc1
#define si_st2	crc2
#ifdef DEBUG
#define DEB(x)	{ if (CFGGET(debug) & D_FDC) { x } }
#else
#define DEB(x)
#endif
static void upd765aUpdateState(Upd765aFdc*);static void upd765aEnterResultPhase(Upd765aFdc*,int);static void upd765aEnterIdleState(Upd765aFdc*);static void upd765aGetDriveAndSide(Upd765aFdc*);static void upd765aSeek(Upd765aFdc*,int);static void upd765aReadId(Upd765aFdc*);static void upd765aWriteData(Upd765aFdc*);static void upd765aReadData(Upd765aFdc*);static void upd765aReadOrWriteComplete(Upd765aFdc*);static DiskSectorInfo*upd765aGetMatchingSector(Upd765aFdc*);static void upd765aIncrementSector(Upd765aFdc*);static Ddrive*upd765aDummyDrive;Upd765aFdc*Upd765aNew(void){Upd765aFdc*chip;if(upd765aDummyDrive==(Ddrive*)0){upd765aDummyDrive=DdriveNew(DDRIVE_TYPE_35,2,80);}chip=Malloc(sizeof(Upd765aFdc),"Upd765aNew");Upd765aReset(chip);return chip;}void Upd765aAttachDrive(Upd765aFdc*chip,Ddrive*ddp){assert(chip->numDrives<NELEM(chip->drives));ddp->id=chip->numDrives;chip->drives[chip->numDrives++]=ddp;}void Upd765aReset(Upd765aFdc*chip){chip->stm=0;chip->st0=0;chip->st1=0;chip->st2=0;chip->st3=0;upd765aEnterIdleState(chip);}uns8 Upd765aControlInByte(Upd765aFdc*chip){uns8 status=chip->stm;DEB(Msg(M_DEBUG,"FDC: stm %02x",status););return status;}void Upd765aDataOutByte(Upd765aFdc*chip,uns8 val){chip->data=val;if((chip->stm&(STM_RDY|STM_DIR))==STM_RDY){chip->stm&=~STM_RDY;upd765aUpdateState(chip);}}uns8 Upd765aDataInByte(Upd765aFdc*chip){if((chip->stm&(STM_RDY|STM_DIR))==(STM_RDY|STM_DIR)){chip->stm&=~STM_RDY;upd765aUpdateState(chip);}return chip->data;}static void upd765aUpdateState(Upd765aFdc*chip){switch(chip->phase){case UPD765A_CMD_PHASE:DEB(Msg(M_DEBUG,"FDC: command[%d] %02x",chip->count,chip->data););chip->command[chip->count]=chip->data;if(chip->count++==0){static int cmdSizes[]={1,1,9,3,2,9,9,2,1,9,2,1,9,6,1,3,1,9,1,1,1,1,9,1,1,9,1,1,1,9,1,1};chip->stm|=STM_BUSY;chip->remaining=cmdSizes[chip->data&0x1f];}if(--chip->remaining==0){chip->phase=UPD765A_EXEC_PHASE;chip->stm|=STM_EXEC;switch(chip->command[0]&0x1f){case 0x02:upd765aGetDriveAndSide(chip);DEB(Msg(M_DEBUG,"FDC: READ TRACK");Msg(M_DEBUG,"      MT %d, MFM %d, SK %d",chip->command[0]>>7,(chip->command[0]>>6)&0x01,(chip->command[0]>>5)&0x01);Msg(M_DEBUG,"      S %d, DS %d",chip->s,chip->d);Msg(M_DEBUG,"      C %d",chip->command[2]);Msg(M_DEBUG,"      H %d",chip->command[3]);Msg(M_DEBUG,"      R %d",chip->command[4]);Msg(M_DEBUG,"      N %d",chip->command[5]);Msg(M_DEBUG,"      EOT %d",chip->command[6]);Msg(M_DEBUG,"      GPL %d",chip->command[7]);Msg(M_DEBUG,"      DTL %d",chip->command[8]););upd765aReadData(chip);break;case 0x03:DEB(Msg(M_DEBUG,"FDC: SPECIFY");Msg(M_DEBUG,"      SRT %d, HUT %d",chip->command[1]>>4,chip->command[1]&0x0f);Msg(M_DEBUG,"      HLT %d, ND %d",chip->command[2]>>1,chip->command[2]&0x01););upd765aEnterIdleState(chip);break;case 0x04:upd765aGetDriveAndSide(chip);DEB(Msg(M_DEBUG,"FDC: SENSE DRIVE STATUS");Msg(M_DEBUG,"      S %d, DS %d",chip->s,chip->d););chip->st3=chip->command[1]&(ST3_SIDE|ST3_DRIVE);if(chip->d>=chip->numDrives){chip->st3|=ST3_FAULT;}else{if(DdriveIsDS(chip->drive)){chip->st3|=ST3_DS;}if(DdriveIsAtTrack0(chip->drive)){chip->st3|=ST3_TRK0;}if(DdriveIsReady(chip->drive)){chip->st3|=ST3_RDY;}if(!DdriveIsLoaded(chip->drive)||DdriveIsWriteProtected(chip->drive)){chip->st3|=ST3_WRPROT;}}chip->result[0]=chip->st3;upd765aEnterResultPhase(chip,1);break;case 0x05:DEB(Msg(M_DEBUG,"FDC: WRITE DATA"););write_data:DEB(Msg(M_DEBUG,"      MT %d, MFM %d, SK %d",chip->command[0]>>7,(chip->command[0]>>6)&0x01,(chip->command[0]>>5)&0x01);Msg(M_DEBUG,"      S %d, DS %d",chip->s,chip->d);Msg(M_DEBUG,"      C %d",chip->command[2]);Msg(M_DEBUG,"      H %d",chip->command[3]);Msg(M_DEBUG,"      R %d",chip->command[4]);Msg(M_DEBUG,"      N %d",chip->command[5]);Msg(M_DEBUG,"      EOT %d",chip->command[6]);Msg(M_DEBUG,"      GPL %d",chip->command[7]);Msg(M_DEBUG,"      DTL %d",chip->command[8]););chip->mt=chip->command[0]>>7;upd765aWriteData(chip);break;case 0x06:upd765aGetDriveAndSide(chip);DEB(Msg(M_DEBUG,"FDC: READ DATA"););read_data:DEB(Msg(M_DEBUG,"      MT %d, MFM %d, SK %d",chip->command[0]>>7,(chip->command[0]>>6)&0x01,(chip->command[0]>>5)&0x01);Msg(M_DEBUG,"      S %d, DS %d",chip->s,chip->d);Msg(M_DEBUG,"      C %d",chip->command[2]);Msg(M_DEBUG,"      H %d",chip->command[3]);Msg(M_DEBUG,"      R %d",chip->command[4]);Msg(M_DEBUG,"      N %d",chip->command[5]);Msg(M_DEBUG,"      EOT %d",chip->command[6]);Msg(M_DEBUG,"      GPL %d",chip->command[7]);Msg(M_DEBUG,"      DTL %d",chip->command[8]););chip->mt=chip->command[0]>>7;upd765aReadData(chip);break;case 0x07:upd765aGetDriveAndSide(chip);DEB(Msg(M_DEBUG,"FDC: RECALIBRATE");Msg(M_DEBUG,"      DS %d",chip->d););upd765aSeek(chip,0);break;case 0x08:DEB(Msg(M_DEBUG,"FDC: SENSE INTERRUPT STATUS"););if(chip->interrupt==0){goto invalid_command;}chip->interrupt=0;chip->stm&=~(STM_BUSY3|STM_BUSY2|STM_BUSY1|STM_BUSY0);chip->result[0]=chip->st0;chip->result[1]=DdriveGetTrack(chip->drive);upd765aEnterResultPhase(chip,2);break;case 0x09:upd765aGetDriveAndSide(chip);DEB(Msg(M_DEBUG,"FDC: WRITE DELETED DATA"););goto write_data;case 0x0a:upd765aGetDriveAndSide(chip);DEB(Msg(M_DEBUG,"FDC: READ SECTOR ID");Msg(M_DEBUG,"      MFM %d",(chip->command[0]>>6)&0x01);Msg(M_DEBUG,"      S %d, DS %d",chip->s,chip->d););upd765aReadId(chip);break;case 0x0c:upd765aGetDriveAndSide(chip);DEB(Msg(M_DEBUG,"FDC: READ DELETED DATA"););goto read_data;case 0x0d:Msg(M_WARN,"FDC: FORMAT TRACK");goto not_implemented;case 0x0f:upd765aGetDriveAndSide(chip);DEB(Msg(M_DEBUG,"FDC: SEEK");Msg(M_DEBUG,"      S %d, DS %d",chip->s,chip->d);Msg(M_DEBUG,"      DTK %d",chip->command[2]););upd765aSeek(chip,chip->command[2]);break;case 0x11:Msg(M_WARN,"FDC: SCAN EQUAL");goto not_implemented;case 0x19:Msg(M_WARN,"FDC: SCAN LOW OR EQUAL");goto not_implemented;case 0x1d:Msg(M_WARN,"FDC: SCAN HIGH OR EQUAL");goto not_implemented;default:invalid_command:not_implemented:chip->result[0]=ST0_INVALCMD;upd765aEnterResultPhase(chip,1);break;}}else{chip->stm|=STM_RDY;}break;case UPD765A_EXEC_PHASE:if((chip->stm&STM_DIR)!=0){chip->data=*chip->datap;}else{*chip->datap=chip->data;}DEB(Msg(M_DEBUG,"FDC: exec[%d] %02x",chip->count,chip->data););chip->datap++;chip->count++;if(--chip->remaining==0){switch(chip->command[0]&0x1f){case 0x02:if(chip->command[4]==chip->command[6]){upd765aReadOrWriteComplete(chip);break;}upd765aIncrementSector(chip);upd765aReadData(chip);break;case 0x0d:assert(0);break;case 0x05:case 0x09:if((chip->command[4]==chip->command[6])&&(chip->mt==0||chip->s==1)){upd765aIncrementSector(chip);upd765aReadOrWriteComplete(chip);}else{upd765aIncrementSector(chip);upd765aWriteData(chip);}break;case 0x06:case 0x0c:if((chip->command[4]==chip->command[6])&&(chip->mt==0||chip->s==1)){upd765aReadOrWriteComplete(chip);break;}if((chip->command[0]&0x20)==0&&chip->dam==((chip->command[0]&0x08)==0?1:0)){chip->st2|=ST2_CTRLMARK;upd765aReadOrWriteComplete(chip);break;}upd765aIncrementSector(chip);upd765aReadData(chip);break;}}else{chip->stm|=STM_RDY;}break;case UPD765A_RESULT_PHASE:DEB(Msg(M_DEBUG,"FDC: result[%d] %02x",chip->count,chip->result[chip->count]););chip->data=chip->result[chip->count++];if(--chip->remaining==0){upd765aEnterIdleState(chip);}else{chip->stm|=STM_RDY;}break;}}static void upd765aEnterResultPhase(Upd765aFdc*chip,int count){chip->phase=UPD765A_RESULT_PHASE;chip->stm&=~STM_EXEC;chip->stm|=(STM_RDY|STM_DIR);chip->count=0;chip->remaining=count;}static void upd765aEnterIdleState(Upd765aFdc*chip){chip->phase=UPD765A_CMD_PHASE;chip->stm&=~(STM_DIR|STM_EXEC|STM_BUSY);chip->stm|=STM_RDY;chip->count=0;}static void upd765aGetDriveAndSide(Upd765aFdc*chip){chip->d=chip->command[1]&ST3_DRIVE;chip->drive=chip->d<chip->numDrives?chip->drives[chip->d]:upd765aDummyDrive;chip->s=(chip->command[1]&ST3_SIDE)>>2;DdriveSetSide(chip->drive,chip->s);}static void upd765aSeek(Upd765aFdc*chip,int track){DdriveStatus status=DDRIVE_STATUS_NOTREADY;chip->stm|=(1<<chip->d);chip->st0=chip->command[1]&(ST3_SIDE|ST3_DRIVE);chip->st1=0;chip->st2=0;if(DdriveIsReady(chip->drive)){status=DdriveSeek(chip->drive,track);}switch(status){case DDRIVE_STATUS_OK:break;case DDRIVE_STATUS_NOTREADY:chip->st0|=(ST0_NOTFINISH|ST0_NOTRDY);break;case DDRIVE_STATUS_SEEKFAIL:chip->st0|=ST0_NOTFINISH;break;default:assert(0);}chip->st0|=ST0_SEEKEND;chip->stm&=~(1<<chip->d);chip->interrupt=1;upd765aEnterIdleState(chip);}static void upd765aReadId(Upd765aFdc*chip){DdriveStatus status=DDRIVE_STATUS_NOTREADY;DiskSectorInfo*sip=(DiskSectorInfo*)chip->databuff;chip->st0=chip->command[1]&(ST3_SIDE|ST3_DRIVE);chip->st1=0;chip->st2=0;if(DdriveIsReady(chip->drive)){status=DdriveReadId(chip->drive,sip);}switch(status){case DDRIVE_STATUS_OK:chip->st1&=~ST1_ENDCYL;if(DdriveIsIndexHole(chip->drive)){chip->st1|=ST1_ENDCYL;}break;case DDRIVE_STATUS_NOTREADY:chip->st0|=(ST0_STOPPED|ST0_NOTRDY);break;case DDRIVE_STATUS_SEEKFAIL:case DDRIVE_STATUS_NOADDR:chip->st0|=ST0_NOTFINISH;chip->st1|=ST1_ADDRMARK;break;default:assert(0);break;}chip->result[0]=chip->st0;chip->result[1]=chip->st1;chip->result[2]=chip->st2;if(status==DDRIVE_STATUS_OK){chip->result[3]=sip->track;chip->result[4]=sip->head;chip->result[5]=sip->sector;chip->result[6]=sip->bps;}else{chip->result[3]=0;chip->result[4]=0;chip->result[5]=1;chip->result[6]=2;}chip->interrupt=1;upd765aEnterResultPhase(chip,7);}static void upd765aWriteData(Upd765aFdc*chip){chip->st0=chip->command[1]&(ST3_SIDE|ST3_DRIVE);chip->st1=0;chip->st2=0;if(DdriveIsReady(chip->drive)){DiskSectorInfo*sip;if(DdriveIsWriteProtected(chip->drive)){chip->st1|=ST1_WRPROT;}else if((sip=upd765aGetMatchingSector(chip))!=(DiskSectorInfo*)0){sip->si_st2&=~ST2_CTRLMARK;if((chip->command[0]&0x08)!=0){sip->si_st2|=ST2_CTRLMARK;}chip->count=0;chip->remaining=chip->command[5]==0?chip->command[8]:128<<chip->command[5];chip->datap=DdriveWriteSector(chip->drive,sip->sector);chip->stm|=STM_RDY;chip->stm&=~STM_DIR;return;}}else{chip->st0|=(ST0_STOPPED|ST0_NOTRDY);}upd765aReadOrWriteComplete(chip);}static void upd765aReadData(Upd765aFdc*chip){chip->st0=chip->command[1]&(ST3_SIDE|ST3_DRIVE);chip->st1=0;chip->st2=0;if(DdriveIsReady(chip->drive)){DiskSectorInfo*sip;while((sip=upd765aGetMatchingSector(chip))!=(DiskSectorInfo*)0){if((chip->command[0]&0x20)!=0&&chip->dam==((chip->command[0]&0x08)==0?1:0)){if((chip->command[4]==chip->command[6])&&(chip->mt==0||chip->s==1)){break;}upd765aIncrementSector(chip);}else{chip->count=0;chip->remaining=chip->command[5]==0?chip->command[8]:128<<chip->command[5];chip->datap=DdriveReadSector(chip->drive,sip->sector);chip->stm|=(STM_RDY|STM_DIR);return;}}}else{chip->st0|=(ST0_STOPPED|ST0_NOTRDY);}upd765aReadOrWriteComplete(chip);}static void upd765aReadOrWriteComplete(Upd765aFdc*chip){chip->result[0]=chip->st0;chip->result[1]=chip->st1;chip->result[2]=chip->st2;chip->result[3]=chip->command[2];chip->result[4]=chip->command[3];chip->result[5]=chip->command[4];chip->result[6]=chip->command[5];chip->interrupt=1;upd765aEnterResultPhase(chip,7);}static DiskSectorInfo*upd765aGetMatchingSector(Upd765aFdc*chip){DiskSectorInfo*sip=(DiskSectorInfo*)chip->databuff;int rotations=2;do{if(DdriveReadId(chip->drive,sip)==DDRIVE_STATUS_OK){if(sip->sector==chip->command[4]){if(sip->track==chip->command[2]){if(sip->head==chip->command[3]&&sip->bps==chip->command[5]){if(sip->sector==chip->command[6]){chip->st1|=ST1_ENDCYL;}chip->dam=(sip->si_st2&ST2_CTRLMARK)>>6;return sip;}}else{chip->st1|=ST1_NODATA;chip->st2|=ST2_ERRCYLTRK;if(sip->track==0xff){chip->st2|=ST2_BADCYL;}return(DiskSectorInfo*)0;}}}if(DdriveIsIndexHole(chip->drive)){rotations--;}}while(rotations>0);chip->st0|=ST0_NOTFINISH;chip->st1|=ST1_NODATA;return(DiskSectorInfo*)0;}static void upd765aIncrementSector(Upd765aFdc*chip){if(chip->mt!=0&&DdriveIsIndexHole(chip->drive)){chip->s=1;DdriveSetSide(chip->drive,chip->s);chip->command[4]=1;chip->command[3]=1;}else{chip->command[4]++;}}
