/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char ay8912_c[]="$Id: ay8912.c,v 4.37 2006/12/22 14:29:13 erik Rel $";
#endif
#include <stdlib.h>
#include <assert.h>
#include "z80/z80.h"
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#ifndef EVALUATION
#include "fileio/psg.h"
#endif
#include "sound/sound.h"
typedef struct _AY8912{uns8 regs[16];unsigned char*buf;unsigned int lastPos;unsigned int updateStep;int periodA,periodB,periodC,periodN,periodE;int countA,countB,countC,countN,countE;unsigned int volA,volB,volC,volE;unsigned char envelopeA,envelopeB,envelopeC;unsigned char outputA,outputB,outputC,outputN;signed char countEnv;unsigned char hold,alternate,attack,holding;int rng;unsigned int volTable[32];}AY8912;
#define AY_AFINE	0
#define AY_ACOARSE	1
#define AY_BFINE	2
#define AY_BCOARSE	3
#define AY_CFINE	4
#define AY_CCOARSE	5
#define AY_NOISEPER	6
#define AY_ENABLE	7
#define AY_AVOL	8
#define AY_BVOL	9
#define AY_CVOL	10
#define AY_EFINE	11
#define AY_ECOARSE	12
#define AY_ESHAPE	13
#define AY_PORTA	14
#define AY_PORTB	15
#define AY_CLOCK	1773400.0
#define AY_STEP	0x8000
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_SOUND) != 0) { x } }
#else
#define DEB(x)
#endif
static void ay8912WriteRegister(uns8,uns8);static void ay8912Update(unsigned int);static void ay8912SetGain(int);static AY8912*ayP=(AY8912*)0;void Ay8912Init(void){if(ayP==(AY8912*)0){ayP=Malloc(sizeof(AY8912),"Ay8912Init");}ayP->updateStep=(double)AY_STEP*SoundDataP->sampleRate*8.0/AY_CLOCK;DEB(Msg(M_DEBUG,"SND: updateStep       %u",ayP->updateStep););if(ayP->buf==(unsigned char*)0){ay8912SetGain(0x00);ayP->periodA=ayP->periodB=ayP->periodC=ayP->updateStep;ayP->periodN=ayP->updateStep;ayP->periodE=ayP->updateStep>>1;ayP->countA=ayP->countB=ayP->countC=ayP->updateStep;ayP->countN=1;ayP->countE=1;ayP->rng=1;}else{free(ayP->buf);}ayP->buf=Malloc(sizeof(unsigned char)*(size_t)(SoundDataP->samplesPerFrame*3),"Ay8912Init");SoundDataP->ayBuf=ayP->buf;ayP->lastPos=0;}void Ay8912Out(uns8 reg,uns8 val){if(reg==AY_ESHAPE||val!=ayP->regs[reg]){unsigned int pos=TSTATES/SoundDataP->tstatesPerSample;if(pos!=ayP->lastPos){if(pos>SoundDataP->samplesPerFrame){pos=SoundDataP->samplesPerFrame;}ay8912Update(pos);}}ay8912WriteRegister(reg,val);
#ifndef EVALUATION
PsgOutByte(reg,val);
#endif
}static void ay8912WriteRegister(uns8 reg,uns8 val){int old;assert(reg<sizeof(ayP->regs));ayP->regs[reg]=val;switch(reg){
#define SET_CHANNEL_FREQ(ch) case AY_##ch##COARSE:	ayP->regs[AY_##ch##COARSE] &= 0x0f;	case AY_##ch##FINE:	old = ayP->period##ch;	ayP->period##ch = ayP->regs[AY_##ch##FINE] + (ayP->regs[AY_##ch##COARSE] << 8);	if (ayP->period##ch == 0)	{	ayP->period##ch = 1;	}	ayP->period##ch *= ayP->updateStep;	ayP->count##ch += ayP->period##ch - old;	if (ayP->count##ch <= 0)	{	ayP->count##ch = 1;	}	break;
SET_CHANNEL_FREQ(A);SET_CHANNEL_FREQ(B);SET_CHANNEL_FREQ(C);case AY_NOISEPER:ayP->regs[AY_NOISEPER]&=0x1f;old=ayP->periodN;ayP->periodN=ayP->regs[AY_NOISEPER];if(ayP->periodN==0){ayP->periodN=1;}ayP->periodN*=ayP->updateStep;ayP->countN+=ayP->periodN-old;if(ayP->countN<=0){ayP->countN=1;}break;case AY_ENABLE:break;
#define SET_CHANNEL_VOL(ch) case AY_##ch##VOL:	ayP->regs[AY_##ch##VOL] &= 0x1f;	ayP->envelope##ch = ayP->regs[AY_##ch##VOL] & 0x10;	if (ayP->envelope##ch != 0)	{	ayP->vol##ch = ayP->volE;	}	else	{	ayP->vol##ch = ayP->volTable[ayP->regs[AY_##ch##VOL] != 0 ? (ayP->regs[AY_##ch##VOL] << 1) + 1 : 0];	}	break;
SET_CHANNEL_VOL(A);SET_CHANNEL_VOL(B);SET_CHANNEL_VOL(C);case AY_EFINE:case AY_ECOARSE:old=ayP->periodE;ayP->periodE=ayP->regs[AY_EFINE]+(ayP->regs[AY_ECOARSE]<<8);if(ayP->periodE==0){ayP->periodE=ayP->updateStep>>1;}else{ayP->periodE*=ayP->updateStep;}ayP->countE+=ayP->periodE-old;if(ayP->countE<=0){ayP->countE=1;}break;case AY_ESHAPE:ayP->regs[AY_ESHAPE]&=0x0f;ayP->attack=(ayP->regs[AY_ESHAPE]&0x04)!=0?0x1f :0x00;if((ayP->regs[AY_ESHAPE]&0x08)==0){ayP->hold=1;ayP->alternate=ayP->attack;}else{ayP->hold=ayP->regs[AY_ESHAPE]&0x01;ayP->alternate=ayP->regs[AY_ESHAPE]&0x02;}ayP->countE=ayP->periodE;ayP->countEnv=0x1f;ayP->holding=0;ayP->volE=ayP->volTable[ayP->countEnv^ayP->attack];if(ayP->envelopeA!=0){ayP->volA=ayP->volE;}if(ayP->envelopeB!=0){ayP->volB=ayP->volE;}if(ayP->envelopeC!=0){ayP->volC=ayP->volE;}break;}}void Ay8912FillBuffer(void){if(ayP->lastPos<SoundDataP->samplesPerFrame){ay8912Update(SoundDataP->samplesPerFrame);}ayP->lastPos=0;}static void ay8912Update(unsigned int pos){unsigned char outn;unsigned char*bufferA=&ayP->buf[ayP->lastPos];unsigned char*bufferB=bufferA+SoundDataP->samplesPerFrame;unsigned char*bufferC=bufferB+SoundDataP->samplesPerFrame;int length,left,nextevent;int volA,volB,volC;assert(pos<=SoundDataP->samplesPerFrame);length=pos-ayP->lastPos;
#define PROCESS_OUTPUT(ch,mask) if ((ayP->regs[AY_ENABLE] & (mask)) != 0)	{	if (ayP->count##ch <= length * AY_STEP)	{	ayP->count##ch += length * AY_STEP;	}	ayP->output##ch = 1;	}	else if (ayP->regs[AY_##ch##VOL] == 0)	{	if (ayP->count##ch <= length * AY_STEP)	{	ayP->count##ch += length * AY_STEP;	}	}
PROCESS_OUTPUT(A,0x01);PROCESS_OUTPUT(B,0x02);PROCESS_OUTPUT(C,0x04);if((ayP->regs[AY_ENABLE]&0x38)==0x38){if(ayP->countN<=length*AY_STEP){ayP->countN+=length*AY_STEP;}}outn=(ayP->outputN|ayP->regs[AY_ENABLE]);while(length--!=0){volA=volB=volC=0;left=AY_STEP;do{nextevent=ayP->countN<left?ayP->countN:left;
#define PROCESS_CHANNEL(ch,mask) if ((outn & (mask)) != 0)	{	if (ayP->output##ch != 0)	{	vol##ch += ayP->count##ch;	}	ayP->count##ch -= nextevent;	while (ayP->count##ch <= 0)	{	ayP->count##ch += ayP->period##ch;	if (ayP->count##ch > 0)	{	ayP->output##ch ^= 1;	if (ayP->output##ch != 0)	{	vol##ch += ayP->period##ch;	}	break;	}	ayP->count##ch += ayP->period##ch;	vol##ch += ayP->period##ch;	}	if (ayP->output##ch != 0)	{	vol##ch -= ayP->count##ch;	}	}	else	{	ayP->count##ch -= nextevent;	while (ayP->count##ch <= 0)	{	ayP->count##ch += ayP->period##ch;	if (ayP->count##ch > 0)	{	ayP->output##ch ^= 1;	break;	}	ayP->count##ch += ayP->period##ch;	}	}
PROCESS_CHANNEL(A,0x08);PROCESS_CHANNEL(B,0x10);PROCESS_CHANNEL(C,0x20);ayP->countN-=nextevent;if(ayP->countN<=0){if(((ayP->rng+1)&2)!=0){ayP->outputN=~ayP->outputN;outn=ayP->outputN|ayP->regs[AY_ENABLE];}if((ayP->rng&1)!=0){ayP->rng^=0x28000;}ayP->rng>>=1;ayP->countN+=ayP->periodN;}left-=nextevent;}while(left>0);if(ayP->holding==0){if((ayP->countE-=AY_STEP)<=0){do{ayP->countEnv--;ayP->countE+=ayP->periodE;}while(ayP->countE<=0);if(ayP->countEnv<0){if(ayP->hold!=0){if(ayP->alternate!=0){ayP->attack^=0x1f;}ayP->holding=1;ayP->countEnv=0;}else{if(ayP->alternate!=0&&(ayP->countEnv&0x20)!=0){ayP->attack^=0x1f;}ayP->countEnv&=0x1f;}}ayP->volE=ayP->volTable[ayP->countEnv^ayP->attack];if(ayP->envelopeA!=0){ayP->volA=ayP->volE;}if(ayP->envelopeB!=0){ayP->volB=ayP->volE;}if(ayP->envelopeC!=0){ayP->volC=ayP->volE;}}}volA*=ayP->volA;volB*=ayP->volB;volC*=ayP->volC;*bufferA++=(volA/AY_STEP)>>8;*bufferB++=(volB/AY_STEP)>>8;*bufferC++=(volC/AY_STEP)>>8;}ayP->lastPos=pos;}static void ay8912SetGain(int gain){int i;double out;gain&=0xff;out=21845.0;while(gain-->0){out*=1.023292992;}for(i=31;i>0;i--){ayP->volTable[i]=out>21845.0?21845U:(unsigned int)out;out/=1.188502227;}ayP->volTable[0]=0;}
