/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: sound.c,v 4.17 2006/12/22 14:29:13 erik Rel $";
#endif
#include <stdlib.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#ifndef EVALUATION
#include "fileio/psg.h"
#include "fileio/wav.h"
#endif
#include "sound/sound.h"
#define mixerSize	((int)(SoundDataP->samplesPerFrame))
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_SOUND) != 0) { x } }
#else
#define DEB(x)
#endif
static void soundInit2(void);static int soundOpenOrClose(void);static void soundOpen(void);static void soundClose(void);void(*SpeakerOutByte)(uns8);void(*Ay8912OutByte)(uns8,uns8);SoundData*SoundDataP;static unsigned char*mixerBuffer;static int mixerStep,mixerRemain;void SoundInit(void){SoundDataP=Malloc(sizeof(SoundData),"SoundInit");SoundDataP->state=SOUND_STATE_CLOSED;SoundSetSpeaker(CFGGET(sound)>0?CFGGET(soundVolumeSp)/25:0U);SoundSetAy(CFGGET(sound)>0?CFGGET(soundVolumeAy)/25:0U);atexit(soundClose);}static void soundInit2(void){if(SoundDataP->state==SOUND_STATE_OPENED){SoundDataP->samplesPerFrame=SoundDataP->sampleRate*2/CFGGET(speed);SoundDataP->tstatesPerSample=Machine->tstatesPerFrame/SoundDataP->samplesPerFrame;DEB(Msg(M_DEBUG,"SND: tstatesPerSample %u",SoundDataP->tstatesPerSample);Msg(M_DEBUG,"SND: samplesPerFrame  %u",SoundDataP->samplesPerFrame););SpeakerInit();Ay8912Init();mixerStep=(SoundDataP->sampleBits>>3)<<SoundDataP->stereo;mixerRemain=0;if(mixerBuffer!=(unsigned char*)0){free(mixerBuffer);}mixerBuffer=Malloc((size_t)(SoundDataP->samplesPerFrame*mixerStep),"SoundControl");}}static int soundOpenOrClose(void){if(CFGGET(fastmode)==0&&(SoundDataP->spVol+SoundDataP->ayVol)>0){soundOpen();if(SoundDataP->state==SOUND_STATE_OPENED){return 1;}}else{soundClose();}return 0;}static void soundOpen(void){if(SoundDataP->state>=SOUND_STATE_CLOSED){int sample,bits,stereo,nonblocking;
#ifdef SOUND_NETWORK
DEB(Msg(M_DEBUG,"SND: connecting to audio server"););
#else
DEB(Msg(M_DEBUG,"SND: opening audio device"););
#endif
sample=CFGGET(soundSample);bits=CFGGET(soundBits);stereo=1;nonblocking=CFGGET(soundNonblocking);switch(OsdOpenAudioDevice(CFGGET(soundDev),&nonblocking,&sample,&bits,&stereo)){case 0:DEB(Msg(M_DEBUG,"SND: sample rate      %d",sample);Msg(M_DEBUG,"SND: sample bits      %d",bits);Msg(M_DEBUG,"SND: stereo           %d",stereo);Msg(M_DEBUG,"SND: nonblocking      %d",nonblocking););SoundDataP->state=SOUND_STATE_OPENED;CFGSET(soundSample,sample);CFGSET(soundBits,bits);if(stereo==0){CFGSET(soundStereo,AY8912_MODE_MONO);}CFGSET(soundNonblocking,nonblocking);SoundDataP->sampleRate=sample;SoundDataP->sampleBits=bits;SoundDataP->stereo=stereo;SoundDataP->nonblocking=nonblocking;soundInit2();break;case-1:break;case-2:Msg(M_WARN,"sound initialisation failed");SoundDataP->state=SOUND_STATE_NONEXIST;CFGSET(sound,-1);}}}static void soundClose(void){if(SoundDataP->state==SOUND_STATE_OPENED){DEB(Msg(M_DEBUG,"SND: closing audio device"););(void)OsdDrainAudioDevice();OsdCloseAudioDevice();SoundDataP->state=SOUND_STATE_CLOSED;}}void SoundControl(SoundAction cmd){switch(cmd){case SOUND_ACTION_INIT:soundInit2();break;case SOUND_ACTION_SUSPEND:if(SoundDataP->state==SOUND_STATE_OPENED){(void)OsdDrainAudioDevice();}break;case SOUND_ACTION_RESUME:break;case SOUND_ACTION_ON:SoundSetSpeaker(SoundDataP->spVol);SoundSetAy(SoundDataP->ayVol);break;case SOUND_ACTION_OFF:SpeakerOutByte=MachineDummyOutByte;Ay8912OutByte=(void(*)(uns8,uns8))MachineDummyOutByte;soundClose();break;default:assert(0);}}void SoundSetSpeaker(unsigned int volume){SoundDataP->spVol=volume;if(soundOpenOrClose()==0||volume==0){SpeakerOutByte=MachineDummyOutByte;}else{SpeakerOutByte=SpeakerOut;}}void SoundSetAy(unsigned int volume){SoundDataP->ayVol=volume;if(soundOpenOrClose()==0||volume==0){Ay8912OutByte=(void(*)(uns8,uns8))MachineDummyOutByte;}else{Ay8912OutByte=Ay8912Out;}}void SoundPlayBuffer(void){if(SoundDataP->state==SOUND_STATE_OPENED){unsigned char*bpSp,*bpAyA,*bpAyB,*bpAyC;unsigned int volSp,volAy;int i;if(mixerRemain>0){unsigned char*bpOut=&mixerBuffer[(mixerSize-mixerRemain-1)*mixerStep];while(mixerRemain>0){if((i=OsdPlayAudioBuffer(bpOut,mixerRemain))<0){goto error;}mixerRemain-=i;bpOut+=i*mixerStep;}}if((volSp=SoundDataP->spVol)>0){SpeakerFillBuffer();}if((volAy=SoundDataP->ayVol)>0){Ay8912FillBuffer();}i=mixerSize;bpSp=SoundDataP->spBuf;bpAyA=SoundDataP->ayBuf;bpAyB=bpAyA+SoundDataP->samplesPerFrame;bpAyC=bpAyB+SoundDataP->samplesPerFrame;if(SoundDataP->sampleBits==16){unsigned short*bpOut=(unsigned short*)mixerBuffer;if(!SoundIsStereo()){unsigned int val;while(i-->0){val=(unsigned int)*bpSp++*volSp+(unsigned int)*bpAyA++*volAy+(unsigned int)*bpAyB++*volAy+(unsigned int)*bpAyC++*volAy;*bpOut++=(unsigned short)(val<<4);}}
#ifndef EVALUATION
else if(CFGGET(soundStereo)==AY8912_MODE_ACB){unsigned int valL,valR;while(i-->0){valL=valR=(unsigned int)*bpSp*volSp+(unsigned int)*bpAyC*volAy;valL+=(unsigned int)*bpAyA*volAy;valR+=(unsigned int)*bpAyB*volAy;
#ifdef SOUND_RL_CHANNEL
*bpOut++=(unsigned short)(valR<<4);*bpOut++=(unsigned short)(valL<<4);
#else 
*bpOut++=(unsigned short)(valL<<4);*bpOut++=(unsigned short)(valR<<4);
#endif
bpSp++;bpAyA++;bpAyB++;bpAyC++;}}else if(CFGGET(soundStereo)==AY8912_MODE_ABC){unsigned int valL,valR;while(i-->0){valL=valR=(unsigned int)*bpSp*volSp+(unsigned int)*bpAyB*volAy;valL+=(unsigned int)*bpAyA*volAy;valR+=(unsigned int)*bpAyC*volAy;
#ifdef SOUND_RL_CHANNEL
*bpOut++=(unsigned short)(valR<<4);*bpOut++=(unsigned short)(valL<<4);
#else 
*bpOut++=(unsigned short)(valL<<4);*bpOut++=(unsigned short)(valR<<4);
#endif
bpSp++;bpAyA++;bpAyB++;bpAyC++;}}
#endif
else{unsigned int val;while(i-->0){val=(unsigned int)*bpSp++*volSp+(unsigned int)*bpAyA++*volAy+(unsigned int)*bpAyB++*volAy+(unsigned int)*bpAyC++*volAy;*bpOut++=(unsigned short)(val<<4);*bpOut++=(unsigned short)(val<<4);}}}else{unsigned char*bpOut=mixerBuffer;if(!SoundIsStereo()){unsigned int val;while(i-->0){val=(unsigned int)*bpSp++*volSp+(unsigned int)*bpAyA++*volAy+(unsigned int)*bpAyB++*volAy+(unsigned int)*bpAyC++*volAy;*bpOut++=(unsigned char)(val>>4);}}
#ifndef EVALUATION
else if(CFGGET(soundStereo)==AY8912_MODE_ACB){unsigned int valL,valR;while(i-->0){valL=valR=(unsigned int)*bpSp*volSp+(unsigned int)*bpAyC*volAy;valL+=(unsigned int)*bpAyA*volAy;valR+=(unsigned int)*bpAyB*volAy;
#ifdef SOUND_RL_CHANNEL
*bpOut++=(unsigned char)(valR>>4);*bpOut++=(unsigned char)(valL>>4);
#else 
*bpOut++=(unsigned char)(valL>>4);*bpOut++=(unsigned char)(valR>>4);
#endif
bpSp++;bpAyA++;bpAyB++;bpAyC++;}}else if(CFGGET(soundStereo)==AY8912_MODE_ABC){unsigned int valL,valR;while(i-->0){valL=valR=(unsigned int)*bpSp*volSp+(unsigned int)*bpAyB*volAy;valL+=(unsigned int)*bpAyA*volAy;valR+=(unsigned int)*bpAyC*volAy;
#ifdef SOUND_RL_CHANNEL
*bpOut++=(unsigned char)(valR>>4);*bpOut++=(unsigned char)(valL>>4);
#else 
*bpOut++=(unsigned char)(valL>>4);*bpOut++=(unsigned char)(valR>>4);
#endif
bpSp++;bpAyA++;bpAyB++;bpAyC++;}}
#endif
else{unsigned int val;while(i-->0){val=(unsigned int)*bpSp++*volSp+(unsigned int)*bpAyA++*volAy+(unsigned int)*bpAyB++*volAy+(unsigned int)*bpAyC++*volAy;*bpOut++=(unsigned char)(val>>4);*bpOut++=(unsigned char)(val>>4);}}}
#ifndef EVALUATION
(void)WavWrite(mixerBuffer,mixerSize*mixerStep);PsgWrite();
#endif
if(SoundDataP->nonblocking!=0){if((i=OsdGetAudioFreespace())<0){goto error;}if(i<mixerSize){DEB(Msg(M_DEBUG,"SND: free %d<%d",i,mixerSize););}if(i>mixerSize){i=mixerSize;}}else{i=mixerSize;}if((i=OsdPlayAudioBuffer(mixerBuffer,i))<0){error:SoundSetSpeaker(0);SoundSetAy(0);}else{mixerRemain=mixerSize-i;if(mixerRemain>0){DEB(Msg(M_DEBUG,"SND: written %d<%d",i,mixerSize););}}}}
