/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: sound_oss.c,v 4.9 2006/12/22 14:29:14 erik Rel $";
#endif
#include <sys/types.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>
#include <assert.h>
#include SOUNDCARD_H_PATH
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "sound/sound.h"
#define CALCULATE_FRAGMENTS
#ifdef WORDS_BIGENDIAN
#define AFMT_U16	AFMT_U16_BE
#else
#define AFMT_U16	AFMT_U16_LE
#endif
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_SOUND) != 0) { x } }
#else
#define DEB(x)
#endif
static int audioFd=-1;static int audioShift;int OsdOpenAudioDevice(const char*device,int*nonblocking,int*sample,int*bits,int*stereo){audio_buf_info info;
#ifdef CALCULATE_FRAGMENTS
int fragSize,fragCount;
#endif
int i,err;if((audioFd=open(device,O_WRONLY|O_NONBLOCK))<0){Msg(M_PERR,"couldn't open audio device <%s> for writing",device);return(errno==EBUSY||errno==EAGAIN?-1:-2);}if(*nonblocking==0){i=fcntl(audioFd,F_GETFL);if(i<0||fcntl(audioFd,F_SETFL,i&~O_NONBLOCK)<0){Msg(M_PERR,"couldn't set blocking mode");*nonblocking=1;}}i=*bits==16?AFMT_U16:AFMT_U8;if((err=ioctl(audioFd,SNDCTL_DSP_SETFMT,&i))<0||i!=(*bits==16?AFMT_U16:AFMT_U8)){if(*bits==16){Msg(M_WARN,"couldn't set sample resolution to 16 Bit, trying 8 Bit");*bits=8;i=AFMT_U8;err=ioctl(audioFd,SNDCTL_DSP_SETFMT,&i);if(i!=AFMT_U8){err=-1;}}}if(err<0){Msg(M_PERR,"couldn't set sample resolution to %d Bit",*bits);goto error;}audioShift=*bits>>4;i=1;if(ioctl(audioFd,SNDCTL_DSP_STEREO,&i)<0||i!=1){Msg(M_WARN,"couldn't set stereo output, forcing mono");i=0;if(ioctl(audioFd,SNDCTL_DSP_STEREO,&i)<0||i){Msg(M_PERR,"couldn't set mono output");goto error;}}*stereo=i;audioShift+=*stereo;i=*sample;if(ioctl(audioFd,SNDCTL_DSP_SPEED,&i)<0){Msg(M_PERR,"couldn't set sample rate to %d Hz",i);goto error;}*sample=i;if(ioctl(audioFd,SNDCTL_DSP_GETOSPACE,&info)<0){Msg(M_PERR,"couldn't get audio informaions");}DEB(Msg(M_DEBUG,"SND: frags size       %d",info.fragsize);Msg(M_DEBUG,"SND: frags count      %d",info.fragstotal););
#ifdef CALCULATE_FRAGMENTS
fragSize=256<<audioShift;fragCount=((((*sample<<audioShift)+fragSize-1)/fragSize)+1)&~1;for(i=1;1<<i<fragSize;i++);i|=fragCount<<16;DEB(Msg(M_DEBUG,"SND: new frags size   %d",fragSize);Msg(M_DEBUG,"SND: new frags count  %d",fragCount););if(ioctl(audioFd,SNDCTL_DSP_SETFRAGMENT,&i)<0){Msg(M_PERR,"couldn't set fragment size");goto error;}
#endif
return 0;error:(void)close(audioFd);audioFd=-1;return-1;}void OsdCloseAudioDevice(void){assert(audioFd>=0);(void)close(audioFd);audioFd=-1;}int OsdDrainAudioDevice(void){return(ioctl(audioFd,SNDCTL_DSP_POST,(void*)0));}int OsdGetAudioFreespace(void){audio_buf_info info;return(ioctl(audioFd,SNDCTL_DSP_GETOSPACE,&info)<0?errno==EPIPE?0:-1:info.bytes>>audioShift);}int OsdPlayAudioBuffer(unsigned char*buf,int len){int written,remain;for(written=0,remain=len<<audioShift;remain>0;buf+=written,remain-=written){do{written=(int)write(audioFd,buf,(size_t)remain);}while(written<0&&errno==EINTR);if(written<0){break;}}if(written<0&&errno!=EAGAIN){Msg(M_PERR,"couldn't write to audio device");return written;}return len-(remain>>audioShift);}
