/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: sound_sunos.c,v 4.9 2006/12/22 14:29:14 erik Rel $";
#endif
#include <sys/types.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sun/audioio.h>
#include <errno.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "sound/lin2ulaw.h"
#include "sound/sound.h"
#define ABSDIFF(x,y)	((x > y) ? (x - y) : (y - x))
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_SOUND) != 0) { x } }
#else
#define DEB(x)
#endif
static int audioFd=-1;static int audioBits,audioShift,audioQueueSize;
#ifdef AUDIO_GETDEV
static int samprates[11]={8000,9600,11025,16000,18900,22050,32000,37800,44100,48000,100000};
#endif
int OsdOpenAudioDevice(const char*device,int*nonblocking,int*sample,int*bits,int*stereo){
#ifdef AUDIO_GETDEV
int audioDev;audio_info_t audioInf;int save,err;
#endif
if((audioFd=open(device,O_WRONLY|O_NDELAY|O_NONBLOCK))<0){Msg(M_PERR,"couldn't open audio device <%s> for writing",device);return(errno==EBUSY||errno==EAGAIN?-1:-2);}(void)ioctl(audioFd,AUDIO_DRAIN,0);
#ifdef AUDIO_GETDEV
if(ioctl(audioFd,AUDIO_GETDEV,&audioDev)<0){Msg(M_PERR,"couldn't get audio device type");goto error;}DEB(Msg(M_DEBUG,"SND: audio device is %d (%s)",audioDev,audioDev==1?"AMD":audioDev==2?"Dbri with speakerbox":audioDev==3?"Dbri with internal speaker":"unknown"););if(ioctl(audioFd,AUDIO_GETINFO,&audioInf)<0){Msg(M_PERR,"couldn't get audio settings");goto error;}DEB(Msg(M_DEBUG,"SND: sample rate      %d",audioInf.play.sample_rate);Msg(M_DEBUG,"SND: sample bits      %d",audioInf.play.precision);Msg(M_DEBUG,"SND: channels         %d",audioInf.play.channels);Msg(M_DEBUG,"SND: encoding         %s",audioInf.play.encoding==AUDIO_ENCODING_ULAW?"ulaw":audioInf.play.encoding==AUDIO_ENCODING_ALAW?"alaw":audioInf.play.encoding==AUDIO_ENCODING_LINEAR?"linear":"unknown"););if(audioDev==AUDIO_DEV_AMD){if(*sample!=8000){Msg(M_WARN,"audio device supports 8000 Hz sample rate only");*sample=8000;}}else{int i,j,k;j=50000;for(i=0;i<NELEM(samprates);i++){if((k=ABSDIFF(*sample,samprates[i]))==0){break;}else{if(k>j){Msg(M_WARN,"sample rate set to nearest legal value %d",samprates[i-1]);*sample=samprates[i-1];break;}j=k;}}}save=audioInf.play.sample_rate;audioInf.play.sample_rate=*sample;if(ioctl(audioFd,AUDIO_SETINFO,&audioInf)<0){Msg(M_PERR,"couldn't set sample rate to %d Hz, using default sample rate %d Hz",*sample,save);audioInf.play.sample_rate=save;*sample=save;}save=audioInf.play.precision;audioInf.play.precision=*bits;audioInf.play.encoding=*bits==16?AUDIO_ENCODING_LINEAR:AUDIO_ENCODING_ULAW;if((err=ioctl(audioFd,AUDIO_SETINFO,&audioInf))<0){if(*bits==16){Msg(M_WARN,"couldn't set sample resolution to 16 Bit, trying 8 Bit");*bits=8;audioInf.play.precision=*bits;audioInf.play.encoding=AUDIO_ENCODING_ULAW;err=ioctl(audioFd,AUDIO_SETINFO,&audioInf);}}if(err<0){Msg(M_PERR,"couldn't set sample resolution to %d Bit",*bits);goto error;}audioInf.play.channels=2;if(ioctl(audioFd,AUDIO_SETINFO,&audioInf)<0){Msg(M_WARN,"couldn't set stereo output, forcing mono");audioInf.play.channels=1;if(ioctl(audioFd,AUDIO_SETINFO,&audioInf)<0){Msg(M_PERR,"couldn't set mono output");goto error;}}*stereo=audioInf.play.channels==2;
#else
DEB(Msg(M_DEBUG,"SND: audio device is AMD 79c30 (old version)");Msg(M_DEBUG,"SND: sample rate      8000");Msg(M_DEBUG,"SND: sample bits      8");Msg(M_DEBUG,"SND: channels         1");Msg(M_DEBUG,"SND: encoding         ulaw"););if(*sample!=8000){Msg(M_WARN,"this audio device supports 8000 Hz sample rate only");*sample=8000;}*bits=8;*stereo=0;
#endif 
audioBits=*bits;audioShift=*bits>>4;audioShift+=*stereo;audioQueueSize=*sample;if(*nonblocking!=0){int flags=fcntl(audioFd,F_GETFL);if(flags<0||fcntl(audioFd,F_SETFL,flags|O_NONBLOCK)<0){Msg(M_PERR,"couldn't set nonblocking mode");goto error;}}return 0;error:(void)close(audioFd);audioFd=-1;return-1;}void OsdCloseAudioDevice(void){assert(audioFd>=0);(void)close(audioFd);audioFd=-1;}int OsdDrainAudioDevice(void){return(ioctl(audioFd,AUDIO_DRAIN,0));}int OsdGetAudioFreespace(void){return audioQueueSize;}int OsdPlayAudioBuffer(unsigned char*buf,int len){int written,remain=len<<audioShift,i;if(audioBits==8){for(i=0;i<remain;i++){buf[i]=Lin2Ulaw[buf[i]^0x80];}}for(written=0;remain>0;buf+=written,remain-=written){do{written=(int)write(audioFd,buf,(size_t)remain);}while(written<0&&errno==EINTR);if(written<0){break;}}if(written<0&&errno!=EAGAIN){Msg(M_PERR,"couldn't write to audio device");return written;}return len-(remain>>audioShift);}
