/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: tape.c,v 4.48 2006/12/22 14:28:54 erik Rel $";
#endif
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "z80/z80.h"
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "event.h"
#include "main.h"
#include "ui/poke.h"
#include "sound/sound.h"
#include "fileio/tap.h"
#include "fileio/tzx.h"
#include "ui/ui.h"
#include "fileio/voc.h"
#include "ui/vtp.h"
#include "tape.h"
enum{TAPE_STATUS_OK,TAPE_STATUS_FAILED,TAPE_STATUS_STOP,TAPE_STATUS_STOP48,TAPE_STATUS_EOB,TAPE_STATUS_EOT};enum{BLOCK_STATE_PILOT,BLOCK_STATE_SYNC1,BLOCK_STATE_SYNC2,BLOCK_STATE_DATA1,BLOCK_STATE_DATA2,BLOCK_STATE_PAUSE};
#define MS	3500
#define INVALID	-1U
#define SET_IN_LOOP
#define RESET_IN_LOOP	{ loopStart = INVALID; }
#define IN_LOOP	( loopStart != INVALID )
#define SET_IN_CALL
#define RESET_IN_CALL	{ callStart = INVALID; }
#define IN_CALL	( callStart != INVALID )
#define tapeInNextBlock()	TapeInSetBlock(TapeDataP->in.recordCurrent + 1)
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_TAPE) != 0) { x } }
#else
#define DEB(x)
#endif
static void tapeDestroyRecord(TapeRecord*);static int tapeInPrepareBlock(void);static void tapeInGetBit(unsigned int);static int tapeInNextEdge(unsigned int*tstates);static int tapeInSelection(TapeBlockSelect*);static void tapeInReadBlock(uns8*,int);static void tapeOutWriteBlock(void);TapeData*TapeDataP;static TapeBlockType blockType;static int blockState;static unsigned int loopStart,loopRepeat,callStart,callCount;static int*callOffsets;static unsigned int pilotPulses,pilotLen,sync1Len,sync2Len,bit0Len,bit1Len,curBit,pauseLen,*pulseLens;static uns8*blockPtr,bitMask;static unsigned int blockLen,lastByte,bitsLeft;
#define pulseLen	pilotLen
#define pulseCount	pilotPulses
#define bitLen	pilotLen
void TapeInit(void){TapeDataP=Malloc(sizeof(TapeData),"TapeInit");TapeDataP->state=(CFGGET(tapeQuickload)>0?TS_QUICKLOAD:0)|(CFGGET(tapeFastload)>0?TS_FASTLOAD:0)|(CFGGET(tapeAutostop)>0?TS_AUTOSTOP:0)|(CFGGET(tapeNoise)>0?TS_NOISE:0)|(CFGGET(tapeQuicksave)>0?TS_QUICKSAVE:0);TapeDataP->in.filename=(const char*)0;TapeDataP->in.recordList=(Llist*)0;TapeDataP->in.recordCount=0;TapeDataP->in.ear=0;TapeDataP->out.filename=(const char*)0;atexit((void(*)(void))TapeOutDetach);}int TapeInSetBlock(unsigned int block){if(block>=TapeDataP->in.recordCount){return-1;}TapeDataP->in.recordCurrent=block;if((block+1)==TapeDataP->in.recordCount){TapeDataP->state|=TS_ENDIN;return 1;}TapeDataP->state&=~TS_ENDIN;return 0;}unsigned int TapeInSetRecord(TapeRecord*record){TapeDataP->in.recordList=AppendElemList(TapeDataP->in.recordList,record);return++TapeDataP->in.recordCount;}TapeRecord*TapeInGetRecord(unsigned int number){return((TapeRecord*)RetrieveElemList(TapeDataP->in.recordList,number));}int TapeInAttach(const char*filename){int(*func)(const char*);int ret;(void)TapeInDetach();switch(GetFileType(filename)){case FILE_TYPE_TAP:func=TapRead;break;case FILE_TYPE_TZX:func=TzxRead;break;case FILE_TYPE_VOC:func=VocRead;break;default:assert(0);func=(int(*)(const char*))0;break;}ret=func(filename);if(ret==0){TapeDataP->state|=TS_FILEIN;TapeDataP->in.filename=filename;}else{free((void*)filename);DestroyList(TapeDataP->in.recordList,(void(*)(void*))tapeDestroyRecord);TapeDataP->in.recordList=(Llist*)0;TapeDataP->in.recordCount=0;}RESET_IN_LOOP;RESET_IN_CALL;return ret;}int TapeInDetach(void){if((TapeDataP->state&TS_FILEIN)!=0){assert((TapeDataP->state&TS_PLAY)==0);TapeDataP->state&=~(TS_FILEIN|TS_PLAY|TS_ERRIN|TS_ENDIN);free((void*)TapeDataP->in.filename);TapeDataP->in.filename=(const char*)0;DestroyList(TapeDataP->in.recordList,(void(*)(void*))tapeDestroyRecord);TapeDataP->in.recordList=(Llist*)0;TapeDataP->in.recordCount=0;TapeDataP->in.recordCurrent=0;}return 0;}void TapeInPlay(void){TapeRecord*tr;assert((TapeDataP->state&TS_FILEIN)!=0&&(TapeDataP->state&(TS_PLAY|TS_ERRIN|TS_ENDIN))==0);tr=TapeInGetRecord(TapeInGetBlock());assert(tr!=(TapeRecord*)0);if((tr->type!=TAPE_BLOCK_PAUSE&&tr->type!=TAPE_BLOCK_STOP48)||tapeInNextBlock()==0){if(tapeInPrepareBlock()==TAPE_STATUS_OK){TapeDataP->state|=TS_PLAY;if((TapeDataP->state&TS_FASTLOAD)!=0){CFGSET(fastmode,CFGGET(fastmode)|2);SoundControl(SOUND_ACTION_OFF);}TapeDataP->in.ear=0;tapeInGetBit(TSTATES);}}VtpUpdateInputFile();}void TapeInStop(void){assert((TapeDataP->state&TS_PLAY)!=0);TapeDataP->state&=~TS_PLAY;if((TapeDataP->state&TS_FASTLOAD)!=0){CFGSET(fastmode,CFGGET(fastmode)&~2);if(CFGGET(fastmode)==0){SoundControl(SOUND_ACTION_ON);}}VtpUpdateInputFile();}int TapeInCheckTrap(int page,unsigned int addr){int ret=0;if(addr==Machine->traps.load.addr&&page==Machine->traps.load.page&&(TapeDataP->state&TS_QUICKLOAD)!=0){DEB(Msg(M_DEBUG,"TAPE: %s request: id = %02x, addr = %04X, length = %5d",F1&FLAG_C?"load":"verify",A1,IX,DE););if(((TapeDataP->state&TS_FILEIN)==0||(TapeDataP->state&TS_ENDIN)!=0)&&CFGGET(smartAttach)!=0){VtpNewInputFile();}if((TapeDataP->state&TS_FILEIN)!=0){if((TapeDataP->state&(TS_PLAY|TS_ENDIN|TS_ERRIN|TS_AUTOSTOP))==0){TapeInPlay();}while((TapeGetState()&(TS_ERRIN|TS_ENDIN|TS_PLAY))==TS_PLAY){if(blockType==TAPE_BLOCK_PAUSE||blockType==TAPE_BLOCK_STOP48||blockState==BLOCK_STATE_PAUSE){if(tapeInNextBlock()==0){(void)tapeInPrepareBlock();}continue;}if(blockType==TAPE_BLOCK_ROM){if(blockState<BLOCK_STATE_DATA1&&(blockLen==0||*blockPtr==A1)){tapeInReadBlock(blockPtr,(int)blockLen);blockState=BLOCK_STATE_PAUSE;ret=1;}}break;}}}return ret;}int TapeOutSetBlock(unsigned int block){if(block>=TapeDataP->out.recordCount){return-1;}TapeDataP->out.recordCurrent=block;if((block+1)==TapeDataP->out.recordCount){TapeDataP->state|=TS_ENDOUT;return 1;}TapeDataP->state&=~TS_ENDOUT;return 0;}unsigned int TapeOutSetRecord(TapeRecord*record){TapeDataP->out.recordList=AppendElemList(TapeDataP->out.recordList,record);return++TapeDataP->out.recordCount;}TapeRecord*TapeOutGetRecord(unsigned int number){return((TapeRecord*)RetrieveElemList(TapeDataP->out.recordList,number));}int TapeOutAttach(const char*filename){TapeRecord*tr;TapeBlockComment*block;unsigned int length;char buf[512];int pos;TapeOutDetach();TapeDataP->state|=TS_FILEOUT;TapeDataP->out.filename=filename;tr=Malloc(sizeof(TapeRecord),"TapeOutAttach");tr->bnr=TapeOutSetRecord(tr)-1;tr->type=TAPE_BLOCK_COMMENT;block=&(tr->block.comment);length=(unsigned int)strlen(Version)+14;block->text=Malloc(sizeof(char)*(size_t)(length+1),"TapeOutAttach");(void)sprintf((char*)block->text,"Created with %s",Version);(void)sprintf(buf,"%.*s",length>30?30:length,(char*)block->text);tr->shortDesc=Strdup(buf,"TapeOutAttach");pos=sprintf(buf,"Description\n""Text: ");FormatLine(block->text,length,&buf[pos],6);tr->longDesc=Strdup(buf,"TapeOutAttach");return 0;}int TapeOutDetach(void){int(*func)(const char*);int ret=0;if((TapeDataP->state&TS_FILEOUT)!=0){switch(GetFileType(TapeDataP->out.filename)){case FILE_TYPE_TAP:func=TapWrite;break;case FILE_TYPE_TZX:func=TzxWrite;break;case FILE_TYPE_VOC:func=VocWrite;break;default:assert(0);func=(int(*)(const char*))0;break;}ret=func(TapeDataP->out.filename);TapeDataP->state&=~(TS_FILEOUT|TS_ERROUT|TS_ENDOUT);free((void*)TapeDataP->out.filename);TapeDataP->out.filename=(const char*)0;DestroyList(TapeDataP->out.recordList,(void(*)(void*))tapeDestroyRecord);TapeDataP->out.recordList=(Llist*)0;TapeDataP->out.recordCount=0;TapeDataP->out.recordCurrent=0;}return ret;}int TapeOutCheckTrap(int page,unsigned int addr){if(addr==Machine->traps.save.addr&&page==Machine->traps.save.page&&(TapeDataP->state&TS_QUICKSAVE)!=0){DEB(Msg(M_DEBUG,"TAPE: %stape saving: addr = %04X, length = %04X",A!=0?"headerless ":"",IX,DE););if((TapeDataP->state&TS_FILEOUT)==0&&CFGGET(smartAttach)!=0){VtpNewOutputFile();}if((TapeDataP->state&TS_FILEOUT)!=0){tapeOutWriteBlock();return 1;}}return 0;}
#ifndef EVALUATION
unsigned int TapeInGetPoke(unsigned int bnr,Llist**pokeList){TapeBlockCustomPokes*block=&((TapeInGetRecord(bnr))->block.custom.subblock.pokes);PokeRecord*pr=block->pokes;unsigned int i;for(i=block->count;i>0;i--){PokeRecord*pr2=Malloc(sizeof(PokeRecord),"pokeNewFile");(void)memcpy(pr2,pr,sizeof(PokeRecord));*pokeList=AppendElemList(*pokeList,pr2);pr++;}return block->count;}
#endif 
const char*TapeIdentifyBlock(unsigned int length,uns8*data,int standard){static const char*types[]={"Program","Num.Array","Char.Array","Bytes",""};char buf[31];int type,i;if(*data==0&&length==19){data++;type=*data<4?*data:4;data++;for(i=0;i<10;i++){buf[i]=(*data<' '||*data>=0x7f)?'?':*data;data++;}}else{type=4;(void)memset(buf,' ',10);}(void)sprintf(&buf[10],"%c %-11s%6d",standard==0?'*':' ',types[type],length>=2?length-2:0);return(Strdup(buf,"TapeIdentifyBlock"));}static void tapeDestroyRecord(TapeRecord*tr){unsigned int i;switch(tr->type){case TAPE_BLOCK_ROM:free(tr->block.rom.data);free((void*)tr->shortDesc);free((void*)tr->longDesc);break;case TAPE_BLOCK_TURBO:free(tr->block.turbo.data);free((void*)tr->shortDesc);free((void*)tr->longDesc);break;case TAPE_BLOCK_PURE_TONE:free((void*)tr->longDesc);break;case TAPE_BLOCK_PULSES:free(tr->block.pulses.lengths);free((void*)tr->longDesc);break;case TAPE_BLOCK_PURE_DATA:free(tr->block.puredata.data);free((void*)tr->shortDesc);free((void*)tr->longDesc);break;case TAPE_BLOCK_RAW_DATA:free(tr->block.rawdata.data);free((void*)tr->longDesc);break;case TAPE_BLOCK_PAUSE:if(tr->block.pause.length>0){free((void*)tr->longDesc);}break;case TAPE_BLOCK_GROUP_START:free((void*)tr->shortDesc);free((void*)tr->longDesc);break;case TAPE_BLOCK_GROUP_END:break;case TAPE_BLOCK_JUMP:free((void*)tr->shortDesc);free((void*)tr->longDesc);break;case TAPE_BLOCK_LOOP_START:free((void*)tr->longDesc);break;case TAPE_BLOCK_LOOP_END:break;case TAPE_BLOCK_CALL:free(tr->block.call.offsets);free((void*)tr->longDesc);break;case TAPE_BLOCK_RETURN:break;case TAPE_BLOCK_SELECT:for(i=0;i<tr->block.select.count;i++){free((void*)tr->block.select.selections[i].description);}free(tr->block.select.selections);free((void*)tr->longDesc);break;case TAPE_BLOCK_STOP48:break;case TAPE_BLOCK_COMMENT:free((void*)tr->block.comment.text);free((void*)tr->shortDesc);free((void*)tr->longDesc);break;case TAPE_BLOCK_MESSAGE:free((void*)tr->block.message.text);free((void*)tr->longDesc);break;case TAPE_BLOCK_ARCHIVE_INFO:free((void*)tr->longDesc);break;case TAPE_BLOCK_HARDWARE:free((void*)tr->longDesc);break;case TAPE_BLOCK_CUSTOM:switch(tr->block.custom.type){case TAPE_BLOCK_CUSTOM_POKE:free(tr->block.custom.subblock.pokes.pokes);break;case TAPE_BLOCK_CUSTOM_INSTRUCTIONS:free((void*)tr->block.custom.subblock.instructions.text);break;default:free(tr->block.custom.subblock.unknown.data);break;}free((void*)tr->shortDesc);free((void*)tr->longDesc);break;case TAPE_BLOCK_UNSUPPORTED:break;case TAPE_BLOCK_EOT:break;default:assert(0);break;}}static int tapeInPrepareBlock(void){unsigned int bnr;TapeRecord*tr;bnr=TapeInGetBlock();do{VtpUpdateInputFile();tr=TapeInGetRecord(bnr);assert(tr!=(TapeRecord*)0);blockType=tr->type;DEB(Msg(M_DEBUG,"TAPE: curBlock = %3u, blockType = %02x",bnr,blockType););switch(blockType){case TAPE_BLOCK_EOT:DEB(Msg(M_DEBUG,"TAPE: end of tape reached"););return TAPE_STATUS_EOT;case TAPE_BLOCK_ROM:{TapeBlockRom*block=&(tr->block.rom);pauseLen=block->pause;blockLen=block->length;blockPtr=block->data;pilotPulses=*blockPtr!=0?0x0c97 :0x1f7f;pilotLen=2168;sync1Len=667;sync2Len=735;bit0Len=855;bit1Len=1710;lastByte=8;blockState=BLOCK_STATE_PILOT;}return TAPE_STATUS_OK;case TAPE_BLOCK_TURBO:{TapeBlockTurbo*block=&(tr->block.turbo);pauseLen=block->pause;blockLen=block->length;blockPtr=block->data;pilotPulses=block->pilotPulses;pilotLen=block->pilotLength;sync1Len=block->sync1Length;sync2Len=block->sync2Length;bit0Len=block->bit0Length;bit1Len=block->bit1Length;lastByte=block->lastByte;blockState=BLOCK_STATE_PILOT;}return TAPE_STATUS_OK;case TAPE_BLOCK_PURE_TONE:{TapeBlockPureTone*block=&(tr->block.puretone);pulseLen=block->length;pulseCount=block->pulses;}return TAPE_STATUS_OK;case TAPE_BLOCK_PULSES:{TapeBlockPulses*block=&(tr->block.pulses);pulseLens=block->lengths;pulseLen=*pulseLens++;pulseCount=block->count;}return TAPE_STATUS_OK;case TAPE_BLOCK_PURE_DATA:{TapeBlockPureData*block=&(tr->block.puredata);pauseLen=block->pause;blockLen=block->length;blockPtr=block->data;bit0Len=block->bit0Length;bit1Len=block->bit1Length;lastByte=block->lastByte;bitMask=1<<7;bitsLeft=blockLen==1?lastByte:8;curBit=(*blockPtr&bitMask)!=0?bit1Len:bit0Len;blockState=BLOCK_STATE_DATA1;}return TAPE_STATUS_OK;case TAPE_BLOCK_RAW_DATA:{TapeBlockRawData*block=&(tr->block.rawdata);pauseLen=block->pause;blockLen=block->length;blockPtr=block->data;bitLen=block->bitLength;lastByte=block->lastByte;bitMask=1<<7;bitsLeft=blockLen==1?lastByte:8;blockState=BLOCK_STATE_DATA1;}return TAPE_STATUS_OK;case TAPE_BLOCK_PAUSE:{TapeBlockPause*block=&(tr->block.pause);pauseLen=block->length;}return(pauseLen==0?TAPE_STATUS_STOP:TAPE_STATUS_OK);case TAPE_BLOCK_GROUP_START:case TAPE_BLOCK_GROUP_END:bnr++;break;case TAPE_BLOCK_JUMP:{TapeBlockJump*block=&(tr->block.jump);bnr+=block->offset;}break;case TAPE_BLOCK_LOOP_START:{TapeBlockLoopStart*block=&(tr->block.loopstart);SET_IN_LOOP;loopStart=bnr;loopRepeat=block->count;if(loopRepeat<1){Msg(M_ERR,"Invalid value for Loop Count in tape file");return TAPE_STATUS_FAILED;}bnr++;}break;case TAPE_BLOCK_LOOP_END:if(!IN_LOOP){Msg(M_ERR,"Loop End without Loop in tape file");return TAPE_STATUS_FAILED;}if(--loopRepeat!=0){bnr=loopStart+1U;}else{bnr++;RESET_IN_LOOP;}break;case TAPE_BLOCK_CALL:{TapeBlockCall*block=&(tr->block.call);SET_IN_CALL;callStart=bnr;callCount=block->count;callOffsets=block->offsets;bnr+=*callOffsets++;}break;case TAPE_BLOCK_RETURN:if(!IN_CALL){Msg(M_ERR,"Return without Call in tape file");return TAPE_STATUS_FAILED;}if(--callCount>0){bnr=callStart+*callOffsets++;}else{bnr=callStart+1U;RESET_IN_CALL;}break;case TAPE_BLOCK_SELECT:bnr+=tapeInSelection(&(tr->block.select));break;case TAPE_BLOCK_STOP48:return TAPE_STATUS_STOP48;case TAPE_BLOCK_COMMENT:case TAPE_BLOCK_MESSAGE:case TAPE_BLOCK_ARCHIVE_INFO:case TAPE_BLOCK_HARDWARE:case TAPE_BLOCK_CUSTOM:case TAPE_BLOCK_UNSUPPORTED:bnr++;break;default:assert(0);break;}}while(TapeInSetBlock(bnr)>=0);return TAPE_STATUS_FAILED;}static void tapeInGetBit(unsigned int this){if((TapeDataP->state&TS_PLAY)!=0){unsigned int next;int status,stop;status=tapeInNextEdge(&next);stop=(status==TAPE_STATUS_STOP||(status==TAPE_STATUS_STOP48&&CFGGET(machine)<=SP_48_3)||status==TAPE_STATUS_EOT);if(next>0||stop){TapeDataP->in.ear^=B_EAR;if((TapeDataP->state&TS_NOISE)!=0){SpeakerOutByte(TapeDataP->in.ear>>3);}}if(stop||(status==TAPE_STATUS_EOB&&(TapeDataP->state&TS_AUTOSTOP)!=0)||status==TAPE_STATUS_FAILED){if(status==TAPE_STATUS_FAILED){TapeDataP->state|=TS_ERRIN;}TapeInStop();}else{next+=this;(void)EventAdd((EventFunc)tapeInGetBit,(void*)(next%Machine->tstatesPerFrame),next);}}}static int tapeInNextEdge(unsigned int*tstates){int status;status=TAPE_STATUS_OK;*tstates=0;switch(blockType){case TAPE_BLOCK_ROM:case TAPE_BLOCK_TURBO:case TAPE_BLOCK_PURE_DATA:switch(blockState){case BLOCK_STATE_PILOT:*tstates=pilotLen;if(--pilotPulses==0){blockState=BLOCK_STATE_SYNC1;}break;case BLOCK_STATE_SYNC1:*tstates=sync1Len;blockState=BLOCK_STATE_SYNC2;break;case BLOCK_STATE_SYNC2:*tstates=sync2Len;if(blockLen!=0){bitMask=1<<7;bitsLeft=blockLen==1?lastByte:8;curBit=(*blockPtr&bitMask)!=0?bit1Len:bit0Len;blockState=BLOCK_STATE_DATA1;}else{blockState=BLOCK_STATE_PAUSE;}break;case BLOCK_STATE_DATA1:*tstates=curBit;blockState=BLOCK_STATE_DATA2;break;case BLOCK_STATE_DATA2:*tstates=curBit;if(--bitsLeft==0){if(--blockLen==0){blockState=BLOCK_STATE_PAUSE;break;}bitMask=1<<7;bitsLeft=blockLen==1?lastByte:8;blockPtr++;}else{bitMask>>=1;}curBit=(*blockPtr&bitMask)!=0?bit1Len:bit0Len;blockState=BLOCK_STATE_DATA1;break;case BLOCK_STATE_PAUSE:goto pause;}break;case TAPE_BLOCK_PURE_TONE:*tstates=pulseLen;if(--pulseCount==0){goto next;}break;case TAPE_BLOCK_PULSES:*tstates=pulseLen;if(--pulseCount==0){goto next;}else{pulseLen=*pulseLens++;}break;case TAPE_BLOCK_RAW_DATA:switch(blockState){case BLOCK_STATE_DATA1:curBit=*blockPtr&bitMask?0xff :0x00;do{if(--bitsLeft==0){if(--blockLen==0){blockState=BLOCK_STATE_PAUSE;break;}bitMask=1<<7;bitsLeft=blockLen==1?lastByte:8;blockPtr++;}else{bitMask>>=1;}*tstates+=bitLen;}while(((*blockPtr^curBit)&bitMask)==0);break;case BLOCK_STATE_PAUSE:goto pause;default:assert(0);}break;case TAPE_BLOCK_PAUSE:pause:if(pauseLen>0){*tstates=pauseLen*MS;}case TAPE_BLOCK_STOP48:next:(void)tapeInNextBlock();status=tapeInPrepareBlock();if(status==TAPE_STATUS_OK){status=TAPE_STATUS_EOB;}break;default:assert(0);break;}return status;}static int tapeInSelection(TapeBlockSelect*block){int choice=0;const char**choices;unsigned int i;if(block->count==0){Msg(M_WARN,"Selection with zero choices in tape file");return 1;}choices=Malloc(sizeof(char*)*(size_t)block->count,"tapeInSelection");for(i=0;i<block->count;i++){choices[i]=block->selections[i].description;}if((choice=UiBrowseList("Choose an item",choices,(int)block->count,choice))<0){choice=0;}free(choices);return block->selections[choice].offset;}static void tapeInReadBlock(uns8*data,int len){uns8 byte,crc;uns16 swap;int error=0;if(len!=0){{int de=(DE!=0?(int)DE:65536)+2;len=de>len?len:de;}crc=*data++;len--;for(error=1;;DE--,IX++){byte=*data++;crc^=byte;if(--len==0){if(crc==0){error=0;}
#ifdef DEBUG
else if((CFGGET(debug)&D_TAPE)!=0){DEB(Msg(M_DEBUG,"TAPE: checksum error"););}
#endif
break;}if((F1&FLAG_C)!=0){Machine->writeMem(IX,byte);}else{if(byte!=Machine->readMem(IX)){break;}}}}if(error!=0){F1&=~(FLAG_Z|FLAG_C);}else{F1|=FLAG_Z|FLAG_C;}swap=AF;AF=AF1;AF1=swap;RET();}static void tapeOutWriteBlock(void){TapeRecord*tr;TapeBlockRom*block;char buf[512];int pos;uns8*p,crc;tr=Malloc(sizeof(TapeRecord),"tapeOutWriteBlock");tr->bnr=TapeOutSetRecord(tr)-1;tr->type=TAPE_BLOCK_ROM;block=&(tr->block.rom);block->length=(unsigned int)DE+2;block->data=Malloc(sizeof(uns8)*(size_t)block->length,"tapeOutWriteBlock");block->pause=A==0x00 ?100:1000;p=block->data;*p++=crc=A;while(DE--!=0){uns8 byte=Machine->readMem(IX);*p++=byte;IX++;crc^=byte;}*p++=crc;F|=FLAG_C;IX+=DE;DE=0;RET();tr->shortDesc=TapeIdentifyBlock(block->length,block->data,1);pos=sprintf(buf,"Standard\n""Length           : %6u bytes\n",block->length>=2?block->length-2:0);if(block->length!=0){pos+=sprintf(&buf[pos],"Flag             :   0x%02X\n""CheckSum         :   0x%02X\n",(unsigned int)block->data[0],(unsigned int)block->data[block->length-1]);}(void)sprintf(&buf[pos],"Pause after block: %6u ms",block->pause);tr->longDesc=Strdup(buf,"tapeOutWriteBlock");VtpUpdateOutputFile();}
