/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifndef _XZX_TAPE_H
#define _XZX_TAPE_H
#include "util.h"
#include "ui/poke.h"
typedef enum{TAPE_BLOCK_ROM,TAPE_BLOCK_TURBO,TAPE_BLOCK_PURE_TONE,TAPE_BLOCK_PULSES,TAPE_BLOCK_PURE_DATA,TAPE_BLOCK_RAW_DATA,TAPE_BLOCK_PAUSE,TAPE_BLOCK_GROUP_START,TAPE_BLOCK_GROUP_END,TAPE_BLOCK_JUMP,TAPE_BLOCK_LOOP_START,TAPE_BLOCK_LOOP_END,TAPE_BLOCK_CALL,TAPE_BLOCK_RETURN,TAPE_BLOCK_SELECT,TAPE_BLOCK_STOP48,TAPE_BLOCK_COMMENT,TAPE_BLOCK_MESSAGE,TAPE_BLOCK_ARCHIVE_INFO,TAPE_BLOCK_HARDWARE,TAPE_BLOCK_CUSTOM,TAPE_BLOCK_UNSUPPORTED,TAPE_BLOCK_EOT}TapeBlockType;typedef struct _tapeBlockRom{unsigned int length;uns8*data;unsigned int pause;}TapeBlockRom;typedef struct _tapeBlockTurbo{unsigned int length;unsigned int lastByte;uns8*data;unsigned int pause;unsigned int pilotLength,pilotPulses;unsigned int sync1Length,sync2Length;unsigned int bit0Length,bit1Length;}TapeBlockTurbo;typedef struct _tapeBlockPureTone{unsigned int length;unsigned int pulses;}TapeBlockPureTone;typedef struct _tapeBlockPulses{unsigned int count;unsigned int*lengths;}TapeBlockPulses;typedef struct _tapeBlockPureData{unsigned int length;unsigned int lastByte;uns8*data;unsigned int pause;unsigned int bit0Length,bit1Length;}TapeBlockPureData;typedef struct _tapeBlockRawData{unsigned int length;unsigned int lastByte;uns8*data;unsigned int pause;unsigned int bitLength;}TapeBlockRawData;typedef struct _tapeBlockPause{unsigned int length;}TapeBlockPause;typedef struct _tapeBlockJump{int offset;}TapeBlockJump;typedef struct _tapeBlockLoopStart{unsigned int count;}TapeBlockLoopStart;typedef struct _tapeBlockCall{unsigned int count;int*offsets;}TapeBlockCall;typedef struct _tapeBlockSelect{unsigned int count;struct{const char*description;int offset;}*selections;}TapeBlockSelect;typedef struct _tapeBlockComment{const char*text;}TapeBlockComment;typedef struct _tapeBlockMessage{unsigned int time;const char*text;}TapeBlockMessage;typedef enum{TAPE_BLOCK_CUSTOM_POKE,TAPE_BLOCK_CUSTOM_INSTRUCTIONS,TAPE_BLOCK_CUSTOM_SCREEN,TAPE_BLOCK_CUSTOM_ZXEDIT,TAPE_BLOCK_CUSTOM_PICTURE,TAPE_BLOCK_CUSTOM_UNSUPPORTED}TapeBlockCustomType;typedef struct _tapeBlockCustomPokes{unsigned int count;PokeRecord*pokes;}TapeBlockCustomPokes;typedef struct _tapeBlockCustomInstructions{const char*text;}TapeBlockCustomInstructions;typedef struct _tapeBlockCustomUnknown{unsigned int length;uns8*data;}TapeBlockCustomUnknown;typedef struct _tapeBlockCustom{TapeBlockCustomType type;union{TapeBlockCustomPokes pokes;TapeBlockCustomInstructions instructions;TapeBlockCustomUnknown unknown;}subblock;}TapeBlockCustom;typedef struct _tapeRecord{unsigned int bnr;TapeBlockType type;const char*shortDesc;const char*longDesc;union{TapeBlockRom rom;TapeBlockTurbo turbo;TapeBlockPureTone puretone;TapeBlockPulses pulses;TapeBlockPureData puredata;TapeBlockRawData rawdata;TapeBlockPause pause;TapeBlockJump jump;TapeBlockLoopStart loopstart;TapeBlockCall call;TapeBlockSelect select;TapeBlockComment comment;TapeBlockMessage message;TapeBlockCustom custom;}block;}TapeRecord;
#define TS_QUICKLOAD	(1<< 0)
#define TS_FASTLOAD	(1<< 1)
#define TS_AUTOSTOP	(1<< 2)
#define TS_NOISE	(1<< 3)
#define TS_QUICKSAVE	(1<< 4)
#define TS_PLAY	(1<< 5)
#define TS_FILEIN	(1<< 6)
#define TS_ERRIN	(1<< 7)
#define TS_ENDIN	(1<< 8)
#define TS_FILEOUT	(1<< 9)
#define TS_ERROUT	(1<<10)
#define TS_ENDOUT	(1<<11)
typedef struct _TapeData{unsigned int state;struct{const char*filename;Llist*recordList;unsigned int recordCount;unsigned int recordCurrent;uns8 ear;}in;struct{const char*filename;Llist*recordList;unsigned int recordCount;unsigned int recordCurrent;}out;}TapeData;
#define TapeGetState()	(TapeDataP->state)
#define TapeSetState(s)	TapeDataP->state = (s)
#define TapeInGetBlock()	(TapeDataP->in.recordCurrent)
#define TapeInGetBlocks()	(TapeDataP->in.recordCount)
#define TapeInGetShortDescription(n)	((TapeInGetRecord(n))->shortDesc)
#ifndef EVALUATION
#define TapeInGetLongDescription(n)	((TapeInGetRecord(n))->longDesc)
#else
#define TapeInGetLongDescription(n)	"Tape information is not available\n" "in the evaluaton version.\n\n" "See the file README for details."
#endif
#define TapeInGetInstructions(n)	((TapeInGetRecord(n))->block.custom.subblock.instructions.text)
#define TapeInGetName()	(TapeDataP->in.filename)
#define TapeInGetBit()	(TapeDataP->in.ear)
#define TapeOutGetBlock()	(TapeDataP->out.recordCurrent)
#define TapeOutGetBlocks()	(TapeDataP->out.recordCount)
#define TapeOutGetShortDescription(n)	((TapeOutGetRecord(n))->shortDesc)
#define TapeOutGetLongDescription(n)	((TapeOutGetRecord(n))->longDesc)
#define TapeOutGetName()	(TapeDataP->out.filename)
extern void TapeInit(void);extern int TapeInSetBlock(unsigned int);extern unsigned int TapeInSetRecord(TapeRecord*);extern TapeRecord*TapeInGetRecord(unsigned int);extern int TapeInAttach(const char*);extern int TapeInDetach(void);extern void TapeInPlay(void);extern void TapeInStop(void);extern int TapeInCheckTrap(int,unsigned int);extern int TapeOutSetBlock(unsigned int);extern unsigned int TapeOutSetRecord(TapeRecord*);extern TapeRecord*TapeOutGetRecord(unsigned int);extern int TapeOutAttach(const char*);extern int TapeOutDetach(void);extern int TapeOutCheckTrap(int,unsigned int);extern unsigned int TapeInGetPoke(unsigned int,Llist**);extern const char*TapeIdentifyBlock(unsigned int,uns8*,int);extern TapeData*TapeDataP;
#endif 

