/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: menu.c,v 4.120 2006/12/22 14:29:16 erik Rel $";
#endif
#include <Xm/Xm.h>
#include <Xm/CascadeBG.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/SeparatoG.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "ui/resource.h"
#include "util.h"
#ifdef XZX_DIDAKTIK
#include "machine/didaktikd80.h"
#endif
#ifdef XZX_IF1
#include "machine/if1.h"
#endif
#ifdef HAVE_XPM
#include "ui/kbd.h"
#endif
#include "machine/machine.h"
#include "monitor/monitor.h"
#include "ui/options.h"
#ifdef XZX_OPUS
#include "machine/opus.h"
#endif
#include "machine/plus3.h"
#ifdef XZX_PLUSD
#include "machine/plusd.h"
#endif
#include "ui/poke.h"
#include "fileio/psg.h"
#include "ui/sbar.h"
#include "fileio/scr.h"
#include "snapshot.h"
#include "sound/sound.h"
#ifdef XZX_TRDOS
#include "machine/trdos.h"
#endif
#include "ui/ui.h"
#include "ui/vtp.h"
#include "fileio/wav.h"
#ifndef DEBUG
#include "ui/xcons.h"
#endif
#ifdef XZX_XMOUSE
#include "ui/xmouse.h"
#endif
#ifdef HAVE_XPM
#include "fileio/xpm.h"
#endif
#include "ui/xwin.h"
#ifdef HAVE_VIDMODE
#include "ui/xwin86.h"
#endif
#include "ui/menu.h"
typedef struct _menuItem{const char*label;WidgetClass*class;char mnemonic;const char*accelerator;const char*accelText;void(*callback)(Widget,XtPointer,XtPointer);XPointer callbackData;struct _menuItem*subItems;Widget widget;}menuItem;
#define MENUBEGIN(a)	static menuItem a[] = {
#define MENUITEM(a,b,c,d,e,f,g,h)	{ (const char *)a,(WidgetClass *)b,(char)c, (const char *)d,(const char *)e,(XtCallbackProc)f, (XPointer)g,(struct _menuItem *)h,0 },
#define MENUSEP	MENUITEM("sep",&xmSeparatorGadgetClass,0,0,0,0,0,0)
#define MENUEND	MENUITEM(0,0,0,0,0,0,0,0) };
#define PB_CLASS	&xmPushButtonGadgetClass
#define TB_CLASS	&xmToggleButtonGadgetClass
#define CB_CLASS	&xmCascadeButtonGadgetClass
enum{HELP_COPYRIGHT,HELP_README,HELP_FAQ,HELP_KEYPIC};static Widget menuCreateMenu(Widget,int,const char*,char,Boolean,menuItem*);static void menuFileSnapshotRead(void);static void menuFileSnapshotWrite(void);static void menuFileDiskInsert(Widget,XtPointer,XtPointer);static void menuFileDiskRemove(Widget,XtPointer,XtPointer);
#ifdef XZX_IF1
static void menuFileCartridgeInsert(Widget,XtPointer,XtPointer);static void menuFileCartridgeRemove(Widget,XtPointer,XtPointer);
#endif
static void menuFileScreendumpRead(void);static void menuFileScreendumpWrite(Widget,XtPointer,XtPointer);static void menuFileAudioWrite(Widget,XtPointer,XtPointer);static void menuFileAudioClose(Widget,XtPointer,XtPointer);static void menuFileQuit(void);static void menuMiscReset(void);static void menuMiscNmi(void);static void menuMiscPause(Widget,XtPointer,XtPointer);
#ifdef HAVE_VIDMODE
static void menuMiscFullscreen(Widget,XtPointer,XtPointer);
#endif
#ifdef XZX_XMOUSE
static void menuMiscPointerGrab(Widget,XtPointer,XtPointer);
#endif
static void menuMiscMonitor(Widget,XtPointer,XtPointer);static void menuMiscVtp(Widget,XtPointer,XtPointer);
#ifndef DEBUG
static void menuMiscConsole(Widget,XtPointer,XtPointer);
#endif
static void menuOptionsSaveOnExit(Widget,XtPointer,XtPointer);static void menuOptionsSaveNow(void);static void menuHelp(Widget,XtPointer,XtPointer);static void menuHelpCallback(Widget,XtPointer,XtPointer);MENUBEGIN(diskInsertItems)MENUITEM("A...",PB_CLASS,'A',"Ctrl<Key>a","Ctrl+A",menuFileDiskInsert,0,0)MENUITEM("B...",PB_CLASS,'B',"Ctrl<Key>b","Ctrl+B",menuFileDiskInsert,1,0)
#if XZX_DISK_DRIVES > 2
MENUITEM("C...",PB_CLASS,'C',"Ctrl<Key>c","Ctrl+C",menuFileDiskInsert,2,0)MENUITEM("D...",PB_CLASS,'D',"Ctrl<Key>d","Ctrl+D",menuFileDiskInsert,3,0)
#endif
MENUEND MENUBEGIN(diskRemoveItems)MENUITEM("A",PB_CLASS,'A',0,0,menuFileDiskRemove,0,0)MENUITEM("B",PB_CLASS,'B',0,0,menuFileDiskRemove,1,0)
#if XZX_DISK_DRIVES > 2
MENUITEM("C",PB_CLASS,'C',0,0,menuFileDiskRemove,2,0)MENUITEM("D",PB_CLASS,'D',0,0,menuFileDiskRemove,3,0)
#endif
MENUEND
#ifdef XZX_IF1
MENUBEGIN(mdrInsertItems)MENUITEM("1...",PB_CLASS,'1',"Ctrl<Key>1","Ctrl+1",menuFileCartridgeInsert,0,0)
#if XZX_IF1_DRIVES > 1
MENUITEM("2...",PB_CLASS,'2',"Ctrl<Key>2","Ctrl+2",menuFileCartridgeInsert,1,0)
#endif
#if XZX_IF1_DRIVES > 2
MENUITEM("3...",PB_CLASS,'3',"Ctrl<Key>3","Ctrl+3",menuFileCartridgeInsert,2,0)
#endif
#if XZX_IF1_DRIVES > 3
MENUITEM("4...",PB_CLASS,'4',"Ctrl<Key>4","Ctrl+4",menuFileCartridgeInsert,3,0)
#endif
#if XZX_IF1_DRIVES > 4
MENUITEM("5...",PB_CLASS,'5',"Ctrl<Key>5","Ctrl+5",menuFileCartridgeInsert,4,0)
#endif
#if XZX_IF1_DRIVES > 5
MENUITEM("6...",PB_CLASS,'6',"Ctrl<Key>6","Ctrl+6",menuFileCartridgeInsert,5,0)
#endif
#if XZX_IF1_DRIVES > 6
MENUITEM("7...",PB_CLASS,'7',"Ctrl<Key>7","Ctrl+7",menuFileCartridgeInsert,6,0)
#endif
#if XZX_IF1_DRIVES > 7
MENUITEM("8...",PB_CLASS,'8',"Ctrl<Key>8","Ctrl+8",menuFileCartridgeInsert,7,0)
#endif
MENUEND MENUBEGIN(mdrRemoveItems)MENUITEM("1",PB_CLASS,'1',0,0,menuFileCartridgeRemove,0,0)
#if XZX_IF1_DRIVES > 1
MENUITEM("2",PB_CLASS,'2',0,0,menuFileCartridgeRemove,1,0)
#endif
#if XZX_IF1_DRIVES > 2
MENUITEM("3",PB_CLASS,'3',0,0,menuFileCartridgeRemove,2,0)
#endif
#if XZX_IF1_DRIVES > 3
MENUITEM("4",PB_CLASS,'4',0,0,menuFileCartridgeRemove,3,0)
#endif
#if XZX_IF1_DRIVES > 4
MENUITEM("5",PB_CLASS,'5',0,0,menuFileCartridgeRemove,4,0)
#endif
#if XZX_IF1_DRIVES > 5
MENUITEM("6",PB_CLASS,'6',0,0,menuFileCartridgeRemove,5,0)
#endif
#if XZX_IF1_DRIVES > 6
MENUITEM("7",PB_CLASS,'7',0,0,menuFileCartridgeRemove,6,0)
#endif
#if XZX_IF1_DRIVES > 7
MENUITEM("8",PB_CLASS,'8',0,0,menuFileCartridgeRemove,7,0)
#endif
MENUEND
#endif 
#ifdef HAVE_XPM
MENUBEGIN(screenSaveItems)MENUITEM("as SCR...",PB_CLASS,'S',"<Key>F4","F4",menuFileScreendumpWrite,FILE_TYPE_SCR,0)MENUITEM("as XPM...",PB_CLASS,'X',0,0,menuFileScreendumpWrite,FILE_TYPE_XPM,0)MENUEND
#endif 
MENUBEGIN(audioSaveItems)MENUITEM("as PSG...",PB_CLASS,'P',0,0,menuFileAudioWrite,FILE_TYPE_PSG,0)MENUITEM("Close PSG",PB_CLASS,'C',0,0,menuFileAudioClose,FILE_TYPE_PSG,0)MENUITEM("as WAV...",PB_CLASS,'W',0,0,menuFileAudioWrite,FILE_TYPE_WAV,0)MENUITEM("Close WAV",PB_CLASS,'l',0,0,menuFileAudioClose,FILE_TYPE_WAV,0)MENUEND MENUBEGIN(fileItems)MENUITEM("Load snapshot...",PB_CLASS,'L',"<Key>F3","F3",menuFileSnapshotRead,0,0)MENUITEM("Save snapshot...",PB_CLASS,'S',"<Key>F2","F2",menuFileSnapshotWrite,0,0)MENUSEP	MENUITEM("Insert disk",CB_CLASS,'I',0,0,0,0,diskInsertItems)MENUITEM("Remove disk",CB_CLASS,'R',0,0,0,0,diskRemoveItems)MENUSEP
#ifdef XZX_IF1
MENUITEM("Insert cartridge",CB_CLASS,'n',0,0,0,0,mdrInsertItems)MENUITEM("Remove cartridge",CB_CLASS,'e',0,0,0,0,mdrRemoveItems)MENUSEP
#endif
MENUITEM("Load screen...",PB_CLASS,'o',0,0,menuFileScreendumpRead,0,0)
#ifdef HAVE_XPM
MENUITEM("Save screen",CB_CLASS,'a',0,0,0,0,screenSaveItems)
#else
MENUITEM("Save screen...",PB_CLASS,'a',"<Key>F4","F4",menuFileScreendumpWrite,0,0)
#endif
MENUSEP	MENUITEM("Save audio",CB_CLASS,'v',0,0,0,0,audioSaveItems)MENUSEP	MENUITEM("Quit",PB_CLASS,'Q',"Ctrl<Key>q","Ctrl+Q",menuFileQuit,0,0)MENUEND MENUBEGIN(miscItems)MENUITEM("Reset",PB_CLASS,'R',"<Key>F5","F5",menuMiscReset,0,0)MENUITEM("NMI",PB_CLASS,'N',"<Key>F6","F6",menuMiscNmi,0,0)MENUITEM("Pause",TB_CLASS,'P',"Ctrl<Key>h","Ctrl+H",menuMiscPause,0,0)
#if defined(HAVE_VIDMODE) || defined(XZX_XMOUSE)
MENUSEP
#ifdef HAVE_VIDMODE
MENUITEM("Fullscreen",TB_CLASS,'F',"Ctrl<Key>f","Ctrl+F",menuMiscFullscreen,0,0)
#endif
#ifdef XZX_XMOUSE
MENUITEM("Grab pointer",TB_CLASS,'G',"Ctrl<Key>g","Ctrl+G",menuMiscPointerGrab,0,0)
#endif
#endif
MENUSEP
#ifndef DEBUG
MENUITEM("Console",TB_CLASS,'C',0,0,menuMiscConsole,0,0)
#endif
MENUITEM("Monitor",TB_CLASS,'M',"<Key>F7","F7",menuMiscMonitor,0,0)MENUITEM("Tape manager",TB_CLASS,'T',"<Key>F8","F8",menuMiscVtp,0,0)MENUITEM("Poke manager...",PB_CLASS,'o',0,0,PokeManager,0,0)MENUEND MENUBEGIN(optionItems)MENUITEM("Change...",PB_CLASS,'C',"<Key>F9","F9",OptionsDialog,0,0)MENUITEM("Save on exit",TB_CLASS,'x',0,0,menuOptionsSaveOnExit,0,0)MENUITEM("Save now",PB_CLASS,'n',0,0,menuOptionsSaveNow,0,0)MENUEND MENUBEGIN(helpItems)
#ifdef HAVE_XPM
MENUITEM("Keyboard",TB_CLASS,'K',"<Key>F1","F1",menuHelp,HELP_KEYPIC,0)
#endif
MENUITEM("Copyright",PB_CLASS,'C',0,0,menuHelp,HELP_COPYRIGHT,0)MENUITEM("Readme",PB_CLASS,'R',0,0,menuHelp,HELP_README,0)MENUITEM("FAQ",PB_CLASS,'F',0,0,menuHelp,HELP_FAQ,0)MENUSEP	MENUITEM("About",PB_CLASS,'A',0,0,UiShowAbout,0,0)MENUEND static Boolean menuSaveOnExit=True;Widget MenuCreateBar(Widget parent){Widget bar,helpmenu;bar=XmCreateMenuBar(parent,(char*)"bar",(ArgList)0,0);(void)menuCreateMenu(bar,XmMENU_PULLDOWN,"File",0,False,fileItems);(void)menuCreateMenu(bar,XmMENU_PULLDOWN,"Misc",0,False,miscItems);(void)menuCreateMenu(bar,XmMENU_PULLDOWN,"Options",0,False,optionItems);helpmenu=menuCreateMenu(bar,XmMENU_PULLDOWN,"Help",0,False,helpItems);XtVaSetValues(bar,XmNmenuHelpWidget,helpmenu,XtNull);XtSetSensitive(audioSaveItems[1].widget,False);XtSetSensitive(audioSaveItems[3].widget,False);XmToggleButtonGadgetSetState(optionItems[1].widget,menuSaveOnExit,False);return bar;}static Widget menuCreateMenu(Widget parent,int menuType,const char*menuTitle,char menuMnemonic,Boolean tearOff,menuItem*items){Widget menu,cascade=(Widget)0,widget;XmString str;Arg args[2];Cardinal i;if(menuType==XmMENU_PULLDOWN||menuType==XmMENU_OPTION){menu=XmCreatePulldownMenu(parent,(char*)menuTitle,(ArgList)0,0);}else{assert(menuType==XmMENU_POPUP);menu=XmCreatePopupMenu(parent,(char*)menuTitle,(ArgList)0,0);}if(tearOff==True){XtVaSetValues(menu,XmNtearOffModel,XmTEAR_OFF_ENABLED,XtNull);}if(menuType==XmMENU_PULLDOWN){i=0;XtSetArg(args[i],XmNsubMenuId,menu);i++;if(menuMnemonic!='\0'){XtSetArg(args[i],XmNmnemonic,menuMnemonic);i++;}cascade=XmCreateCascadeButtonGadget(parent,(char*)menuTitle,args,i);XtManageChild(cascade);}else if(menuType==XmMENU_OPTION){i=0;XtSetArg(args[i],XmNsubMenuId,menu);i++;if(menuMnemonic!='\0'){XtSetArg(args[i],XmNmnemonic,menuMnemonic);i++;}cascade=XmCreateOptionMenu(parent,(char*)menuTitle,args,i);XtManageChild(cascade);}for(i=0;items[i].label!=(const char*)0;i++){if(items[i].subItems!=(menuItem*)0){assert(menuType!=XmMENU_OPTION);widget=menuCreateMenu(menu,XmMENU_PULLDOWN,items[i].label,items[i].mnemonic,tearOff,items[i].subItems);}else{widget=XtVaCreateManagedWidget(items[i].label,*items[i].class,menu,XtNull);}items[i].widget=widget;if(items[i].mnemonic!='\0'){XtVaSetValues(widget,XmNmnemonic,items[i].mnemonic,XtNull);}if(items[i].accelerator!=(const char*)0){str=XmStringCreateLtoR((char*)(items[i].accelText),XmFONTLIST_DEFAULT_TAG);XtVaSetValues(widget,XmNaccelerator,items[i].accelerator,XmNacceleratorText,str,XtNull);XmStringFree(str);}if(items[i].callback!=XtNull){XtAddCallback(widget,(items[i].class==&xmToggleButtonWidgetClass||items[i].class==&xmToggleButtonGadgetClass)?XmNvalueChangedCallback:XmNactivateCallback,items[i].callback,items[i].callbackData);}}return(menuType==XmMENU_POPUP?menu:cascade);}static void menuFileSnapshotRead(void){int types[4]={FILE_TYPE_Z80,FILE_TYPE_SLT,FILE_TYPE_SNA,FILE_TYPE_END};char*filename;if((filename=UiFileSelector("Load Snapshot",1,types))!=(char*)0){(void)SnapshotRead(filename);free(filename);}}static void menuFileSnapshotWrite(void){int types[4]={FILE_TYPE_Z80,FILE_TYPE_SLT,FILE_TYPE_SNA,FILE_TYPE_END};char*filename;if(CFGGET(machine)>SP_48_3){types[2]=FILE_TYPE_END;}if((filename=UiFileSelector("Save Snapshot",0,types))!=(char*)0){(void)SnapshotWrite(filename);free(filename);}}static void menuFileDiskInsert(Widget w,XtPointer client,XtPointer call){USED(w);USED(call);if(CFGGET(machine)==SP_3){(void)Plus3InsertDisk((int)client);}
#ifdef XZX_DIDAKTIK
else if(CFGGET(machine)==SP_DIDAKTIK){(void)DidaktikD80InsertDisk((int)client);}
#endif
#ifdef XZX_OPUS
else if(CFGGET(opus)>0){(void)OpusInsertDisk((int)client);}
#endif
#ifdef XZX_TRDOS
else if(CFGGET(trdos)>0||PENTAGON_OR_SCORPION){(void)TrdosInsertDisk((int)client);}
#endif
#ifdef XZX_PLUSD
else if(CFGGET(plusd)>0){(void)PlusdInsertDisk((int)client);}
#endif
}static void menuFileDiskRemove(Widget w,XtPointer client,XtPointer call){USED(w);USED(call);if(CFGGET(machine)==SP_3){Plus3EjectDisk((int)client);}
#ifdef XZX_DIDAKTIK
else if(CFGGET(machine)==SP_DIDAKTIK){DidaktikD80EjectDisk((int)client);}
#endif
#ifdef XZX_OPUS
else if(CFGGET(opus)>0){(void)OpusEjectDisk((int)client);}
#endif
#ifdef XZX_TRDOS
else if(CFGGET(trdos)>0||PENTAGON_OR_SCORPION){TrdosEjectDisk((int)client);}
#endif
#ifdef XZX_PLUSD
else if(CFGGET(plusd)>0){PlusdEjectDisk((int)client);}
#endif
}void MenuFileDiskUpdate(void){char**resource=(char**)0;Boolean ins,rem;int drives=0,i;if(CFGGET(machine)==SP_3){resource=CFGGET(sp3Disks);drives=2;}
#ifdef XZX_DIDAKTIK
else if(CFGGET(machine)==SP_DIDAKTIK){resource=CFGGET(didaktikDisks);drives=2;}
#endif
#ifdef XZX_OPUS
else if(CFGGET(opus)>0){resource=CFGGET(opusDisks);drives=2;}
#endif
#ifdef XZX_TRDOS
else if(CFGGET(trdos)>0||PENTAGON_OR_SCORPION){resource=CFGGET(trdosDisks);drives=4;}
#endif
#ifdef XZX_PLUSD
else if(CFGGET(plusd)>0){resource=CFGGET(plusdDisks);drives=2;}
#endif
for(i=0;i<XZX_DISK_DRIVES;i++){if(i>=drives){ins=False;rem=False;}else{ins=True;rem=resource[i]!=(char*)0?True:False;}XtSetSensitive(diskInsertItems[i].widget,ins);XtSetSensitive(diskRemoveItems[i].widget,rem);}SbarSetLed(drives>0?LED_SHOW:LED_HIDE);}
#ifdef XZX_IF1
static void menuFileCartridgeInsert(Widget w,XtPointer client,XtPointer call){USED(w);USED(call);assert(CFGGET(if1)>0);(void)If1InsertCart((int)client);}
#endif 
#ifdef XZX_IF1
static void menuFileCartridgeRemove(Widget w,XtPointer client,XtPointer call){USED(w);USED(call);assert(CFGGET(if1)>0);If1EjectCart((int)client);}
#endif 
#ifdef XZX_IF1
void MenuFileCartridgeUpdate(void){Boolean ins,rem;int i;ins=CFGGET(if1)<=0?False:True;for(i=0;i<XZX_IF1_DRIVES;i++){rem=(CFGGET(if1)<=0||CFGGET(if1Carts[i])==(char*)0)?False:True;XtSetSensitive(mdrInsertItems[i].widget,ins);XtSetSensitive(mdrRemoveItems[i].widget,rem);}}
#endif 
static void menuFileScreendumpRead(void){int types[2]={FILE_TYPE_SCR,FILE_TYPE_END};char*filename;if((filename=UiFileSelector("Load Screenshot",1,types))!=(char*)0){(void)ScrRead(filename);free(filename);}}static void menuFileScreendumpWrite(Widget w,XtPointer client,XtPointer call){int types[2]={FILE_TYPE_SCR,FILE_TYPE_END};int(*func)(const char*)=ScrWrite;char*filename;USED(w);USED(client);USED(call);
#ifdef HAVE_XPM
if((FileType)client==FILE_TYPE_XPM){types[0]=FILE_TYPE_XPM;func=XpmWrite;}
#endif
if((filename=UiFileSelector("Save Screenshot",0,types))!=(char*)0){(void)func(filename);free(filename);}}static void menuFileAudioWrite(Widget w,XtPointer client,XtPointer call){int types[2];char*filename;SoundData*sdp=SoundDataP;USED(call);types[0]=(int)client;types[1]=FILE_TYPE_END;if((filename=UiFileSelector("Save Audio",0,types))!=(char*)0){if(((int)client==FILE_TYPE_PSG?PsgOpen(filename):WavOpen(filename,sdp->sampleRate,sdp->stereo?2U:1U,sdp->sampleBits))==0){XtSetSensitive(w,False);XtSetSensitive((int)client==FILE_TYPE_PSG?audioSaveItems[1].widget:audioSaveItems[3].widget,True);}free(filename);}}static void menuFileAudioClose(Widget w,XtPointer client,XtPointer call){USED(call);if((int)client==FILE_TYPE_PSG){PsgClose();XtSetSensitive(audioSaveItems[0].widget,True);}else{WavClose();XtSetSensitive(audioSaveItems[2].widget,True);}XtSetSensitive(w,False);}static void menuFileQuit(void){if(UiShowMessage(DIALOG_ASK,"Do you really want to quit?",(const char*)0,(const char*)0,DIALOG_CANCEL)==DIALOG_OK){if(menuSaveOnExit==True&&UiShowMessage(DIALOG_ASK,"Save the current settings?",(const char*)0,(const char*)0,DIALOG_CANCEL)==DIALOG_OK){if(ResourceWrite()<0){(void)UiShowMessage(DIALOG_ERROR,"Could not save sattings!",(const char*)0,(const char*)0,DIALOG_OK);}}exit(0);}}static void menuMiscReset(void){Machine->reset();}static void menuMiscNmi(void){Machine->nmi();}static void menuMiscPause(Widget w,XtPointer client,XtPointer call){static int paused;USED(client);if(((XmToggleButtonCallbackStruct*)call)->set==True){char*title,*saveTitle;(void)UiEnter();SoundControl(SOUND_ACTION_OFF);paused=1;XtVaGetValues(XwinDataP->topLevel,XtNtitle,&title,XtNull);saveTitle=Strdup(title,"menuMiscPause");title=Malloc(sizeof(char)*(strlen(saveTitle)+(size_t)10),"menuMiscPause");(void)sprintf(title,"%s (paused)",saveTitle);XtVaSetValues(XwinDataP->topLevel,XtNtitle,title,XtNull);while(paused!=0){XtAppProcessEvent(XtWidgetToApplicationContext(w),XtIMAll);}XtVaSetValues(XwinDataP->topLevel,XtNtitle,saveTitle,XtNull);free(saveTitle);free(title);SoundControl(SOUND_ACTION_ON);(void)UiLeave();}else{paused=0;}}
#ifdef HAVE_VIDMODE
static void menuMiscFullscreen(Widget w,XtPointer client,XtPointer call){USED(client);if(((XmToggleButtonCallbackStruct*)call)->set==True){if(Xwin86EnterFullscreenMode()<0){XmToggleButtonGadgetSetState(w,False,False);
#ifndef EVALUATION
XtSetSensitive(w,False);
#endif
}}else{(void)Xwin86LeaveFullscreenMode();}}
#endif
#ifdef XZX_XMOUSE
static void menuMiscPointerGrab(Widget w,XtPointer client,XtPointer call){USED(client);if(((XmToggleButtonCallbackStruct*)call)->set==True){if(XmouseGrab()<0){XmToggleButtonGadgetSetState(w,False,False);}}else{XmouseUngrab();}}
#endif
static void menuMiscMonitor(Widget w,XtPointer client,XtPointer call){USED(w);USED(client);if(((XmToggleButtonCallbackStruct*)call)->set==True){MonitorShow();}else{MonitorHide();}}static void menuMiscVtp(Widget w,XtPointer client,XtPointer call){USED(w);USED(client);if(((XmToggleButtonCallbackStruct*)call)->set==True){VtpShow();}else{VtpHide();}}
#ifndef DEBUG
static void menuMiscConsole(Widget w,XtPointer client,XtPointer call){USED(w);USED(client);if(((XmToggleButtonCallbackStruct*)call)->set==True){XconsShow();}else{XconsHide();}}
#endif 
static void menuOptionsSaveOnExit(Widget w,XtPointer client,XtPointer call){USED(w);USED(client);menuSaveOnExit=((XmToggleButtonCallbackStruct*)call)->set;}static void menuOptionsSaveNow(void){int type;const char*msg;if(ResourceWrite()<0){type=DIALOG_ERROR;msg="Could not save sattings!";}else{type=DIALOG_INFO;msg="Settings saved successfully.";}(void)UiShowMessage(type,msg,(const char*)0,(const char*)0,DIALOG_OK);}static void menuHelp(Widget w,XtPointer client,XtPointer call){const char*file;USED(call);switch((int)client){default:case HELP_COPYRIGHT:file="COPYRIGHT";break;case HELP_README:file="README";break;case HELP_FAQ:file="FAQ";break;
#ifdef HAVE_XPM
case HELP_KEYPIC:if(((XmToggleButtonCallbackStruct*)call)->set==True){KbdShow(w);}else{KbdHide(w);}return;
#endif
}if(UiShowTextFile(file,menuHelpCallback,(XtPointer)w)!=0){(void)UiShowMessage(DIALOG_ERROR,"Could not open file!",(const char*)0,(const char*)0,DIALOG_OK);}else{XtSetSensitive(w,False);}}static void menuHelpCallback(Widget w,XtPointer client,XtPointer call){USED(w);USED(call);XtSetSensitive((Widget)client,True);}
