/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: options.c,v 4.58 2006/12/22 14:29:16 erik Rel $";
#endif
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/LabelG.h>
#include <Xm/Notebook.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/SelectioB.h>
#include <Xm/SeparatoG.h>
#include <Xm/SSpinB.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <sys/param.h>
#include <stdlib.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "ui/ui.h"
#include "emul.h"
#ifdef XZX_IF1
#include "machine/if1.h"
#endif
#ifdef XZX_KMOUSE
#include "machine/kmouse.h"
#endif
#ifdef XZX_LIGHTGUN
#include "machine/lightgun.h"
#endif
#include "ui/menu.h"
#ifdef XZX_OPUS
#include "machine/opus.h"
#endif
#ifdef XZX_PRINTER
#include "printer/printer.h"
#endif
#ifdef XZX_IF1_RS232
#include "rs232.h"
#endif
#include "sound/sound.h"
#include "tape.h"
#include "ui/toolhelp.h"
#include "ui/ui.h"
#include "ui/xwin.h"
#include "ui/options.h"
enum{OPTIONS_NOP,OPTIONS_APPLY,OPTIONS_DISCARD};typedef struct _optionsPage{int pageNumber;const char*pageLabel;void(*pageCreate)(Widget,struct _optionsPage*);void(*pageCallback)(Widget,XtPointer,XtPointer);void(*pageUpdate)(struct _optionsPage*);void(*pageCommit)(struct _optionsPage*);void*pageData;}optionsPage;static void optionsDialogCallback(Widget,XtPointer,XtPointer);static void optionsMiscCreate(Widget,optionsPage*);static void optionsMiscCallback(Widget,XtPointer,XtPointer);static void optionsMiscUpdate(optionsPage*);static void optionsMiscCommit(optionsPage*);static void optionsHardwareCreate(Widget,optionsPage*);static void optionsHardwareCallback(Widget,XtPointer,XtPointer);static void optionsHardwareUpdate(optionsPage*);static void optionsHardwareCommit(optionsPage*);static void optionsTapeCreate(Widget,optionsPage*);static void optionsTapeCallback(Widget,XtPointer,XtPointer);static void optionsTapeUpdate(optionsPage*);static void optionsTapeCommit(optionsPage*);static void optionsSoundCreate(Widget,optionsPage*);static void optionsSoundCallback(Widget,XtPointer,XtPointer);static void optionsSoundUpdate(optionsPage*);static void optionsSoundCommit(optionsPage*);static void optionsJoystickCreate(Widget,optionsPage*);static void optionsJoystickCallback(Widget,XtPointer,XtPointer);static void optionsJoystickUpdate(optionsPage*);static void optionsJoystickCommit(optionsPage*);
#ifdef XZX_IF1
static void optionsIf1Create(Widget,optionsPage*);static void optionsIf1Callback(Widget,XtPointer,XtPointer);static void optionsIf1Update(optionsPage*);static void optionsIf1Commit(optionsPage*);
#endif
#ifdef XZX_PRINTER
static void optionsPrinterCreate(Widget,optionsPage*);static void optionsPrinterCallback(Widget,XtPointer,XtPointer);static void optionsPrinterUpdate(optionsPage*);static void optionsPrinterCommit(optionsPage*);
#endif
#ifdef XZX_NETWORK
static void optionsNetworkCreate(Widget,optionsPage*);static void optionsNetworkCallback(Widget,XtPointer,XtPointer);static void optionsNetworkUpdate(optionsPage*);static void optionsNetworkCommit(optionsPage*);
#endif
#ifdef DEBUG
static void optionsDebugCreate(Widget,optionsPage*);static void optionsDebugCallback(Widget,XtPointer,XtPointer);static void optionsDebugUpdate(optionsPage*);static void optionsDebugCommit(optionsPage*);
#endif
static optionsPage optionsPages[]={{1,"Misc",optionsMiscCreate,optionsMiscCallback,optionsMiscUpdate,optionsMiscCommit,0},{2,"Hardware",optionsHardwareCreate,optionsHardwareCallback,optionsHardwareUpdate,optionsHardwareCommit,0},
#ifdef XZX_IF1
{3,"Interface I",optionsIf1Create,optionsIf1Callback,optionsIf1Update,optionsIf1Commit,0},
#endif
{4,"Sound",optionsSoundCreate,optionsSoundCallback,optionsSoundUpdate,optionsSoundCommit,0},
#ifdef XZX_PRINTER
{5,"Printer",optionsPrinterCreate,optionsPrinterCallback,optionsPrinterUpdate,optionsPrinterCommit,0},
#endif
{6,"Joystick",optionsJoystickCreate,optionsJoystickCallback,optionsJoystickUpdate,optionsJoystickCommit,0},{7,"Tape",optionsTapeCreate,optionsTapeCallback,optionsTapeUpdate,optionsTapeCommit,0},
#ifdef XZX_NETWORK
{8,"Network",optionsNetworkCreate,optionsNetworkCallback,optionsNetworkUpdate,optionsNetworkCommit,0},
#endif
#ifdef DEBUG
{9,"Debug",optionsDebugCreate,optionsDebugCallback,optionsDebugUpdate,optionsDebugCommit,0},
#endif
};void OptionsDialog(void){static Widget dialog=(Widget)0;static int answer;optionsPage*opp;if(dialog==(Widget)0){XmString title,cancelLabel;Widget notebook,form,tab;Arg arg[7];Cardinal i;int j;title=XmStringCreateLtoR((char*)"Options",XmFONTLIST_DEFAULT_TAG);cancelLabel=XmStringCreateLtoR((char*)"Discard",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNnoResize,True);i++;XtSetArg(arg[i],XmNautoUnmanage,False);i++;XtSetArg(arg[i],XmNdeleteResponse,XmDO_NOTHING);i++;XtSetArg(arg[i],XmNdialogStyle,XmDIALOG_FULL_APPLICATION_MODAL);i++;XtSetArg(arg[i],XmNdialogTitle,title);i++;XtSetArg(arg[i],XmNcancelLabelString,cancelLabel);i++;XtSetArg(arg[i],XtNkeyboardFocusPolicy,XmPOINTER);i++;assert(i<=XtNumber(arg));dialog=XmCreatePromptDialog(XwinDataP->topLevel,(char*)"options",arg,i);XmStringFree(title);XmStringFree(cancelLabel);XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_COMMAND_TEXT));XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_PROMPT_LABEL));XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_OK_BUTTON));XtManageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_APPLY_BUTTON));XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_HELP_BUTTON));XtAddCallback(dialog,XmNapplyCallback,optionsDialogCallback,&answer);XtAddCallback(dialog,XmNcancelCallback,optionsDialogCallback,&answer);i=0;XtSetArg(arg[i],XmNuserData,&optionsPages);i++;assert(i<=XtNumber(arg));notebook=XmCreateNotebook(dialog,(char*)"notebook",arg,i);for(opp=optionsPages,j=1;opp<&optionsPages[NELEM(optionsPages)];opp++,j++){opp->pageNumber=j;i=0;XtSetArg(arg[i],XmNnotebookChildType,XmPAGE);i++;XtSetArg(arg[i],XmNpageNumber,j);i++;XtSetArg(arg[i],XmNmarginHeight,5);i++;XtSetArg(arg[i],XmNmarginWidth,5);i++;assert(i<=XtNumber(arg));form=XmCreateForm(notebook,(char*)"form",arg,i);i=0;XtSetArg(arg[i],XmNnotebookChildType,XmMAJOR_TAB);i++;XtSetArg(arg[i],XmNpageNumber,j);i++;assert(i<=XtNumber(arg));tab=XmCreatePushButtonGadget(notebook,(char*)opp->pageLabel,arg,i);opp->pageCreate(form,opp);XtManageChild(tab);XtManageChild(form);}XtManageChild(notebook);}for(opp=optionsPages;opp<&optionsPages[NELEM(optionsPages)];opp++){opp->pageUpdate(opp);}UiShowDialog(dialog,&answer);}static void optionsDialogCallback(Widget w,XtPointer client,XtPointer call){optionsPage*opp;USED(w);switch(((XmAnyCallbackStruct*)call)->reason){case XmCR_APPLY:*((int*)client)=OPTIONS_APPLY;for(opp=optionsPages;opp<&optionsPages[NELEM(optionsPages)];opp++){opp->pageCommit(opp);}break;case XmCR_CANCEL:*((int*)client)=OPTIONS_DISCARD;break;}}typedef struct _optionsMiscData{Widget speedWidget,fastWidget;Widget refreshWidget,cacheWidget,xsyncWidget;Widget smartWidget,warn48Widget;}optionsMiscData;static void optionsMiscCreate(Widget parent,optionsPage*op){static optionsMiscData optionsMiscDatas;optionsMiscData*dp;Widget sep,w;XmString label;Arg arg[13];Cardinal i;op->pageData=dp=&optionsMiscDatas;XtVaSetValues(parent,XmNfractionBase,100,XtNull);label=XmStringCreateLtoR((char*)"Emulation speed (%)",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNorientation,XmHORIZONTAL);i++;XtSetArg(arg[i],XmNprocessingDirection,XmMAX_ON_RIGHT);i++;XtSetArg(arg[i],XmNminimum,10);i++;XtSetArg(arg[i],XmNmaximum,200);i++;XtSetArg(arg[i],XmNscaleMultiple,10);i++;XtSetArg(arg[i],XmNshowValue,True);i++;XtSetArg(arg[i],XmNtitleString,label);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,45);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateScale(parent,(char*)"scale",arg,i);XtManageChild(w);dp->speedWidget=w;XmStringFree(label);ToolhelpAddWidget(w,"Control emulation speed. The default\n""setting is 100, which causes the\n""emulation to never run faster than the\n""real machine. A higher value allows the\n""emulator to run faster, a lower one may\n""force it to run slower.");i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNleftOffset,3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButton(parent,(char*)"Fast mode",arg,i);XtManageChild(w);dp->fastWidget=w;ToolhelpAddWidget(w,"Special speed mode, which disables any\n""speed limit, turns sound emulation off\n""and uses a 1/10 refresh rate, so that\n""the emulator will run at the maximum\n""possible speed.");label=XmStringCreateLtoR((char*)"Refresh rate (1/n)",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNorientation,XmHORIZONTAL);i++;XtSetArg(arg[i],XmNprocessingDirection,XmMAX_ON_RIGHT);i++;XtSetArg(arg[i],XmNminimum,1);i++;XtSetArg(arg[i],XmNmaximum,50);i++;XtSetArg(arg[i],XmNscaleMultiple,1);i++;XtSetArg(arg[i],XmNshowValue,True);i++;XtSetArg(arg[i],XmNtitleString,label);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,50);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,95);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateScale(parent,(char*)"scale",arg,i);XtManageChild(w);dp->refreshWidget=w;XmStringFree(label);ToolhelpAddWidget(w,"Control rate of frame update. The value\n""ranges from \"1/1\" (update 1/1 of the\n""frames of the real machine, that is 50\n""frames per second) to \"1/50\" (update 1\n""every 50 frames).");i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,50);i++;XtSetArg(arg[i],XmNleftOffset,-3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButton(parent,(char*)"Video cache",arg,i);XtManageChild(w);dp->cacheWidget=w;ToolhelpAddWidget(w,"Video cache can speed up the emulation\n""when little graphics activity is going\n""on. It is especially useful when the\n""emulator is running across a network, as\n""it can reduce the amount of bandwidth\n""required.");i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,50);i++;XtSetArg(arg[i],XmNleftOffset,-3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButton(parent,(char*)"XSync",arg,i);XtManageChild(w);dp->xsyncWidget=w;ToolhelpAddWidget(w,"Call the X11 function XSync before\n""updating the emulation window. This\n""might be necessary on low-end systems\n""to prevent it from stealing so many\n""system resources that it would become\n""impossible to interact with it.");i=0;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;XtSetArg(arg[i],XmNtopOffset,15);i++;assert(i<=XtNumber(arg));sep=XmCreateSeparatorGadget(parent,(char*)"sep",arg,i);XtManageChild(sep);i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNleftOffset,3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,sep);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButtonGadget(parent,(char*)"Smart attach of media",arg,i);XtManageChild(w);dp->smartWidget=w;i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNleftOffset,3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButtonGadget(parent,(char*)"Warn if a 48k program accesses\nports from the 128K machinery",arg,i);XtManageChild(w);dp->warn48Widget=w;}static void optionsMiscCallback(Widget w,XtPointer client,XtPointer call){USED(w);USED(client);USED(call);}static void optionsMiscUpdate(optionsPage*op){optionsMiscData*dp=(optionsMiscData*)(op->pageData);XmScaleSetValue(dp->speedWidget,(int)CFGGET(speed));XmToggleButtonSetState(dp->fastWidget,(CFGGET(fastmode)&1)!=0?True:False,False);XmScaleSetValue(dp->refreshWidget,CFGGET(refresh));XmToggleButtonSetState(dp->cacheWidget,CFGGET(cache)!=0?True:False,False);XmToggleButtonSetState(dp->xsyncWidget,CFGGET(xsync)!=0?True:False,False);XmToggleButtonSetState(dp->warn48Widget,CFGGET(warn48)!=0?True:False,False);XmToggleButtonSetState(dp->smartWidget,CFGGET(smartAttach)!=0?True:False,False);}static void optionsMiscCommit(optionsPage*op){optionsMiscData*dp=(optionsMiscData*)(op->pageData);int val;XmScaleGetValue(dp->speedWidget,&val);val=((val+5)/10)*10;if((unsigned int)val!=CFGGET(speed)){CFGSET(speed,(unsigned int)val);EmulSetSpeed((unsigned int)val);}val=(CFGGET(fastmode)&~1)|(XmToggleButtonGetState(dp->fastWidget)==True?1:0);if(val!=CFGGET(fastmode)){CFGSET(fastmode,val);SoundControl(val==0?SOUND_ACTION_ON:SOUND_ACTION_OFF);}XmScaleGetValue(dp->refreshWidget,&val);CFGSET(refresh,val);CFGSET(cache,XmToggleButtonGetState(dp->cacheWidget)==True?1:0);CFGSET(xsync,XmToggleButtonGetState(dp->xsyncWidget)==True?1:0);CFGSET(warn48,XmToggleButtonGetState(dp->warn48Widget)==True?1:0);CFGSET(smartAttach,XmToggleButtonGetState(dp->smartWidget)==True?1:0);}enum{HW_SP482,HW_SP483,HW_SP128,HW_SP3,HW_PENTAGON,HW_SCORPION,HW_DIDAKTIK,HW_MAX_OPTIONS};enum{HW_RESET,HW_IF1,HW_OPUS,HW_TRDOS,HW_PLUSD,HW_MF,HW_USPEECH,HW_LIGHTGUN,HW_KMOUSE,HW_MAX_BUTTONS};enum{W_HW_SP482,W_HW_SP483,W_HW_SP128,W_HW_SP3,W_HW_PENTAGON,W_HW_SCORPION,W_HW_DIDAKTIK,W_HW_RESET,W_HW_IF1,W_HW_OPUS,W_HW_TRDOS,W_HW_PLUSD,W_HW_MF,W_HW_USPEECH,W_HW_LIGHTGUN,W_HW_KMOUSE};typedef struct _optionsHardwareData{Widget modelWidget;int modelValue;struct{const char*label;Boolean sensitive;Widget widget;}modelOptions[HW_MAX_OPTIONS];struct{const char*label;Boolean sensitive;Widget widget;}hardwareButtons[HW_MAX_BUTTONS];
#ifdef XZX_MF
Boolean mf1Active,mf128Active,mf3Active;
#endif
}optionsHardwareData;static void optionsHardwareCreate(Widget parent,optionsPage*op){static optionsHardwareData optionsHardwareDatas={(Widget)0,0,{{"Spectrum 48K Issue 2",True,(Widget)0},{"Spectrum 48K Issue 3",True,(Widget)0},{"Spectrum 128K",True,(Widget)0},{"Spectrum +3",True,(Widget)0},
#ifdef XZX_PENTAGON
{"Pentagon 128",True,(Widget)0},
#else
{"Pentagon 128",False,(Widget)0},
#endif
#ifdef XZX_SCORPION
{"Scorpion 256",True,(Widget)0},
#else
{"Scorpion 256",False,(Widget)0},
#endif
#ifdef XZX_DIDAKTIK
{"Didaktik Kompakt 128K",True,(Widget)0},
#else
{"Didaktik Kompakt 128K",False,(Widget)0},
#endif
},{{"Reset upon change",True,(Widget)0},
#ifdef XZX_IF1
{"Interface I",True,(Widget)0},
#else
{"Interface I",False,(Widget)0},
#endif
#ifdef XZX_OPUS
{"Opus Discovery",True,(Widget)0},
#else
{"Opus Discovery",False,(Widget)0},
#endif
#ifdef XZX_TRDOS
{"Beta 128",True,(Widget)0},
#else
{"Beta 128",False,(Widget)0},
#endif
#ifdef XZX_PLUSD
{"+D",True,(Widget)0},
#else
{"+D",False,(Widget)0},
#endif
#ifdef XZX_MF
{"Multiface",True,(Widget)0},
#else
{"Multiface",False,(Widget)0},
#endif
{"Currah uSpeech",False,(Widget)0},
#ifdef XZX_LIGHTGUN
{"Lightgun",True,(Widget)0},
#else
{"Lightgun",False,(Widget)0},
#endif
#ifdef XZX_KMOUSE
{"Kempston mouse",True,(Widget)0},
#else
{"Kempston mouse",False,(Widget)0},
#endif
}
#ifdef XZX_MF
,False,False,False
#endif
};optionsHardwareData*dp;Widget menu,cascade,sep,w;XmString label;Arg arg[7];Cardinal i;int pos,j;op->pageData=dp=&optionsHardwareDatas;menu=XmCreatePulldownMenu(parent,(char*)"pd",(ArgList)0,0);label=XmStringCreateLtoR((char*)"Model",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNsubMenuId,menu);i++;XtSetArg(arg[i],XmNlabelString,label);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));cascade=XmCreateOptionMenu(parent,(char*)"om",arg,i);XmStringFree(label);for(j=HW_SP482;j<HW_MAX_OPTIONS;j++){i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNsensitive,dp->modelOptions[j].sensitive);i++;assert(i<=XtNumber(arg));w=XmCreatePushButton(menu,(char*)(dp->modelOptions[j].label),arg,i);XtManageChild(w);if(dp->modelOptions[j].sensitive==True){XtAddCallback(w,XmNactivateCallback,optionsHardwareCallback,W_HW_SP482+j);}dp->modelOptions[j].widget=w;}XtManageChild(cascade);dp->modelWidget=cascade;i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNset,True);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,cascade);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButton(parent,(char*)(dp->hardwareButtons[HW_RESET].label),arg,i);XtManageChild(w);dp->hardwareButtons[HW_RESET].widget=w;ToolhelpAddWidget(w,"Usually it is a good idea to reset the\n""computer when changes are made to the\n""emulated hardware, because of software\n""initialisation. By default, a hardware\n""change (except from the extra hardware)\n""causes a reset. If this behaviour is not\n""desired, it can be switched off. The\n""emulator will try to lift the current\n""state to the new environment as well as\n""possible.");i=0;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;XtSetArg(arg[i],XmNtopOffset,20);i++;assert(i<=XtNumber(arg));sep=XmCreateSeparatorGadget(parent,(char*)"sep",arg,i);XtManageChild(sep);XtVaSetValues(parent,XmNfractionBase,2,XtNull);for(j=HW_IF1,pos=0,w=sep;j<HW_MAX_BUTTONS;j++){if(j==HW_MF){pos++;w=sep;}i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNsensitive,dp->hardwareButtons[j].sensitive);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,pos);i++;XtSetArg(arg[i],XmNleftOffset,5);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButtonGadget(parent,(char*)(dp->hardwareButtons[j].label),arg,i);XtManageChild(w);XtAddCallback(w,XmNvalueChangedCallback,optionsHardwareCallback,W_HW_RESET+j);dp->hardwareButtons[j].widget=w;}}static void optionsHardwareCallback(Widget w,XtPointer client,XtPointer call){Boolean set=((XmToggleButtonCallbackStruct*)call)->set!=0?True:False;Boolean s_if1,s_opus,s_trdos,s_plusd,s_mf,s_uspeech,s_lightgun,s_kmouse;Boolean v_if1,v_opus,v_trdos,v_plusd,v_mf,v_uspeech,v_lightgun,v_kmouse;optionsPage*op;optionsHardwareData*dp;XtVaGetValues(w,XmNuserData,&op,XtNull);dp=(optionsHardwareData*)(op->pageData);assert(dp!=(optionsHardwareData*)0);s_if1=XtIsSensitive(dp->hardwareButtons[HW_IF1].widget);s_opus=XtIsSensitive(dp->hardwareButtons[HW_OPUS].widget);s_trdos=XtIsSensitive(dp->hardwareButtons[HW_TRDOS].widget);s_plusd=XtIsSensitive(dp->hardwareButtons[HW_PLUSD].widget);s_mf=XtIsSensitive(dp->hardwareButtons[HW_MF].widget);s_uspeech=XtIsSensitive(dp->hardwareButtons[HW_USPEECH].widget);s_lightgun=XtIsSensitive(dp->hardwareButtons[HW_LIGHTGUN].widget);s_kmouse=XtIsSensitive(dp->hardwareButtons[HW_KMOUSE].widget);v_if1=XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_IF1].widget);v_opus=XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_OPUS].widget);v_trdos=XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_TRDOS].widget);v_plusd=XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_PLUSD].widget);v_mf=XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_MF].widget);v_uspeech=XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_USPEECH].widget);v_lightgun=XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_LIGHTGUN].widget);v_kmouse=XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_KMOUSE].widget);switch((int)client){case W_HW_SP482:case W_HW_SP483:case W_HW_SP128:case W_HW_SP3:case W_HW_PENTAGON:case W_HW_SCORPION:case W_HW_DIDAKTIK:dp->modelValue=(int)client-W_HW_SP482;
#ifdef XZX_IF1
if(CFGGET(if1)>=0){s_if1=dp->modelValue>HW_SP128?False:True;}
#endif
#ifdef XZX_OPUS
if(CFGGET(opus)>=0){s_opus=dp->modelValue>HW_SP128?False:True;}
#endif
#ifdef XZX_TRDOS
if(CFGGET(trdos)>=0){s_trdos=dp->modelValue>HW_SP128?False:True;}
#endif
#ifdef XZX_PLUSD
if(CFGGET(plusd)>=0){s_plusd=dp->modelValue>HW_SP128?False:True;}
#endif
#ifdef XZX_MF
s_mf=False;if(dp->modelValue<=W_HW_SP483){s_mf=dp->mf1Active;}else if(dp->modelValue==W_HW_SP128){s_mf=dp->mf128Active;}else if(dp->modelValue==W_HW_SP3){s_mf=dp->mf3Active;}
#endif
#ifdef XZX_LIGHTGUN
s_lightgun=dp->modelValue==HW_SP128||dp->modelValue==HW_SP3?True:False;
#endif
#ifdef XZX_KMOUSE
s_kmouse=dp->modelValue==HW_PENTAGON||dp->modelValue==HW_SCORPION?True:False;
#endif
break;case W_HW_IF1:v_if1=set;if(v_if1==True){v_opus=v_plusd=False;}break;case W_HW_OPUS:v_opus=set;if(v_opus==True){v_if1=v_trdos=v_plusd=False;}break;case W_HW_TRDOS:v_trdos=set;if(v_trdos==True){v_opus=v_plusd=v_mf=False;}break;case W_HW_PLUSD:v_plusd=set;if(v_plusd==True){v_if1=v_opus=v_trdos=v_mf=False;}break;case W_HW_MF:v_mf=set;if(v_mf==True){v_trdos=v_plusd=False;}break;case W_HW_USPEECH:v_uspeech=set;break;case W_HW_LIGHTGUN:v_lightgun=set;break;case W_HW_KMOUSE:v_kmouse=set;break;}XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_IF1].widget,s_if1==True?v_if1:False,False);XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_OPUS].widget,s_opus==True?v_opus:False,False);XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_TRDOS].widget,s_trdos==True?v_trdos:False,False);XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_PLUSD].widget,s_plusd==True?v_plusd:False,False);XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_MF].widget,s_mf==True?v_mf:False,False);XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_USPEECH].widget,s_uspeech==True?v_uspeech:False,False);XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_LIGHTGUN].widget,s_lightgun==True?v_lightgun:False,False);XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_KMOUSE].widget,s_kmouse==True?v_kmouse:False,False);XtSetSensitive(dp->hardwareButtons[HW_IF1].widget,s_if1);XtSetSensitive(dp->hardwareButtons[HW_OPUS].widget,s_opus);XtSetSensitive(dp->hardwareButtons[HW_TRDOS].widget,s_trdos);XtSetSensitive(dp->hardwareButtons[HW_PLUSD].widget,s_plusd);XtSetSensitive(dp->hardwareButtons[HW_MF].widget,s_mf);XtSetSensitive(dp->hardwareButtons[HW_USPEECH].widget,s_uspeech);XtSetSensitive(dp->hardwareButtons[HW_LIGHTGUN].widget,s_lightgun);XtSetSensitive(dp->hardwareButtons[HW_KMOUSE].widget,s_kmouse);}static void optionsHardwareUpdate(optionsPage*op){optionsHardwareData*dp=(optionsHardwareData*)(op->pageData);int model=CFGGET(machine);
#if !defined(XZX_PENTAGON) && !defined(XZX_SCORPION)
if(model>SP_3){model+=2;}
#elif !defined(XZX_PENTAGON)
if(model>SP_3){model++;}
#elif !defined(XZX_SCORPION)
if(model>SP_PENTAGON){model++;}
#endif
dp->modelValue=model;XtVaSetValues(dp->modelWidget,XmNmenuHistory,dp->modelOptions[model].widget,XtNull);
#ifdef XZX_IF1
XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_IF1].widget,CFGGET(if1)>0?True:False,False);XtSetSensitive(dp->hardwareButtons[HW_IF1].widget,CFGGET(if1)>=0&&CFGGET(machine)<=SP_128?True:False);
#endif
#ifdef XZX_OPUS
XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_OPUS].widget,CFGGET(opus)>0?True:False,False);XtSetSensitive(dp->hardwareButtons[HW_OPUS].widget,CFGGET(machine)<=SP_128?True:False);
#endif
#ifdef XZX_TRDOS
XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_TRDOS].widget,CFGGET(trdos)>0?True:False,False);XtSetSensitive(dp->hardwareButtons[HW_TRDOS].widget,CFGGET(trdos)>=0&&CFGGET(machine)<=SP_128?True:False);
#endif
#ifdef XZX_PLUSD
XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_PLUSD].widget,CFGGET(plusd)>0?True:False,False);XtSetSensitive(dp->hardwareButtons[HW_PLUSD].widget,CFGGET(plusd)>=0&&CFGGET(machine)<=SP_128?True:False);
#endif
#ifdef XZX_MF
dp->mf1Active=CFGGET(mf1)>=0?True:False;dp->mf128Active=CFGGET(mf128)>=0?True:False;dp->mf3Active=CFGGET(mf3)>=0?True:False;{Boolean set=False,sensitive=False;if(CFGGET(machine)<=SP_48_3){set=CFGGET(mf1)>0?True:False;sensitive=dp->mf1Active;}else if(CFGGET(machine)==SP_128){set=CFGGET(mf128)>0?True:False;sensitive=dp->mf128Active;}else if(CFGGET(machine)==SP_3){set=CFGGET(mf3)>0?True:False;sensitive=dp->mf3Active;}XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_MF].widget,set,False);XtSetSensitive(dp->hardwareButtons[HW_MF].widget,sensitive);}
#endif
#ifdef XZX_LIGHTGUN
XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_LIGHTGUN].widget,CFGGET(lightgun)>0?True:False,False);XtSetSensitive(dp->hardwareButtons[HW_LIGHTGUN].widget,CFGGET(machine)==SP_128||CFGGET(machine)==SP_3?True:False);
#endif
#ifdef XZX_KMOUSE
XmToggleButtonGadgetSetState(dp->hardwareButtons[HW_KMOUSE].widget,CFGGET(kmouse)>0?True:False,False);XtSetSensitive(dp->hardwareButtons[HW_KMOUSE].widget,PENTAGON_OR_SCORPION?True:False);
#endif
}static void optionsHardwareCommit(optionsPage*op){optionsHardwareData*dp=(optionsHardwareData*)(op->pageData);int model=dp->modelValue;
#ifndef XZX_SCORPION
if(dp->modelValue>HW_SCORPION){model--;}
#endif
#ifndef XZX_PENTAGON
if(dp->modelValue>HW_PENTAGON){model--;}
#endif
#ifdef XZX_IF1
if(CFGGET(if1)>=0){CFGSET(if1,XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_IF1].widget)==True?1:0);MenuFileCartridgeUpdate();}
#endif
#ifdef XZX_OPUS
if(CFGGET(opus)>=0){CFGSET(opus,XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_OPUS].widget)==True?1:0);}
#endif
#ifdef XZX_TRDOS
if(CFGGET(trdos)>=0){CFGSET(trdos,XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_TRDOS].widget)==True?1:0);}
#endif
#ifdef XZX_PLUSD
if(CFGGET(plusd)>=0){CFGSET(plusd,XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_PLUSD].widget)==True?1:0);}
#endif
#ifdef XZX_MF
{int set=XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_MF].widget)==True?1:0;if(model<=SP_48_3){if(dp->mf1Active){CFGSET(mf1,set);}}else if(model==SP_128){if(dp->mf128Active){CFGSET(mf128,set);}}else if(model==SP_3){if(dp->mf3Active){CFGSET(mf3,set);}}}
#endif
#ifdef XZX_LIGHTGUN
{if(XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_LIGHTGUN].widget)==True){LightgunOn();}else{LightgunOff();}}
#endif
#ifdef XZX_KMOUSE
{if(XmToggleButtonGadgetGetState(dp->hardwareButtons[HW_KMOUSE].widget)==True){KmouseOn();}else{KmouseOff();}}
#endif
if(model!=CFGGET(machine)){MachineSelect(model,XmToggleButtonGetState(dp->hardwareButtons[HW_RESET].widget)==True?1:0);}MenuFileDiskUpdate();}enum{TAPE_QUICK_LOAD,TAPE_FAST,TAPE_STOP,TAPE_NOISE,TAPE_QUICK_SAVE,TAPE_MAX_BUTTONS};enum{W_TAPE_QUICK_LOAD,W_TAPE_FAST,W_TAPE_STOP,W_TAPE_NOISE,W_TAPE_QUICK_SAVE};typedef struct _optionsTapeData{struct{const char*label;Boolean sensitive;Widget widget;}buttons[TAPE_MAX_BUTTONS];}optionsTapeData;static void optionsTapeCreate(Widget parent,optionsPage*op){static optionsTapeData optionsTapeDatas={{{"Quick loading of blocks",True,(Widget)0},{"Fast mode while loading",True,(Widget)0},{"Stop playback after block",True,(Widget)0},{"Noise while loading",True,(Widget)0},{"Quick saving of blocks",True,(Widget)0},}};optionsTapeData*dp;Widget w=(Widget)0;Arg arg[6];Cardinal i;unsigned int j;op->pageData=dp=&optionsTapeDatas;for(j=TAPE_QUICK_LOAD;j<TAPE_MAX_BUTTONS;j++){i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNsensitive,dp->buttons[j].sensitive);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;if(w==(Widget)0){XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;}else{XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;if(j==TAPE_QUICK_SAVE){XtSetArg(arg[i],XmNtopOffset,10);i++;}}assert(i<=XtNumber(arg));w=XmCreateToggleButtonGadget(parent,(char*)(dp->buttons[j].label),arg,i);XtManageChild(w);if(dp->buttons[j].sensitive==True){XtAddCallback(w,XmNvalueChangedCallback,optionsTapeCallback,W_TAPE_QUICK_LOAD+j);}dp->buttons[j].widget=w;}}static void optionsTapeCallback(Widget w,XtPointer client,XtPointer call){optionsPage*op;optionsTapeData*dp;XtVaGetValues(w,XmNuserData,&op,XtNull);dp=(optionsTapeData*)(op->pageData);assert(dp!=(optionsTapeData*)0);switch((int)client){case W_TAPE_QUICK_LOAD:case W_TAPE_STOP:case W_TAPE_NOISE:case W_TAPE_QUICK_SAVE:break;case W_TAPE_FAST:{Boolean state=((XmToggleButtonCallbackStruct*)call)->set!=0?True:False;Widget button=dp->buttons[TAPE_NOISE].widget;if(state==True){XmToggleButtonGadgetSetState(button,False,False);}XtSetSensitive(button,!state);}break;}}static void optionsTapeUpdate(optionsPage*op){optionsTapeData*dp=(optionsTapeData*)(op->pageData);Boolean faststate;XmToggleButtonGadgetSetState(dp->buttons[TAPE_QUICK_LOAD].widget,CFGGET(tapeQuickload)==1?True:False,False);faststate=CFGGET(tapeFastload)==1?True:False;XmToggleButtonGadgetSetState(dp->buttons[TAPE_FAST].widget,faststate,False);XmToggleButtonGadgetSetState(dp->buttons[TAPE_NOISE].widget,CFGGET(tapeNoise)==1?True:False,False);XtSetSensitive(dp->buttons[TAPE_NOISE].widget,!faststate);XmToggleButtonGadgetSetState(dp->buttons[TAPE_QUICK_SAVE].widget,CFGGET(tapeQuicksave)==1?True:False,False);}static void optionsTapeCommit(optionsPage*op){optionsTapeData*dp=(optionsTapeData*)(op->pageData);unsigned int tapestate=TapeGetState();int savestate;if(XmToggleButtonGadgetGetState(dp->buttons[TAPE_QUICK_LOAD].widget)==True){CFGSET(tapeQuickload,1);tapestate|=TS_QUICKLOAD;}else{CFGSET(tapeQuickload,0);tapestate&=~TS_QUICKLOAD;}savestate=CFGGET(tapeFastload);if(XmToggleButtonGadgetGetState(dp->buttons[TAPE_FAST].widget)==True){CFGSET(tapeFastload,1);tapestate|=TS_FASTLOAD;}else{CFGSET(tapeFastload,0);tapestate&=~TS_FASTLOAD;}if(XmToggleButtonGadgetGetState(dp->buttons[TAPE_STOP].widget)==True){CFGSET(tapeAutostop,1);tapestate|=TS_AUTOSTOP;}else{CFGSET(tapeAutostop,0);tapestate&=~TS_AUTOSTOP;}if(XmToggleButtonGadgetGetState(dp->buttons[TAPE_NOISE].widget)==True){CFGSET(tapeNoise,1);tapestate|=TS_NOISE;}else{CFGSET(tapeNoise,0);tapestate&=~TS_NOISE;}if(XmToggleButtonGadgetGetState(dp->buttons[TAPE_QUICK_SAVE].widget)==True){CFGSET(tapeQuicksave,1);tapestate|=TS_QUICKSAVE;}else{CFGSET(tapeQuicksave,0);tapestate&=~TS_QUICKSAVE;}if(savestate!=CFGGET(tapeFastload)){if((tapestate&(TS_PLAY|TS_FASTLOAD))==(TS_PLAY|TS_FASTLOAD)){CFGSET(fastmode,CFGGET(fastmode)|2);}else{CFGSET(fastmode,CFGGET(fastmode)&~2);}SoundControl(CFGGET(fastmode)==0?SOUND_ACTION_ON:SOUND_ACTION_OFF);}TapeSetState(tapestate);}enum{SOUND_SP,SOUND_AY,SOUND_USPEECH,SOUND_MAX_SCALES};enum{SOUND_MONO,SOUND_ACB,SOUND_ABC,SOUND_MAX_RADIOS};typedef struct _optionsSoundData{struct{const char*label;Boolean sensitive;Widget widget;const char*toolhelp;}scales[SOUND_MAX_SCALES];struct{const char*label;Widget widget;}radios[SOUND_MAX_RADIOS];}optionsSoundData;static void optionsSoundCreate(Widget parent,optionsPage*op){static optionsSoundData optionsSoundDatas={{{"SP",True,(Widget)0,"Speaker volume"},{"AY",True,(Widget)0,"AY-8912 volume"},{"US",False,(Widget)0,(const char*)0}},{{"Mono",(Widget)0},{"ACB",(Widget)0},{"ABC",(Widget)0}}};optionsSoundData*dp;Widget form,radio,sep,w;Arg arg[12];Cardinal i;unsigned int j;op->pageData=dp=&optionsSoundDatas;XtVaSetValues(parent,XmNfractionBase,100,XtNull);for(j=SOUND_SP,form=(Widget)0;j<SOUND_MAX_SCALES;j++){i=0;if(form==(Widget)0){XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNleftOffset,10);i++;}else{XtSetArg(arg[i],XmNleftAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNleftWidget,form);i++;XtSetArg(arg[i],XmNleftOffset,20);i++;}XtSetArg(arg[i],XmNsensitive,dp->scales[j].sensitive);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNbottomPosition,65);i++;assert(i<=XtNumber(arg));form=XmCreateForm(parent,(char*)"form",arg,i);i=0;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabelGadget(form,(char*)dp->scales[j].label,arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNorientation,XmVERTICAL);i++;XtSetArg(arg[i],XmNprocessingDirection,XmMAX_ON_TOP);i++;XtSetArg(arg[i],XmNminimum,0);i++;XtSetArg(arg[i],XmNmaximum,100);i++;XtSetArg(arg[i],XmNscaleMultiple,10);i++;XtSetArg(arg[i],XmNshowValue,False);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateScale(form,(char*)"scale",arg,i);XtManageChild(w);dp->scales[j].widget=w;XmScaleSetTicks(w,10,1,0,4,2,0);if(dp->scales[j].sensitive==True){ToolhelpAddWidget(w,dp->scales[j].toolhelp);}XtManageChild(form);}i=0;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNleftWidget,form);i++;XtSetArg(arg[i],XmNleftOffset,25);i++;assert(i<=XtNumber(arg));sep=XmCreateSeparatorGadget(parent,(char*)"sep",arg,i);XtManageChild(sep);i=0;XtSetArg(arg[i],XmNalignment,XmALIGNMENT_BEGINNING);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNleftWidget,sep);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabelGadget(parent,(char*)"AY mode",arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNmarginWidth,0);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNleftWidget,sep);i++;XtSetArg(arg[i],XmNleftOffset,-3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;assert(i<=XtNumber(arg));radio=XmCreateRadioBox(parent,(char*)"rb",arg,i);for(j=SOUND_MONO;j<SOUND_MAX_RADIOS;j++){w=XmCreateToggleButtonGadget(radio,(char*)(dp->radios[j].label),(ArgList)0,0);XtManageChild(w);dp->radios[j].widget=w;}XmToggleButtonGadgetSetState(dp->radios[SOUND_MONO].widget,True,True);XtManageChild(radio);}static void optionsSoundCallback(Widget w,XtPointer client,XtPointer call){USED(w);USED(client);USED(call);}static void optionsSoundUpdate(optionsPage*op){optionsSoundData*dp=(optionsSoundData*)(op->pageData);Boolean sensitive=CFGGET(sound)>=0?True:False;unsigned int val;val=SoundGetSpeaker()*25;XmScaleSetValue(dp->scales[SOUND_SP].widget,(int)val);XtSetSensitive(dp->scales[SOUND_SP].widget,sensitive);val=SoundGetAY()*25;XmScaleSetValue(dp->scales[SOUND_AY].widget,(int)val);XtSetSensitive(dp->scales[SOUND_AY].widget,sensitive);XmToggleButtonGadgetSetState(dp->radios[CFGGET(soundStereo)].widget,True,True);XtSetSensitive(dp->radios[SOUND_MONO].widget,sensitive);
#ifdef EVALUATION
sensitive=False;
#endif
XtSetSensitive(dp->radios[SOUND_ACB].widget,sensitive);XtSetSensitive(dp->radios[SOUND_ABC].widget,sensitive);}static void optionsSoundCommit(optionsPage*op){optionsSoundData*dp=(optionsSoundData*)(op->pageData);unsigned int val;if(XtIsSensitive(XtParent(dp->radios[SOUND_ACB].widget))==True){int j;for(j=SOUND_MONO;j<SOUND_MAX_RADIOS;j++){if(XmToggleButtonGadgetGetState(dp->radios[j].widget)==True){break;}}assert(j<SOUND_MAX_RADIOS);if(j>AY8912_MODE_MONO&&!SoundIsStereo()){SoundControl(SOUND_ACTION_OFF);}CFGSET(soundStereo,j);}XmScaleGetValue(dp->scales[SOUND_SP].widget,(int*)&val);val/=25;SoundSetSpeaker(val);CFGSET(sound,val>0?1:0);XmScaleGetValue(dp->scales[SOUND_AY].widget,(int*)&val);val/=25;SoundSetAy(val);CFGSET(sound,val>0?1:CFGGET(sound));}enum{JOY_UP,JOY_DOWN,JOY_LEFT,JOY_RIGHT,JOY_FIRE,JOY_MAX_KEYS};enum{JOY_TOLERANCE,JOY_AUTOFIRE,JOY_JUGGLE,JOY_MAX_SPINS};enum{JOY_SINCLAIR1,JOY_SINCLAIR2,JOY_CURSOR,JOY_KEMPSTON,JOY_MAX_OPTIONS};enum{W_JOY_KEYBOARD,W_JOY_UP,W_JOY_DOWN,W_JOY_LEFT,W_JOY_RIGHT,W_JOY_FIRE,W_JOY_JOYSTICK,W_JOY_SINCLAIR1,W_JOY_SINCLAIR2,W_JOY_CURSOR,W_JOY_KEMPSTON};typedef struct _optionsJoystickData{Widget keyWidget;struct{const char*label;Widget widget;}keys[JOY_MAX_KEYS];Widget joyWidget;struct{const char*label;Widget widget;}joySpins[JOY_MAX_SPINS];Widget joyType;int joyValue;struct{const char*label;Widget widget;}joyOptions[JOY_MAX_OPTIONS];}optionsJoystickData;static void optionsJoystickCreate(Widget parent,optionsPage*op){static optionsJoystickData optionsJoystickDatas={(Widget)0,{{"Up",(Widget)0},{"Down",(Widget)0},{"Left",(Widget)0},{"Right",(Widget)0},{"Fire",(Widget)0},},(Widget)0,{{"Tolerance",(Widget)0},{"Autofire",(Widget)0},{"L-R Juggle",(Widget)0},},(Widget)0,0,{{"Sinclair 1",(Widget)0},{"Sinclair 2",(Widget)0},{"Cursor",(Widget)0},{"Kempston",(Widget)0},}};optionsJoystickData*dp;Widget form,sep,menu,cascade,tf,w;XmString label;Arg arg[16];Cardinal i;unsigned int j;op->pageData=dp=&optionsJoystickDatas;XtVaSetValues(parent,XmNfractionBase,100,XtNull);i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNset,True);i++;XtSetArg(arg[i],XmNindicatorType,XmONE_OF_MANY);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButtonGadget(parent,(char*)"Keyboard",arg,i);XtManageChild(w);XtAddCallback(w,XmNvalueChangedCallback,optionsJoystickCallback,W_JOY_KEYBOARD);dp->keyWidget=w;for(j=JOY_UP;j<JOY_MAX_KEYS;j++){i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,45);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;assert(i<=XtNumber(arg));form=XmCreateForm(parent,(char*)"form",arg,i);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabelGadget(form,(char*)dp->keys[j].label,arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNcolumns,8);i++;XtSetArg(arg[i],XmNcursorPositionVisible,False);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,30);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateTextField(form,(char*)"text",arg,i);XtManageChild(w);XtAddCallback(w,XmNfocusCallback,optionsJoystickCallback,W_JOY_UP+j);XtAddCallback(w,XmNlosingFocusCallback,optionsJoystickCallback,W_JOY_UP+j);dp->keys[j].widget=w;XtManageChild(form);}i=0;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,form);i++;assert(i<=XtNumber(arg));sep=XmCreateSeparatorGadget(parent,(char*)"sep",arg,i);XtManageChild(sep);i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNsensitive,False);i++;XtSetArg(arg[i],XmNset,False);i++;XtSetArg(arg[i],XmNindicatorType,XmONE_OF_MANY);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,50);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButtonGadget(parent,(char*)"Joystick",arg,i);XtManageChild(w);XtAddCallback(w,XmNvalueChangedCallback,optionsJoystickCallback,W_JOY_JOYSTICK);dp->joyWidget=w;for(j=JOY_TOLERANCE;j<JOY_MAX_SPINS;j++){int min=j==JOY_TOLERANCE?0:1;int max=j==JOY_TOLERANCE?50:500;i=0;
#ifndef HAVE_JOYSTICK
XtSetArg(arg[i],XmNsensitive,False);i++;
#endif
XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,50);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;assert(i<=XtNumber(arg));form=XmCreateForm(parent,(char*)"form",arg,i);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabelGadget(form,(char*)dp->joySpins[j].label,arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNarrowLayout,XmARROWS_END);i++;XtSetArg(arg[i],XmNspinBoxChildType,XmNUMERIC);i++;XtSetArg(arg[i],XmNpositionType,XmPOSITION_VALUE);i++;XtSetArg(arg[i],XmNcolumns,3);i++;XtSetArg(arg[i],XmNposition,min);i++;XtSetArg(arg[i],XmNminimumValue,min);i++;XtSetArg(arg[i],XmNmaximumValue,max);i++;XtSetArg(arg[i],XmNincrementValue,1);i++;XtSetArg(arg[i],XmNwrap,False);i++;XtSetArg(arg[i],XmNeditable,False);i++;XtSetArg(arg[i],XmNshadowThickness,0);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,45);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateSimpleSpinBox(form,(char*)"sb",arg,i);XtManageChild(w);dp->joySpins[j].widget=w;XtVaGetValues(w,XmNtextField,&tf,XtNull);assert(tf!=(Widget)0);XtVaSetValues(tf,XmNcursorPositionVisible,False,XtNull);XtManageChild(form);}menu=XmCreatePulldownMenu(parent,(char*)"pd",(ArgList)0,0);label=XmStringCreateLtoR((char*)"Type",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNsubMenuId,menu);i++;XtSetArg(arg[i],XmNlabelString,label);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,50);i++;XtSetArg(arg[i],XmNleftOffset,-3);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNbottomWidget,sep);i++;assert(i<=XtNumber(arg));cascade=XmCreateOptionMenu(parent,(char*)"om",arg,i);XmStringFree(label);for(j=JOY_SINCLAIR1;j<JOY_MAX_OPTIONS;j++){i=0;XtSetArg(arg[i],XmNuserData,op);i++;assert(i<=XtNumber(arg));w=XmCreatePushButton(menu,(char*)(dp->joyOptions[j].label),arg,i);XtManageChild(w);XtAddCallback(w,XmNactivateCallback,optionsJoystickCallback,W_JOY_SINCLAIR1+j);dp->joyOptions[j].widget=w;}XtManageChild(cascade);dp->joyType=cascade;}static void optionsJoystickCallback(Widget w,XtPointer client,XtPointer call){optionsPage*op;optionsJoystickData*dp;KeySym key;char*str;XtVaGetValues(w,XmNuserData,&op,XtNull);dp=(optionsJoystickData*)(op->pageData);assert(dp!=(optionsJoystickData*)0);switch((int)client){case W_JOY_KEYBOARD:
#ifdef HAVE_JOYSTICK
if(XtIsSensitive(dp->joyWidget)==True){XmToggleButtonGadgetSetState(dp->joyWidget,False,False);}else
#endif
{XmToggleButtonGadgetSetState(w,True,False);}break;case W_JOY_UP:case W_JOY_DOWN:case W_JOY_LEFT:case W_JOY_RIGHT:case W_JOY_FIRE:if(((XmAnyCallbackStruct*)call)->reason==XmCR_FOCUS){XtVaSetValues(w,XmNcursorPositionVisible,True,XtNull);}else{XtVaSetValues(w,XmNcursorPositionVisible,False,XtNull);XmTextFieldClearSelection(w,CurrentTime);str=XmTextFieldGetString(w);key=XStringToKeysym(str);XtFree(str);if(key==NoSymbol){(void)XBell(XtDisplayOfObject(w),5);}}break;
#ifdef HAVE_JOYSTICK
case W_JOY_JOYSTICK:XmToggleButtonGadgetSetState(dp->keyWidget,False,False);break;
#endif
case W_JOY_SINCLAIR1:case W_JOY_SINCLAIR2:case W_JOY_CURSOR:case W_JOY_KEMPSTON:dp->joyValue=(int)client-W_JOY_SINCLAIR1;break;}}static void optionsJoystickUpdate(optionsPage*op){optionsJoystickData*dp=(optionsJoystickData*)(op->pageData);
#ifdef HAVE_JOYSTICK
Boolean sensitive;
#ifndef HAVE_LIBJSW
int tolerance;
#endif
int autofire,juggle;
#endif
unsigned int j;for(j=JOY_UP;j<JOY_MAX_KEYS;j++){XmTextFieldSetString(dp->keys[j].widget,XKeysymToString(CFGGET(joyKeys[j])));}
#ifdef HAVE_JOYSTICK
sensitive=CFGGET(joy)>=0?True:False;
#ifndef HAVE_LIBJSW
tolerance=CFGGET(joyTolerance);
#endif
autofire=CFGGET(joyAutofire);juggle=CFGGET(joyJuggle);XtSetSensitive(dp->joyWidget,sensitive);
#ifdef HAVE_LIBJSW
XtSetSensitive(XtParent(dp->joySpins[JOY_TOLERANCE].widget),False);
#else
XtSetSensitive(XtParent(dp->joySpins[JOY_TOLERANCE].widget),sensitive);
#endif
XtSetSensitive(XtParent(dp->joySpins[JOY_AUTOFIRE].widget),sensitive==True&&autofire>0);XtSetSensitive(XtParent(dp->joySpins[JOY_JUGGLE].widget),sensitive==True&&juggle>0);XmToggleButtonGadgetSetState(dp->joyWidget,CFGGET(joy)>0?True:False,True);
#ifndef HAVE_LIBJSW
XtVaSetValues(dp->joySpins[JOY_TOLERANCE].widget,XmNposition,tolerance,XtNull);
#endif
XtVaSetValues(dp->joySpins[JOY_AUTOFIRE].widget,XmNposition,autofire>0?autofire:-autofire,XtNull);XtVaSetValues(dp->joySpins[JOY_JUGGLE].widget,XmNposition,juggle>0?juggle:-juggle,XtNull);
#endif
dp->joyValue=CFGGET(joyType);XtVaSetValues(dp->joyType,XmNmenuHistory,dp->joyOptions[dp->joyValue].widget,XtNull);}static void optionsJoystickCommit(optionsPage*op){optionsJoystickData*dp=(optionsJoystickData*)(op->pageData);KeySym key;char*str;unsigned int j;for(j=JOY_UP;j<JOY_MAX_KEYS;j++){str=XmTextFieldGetString(dp->keys[j].widget);key=XStringToKeysym(str);XtFree(str);if(key!=NoSymbol){CFGSET(joyKeys[j],key);}}
#ifdef HAVE_JOYSTICK
if(CFGGET(joy)>=0){CFGSET(joy,XmToggleButtonGadgetGetState(dp->joyWidget)==True?1:0);
#ifndef HAVE_LIBJSW
XtVaGetValues(dp->joySpins[JOY_TOLERANCE].widget,XmNposition,&CFGGET(joyTolerance),XtNull);
#endif
if(CFGGET(joyAutofire)>0){XtVaGetValues(dp->joySpins[JOY_AUTOFIRE].widget,XmNposition,&CFGGET(joyAutofire),XtNull);}if(CFGGET(joyJuggle)>0){XtVaGetValues(dp->joySpins[JOY_JUGGLE].widget,XmNposition,&CFGGET(joyJuggle),XtNull);}}
#endif
CFGSET(joyType,dp->joyValue);}
#ifdef XZX_IF1
enum{IF1_NONE,IF1_RS232,IF1_PRT,IF1_MAX_OPTIONS};enum{NET_NONE,NET_INTERNET,NET_MAX_OPTIONS};enum{W_IF1_NONE,W_IF1_RS232,W_IF1_PRT,W_NET_NONE,W_NET_TCPIP};typedef struct _optionsIf1Data{Widget rs232Widget;int rs232Value;struct{const char*label;Boolean sensitive;Widget widget;}rs232Options[IF1_MAX_OPTIONS];Widget netWidget;int netValue;struct{const char*label;Boolean sensitive;Widget widget;}netOptions[NET_MAX_OPTIONS];}optionsIf1Data;static void optionsIf1Create(Widget parent,optionsPage*op){static optionsIf1Data optionsIf1Datas={(Widget)0,0,{{"none",True,(Widget)0},
#ifdef XZX_IF1_RS232
{"serial port",True,(Widget)0},
#else
{"serial port",False,(Widget)0},
#endif
#ifdef XZX_IF1_PRINTER
{"printer emulation",True,(Widget)0},
#else
{"printer emulation",False,(Widget)0},
#endif
},(Widget)0,0,{{"none",True,(Widget)0},
#ifdef XZX_IF1_NETWORK
{"internet",True,(Widget)0},
#else
{"internet",False,(Widget)0},
#endif
}};optionsIf1Data*dp;Widget menu,cascade,w;XmString label;Arg arg[5];Cardinal i;int j;op->pageData=dp=&optionsIf1Datas;menu=XmCreatePulldownMenu(parent,(char*)"pd",(ArgList)0,0);label=XmStringCreateLtoR((char*)"RS232 mode",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNsubMenuId,menu);i++;XtSetArg(arg[i],XmNlabelString,label);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));cascade=XmCreateOptionMenu(parent,(char*)"om",arg,i);XmStringFree(label);for(j=IF1_NONE;j<IF1_MAX_OPTIONS;j++){i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNsensitive,dp->rs232Options[j].sensitive);i++;assert(i<=XtNumber(arg));w=XmCreatePushButton(menu,(char*)(dp->rs232Options[j].label),arg,i);XtManageChild(w);if(dp->rs232Options[j].sensitive==True){XtAddCallback(w,XmNactivateCallback,optionsIf1Callback,W_IF1_NONE+j);}dp->rs232Options[j].widget=w;}XtManageChild(cascade);dp->rs232Widget=cascade;menu=XmCreatePulldownMenu(parent,(char*)"pd",(ArgList)0,0);label=XmStringCreateLtoR((char*)"Network mode",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNsubMenuId,menu);i++;XtSetArg(arg[i],XmNlabelString,label);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,cascade);i++;assert(i<=XtNumber(arg));cascade=XmCreateOptionMenu(parent,(char*)"om",arg,i);XmStringFree(label);for(j=NET_NONE;j<NET_MAX_OPTIONS;j++){i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNsensitive,dp->netOptions[j].sensitive);i++;assert(i<=XtNumber(arg));w=XmCreatePushButton(menu,(char*)(dp->netOptions[j].label),arg,i);XtManageChild(w);if(dp->netOptions[j].sensitive==True){XtAddCallback(w,XmNactivateCallback,optionsIf1Callback,W_NET_NONE+j);}dp->netOptions[j].widget=w;}XtManageChild(cascade);dp->netWidget=cascade;}static void optionsIf1Callback(Widget w,XtPointer client,XtPointer call){optionsPage*op;optionsIf1Data*dp;USED(call);XtVaGetValues(w,XmNuserData,&op,XtNull);dp=(optionsIf1Data*)(op->pageData);assert(dp!=(optionsIf1Data*)0);switch((int)client){case W_IF1_NONE:case W_IF1_RS232:case W_IF1_PRT:dp->rs232Value=(int)client-W_IF1_NONE;break;}}static void optionsIf1Update(optionsPage*op){optionsIf1Data*dp=(optionsIf1Data*)(op->pageData);Boolean sensitive=CFGGET(if1)>=0?True:False;dp->rs232Value=CFGGET(if1Rs232Mode);XtVaSetValues(dp->rs232Widget,XmNmenuHistory,dp->rs232Options[dp->rs232Value].widget,XtNull);
#ifdef XZX_IF1_RS232
XtSetSensitive(dp->rs232Options[IF1_RS232].widget,Rs232IsActive()>=0?True:False);
#endif
XtSetSensitive(dp->rs232Widget,sensitive);}static void optionsIf1Commit(optionsPage*op){optionsIf1Data*dp=(optionsIf1Data*)(op->pageData);If1Rs232Set(dp->rs232Value);}
#endif 
#ifdef XZX_PRINTER
enum{PRT_NONE=PRINTER_MODE_NONE,PRT_FILE=PRINTER_MODE_FILE,PRT_CMD=PRINTER_MODE_CMD,PRT_MAX_OPTIONS};enum{W_PRT_NONE=PRINTER_MODE_NONE,W_PRT_FILE=PRINTER_MODE_FILE,W_PRT_CMD=PRINTER_MODE_CMD,W_PRT_FILE_BUTTON,W_PRT_CMD_FIELD};typedef struct _optionsPrinterData{Widget prtWidget;int prtValue;struct{const char*label;Widget widget;}prtOptions[PRT_MAX_OPTIONS];Widget file;const char*fileValue;Widget cmd;int cmdChanged;}optionsPrinterData;static void optionsPrinterCreate(Widget parent,optionsPage*op){static optionsPrinterData optionsPrinterDatas={(Widget)0,0,{{"none",(Widget)0},{"file",(Widget)0},{"command",(Widget)0},},(Widget)0,(const char*)0,(Widget)0,0};optionsPrinterData*dp;Widget form,menu,cascade,w;XmString label;Arg arg[11];Cardinal i;int j;op->pageData=dp=&optionsPrinterDatas;XtVaSetValues(parent,XmNfractionBase,100,XtNull);menu=XmCreatePulldownMenu(parent,(char*)"pd",(ArgList)0,0);label=XmStringCreateLtoR((char*)"Printer mode",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNsubMenuId,menu);i++;XtSetArg(arg[i],XmNlabelString,label);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));cascade=XmCreateOptionMenu(parent,(char*)"om",arg,i);XmStringFree(label);for(j=PRT_NONE;j<PRT_MAX_OPTIONS;j++){i=0;XtSetArg(arg[i],XmNuserData,op);i++;assert(i<=XtNumber(arg));w=XmCreatePushButton(menu,(char*)(dp->prtOptions[j].label),arg,i);XtManageChild(w);XtAddCallback(w,XmNactivateCallback,optionsPrinterCallback,W_PRT_NONE+j);dp->prtOptions[j].widget=w;}XtManageChild(cascade);dp->prtWidget=cascade;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,cascade);i++;XtSetArg(arg[i],XmNtopOffset,0);i++;assert(i<=XtNumber(arg));form=XmCreateForm(parent,(char*)"form",arg,i);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNleftOffset,3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabelGadget(form,(char*)"File",arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNcolumns,20);i++;XtSetArg(arg[i],XmNmaxLength,MAXPATHLEN);i++;XtSetArg(arg[i],XmNeditable,False);i++;XtSetArg(arg[i],XmNcursorPositionVisible,False);i++;XtSetArg(arg[i],XmNtraversalOn,False);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,25);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateTextField(form,(char*)"text",arg,i);XtManageChild(w);dp->file=w;i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNmarginHeight,0);i++;XtSetArg(arg[i],XmNmarginWidth,5);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNleftWidget,w);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreatePushButton(form,(char*)"...",arg,i);XtAddCallback(w,XmNactivateCallback,optionsPrinterCallback,W_PRT_FILE_BUTTON);XtManageChild(w);XtManageChild(form);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,form);i++;XtSetArg(arg[i],XmNtopOffset,3);i++;assert(i<=XtNumber(arg));form=XmCreateForm(parent,(char*)"form",arg,i);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNleftOffset,3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabelGadget(form,(char*)"Command",arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNcolumns,20);i++;XtSetArg(arg[i],XmNmaxLength,255);i++;XtSetArg(arg[i],XmNeditable,True);i++;XtSetArg(arg[i],XmNcursorPositionVisible,False);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,25);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateTextField(form,(char*)"text",arg,i);XtManageChild(w);XtAddCallback(w,XmNfocusCallback,optionsPrinterCallback,W_PRT_CMD_FIELD);XtAddCallback(w,XmNlosingFocusCallback,optionsPrinterCallback,W_PRT_CMD_FIELD);XtAddCallback(w,XmNvalueChangedCallback,optionsPrinterCallback,W_PRT_CMD_FIELD);dp->cmd=w;XtManageChild(form);}static void optionsPrinterCallback(Widget w,XtPointer client,XtPointer call){optionsPage*op;optionsPrinterData*dp;XtVaGetValues(w,XmNuserData,&op,XtNull);dp=(optionsPrinterData*)(op->pageData);assert(dp!=(optionsPrinterData*)0);switch((int)client){case W_PRT_NONE:case W_PRT_FILE:case W_PRT_CMD:dp->prtValue=(int)client-W_PRT_NONE;break;case W_PRT_FILE_BUTTON:{int types[2]={FILE_TYPE_ANY,FILE_TYPE_END};char*filename;if((filename=UiFileSelector("Printer Dump",0,types))!=(char*)0){XmTextFieldSetString(dp->file,(char*)GetBaseName(filename));if(dp->fileValue){free((void*)dp->fileValue);}dp->fileValue=filename;}}break;case W_PRT_CMD_FIELD:if(((XmAnyCallbackStruct*)call)->reason==XmCR_FOCUS){XtVaSetValues(w,XmNcursorPositionVisible,True,XtNull);}else if(((XmAnyCallbackStruct*)call)->reason==XmCR_LOSING_FOCUS){XtVaSetValues(w,XmNcursorPositionVisible,False,XtNull);XmTextFieldClearSelection(w,CurrentTime);}else{dp->cmdChanged++;}break;}}static void optionsPrinterUpdate(optionsPage*op){optionsPrinterData*dp=(optionsPrinterData*)(op->pageData);dp->prtValue=CFGGET(printerMode);XtVaSetValues(dp->prtWidget,XmNmenuHistory,dp->prtOptions[dp->prtValue].widget,XtNull);dp->fileValue=(const char*)0;XmTextFieldSetString(dp->file,(char*)GetBaseName(CFGGET(printerFile)));dp->cmdChanged=0;XmTextFieldSetString(dp->cmd,(char*)CFGGET(printerCmd));}static void optionsPrinterCommit(optionsPage*op){optionsPrinterData*dp=(optionsPrinterData*)(op->pageData);int mode;const char*file,*cmd;mode=dp->prtValue!=CFGGET(printerMode)?dp->prtValue:-1;file=dp->fileValue;cmd=dp->cmdChanged!=0?XmTextFieldGetString(dp->cmd):(const char*)0;PrinterSet(mode,file,cmd);}
#endif 
#ifdef XZX_NETWORK
enum{NETWORK_VNC_YES,NETWORK_VNC_NO,NETWORK_VNC_ASK,NETWORK_VNC_MAX};typedef struct _optionsNetworkData{Widget vncWidget;int vncValue;struct{const char*label;Boolean sensitive;Widget widget;}vncOptions[NETWORK_VNC_MAX];}optionsNetworkData;static void optionsNetworkCreate(Widget parent,optionsPage*op){static optionsNetworkData optionsNetworkDatas={(Widget)0,0,{{"yes",True,(Widget)0},{"no",True,(Widget)0},{"ask",True,(Widget)0},}};optionsNetworkData*dp;Widget menu,cascade,w;XmString label;Arg arg[4];Cardinal i;int j;op->pageData=dp=&optionsNetworkDatas;menu=XmCreatePulldownMenu(parent,(char*)"pd",(ArgList)0,0);label=XmStringCreateLtoR((char*)"Accept incomming connections",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNsubMenuId,menu);i++;XtSetArg(arg[i],XmNlabelString,label);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));cascade=XmCreateOptionMenu(parent,(char*)"om",arg,i);XmStringFree(label);for(j=NETWORK_VNC_YES;j<NETWORK_VNC_MAX;j++){i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNsensitive,dp->vncOptions[j].sensitive);i++;assert(i<=XtNumber(arg));w=XmCreatePushButton(menu,(char*)(dp->vncOptions[j].label),arg,i);XtManageChild(w);XtAddCallback(w,XmNactivateCallback,optionsNetworkCallback,j);dp->vncOptions[j].widget=w;}XtManageChild(cascade);dp->vncWidget=cascade;}static void optionsNetworkCallback(Widget w,XtPointer client,XtPointer call){}static void optionsNetworkUpdate(optionsPage*op){}static void optionsNetworkCommit(optionsPage*op){}
#endif 
#ifdef DEBUG
typedef struct _optionsDebugData{const char*label;Boolean sensitive;Widget widget;unsigned int mask;unsigned int value;}optionsDebugData;static void optionsDebugCreate(Widget parent,optionsPage*op){static optionsDebugData optionsDebugDatas[]={{"Z80 mnemonics",True,(Widget)0,D_MNEMO,0},{"Z80 registers",True,(Widget)0,D_CPU,0},{"Z80 in/out",True,(Widget)0,D_IO,0},{"Z80 interrupts",True,(Widget)0,D_INTERRUPT,0},{"Z80 illegal opcodes",True,(Widget)0,D_ILLOPS,0},
#ifdef XZX_IF1
{"Interface I emulation",True,(Widget)0,D_IF1,0},
#else
{"Interface I emulation",False,(Widget)0,D_IF1,0},
#endif
{"Disk IF emulation",True,(Widget)0,D_DISKIF,0},{"FDC emulation",True,(Widget)0,D_FDC,0},{"Sound emulation",True,(Widget)0,D_SOUND,0},
#ifdef XZX_PRINTER
{"Printer emulation",True,(Widget)0,D_PRINTER,0},
#else
{"Printer emulation",False,(Widget)0,D_PRINTER,0},
#endif
{"Joystick emulation",True,(Widget)0,D_JOYSTICK,0},
#ifdef XZX_NETWORK
{"Network emulation",True,(Widget)0,D_NETWORK,0},
#else
{"Network emulation",False,(Widget)0,D_NETWORK,0},
#endif
{"Snapshot operations",True,(Widget)0,D_SNAP,0},{"Tape operations",True,(Widget)0,D_TAPE,0},{"Disk operations",True,(Widget)0,D_DISK,0},{"X11 releated things",True,(Widget)0,D_X11,0},{(const char*)0,False,(Widget)0,0,0}};optionsDebugData*dp;Widget rowcol;Arg arg[5];Cardinal i;unsigned int idx;op->pageData=dp=&optionsDebugDatas[0];i=0;XtSetArg(arg[i],XmNnumColumns,2);i++;XtSetArg(arg[i],XmNorientation,XmVERTICAL);i++;XtSetArg(arg[i],XmNpacking,XmPACK_COLUMN);i++;XtSetArg(arg[i],XmNmarginHeight,5);i++;XtSetArg(arg[i],XmNmarginWidth,5);i++;assert(i<=XtNumber(arg));rowcol=XmCreateRowColumn(parent,(char*)"rowcol",arg,i);XtManageChild(rowcol);for(idx=0;dp->label!=(const char*)0;dp++,idx++){i=0;XtSetArg(arg[i],XmNuserData,op);i++;XtSetArg(arg[i],XmNsensitive,dp->sensitive);i++;assert(i<=XtNumber(arg));dp->widget=XmCreateToggleButtonGadget(rowcol,(char*)(dp->label),arg,i);XtManageChild(dp->widget);if(dp->sensitive==True){XtAddCallback(dp->widget,XmNvalueChangedCallback,optionsDebugCallback,idx);}}}static void optionsDebugCallback(Widget w,XtPointer client,XtPointer call){optionsPage*op;optionsDebugData*dp;XtVaGetValues(w,XmNuserData,&op,XtNull);dp=(optionsDebugData*)(op->pageData);assert(dp!=(optionsDebugData*)0);dp+=(unsigned int)client;if(((XmToggleButtonCallbackStruct*)call)->set!=0){dp->value|=dp->mask;}else{dp->value&=~(dp->mask);}}static void optionsDebugUpdate(optionsPage*op){optionsDebugData*dp=(optionsDebugData*)(op->pageData);unsigned int debug=CFGGET(debug);for(;dp->label!=(const char*)0;dp++){if(dp->sensitive==True){dp->value=debug&dp->mask;XmToggleButtonGadgetSetState(dp->widget,dp->value!=0?True:False,False);}}}static void optionsDebugCommit(optionsPage*op){optionsDebugData*dp=(optionsDebugData*)(op->pageData);unsigned int debug=0;for(;dp->label!=(const char*)0;dp++){if(dp->sensitive==True){debug|=dp->value;}}CFGSET(debug,debug);}
#endif 

