/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: screen.c,v 4.77 2006/12/22 14:29:16 erik Rel $";
#endif
#include <stdlib.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "tables.h"
#ifdef HAVE_VNCSERVER
#include "netplay/vncserver.h"
#endif
#include "ui/xwin.h"
#include "ui/screen.h"
#define BORDER_8x1	(1 * BORDER_PIXELS / sizeof(uns32) * sizeof(uns8))
#define SCREEN_8x1	(1 * X_PIXELS / sizeof(uns32) * sizeof(uns8))
#define HSIZE_8x1	(1 * BORDER_8x1 + SCREEN_8x1 + BORDER_8x1)
#define BORDER_8x2	(2 * BORDER_PIXELS / sizeof(uns32) * sizeof(uns8))
#define SCREEN_8x2	(2 * X_PIXELS / sizeof(uns32) * sizeof(uns8))
#define HSIZE_8x2	(2 * HSIZE_8x1)
#define BORDER_8x3	(3 * BORDER_PIXELS / sizeof(uns32) * sizeof(uns8))
#define SCREEN_8x3	(3 * X_PIXELS / sizeof(uns32) * sizeof(uns8))
#define HSIZE_8x3	(3 * HSIZE_8x1)
#define BORDER_16x1	(1 * BORDER_PIXELS / sizeof(uns32) * sizeof(uns16))
#define SCREEN_16x1	(1 * X_PIXELS / sizeof(uns32) * sizeof(uns16))
#define HSIZE_16x1	(1 * BORDER_16x1 + SCREEN_16x1 + BORDER_16x1)
#define BORDER_16x2	(2 * BORDER_PIXELS / sizeof(uns32) * sizeof(uns16))
#define SCREEN_16x2	(2 * X_PIXELS / sizeof(uns32) * sizeof(uns16))
#define HSIZE_16x2	(2 * HSIZE_16x1)
#define BORDER_16x3	(3 * BORDER_PIXELS / sizeof(uns32) * sizeof(uns16))
#define SCREEN_16x3	(3 * X_PIXELS / sizeof(uns32) * sizeof(uns16))
#define HSIZE_16x3	(3 * HSIZE_16x1)
#define BORDER_24x1	(1 * BORDER_PIXELS / sizeof(uns32) * 3)
#define SCREEN_24x1	(1 * X_PIXELS / sizeof(uns32) * 3)
#define HSIZE_24x1	(1 * BORDER_24x1 + SCREEN_24x1 + BORDER_24x1)
#define BORDER_24x2	(2 * BORDER_PIXELS / sizeof(uns32) * 3)
#define SCREEN_24x2	(2 * X_PIXELS / sizeof(uns32) * 3)
#define HSIZE_24x2	(2 * HSIZE_24x1)
#define BORDER_24x3	(3 * BORDER_PIXELS / sizeof(uns32) * 3)
#define SCREEN_24x3	(3 * X_PIXELS / sizeof(uns32) * 3)
#define HSIZE_24x3	(3 * HSIZE_24x1)
#define BORDER_32x1	(1 * BORDER_PIXELS / sizeof(uns32) * sizeof(uns32))
#define SCREEN_32x1	(1 * X_PIXELS / sizeof(uns32) * sizeof(uns32))
#define HSIZE_32x1	(1 * BORDER_32x1 + SCREEN_32x1 + BORDER_32x1)
#define BORDER_32x2	(2 * BORDER_PIXELS / sizeof(uns32) * sizeof(uns32))
#define SCREEN_32x2	(2 * X_PIXELS / sizeof(uns32) * sizeof(uns32))
#define HSIZE_32x2	(2 * HSIZE_32x1)
#define BORDER_32x3	(3 * BORDER_PIXELS / sizeof(uns32) * sizeof(uns32))
#define SCREEN_32x3	(3 * X_PIXELS / sizeof(uns32) * sizeof(uns32))
#define HSIZE_32x3	(3 * HSIZE_32x1)
typedef struct{uns32 l[1];}_C8x1;typedef struct{uns32 l[2];}_C8x2;typedef struct{uns32 l[3];}_C8x3;typedef struct{uns32 l[2];}_C16x1;typedef struct{uns32 l[4];}_C16x2;typedef struct{uns32 l[6];}_C16x3;typedef struct{uns32 l[3];}_C24x1;typedef struct{uns32 l[6];}_C24x2;typedef struct{uns32 l[9];}_C24x3;typedef struct{uns32 l[4];}_C32x1;typedef struct{uns32 l[8];}_C32x2;typedef struct{uns32 l[12];}_C32x3;
#ifdef WORDS_BIGENDIAN
#define WRITE24BIT(p,b1,b2,b3)	*(p)++ = (b3); *(p)++ = (b2); *(p)++ = (b1);
#else
#define WRITE24BIT(p,b1,b2,b3)	*(p)++ = (b1); *(p)++ = (b2); *(p)++ = (b3);
#endif
#define	EXPAND_BOX_X(x)	{ if ((x) < xMin) xMin = (x);	if ((x) > xMax) xMax = (x);	}
#define	EXPAND_BOX_Y(y)	{ if ((y) < yMin) yMin = (y);	if ((y) > yMax) yMax = (y);	}
#define MINIMIZE_BOX()	{ xMax = 0;	yMax = 0;	xMin = COLS - 1;	yMin = Y_PIXELS - 1;	}
#ifdef HAVE_VNCSERVER
#define UPDATE_IMAGE(x,y,w,h)	XwinDataP->putImage(XwinDataP->display, XtWindow(XwinDataP->emuWindow), XwinDataP->gc,	XwinDataP->image, (int)(x), (int)(y), (int)(x), (int)(y),	(unsigned int)(w), (unsigned int)(h), False);	VncServerUpdate((unsigned int)(x), (unsigned int)(y),	(unsigned int)(w), (unsigned int)(h));
#else
#define UPDATE_IMAGE(x,y,w,h)	XwinDataP->putImage(XwinDataP->display, XtWindow(XwinDataP->emuWindow), XwinDataP->gc,	XwinDataP->image, (int)(x), (int)(y), (int)(x), (int)(y),	(unsigned int)(w), (unsigned int)(h), False);
#endif
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_X11) != 0) { x } }
#else
#define DEB(x)
#endif
static void screenLine8x1(unsigned int);static void screenLine8x2(unsigned int);static void screenLine8x3(unsigned int);static void screenLine16x1(unsigned int);static void screenLine16x2(unsigned int);static void screenLine16x3(unsigned int);static void screenLine24x1(unsigned int);static void screenLine24x2(unsigned int);static void screenLine24x3(unsigned int);static void screenLine32x1(unsigned int);static void screenLine32x2(unsigned int);static void screenLine32x3(unsigned int);void(*ScreenUpdateLine)(unsigned int);static uns32*xImagePtr;static void*pix2color[16];static unsigned long border;static int borderChanged=0;static int borderUpdate;static uns32 screenChanged[Y_PIXELS];static int screenUpdate;static unsigned int xMin,xMax,yMin,yMax;static void(*updateLine[][3])(unsigned int)={{screenLine8x1,screenLine8x2,screenLine8x3},{screenLine16x1,screenLine16x2,screenLine16x3},{screenLine24x1,screenLine24x2,screenLine24x3},{screenLine32x1,screenLine32x2,screenLine32x3}};static int flashPhase=0;void ScreenCreateTables(void){unsigned long ink[FLASH],paper[FLASH];int i,j,k,l;ScreenUpdateLine=updateLine[XwinDataP->bytesPerPixel-1][CFGGET(scale)-1];if(CFGGET(monocrome)!=0){unsigned long fg,bg;for(i=0;i<2;i++){for(j=0;j<8;j++){for(k=0;k<8;k++){if(j>k){fg=XwinDataP->colors[BLACK];bg=XwinDataP->colors[WHITE];}else if(j<k){fg=XwinDataP->colors[WHITE];bg=XwinDataP->colors[BLACK];}else{if(j<GREEN||(j>WHITE&&j<BRIGHT_GREEN)){fg=bg=XwinDataP->colors[BLACK];}else{fg=bg=XwinDataP->colors[WHITE];}}ink[(i<<6)+(j<<3)+k]=fg;paper[(i<<6)+(j<<3)+k]=bg;}}}}else{for(i=0;i<2;i++){for(j=0;j<8;j++){for(k=0;k<8;k++){ink[(i<<6)+(j<<3)+k]=XwinDataP->colors[k+(i<<3)];paper[(i<<6)+(j<<3)+k]=XwinDataP->colors[j+(i<<3)];}}}}for(i=0;i<16;i++){pix2color[i]=Malloc((size_t)(128*4*XwinDataP->bytesPerPixel*CFGGET(scale)),"ScreenCreateTables");}switch(XwinDataP->bytesPerPixel){case 1:{uns8 c,*p;for(i=0;i<16;i++){p=(uns8*)pix2color[i];for(j=0;j<FLASH;j++){for(k=0x08;k>0;k>>=1){c=(uns8)(i&k?ink[j]:paper[j]);for(l=0;l<CFGGET(scale);l++){*p++=c;}}}}}break;case 2:{uns16 c,*p;for(i=0;i<16;i++){p=(uns16*)pix2color[i];for(j=0;j<FLASH;j++){for(k=0x08;k>0;k>>=1){c=(uns16)(i&k?ink[j]:paper[j]);for(l=0;l<CFGGET(scale);l++){*p++=c;}}}}}break;case 3:{uns32 c;uns8*p;for(i=0;i<16;i++){p=(uns8*)pix2color[i];for(j=0;j<FLASH;j++){for(k=0x08;k>0;k>>=1){c=(uns32)(i&k?ink[j]:paper[j]);for(l=0;l<CFGGET(scale);l++){WRITE24BIT(p,c&0x0000ff,(c&0x00ff00)>>8,(c&0xff0000)>>16);}}}}}break;case 4:{uns32 c,*p;for(i=0;i<16;i++){p=(uns32*)pix2color[i];for(j=0;j<FLASH;j++){for(k=0x08;k>0;k>>=1){c=(uns32)(i&k?ink[j]:paper[j]);for(l=0;l<CFGGET(scale);l++){*p++=c;}}}}}break;}borderUpdate++;borderChanged=VISIBLE_LINES;screenUpdate=0;ScreenMarkDirtyAll();}void ScreenDestroyTables(void){int i;for(i=0;i<16;i++){if(pix2color[i]!=(void*)0){free(pix2color[i]);}}}void ScreenMarkDirty(uns16 addr){if(addr<(uns16)PIXEL_LENGTH){
#if 1
screenChanged[Pix2Line[addr]]|=1U<<(addr&0x001f);
#else
screenChanged[((addr&0x1800)>>5)|((addr&0x0700)>>8)|((addr&0x00e0)>>2)]|=1U<<(addr&0x001f);
#endif
}else{uns32 changed,*lcp;int i;changed=1U<<(addr&0x001f);lcp=&screenChanged[(addr&0x03e0)>>2];for(i=8;i;i--){*lcp++|=changed;}}}void ScreenMarkDirtyAll(void){(void)memset(screenChanged,0xff,sizeof(screenChanged));}void ScreenDoFlashing(void){uns8*mem;uns32 changed,*lcp;int i,j;flashPhase=!flashPhase;mem=Machine->screen+ATTR_OFFSET;lcp=screenChanged;changed=0;for(i=24;i;i--){for(j=32;j;j--){changed>>=1;if((*mem++&FLASH)!=0){changed|=1U<<31;}}if(changed!=0){for(j=8;j;j--){*lcp++|=changed;}changed=0;}else{lcp+=8;}}}void ScreenSetBorder(uns8 color){border=XwinDataP->colors[color];borderChanged=VISIBLE_LINES;}static void screenLine8x1(unsigned int line){uns32 bc;uns32*bp;_C8x1*sp;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if((uns8)border!=*(uns8*)xImagePtr){borderUpdate++;bp=xImagePtr;bc=border|(border<<8)|(border<<16)|(border<<24);if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=HSIZE_8x1;i>0;i--){*bp++=bc;}xImagePtr+=HSIZE_8x1;return;}else{for(i=BORDER_8x1;i>0;i--){*bp++=bc;}bp+=SCREEN_8x1;for(i=BORDER_8x1;i>0;i--){*bp++=bc;}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C8x1*)(xImagePtr+BORDER_8x1);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp++=*((_C8x1*)pix2color[*pixPtr>>4]+attr);*sp++=*((_C8x1*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;}}}}xImagePtr+=HSIZE_8x1;}static void screenLine8x2(unsigned int line){uns32 bc;uns32*bp;_C8x2*sp,*sp2;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i,j;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if((uns8)border!=*(uns8*)xImagePtr){borderUpdate++;bp=xImagePtr;bc=border|(border<<8)|(border<<16)|(border<<24);if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=2*HSIZE_8x2;i>0;i--){*bp++=bc;}xImagePtr+=2*HSIZE_8x2;return;}else{for(j=2;j>0;j--){for(i=BORDER_8x2;i>0;i--){*bp++=bc;}bp+=SCREEN_8x2;for(i=BORDER_8x2;i>0;i--){*bp++=bc;}}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C8x2*)(xImagePtr+BORDER_8x2);sp2=(_C8x2*)(xImagePtr+BORDER_8x2+HSIZE_8x2);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp2++=*sp++=*((_C8x2*)pix2color[*pixPtr>>4]+attr);*sp2++=*sp++=*((_C8x2*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;sp2+=2;}}}}xImagePtr+=2*HSIZE_8x2;}static void screenLine8x3(unsigned int line){uns32 bc;uns32*bp;_C8x3*sp,*sp2,*sp3;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i,j;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if((uns8)border!=*(uns8*)xImagePtr){borderUpdate++;bp=xImagePtr;bc=border|(border<<8)|(border<<16)|(border<<24);if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=3*HSIZE_8x3;i>0;i--){*bp++=bc;}xImagePtr+=3*HSIZE_8x3;return;}else{for(j=3;j>0;j--){for(i=BORDER_8x3;i>0;i--){*bp++=bc;}bp+=SCREEN_8x3;for(i=BORDER_8x3;i>0;i--){*bp++=bc;}}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C8x3*)(xImagePtr+BORDER_8x3);sp2=(_C8x3*)(xImagePtr+BORDER_8x3+HSIZE_8x3);sp3=(_C8x3*)(xImagePtr+BORDER_8x3+2*HSIZE_8x3);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp3++=*sp2++=*sp++=*((_C8x3*)pix2color[*pixPtr>>4]+attr);*sp3++=*sp2++=*sp++=*((_C8x3*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;sp2+=2;sp3+=2;}}}}xImagePtr+=3*HSIZE_8x3;}static void screenLine16x1(unsigned int line){uns32 bc;uns32*bp;_C16x1*sp;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if((uns16)border!=*(uns16*)xImagePtr){borderUpdate++;bp=xImagePtr;bc=border|(border<<16);if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=HSIZE_16x1;i>0;i--){*bp++=bc;}xImagePtr+=HSIZE_16x1;return;}else{for(i=BORDER_16x1;i>0;i--){*bp++=bc;}bp+=SCREEN_16x1;for(i=BORDER_16x1;i>0;i--){*bp++=bc;}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C16x1*)(xImagePtr+BORDER_16x1);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp++=*((_C16x1*)pix2color[*pixPtr>>4]+attr);*sp++=*((_C16x1*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;}}}}xImagePtr+=HSIZE_16x1;}static void screenLine16x2(unsigned int line){uns32 bc;uns32*bp;_C16x2*sp,*sp2;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i,j;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if((uns16)border!=*(uns16*)xImagePtr){borderUpdate++;bp=xImagePtr;bc=border|(border<<16);if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=2*HSIZE_16x2;i>0;i--){*bp++=bc;}xImagePtr+=2*HSIZE_16x2;return;}else{for(j=2;j>0;j--){for(i=BORDER_16x2;i>0;i--){*bp++=bc;}bp+=SCREEN_16x2;for(i=BORDER_16x2;i>0;i--){*bp++=bc;}}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C16x2*)(xImagePtr+BORDER_16x2);sp2=(_C16x2*)(xImagePtr+BORDER_16x2+HSIZE_16x2);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp2++=*sp++=*((_C16x2*)pix2color[*pixPtr>>4]+attr);*sp2++=*sp++=*((_C16x2*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;sp2+=2;}}}}xImagePtr+=2*HSIZE_16x2;}static void screenLine16x3(unsigned int line){uns32 bc;uns32*bp;_C16x3*sp,*sp2,*sp3;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i,j;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if((uns16)border!=*(uns16*)xImagePtr){borderUpdate++;bp=xImagePtr;bc=border|(border<<16);if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=3*HSIZE_16x3;i>0;i--){*bp++=bc;}xImagePtr+=3*HSIZE_16x3;return;}else{for(j=3;j>0;j--){for(i=BORDER_16x3;i>0;i--){*bp++=bc;}bp+=SCREEN_16x3;for(i=BORDER_16x3;i>0;i--){*bp++=bc;}}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C16x3*)(xImagePtr+BORDER_16x3);sp2=(_C16x3*)(xImagePtr+BORDER_16x3+HSIZE_16x3);sp3=(_C16x3*)(xImagePtr+BORDER_16x3+2*HSIZE_16x3);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp3++=*sp2++=*sp++=*((_C16x3*)pix2color[*pixPtr>>4]+attr);*sp3++=*sp2++=*sp++=*((_C16x3*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;sp2+=2;sp3+=2;}}}}xImagePtr+=3*HSIZE_16x3;}static void screenLine24x1(unsigned int line){uns8 b1,b2,b3,*bp;_C24x1*sp;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if(border!=(*xImagePtr&0x00ffffff)){borderUpdate++;bp=(uns8*)xImagePtr;b1=border&0x0000ff;b2=(border&0x00ff00)>>8;b3=(border&0xff0000)>>16;if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=HSIZE_24x1;i>0;i--){WRITE24BIT(bp,b1,b2,b3);}xImagePtr+=HSIZE_24x1;return;}else{for(i=BORDER_24x1;i>0;i--){WRITE24BIT(bp,b1,b2,b3);}bp+=SCREEN_24x1;for(i=BORDER_24x1;i>0;i--){WRITE24BIT(bp,b1,b2,b3);}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C24x1*)(xImagePtr+BORDER_24x1);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp++=*((_C24x1*)pix2color[*pixPtr>>4]+attr);*sp++=*((_C24x1*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;}}}}xImagePtr+=HSIZE_24x1;}static void screenLine24x2(unsigned int line){uns8 b1,b2,b3,*bp;_C24x2*sp,*sp2;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i,j;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if(border!=(*xImagePtr&0x00ffffff)){borderUpdate++;bp=(uns8*)xImagePtr;b1=border&0x0000ff;b2=(border&0x00ff00)>>8;b3=(border&0xff0000)>>16;if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=2*HSIZE_24x2;i>0;i--){WRITE24BIT(bp,b1,b2,b3);}xImagePtr+=2*HSIZE_24x2;return;}else{for(j=2;j>0;j--){for(i=BORDER_24x2;i>0;i--){WRITE24BIT(bp,b1,b2,b3);}bp+=SCREEN_24x2;for(i=BORDER_24x2;i>0;i--){WRITE24BIT(bp,b1,b2,b3);}}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C24x2*)(xImagePtr+BORDER_24x2);sp2=(_C24x2*)(xImagePtr+BORDER_24x2+HSIZE_24x2);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp2++=*sp++=*((_C24x2*)pix2color[*pixPtr>>4]+attr);*sp2++=*sp++=*((_C24x2*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;sp2+=2;}}}}xImagePtr+=2*HSIZE_24x2;}static void screenLine24x3(unsigned int line){uns8 b1,b2,b3,*bp;_C24x3*sp,*sp2,*sp3;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i,j;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if(border!=(*xImagePtr&0x00ffffff)){borderUpdate++;b1=border&0x0000ff;b2=(border&0x00ff00)>>8;b3=(border&0xff0000)>>16;bp=(uns8*)xImagePtr;if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=3*HSIZE_24x3;i>0;i--){WRITE24BIT(bp,b1,b2,b3);}xImagePtr+=3*HSIZE_24x3;return;}else{for(j=3;j>0;j--){for(i=BORDER_24x3;i>0;i--){WRITE24BIT(bp,b1,b2,b3);}bp+=SCREEN_24x3;for(i=BORDER_24x3;i>0;i--){WRITE24BIT(bp,b1,b2,b3);}}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C24x3*)(xImagePtr+BORDER_24x3);sp2=(_C24x3*)(xImagePtr+BORDER_24x3+HSIZE_24x3);sp3=(_C24x3*)(xImagePtr+BORDER_24x3+2*HSIZE_24x3);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp3++=*sp2++=*sp++=*((_C24x3*)pix2color[*pixPtr>>4]+attr);*sp3++=*sp2++=*sp++=*((_C24x3*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;sp2+=2;sp3+=2;}}}}xImagePtr+=3*HSIZE_24x3;}static void screenLine32x1(unsigned int line){uns32*bp;_C32x1*sp;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if(border!=*xImagePtr){borderUpdate++;bp=xImagePtr;if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=HSIZE_32x1;i>0;i--){*bp++=(uns32)border;}xImagePtr+=HSIZE_32x1;return;}else{for(i=BORDER_32x1;i>0;i--){*bp++=(uns32)border;}bp+=SCREEN_32x1;for(i=BORDER_32x1;i>0;i--){*bp++=(uns32)border;}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C32x1*)(xImagePtr+BORDER_32x1);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp++=*((_C32x1*)pix2color[*pixPtr>>4]+attr);*sp++=*((_C32x1*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;}}}}xImagePtr+=HSIZE_32x1;}static void screenLine32x2(unsigned int line){uns32*bp;_C32x2*sp,*sp2;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i,j;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if(border!=*xImagePtr){borderUpdate++;bp=xImagePtr;if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=2*HSIZE_32x2;i>0;i--){*bp++=(uns32)border;}xImagePtr+=2*HSIZE_32x2;return;}else{for(j=2;j>0;j--){for(i=BORDER_32x2;i>0;i--){*bp++=(uns32)border;}bp+=SCREEN_32x2;for(i=BORDER_32x2;i>0;i--){*bp++=(uns32)border;}}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C32x2*)(xImagePtr+BORDER_32x2);sp2=(_C32x2*)(xImagePtr+BORDER_32x2+HSIZE_32x2);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp2++=*sp++=*((_C32x2*)pix2color[*pixPtr>>4]+attr);*sp2++=*sp++=*((_C32x2*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;sp2+=2;}}}}xImagePtr+=2*HSIZE_32x2;}static void screenLine32x3(unsigned int line){uns32*bp;_C32x3*sp,*sp2,*sp3;uns8*pixPtr,*attrPtr,attr;uns32 changed;unsigned short i,j;if(line==0){xImagePtr=(uns32*)(XwinDataP->image->data);}if(borderChanged!=0){borderChanged--;if(border!=*xImagePtr){borderUpdate++;bp=xImagePtr;if(line<BORDER_LINES||line>=BORDER_LINES+Y_PIXELS){for(i=3*HSIZE_32x3;i>0;i--){*bp++=(uns32)border;}xImagePtr+=3*HSIZE_32x3;return;}else{for(j=3;j>0;j--){for(i=BORDER_32x3;i>0;i--){*bp++=(uns32)border;}bp+=SCREEN_32x3;for(i=BORDER_32x3;i>0;i--){*bp++=(uns32)border;}}goto drawScreenLine;}}}if(line>=BORDER_LINES&&line<BORDER_LINES+Y_PIXELS){drawScreenLine:if((changed=screenChanged[line-=BORDER_LINES])!=0){screenChanged[line]=0;EXPAND_BOX_Y(line);screenUpdate++;sp=(_C32x3*)(xImagePtr+BORDER_32x3);sp2=(_C32x3*)(xImagePtr+BORDER_32x3+HSIZE_32x3);sp3=(_C32x3*)(xImagePtr+BORDER_32x3+2*HSIZE_32x3);pixPtr=Machine->screen+Line2Pix[line];attrPtr=Machine->screen+Line2Attr[line];for(i=0;i<COLS;i++,pixPtr++,attrPtr++,changed>>=1){if((changed&0x1)!=0){EXPAND_BOX_X(i);attr=((*attrPtr&FLASH)&&flashPhase)?Reverse[*attrPtr&~FLASH]:*attrPtr&~FLASH;*sp3++=*sp2++=*sp++=*((_C32x3*)pix2color[*pixPtr>>4]+attr);*sp3++=*sp2++=*sp++=*((_C32x3*)pix2color[*pixPtr&0x0f]+attr);}else{sp+=2;sp2+=2;sp3+=2;}}}}xImagePtr+=3*HSIZE_32x3;}void ScreenRefresh(void){int borderWidth,borderHeight,middleHeight;int flush=0;if(CFGGET(cache)!=0){if(borderUpdate!=0){borderUpdate=0;borderWidth=BORDER_PIXELS*CFGGET(scale);borderHeight=BORDER_LINES*CFGGET(scale);middleHeight=Y_PIXELS*CFGGET(scale);UPDATE_IMAGE(0,0,XwinDataP->hsize,borderHeight);UPDATE_IMAGE(0,borderHeight,borderWidth,middleHeight);UPDATE_IMAGE((BORDER_PIXELS+X_PIXELS)*CFGGET(scale),borderHeight,borderWidth,middleHeight);UPDATE_IMAGE(0,(BORDER_LINES+Y_PIXELS)*CFGGET(scale),XwinDataP->hsize,borderHeight);flush++;}if(screenUpdate!=0){screenUpdate=0;UPDATE_IMAGE((BORDER_PIXELS+xMin*8)*CFGGET(scale),(BORDER_LINES+yMin)*CFGGET(scale),(xMax-xMin+1)*8*CFGGET(scale),(yMax-yMin+1)*CFGGET(scale));MINIMIZE_BOX();flush++;}}else{if((borderUpdate||screenUpdate)!=0){borderUpdate=screenUpdate=0;UPDATE_IMAGE(0,0,XwinDataP->hsize,XwinDataP->vsize);flush++;}}if(flush!=0){if(CFGGET(xsync)!=0){XSync(XwinDataP->display,False);}else{(void)XFlush(XwinDataP->display);}}}
