/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: xkey.c,v 4.17 2006/12/22 14:29:16 erik Rel $";
#endif
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "joystick.h"
#include "keyboard.h"
#include "ui/xkey.h"
#define ROW1	0
#define ROW2	1
#define ROW3	2
#define ROW4	3
#define ROW5	4
#define ROW6	5
#define ROW7	6
#define ROW8	7
#define NONE	8
#define COL1	1
#define COL2	2
#define COL3	4
#define COL4	8
#define COL5	16
enum KeyState{PRESSED,RELEASED,DELAYED};
#define DELAY_FRAMES	5
typedef struct _key{enum KeyState state;unsigned int frame;uns8 row;uns8 col;}key;
#define CAPS_SHIFT	96
#define SYMBOL_SHIFT	97
#define RETURN	98
#define VALID_KEY(key)	((key) >= 0x20 && (key) < 0x80)
#define PRESS(k)	{ key *kp = &keys[(k)];	kp->state = PRESSED;	kp->frame = DELAY_FRAMES;	KeyboardPorts[kp->row] &= ~kp->col;	}
#define RELEASE(k)	{ key *kp = &keys[(k)];	if (kp->state == PRESSED)	{	if (kp->frame == 0)	{	kp->state = RELEASED;	KeyboardPorts[kp->row] |= kp->col;	}	else	{	kp->state = DELAYED;	}	}	}
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_X11) != 0) { x } }
#else
#define DEB(x)
#endif
static key keys[128-32+3]={{RELEASED,0,ROW8,COL1},{RELEASED,0,ROW4,COL1},{RELEASED,0,ROW6,COL1},{RELEASED,0,ROW4,COL3},{RELEASED,0,ROW4,COL4},{RELEASED,0,ROW4,COL5},{RELEASED,0,ROW5,COL5},{RELEASED,0,ROW5,COL4},{RELEASED,0,ROW5,COL3},{RELEASED,0,ROW5,COL2},{RELEASED,0,ROW8,COL5},{RELEASED,0,ROW7,COL3},{RELEASED,0,ROW8,COL4},{RELEASED,0,ROW7,COL4},{RELEASED,0,ROW8,COL3},{RELEASED,0,ROW1,COL5},{RELEASED,0,ROW5,COL1},{RELEASED,0,ROW4,COL1},{RELEASED,0,ROW4,COL2},{RELEASED,0,ROW4,COL3},{RELEASED,0,ROW4,COL4},{RELEASED,0,ROW4,COL5},{RELEASED,0,ROW5,COL5},{RELEASED,0,ROW5,COL4},{RELEASED,0,ROW5,COL3},{RELEASED,0,ROW5,COL2},{RELEASED,0,ROW1,COL2},{RELEASED,0,ROW6,COL2},{RELEASED,0,ROW3,COL4},{RELEASED,0,ROW7,COL2},{RELEASED,0,ROW3,COL5},{RELEASED,0,ROW1,COL4},{RELEASED,0,ROW4,COL2},{RELEASED,0,ROW2,COL1},{RELEASED,0,ROW8,COL5},{RELEASED,0,ROW1,COL4},{RELEASED,0,ROW2,COL3},{RELEASED,0,ROW3,COL3},{RELEASED,0,ROW2,COL4},{RELEASED,0,ROW2,COL5},{RELEASED,0,ROW7,COL5},{RELEASED,0,ROW6,COL3},{RELEASED,0,ROW7,COL4},{RELEASED,0,ROW7,COL3},{RELEASED,0,ROW7,COL2},{RELEASED,0,ROW8,COL3},{RELEASED,0,ROW8,COL4},{RELEASED,0,ROW6,COL2},{RELEASED,0,ROW6,COL1},{RELEASED,0,ROW3,COL1},{RELEASED,0,ROW3,COL4},{RELEASED,0,ROW2,COL2},{RELEASED,0,ROW3,COL5},{RELEASED,0,ROW6,COL4},{RELEASED,0,ROW1,COL5},{RELEASED,0,ROW3,COL2},{RELEASED,0,ROW1,COL1},{RELEASED,0,ROW6,COL1},{RELEASED,0,ROW1,COL2},{RELEASED,0,NONE,NONE},{RELEASED,0,NONE,NONE},{RELEASED,0,NONE,NONE},{RELEASED,0,ROW7,COL5},{RELEASED,0,ROW5,COL1},{RELEASED,0,ROW1,COL3},{RELEASED,0,ROW2,COL1},{RELEASED,0,ROW8,COL5},{RELEASED,0,ROW1,COL4},{RELEASED,0,ROW2,COL3},{RELEASED,0,ROW3,COL3},{RELEASED,0,ROW2,COL4},{RELEASED,0,ROW2,COL5},{RELEASED,0,ROW7,COL5},{RELEASED,0,ROW6,COL3},{RELEASED,0,ROW7,COL4},{RELEASED,0,ROW7,COL3},{RELEASED,0,ROW7,COL2},{RELEASED,0,ROW8,COL3},{RELEASED,0,ROW8,COL4},{RELEASED,0,ROW6,COL2},{RELEASED,0,ROW6,COL1},{RELEASED,0,ROW3,COL1},{RELEASED,0,ROW3,COL4},{RELEASED,0,ROW2,COL2},{RELEASED,0,ROW3,COL5},{RELEASED,0,ROW6,COL4},{RELEASED,0,ROW1,COL5},{RELEASED,0,ROW3,COL2},{RELEASED,0,ROW1,COL3},{RELEASED,0,ROW6,COL5},{RELEASED,0,ROW1,COL2},{RELEASED,0,NONE,NONE},{RELEASED,0,NONE,NONE},{RELEASED,0,NONE,NONE},{RELEASED,0,NONE,NONE},{RELEASED,0,NONE,NONE},{RELEASED,0,ROW1,COL1},{RELEASED,0,ROW8,COL2},{RELEASED,0,ROW7,COL1}};static int unshiftCursor,emulatedShift,trueShift;void XkeyReset(void){key*kp;for(kp=keys;kp<&keys[NELEM(keys)];kp++){kp->state=RELEASED;}unshiftCursor=0;emulatedShift=0;trueShift=0;KeyboardReset();}void XkeyRefresh(void){key*kp;for(kp=keys;kp<&keys[NELEM(keys)];kp++){if(kp->frame!=0){if(--kp->frame==0&&kp->state==DELAYED){kp->state=RELEASED;KeyboardPorts[kp->row]|=kp->col;}}}}void XkeyPress(Widget w,XtPointer client,XEvent*xev,Boolean*cont){KeySym ks,ks0;USED(w);USED(client);USED(cont);(void)XLookupString((XKeyEvent*)xev,(char*)0,0,&ks,(XComposeStatus*)0);ks0=XLookupKeysym((XKeyEvent*)xev,0);DEB(Msg(M_DEBUG,"ks = %04x (%s), ks0 = %04x (%s)",(unsigned int)ks,XKeysymToString(ks),(unsigned int)ks0,XKeysymToString(ks0)););
#ifdef HAVE_JOYSTICK
if(CFGGET(joy)<=0)
#endif
{if(ks0==CFGGET(joyUp)){JOY_PRESS(up);}else if(ks0==CFGGET(joyDown)){JOY_PRESS(down);}else if(ks0==CFGGET(joyLeft)){JOY_PRESS(left);}else if(ks0==CFGGET(joyRight)){JOY_PRESS(right);}else if(ks0==CFGGET(joyFire)){JOY_PRESS(fire);}}switch((int)ks){case XK_Tab:unshiftCursor=!unshiftCursor;break;case XK_backslash:case XK_bar:XkeyReset();break;case XK_Return:PRESS(RETURN);break;case XK_Control_L:PRESS(CAPS_SHIFT);PRESS(SYMBOL_SHIFT);emulatedShift++;break;case XK_Shift_L:PRESS(CAPS_SHIFT);emulatedShift++;break;case XK_Alt_L:case XK_Meta_L:PRESS(SYMBOL_SHIFT);emulatedShift++;break;case XK_BackSpace:case XK_Delete:PRESS(CAPS_SHIFT);PRESS('0'-0x20);break;case XK_Escape:PRESS(CAPS_SHIFT);PRESS('1'-0x20);break;case XK_Up:if(unshiftCursor==0){PRESS(CAPS_SHIFT);}PRESS('7'-0x20);break;case XK_Down:if(unshiftCursor==0){PRESS(CAPS_SHIFT);}PRESS('6'-0x20);break;case XK_Left:if(unshiftCursor==0){PRESS(CAPS_SHIFT);}PRESS('5'-0x20);break;case XK_Right:if(unshiftCursor==0){PRESS(CAPS_SHIFT);}PRESS('8'-0x20);break;case XK_sterling:if(emulatedShift==0){PRESS(SYMBOL_SHIFT);PRESS('`'-0x20);break;}default:if(emulatedShift!=0){if(VALID_KEY(ks0)){PRESS(ks0-0x20);}}else if(VALID_KEY(ks)){if(ks>='A'&&ks<='Z'){PRESS(CAPS_SHIFT);}else if(!((ks>='a'&&ks<='z')||(ks>='0'&&ks<='9')||ks==' ')){PRESS(SYMBOL_SHIFT);}PRESS(ks-0x20);}break;}}void XkeyRelease(Widget w,XtPointer client,XEvent*xev,Boolean*cont){KeySym ks,ks0;USED(w);USED(client);USED(cont);(void)XLookupString((XKeyEvent*)xev,(char*)0,0,&ks,(XComposeStatus*)0);ks0=XLookupKeysym((XKeyEvent*)xev,0);DEB(Msg(M_DEBUG,"ks = %04x (%s), ks0 = %04x (%s)",(unsigned int)ks,XKeysymToString(ks),(unsigned int)ks0,XKeysymToString(ks0)););
#ifdef HAVE_JOYSTICK
if(CFGGET(joy)<=0)
#endif
{if(ks0==CFGGET(joyUp)){JOY_RELEASE(up);}else if(ks0==CFGGET(joyDown)){JOY_RELEASE(down);}else if(ks0==CFGGET(joyLeft)){JOY_RELEASE(left);}else if(ks0==CFGGET(joyRight)){JOY_RELEASE(right);}else if(ks0==CFGGET(joyFire)){JOY_RELEASE(fire);}}switch((int)ks){case XK_Control_R:case XK_Shift_R:case XK_Alt_R:case XK_Meta_R:case XK_Mode_switch:trueShift=xev->xkey.state==1?1:2;break;case XK_Return:RELEASE(RETURN);break;case XK_Control_L:RELEASE(CAPS_SHIFT);RELEASE(SYMBOL_SHIFT);if(emulatedShift!=0){emulatedShift--;}break;case XK_Shift_L:RELEASE(CAPS_SHIFT);if(emulatedShift!=0){emulatedShift--;}break;case XK_Alt_L:case XK_Meta_L:RELEASE(SYMBOL_SHIFT);if(emulatedShift!=0){emulatedShift--;}break;case XK_BackSpace:case XK_Delete:RELEASE(CAPS_SHIFT);RELEASE('0'-0x20);break;case XK_Escape:RELEASE(CAPS_SHIFT);RELEASE('1'-0x20);break;case XK_Up:if(unshiftCursor==0){RELEASE(CAPS_SHIFT);}RELEASE('7'-0x20);break;case XK_Down:if(unshiftCursor==0){RELEASE(CAPS_SHIFT);}RELEASE('6'-0x20);break;case XK_Left:if(unshiftCursor==0){RELEASE(CAPS_SHIFT);}RELEASE('5'-0x20);break;case XK_Right:if(unshiftCursor==0){RELEASE(CAPS_SHIFT);}RELEASE('8'-0x20);break;case XK_sterling:if(emulatedShift==0){RELEASE(SYMBOL_SHIFT);RELEASE('`'-0x20);break;}default:if(emulatedShift!=0){if(VALID_KEY(ks0)){RELEASE(ks0-0x20);}}else if(VALID_KEY(ks)){if(keys[ks-0x20].state!=PRESSED){ks=XLookupKeysym((XKeyEvent*)xev,trueShift);if(!VALID_KEY(ks)||keys[ks-0x20].state!=PRESSED){DEB(Msg(M_DEBUG,"ks = %04x -not fixed-",(unsigned int)ks););break;}DEB(Msg(M_DEBUG,"ks = %04x (%s) -fixed-",(unsigned int)ks,XKeysymToString(ks)););}if(ks>='A'&&ks<='Z'){RELEASE(CAPS_SHIFT);}else if(!((ks>='a'&&ks<='z')||(ks>='0'&&ks<='9')||ks==' ')){RELEASE(SYMBOL_SHIFT);}RELEASE(ks-0x20);}break;}}
