/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef XZX_XMOUSE
#ifndef lint
static char rcsid[]="$Id: xmouse.c,v 4.2 2006/12/22 14:29:16 erik Rel $";
#endif
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <stdio.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "ui/ui.h"
#include "ui/xwin.h"
#include "ui/xmouse.h"
#define MAX_FUNCS	2
#define ZONE_OFFSET	(signed int)BORDER_PIXELS
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_X11) != 0) { x } }
#else
#define DEB(x)
#endif
static void xmouseButton(Widget,XtPointer,XEvent*,Boolean*);static void xmouseMotion(Widget,XtPointer,XEvent*,Boolean*);static void xmouseEnter(Widget,XtPointer,XEvent*,Boolean*);static void xmouseSendEvent(XmouseEvent*);static XmouseFunc funcTable[MAX_FUNCS];static int funcCount;static int x,y,grab;XmouseId XmouseAdd(XmouseFunc func,const char*name){int i;for(i=0;i<NELEM(funcTable);i++){if(funcTable[i]==(XmouseFunc)0){funcTable[i]=func;funcCount++;break;}}if(i==NELEM(funcTable)){Msg(M_FATAL,"function table overflow");}Msg(M_DEBUG,"XmouseAdd: %s -> %d",name,i);return(XmouseId)i;}void XmouseRemove(XmouseId id){int i=(int)id;assert(i>=0&&i<NELEM(funcTable)&&funcTable[i]!=(XmouseFunc)0);Msg(M_DEBUG,"XmouseRemove: %d",id);funcTable[i]=(XmouseFunc)0;funcCount--;}void XmouseOn(void){Msg(M_DEBUG,"XmouseOn: funcCount %d",funcCount);if(funcCount>0){Widget emuWindow=XwinDataP->emuWindow;XtAddEventHandler(emuWindow,PointerMotionMask,False,xmouseMotion,XtNull);XtAddEventHandler(emuWindow,ButtonPressMask|ButtonReleaseMask,False,xmouseButton,XtNull);XtAddEventHandler(emuWindow,EnterWindowMask,False,xmouseEnter,XtNull);if(grab!=0){XSync(XtDisplay(emuWindow),False);}}}void XmouseOff(void){Msg(M_DEBUG,"XmouseOff: funcCount %d",funcCount);if(funcCount>0){Widget emuWindow=XwinDataP->emuWindow;XtRemoveEventHandler(emuWindow,PointerMotionMask,0,xmouseMotion,XtNull);XtRemoveEventHandler(emuWindow,ButtonPressMask|ButtonReleaseMask,0,xmouseButton,XtNull);XtRemoveEventHandler(emuWindow,EnterWindowMask,0,xmouseEnter,XtNull);}}int XmouseGrab(void){Msg(M_DEBUG,"XmouseGrab");grab=1;return-1;}void XmouseUngrab(void){Msg(M_DEBUG,"XmouseUngrab");grab=0;}static void xmouseButton(Widget w,XtPointer client,XEvent*xev,Boolean*cont){XmouseButtonEvent ev;USED(w);USED(client);USED(cont);ev.type=XMOUSE_BUTTON;switch(xev->xbutton.button){case Button1:ev.button=XMOUSE_BUTTON_L;break;case Button2:ev.button=XMOUSE_BUTTON_M;break;case Button3:ev.button=XMOUSE_BUTTON_R;break;default:ev.button=0;break;}ev.state=xev->type==ButtonPress?XMOUSE_BUTTON_PRESS:XMOUSE_BUTTON_RELEASE;DEB(Msg(M_DEBUG,"XMOUSE: %s %x",ev.state==XMOUSE_BUTTON_PRESS?"press":"release",ev.button););xmouseSendEvent((XmouseEvent*)&ev);}static void xmouseMotion(Widget w,XtPointer client,XEvent*xev,Boolean*cont){XmouseMotionEvent ev;USED(w);USED(client);USED(cont);DEB(Msg(M_DEBUG,"XMOUSE: x %d, y %d",xev->xmotion.x,xev->xmotion.y););ev.type=XMOUSE_MOTION;if(x!=xev->xmotion.x||y!=xev->xmotion.y){if(grab!=0&&(x<ZONE_OFFSET||y<ZONE_OFFSET||x>((int)(XwinDataP->hsize)-ZONE_OFFSET)||y>((int)(XwinDataP->vsize)-ZONE_OFFSET))){x=XwinDataP->hsize>>1;y=XwinDataP->vsize>>1;(void)XWarpPointer(XtDisplay(w),XtWindow(w),XtWindow(w),0,0,0U,0U,x,y);}else{x=xev->xmotion.x;y=xev->xmotion.y;}}ev.x=x;ev.y=y;xmouseSendEvent((XmouseEvent*)&ev);}static void xmouseEnter(Widget w,XtPointer client,XEvent*xev,Boolean*cont){USED(client);USED(xev);USED(cont);if(grab!=0){(void)XtGrabPointer(w,True,PointerMotionMask,GrabModeAsync,GrabModeAsync,XtWindow(w),(Cursor)0,CurrentTime);(void)XSync(XtDisplay(w),False);}}static void xmouseSendEvent(XmouseEvent*ev){int i;for(i=0;i<NELEM(funcTable);i++){if(funcTable[i]!=(XmouseFunc)0){funcTable[i](ev);}}}
#endif 

