/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: xwin.c,v 4.22 2006/12/22 14:29:16 erik Rel $";
#endif
#include <Xm/Xm.h>
#include <Xm/RepType.h>
#include <Xm/MainW.h>
#include <Xm/DrawingA.h>
#ifdef HAVE_MITSHM
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#endif
#include <stdlib.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "main.h"
#include "ui/menu.h"
#include "keyboard.h"
#ifdef XZX_KMOUSE
#include "machine/kmouse.h"
#endif
#include "ui/screen.h"
#include "ui/sbar.h"
#ifdef HAVE_VNCSERVER
#include "netplay/vncserver.h"
#endif
#include "ui/xkey.h"
#include "ui/xwin.h"
#include "ui/icon.xbm"
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_X11) != 0) { x } }
#else
#define DEB(x)
#endif
static void xwinQuit(void);static void xwinAllocColors(void);static void xwinFreeColors(void);static void xwinCreateImage(void);static void xwinDestroyImage(void);
#ifdef HAVE_MITSHM
static int xwinErrorHandler(Display*,XErrorEvent*);
#endif
static void xwinEnter(Widget,XtPointer,XEvent*,Boolean*);static void xwinLeave(Widget,XtPointer,XEvent*,Boolean*);static void xwinExpose(Widget,XtPointer,XEvent*,Boolean*);static void xwinConfigure(Widget,XtPointer,XEvent*,Boolean*);XwinData*XwinDataP;static XtAppContext appContext;static Screen*screen;static Window root;static Visual*visual;static unsigned int depth;static Colormap cmap;static struct{Dimension x,y;}sizes[3];
#ifdef HAVE_MITSHM
static Bool mitshm;static XShmSegmentInfo xshminfo;static int attachError;static int(*oldErrorHandler)(Display*,XErrorEvent*);
#endif
void XwinInit(int*argcp,const char**argvp){Widget topWin,menuBar,statusBar;Pixmap iconPixmap;unsigned long white,black;Dimension width,height,incWidth,incHeight;XwinData*xdp;Arg arg[4];int i;xdp=XwinDataP=Malloc(sizeof(XwinData),"XwinInit");XtSetLanguageProc((XtAppContext)0,(XtLanguageProc)0,XtNull);XtToolkitInitialize();if((appContext=XtCreateApplicationContext())==(XtAppContext)0||(xdp->display=XtOpenDisplay(appContext,(String)0,Version,"Xzx",ResourceOptions,ResourceOptionsCount,argcp,(String*)argvp))==(Display*)0||(xdp->topLevel=XtVaAppCreateShell((String)0,"Xzx",applicationShellWidgetClass,xdp->display,(ArgList)0))==(Widget)0){Msg(M_FATAL,"couldn't connect to X display");}atexit(xwinQuit);ResourceRead(argcp,argvp);XmRepTypeInstallTearOffModelConverter();screen=XtScreen(xdp->topLevel);root=RootWindowOfScreen(screen);white=WhitePixelOfScreen(screen);black=BlackPixelOfScreen(screen);for(i=CFGGET(scale);i>1;i--){if(DisplayWidth(xdp->display,XScreenNumberOfScreen(screen))>i*(int)(BORDER_PIXELS+X_PIXELS+BORDER_PIXELS)&&DisplayHeight(xdp->display,XScreenNumberOfScreen(screen))>i*(int)(BORDER_LINES+Y_PIXELS+BORDER_LINES)){break;}}CFGSET(scale,i);xdp->hsize=(unsigned int)i*(BORDER_PIXELS+X_PIXELS+BORDER_PIXELS);xdp->vsize=(unsigned int)i*(BORDER_LINES+Y_PIXELS+BORDER_LINES);{
#ifdef BROKEN_VISUAL_SUPPORT
visual=DefaultVisualOfScreen(screen);depth=DefaultDepthOfScreen(screen);
#else
int visuals[]={PseudoColor,TrueColor,StaticGray};int depths[]={8,16,32,15,24};XVisualInfo visualinfo;int j;for(i=0;i<NELEM(depths);i++){for(j=0;j<NELEM(visuals);j++){if(XMatchVisualInfo(xdp->display,DefaultScreen(xdp->display),depths[i],visuals[j],&visualinfo)!=0){DEB(Msg(M_DEBUG,"visual        : %s",visuals[j]==PseudoColor?"PseudoColor":visuals[j]==TrueColor?"TrueColor":"StaticGray");Msg(M_DEBUG,"colorDepth    : %d",visualinfo.depth););goto found_visual;}}}Msg(M_FATAL,"couldn't detect supported visual");found_visual:visual=visualinfo.visual;depth=(unsigned int)visualinfo.depth;
#endif
xwinAllocColors();}XtVaSetValues(xdp->topLevel,XtNdepth,depth,XtNvisual,visual,XtNcolormap,cmap,XtNbackground,0,XtNborderColor,0,XtNull);{XPixmapFormatValues*pixmaps;int formats;pixmaps=XListPixmapFormats(xdp->display,&formats);for(i=0;i<formats;i++){if((unsigned int)pixmaps[i].depth==depth){if(pixmaps[i].bits_per_pixel==8||pixmaps[i].bits_per_pixel==16||pixmaps[i].bits_per_pixel==24||pixmaps[i].bits_per_pixel==32){DEB(Msg(M_DEBUG,"bitsPerPixel  : %d",pixmaps[i].bits_per_pixel););goto found_pixmap;}}}Msg(M_FATAL,"couldn't detect supported pixmap format");found_pixmap:xdp->bitsPerPixel=pixmaps[i].bits_per_pixel;XFree(pixmaps);xwinCreateImage();}DEB(Msg(M_DEBUG,"bitmapBitOrder: %s first",BitmapBitOrder(xdp->display)==LSBFirst?"LSB":"MSB");Msg(M_DEBUG,"imageByteOrder: %s first",ImageByteOrder(xdp->display)==LSBFirst?"LSB":"MSB"););switch(depth){case 8:xdp->bytesPerPixel=1;break;case 15:case 16:xdp->bytesPerPixel=2;break;case 24:xdp->bytesPerPixel=xdp->image->bits_per_pixel==24?3:4;break;case 32:xdp->bytesPerPixel=4;break;}ScreenCreateTables();iconPixmap=XCreatePixmapFromBitmapData(xdp->display,root,(char*)icon_bits,icon_height,icon_width,black,white,depth);XtVaSetValues(xdp->topLevel,XmNiconPixmap,iconPixmap,XtNull);i=0;XtSetArg(arg[i],XmNshowSeparator,True);i++;topWin=XmCreateMainWindow(xdp->topLevel,(char*)"topWin",arg,(Cardinal)i);XtManageChild(topWin);XtAddEventHandler(topWin,StructureNotifyMask,False,(XtEventHandler)xwinConfigure,XtNull);menuBar=MenuCreateBar(topWin);XtManageChild(menuBar);i=0;XtSetArg(arg[i],XmNunitType,XmPIXELS);i++;XtSetArg(arg[i],XmNheight,xdp->vsize);i++;XtSetArg(arg[i],XmNwidth,xdp->hsize);i++;XtSetArg(arg[i],XmNresizePolicy,XmNONE);i++;xdp->emuWindow=XmCreateDrawingArea(topWin,(char*)"emuWindow",arg,(Cardinal)i);XtManageChild(xdp->emuWindow);XtAddEventHandler(xdp->emuWindow,KeyPressMask,False,(XtEventHandler)XkeyPress,XtNull);XtAddEventHandler(xdp->emuWindow,KeyReleaseMask,False,(XtEventHandler)XkeyRelease,XtNull);XtAddEventHandler(xdp->emuWindow,EnterWindowMask,False,(XtEventHandler)xwinEnter,XtNull);XtAddEventHandler(xdp->emuWindow,LeaveWindowMask,False,(XtEventHandler)xwinLeave,XtNull);XtAddEventHandler(xdp->emuWindow,ExposureMask,False,(XtEventHandler)xwinExpose,XtNull);statusBar=SbarCreateBar(topWin);XtManageChild(statusBar);XmMainWindowSetAreas(topWin,menuBar,(Widget)0,(Widget)0,(Widget)0,xdp->emuWindow);XtVaSetValues(topWin,XmNmessageWindow,statusBar,XtNull);XtRealizeWidget(xdp->topLevel);XtVaGetValues(xdp->topLevel,XtNwidth,&width,XtNheight,&height,XtNull);incWidth=BORDER_PIXELS+X_PIXELS+BORDER_PIXELS;incHeight=BORDER_LINES+Y_PIXELS+BORDER_LINES;for(i=0;i<3;i++){sizes[i].x=width-incWidth*(CFGGET(scale)-1-i);sizes[i].y=height-incHeight*(CFGGET(scale)-1-i);}XtVaSetValues(xdp->topLevel,XtNminWidth,sizes[0].x,XtNmaxWidth,sizes[2].x,XtNwidthInc,incWidth,XtNminHeight,sizes[0].y,XtNmaxHeight,sizes[2].y,XtNheightInc,incHeight,XtNminAspectX,incWidth,XtNmaxAspectX,incWidth,XtNminAspectY,incHeight,XtNmaxAspectY,incHeight,XtNbaseWidth,sizes[0].x-incWidth,XtNbaseHeight,sizes[0].y-incHeight,XtNull);xdp->gc=XCreateGC(xdp->display,XtWindow(xdp->emuWindow),0UL,(XGCValues*)0);(void)XSetGraphicsExposures(xdp->display,xdp->gc,False);}static void xwinQuit(void){ScreenDestroyTables();xwinDestroyImage();xwinFreeColors();if(XwinDataP->gc!=(GC)0){XFreeGC(XwinDataP->display,XwinDataP->gc);}(void)XAutoRepeatOn(XwinDataP->display);XtCloseDisplay(XwinDataP->display);}void XwinSetScale(int newScale){if(newScale!=CFGGET(scale)){DEB(Msg(M_DEBUG,"changing scale from %d to %d",CFGGET(scale),newScale););xwinDestroyImage();ScreenDestroyTables();CFGSET(scale,newScale);XwinDataP->hsize=newScale*(BORDER_PIXELS+X_PIXELS+BORDER_PIXELS);XwinDataP->vsize=newScale*(BORDER_LINES+Y_PIXELS+BORDER_LINES);xwinCreateImage();ScreenCreateTables();
#ifdef HAVE_VNCSERVER
VncServerInit();
#endif
}XtVaSetValues(XwinDataP->topLevel,XtNwidth,sizes[newScale-1].x,XtNheight,sizes[newScale-1].y,XtNull);}static void xwinAllocColors(void){static unsigned short rgbVals[NCOLORS][3]={{0x0000,0x0000,0x0000 },{0x0000,0x0000,0xbfff },{0xbfff,0x0000,0x0000 },{0xbfff,0x0000,0xbfff },{0x0000,0xbfff,0x0000 },{0x0000,0xbfff,0xbfff },{0xbfff,0xbfff,0x0000 },{0xbfff,0xbfff,0xbfff },{0x0000,0x0000,0x0000 },{0x0000,0x0000,0xffff },{0xffff,0x0000,0x0000 },{0xffff,0x0000,0xffff },{0x0000,0xffff,0x0000 },{0x0000,0xffff,0xffff },{0xffff,0xffff,0x0000 },{0xffff,0xffff,0xffff },};XColor xc;int i;if(visual!=DefaultVisualOfScreen(screen)||depth!=(unsigned int)DefaultDepthOfScreen(screen)){again:Msg(M_WARN,"forcing private colormap");CFGSET(private,1);}if(CFGGET(private)!=0){cmap=XCreateColormap(XwinDataP->display,root,visual,AllocNone);}else{cmap=DefaultColormapOfScreen(screen);}if(CFGGET(monocrome)!=0){xc.flags=DoRed|DoGreen|DoBlue;xc.red=rgbVals[BLACK][0];xc.green=rgbVals[BLACK][1];xc.blue=rgbVals[BLACK][2];if(XAllocColor(XwinDataP->display,cmap,&xc)==0){i=BLACK;goto error;}for(i=0;i<(NCOLORS/4);i++){XwinDataP->colors[i]=XwinDataP->colors[i+(NCOLORS/2)]=xc.pixel;}xc.red=rgbVals[BRIGHT_WHITE][0];xc.green=rgbVals[BRIGHT_WHITE][1];xc.blue=rgbVals[BRIGHT_WHITE][2];if(XAllocColor(XwinDataP->display,cmap,&xc)==0){(void)XFreeColors(XwinDataP->display,cmap,XwinDataP->colors,1,0);i=BRIGHT_WHITE;goto error;}for(;i<(NCOLORS/2);i++){XwinDataP->colors[i]=XwinDataP->colors[i+(NCOLORS/2)]=xc.pixel;}}else{xc.flags=DoRed|DoGreen|DoBlue;for(i=0;i<NCOLORS;i++){xc.red=rgbVals[i][0];xc.green=rgbVals[i][1];xc.blue=rgbVals[i][2];if(XAllocColor(XwinDataP->display,cmap,&xc)==0){if(i!=0){(void)XFreeColors(XwinDataP->display,cmap,XwinDataP->colors,i-1,0);}error:assert(CFGGET(private)==0);Msg(M_ERR,"couldn't allocate color %d",i);goto again;}XwinDataP->colors[i]=xc.pixel;}}}static void xwinFreeColors(void){if(cmap!=(Colormap)0){if(CFGGET(monocrome)!=0){(void)XFreeColors(XwinDataP->display,cmap,&(XwinDataP->colors[0]),1,0);(void)XFreeColors(XwinDataP->display,cmap,&(XwinDataP->colors[NCOLORS/4]),1,0);}else{(void)XFreeColors(XwinDataP->display,cmap,XwinDataP->colors,NCOLORS,0);}if(cmap!=DefaultColormapOfScreen(screen)){XFreeColormap(XwinDataP->display,cmap);}}}static void xwinCreateImage(void){XwinData*xdp=XwinDataP;
#ifdef HAVE_MITSHM
int x,y,z;if(XQueryExtension(xdp->display,"MIT-SHM",&x,&y,&z)!=0){(void)XShmQueryVersion(xdp->display,&x,&y,&mitshm);}else{mitshm=False;}if(mitshm!=0){int rc;xdp->image=XShmCreateImage(xdp->display,visual,depth,ZPixmap,(char*)0,&xshminfo,xdp->hsize,xdp->vsize);if(xdp->image==(XImage*)0){goto error;}xshminfo.shmid=shmget(IPC_PRIVATE,(size_t)(xdp->image->bytes_per_line*xdp->image->height),IPC_CREAT|0777);if(xshminfo.shmid<0){goto error;}xshminfo.shmaddr=xdp->image->data=shmat(xshminfo.shmid,0,0);if(xshminfo.shmaddr==(char*)0){goto error;}xshminfo.readOnly=False;attachError=0;oldErrorHandler=XSetErrorHandler(xwinErrorHandler);rc=XShmAttach(xdp->display,&xshminfo);XSync(xdp->display,False);(void)XSetErrorHandler(oldErrorHandler);if(rc!=0&&attachError==0){(void)shmctl(xshminfo.shmid,IPC_RMID,0);xdp->putImage=(PutImageFunc)XShmPutImage;return;}error:Msg(M_ERR,"couldn't create MIT-SHM image");if(xdp->image!=(XImage*)0){if(attachError==0){(void)XShmDetach(xdp->display,&xshminfo);}XDestroyImage(xdp->image);if(xshminfo.shmaddr!=(char*)0){(void)shmdt(xshminfo.shmaddr);}if(xshminfo.shmid>=0){(void)shmctl(xshminfo.shmid,IPC_RMID,0);}}mitshm=False;}
#endif 
xdp->image=XCreateImage(xdp->display,visual,depth,ZPixmap,0,(char*)0,xdp->hsize,xdp->vsize,8,0);if(xdp->image==(XImage*)0){Msg(M_FATAL,"couldn't create X image");}xdp->image->data=Malloc((size_t)(xdp->image->bytes_per_line*xdp->image->height),"xwinCreateImage");xdp->putImage=(PutImageFunc)XPutImage;}static void xwinDestroyImage(void){if(XwinDataP->image!=(XImage*)0){
#ifdef HAVE_MITSHM
if(mitshm!=0){(void)XShmDetach(XwinDataP->display,&xshminfo);XDestroyImage(XwinDataP->image);(void)shmdt(xshminfo.shmaddr);}else
#endif
{XDestroyImage(XwinDataP->image);}}}
#ifdef HAVE_MITSHM
static int xwinErrorHandler(Display*disp,XErrorEvent*ev){if(ev->error_code==BadAccess){attachError=1;return 0;}return(oldErrorHandler(disp,ev));}
#endif 
void XwinProcessEvents(void){XEvent xev;while(XtAppPending(appContext)){XtAppNextEvent(appContext,&xev);
#ifdef DEBUG
if((CFGGET(debug)&D_X11)!=0){static const char*eventNames[]={"","","KeyPress","KeyRelease","ButtonPress","ButtonRelease","MotionNotify","EnterNotify","LeaveNotify","FocusIn","FocusOut","KeymapNotify","Expose","GraphicsExpose","NoExpose","VisibilityNotify","CreateNotify","DestroyNotify","UnmapNotify","MapNotify","MapRequest","ReparentNotify","ConfigureNotify","ConfigureRequest","GravityNotify","ResizeRequest","CirculateNotify","CirculateRequest","PropertyNotify","SelectionClear","SelectionRequest","SelectionNotify","ColormapNotify","ClientMessage","MappingNotify"};assert(xev.type<NELEM(eventNames));Msg(M_DEBUG,"got X event %s",eventNames[xev.type]);}
#endif
(void)XtDispatchEvent(&xev);}}static void xwinEnter(Widget w,XtPointer client,XEvent*xev,Boolean*cont){USED(w);USED(client);USED(cont);if(xev->xcrossing.detail!=NotifyInferior){(void)XAutoRepeatOff(XwinDataP->display);XSync(XwinDataP->display,False);}}static void xwinLeave(Widget w,XtPointer client,XEvent*xev,Boolean*cont){USED(w);USED(client);USED(cont);if(xev->xcrossing.detail!=NotifyInferior){(void)XAutoRepeatOn(XwinDataP->display);XkeyReset();XSync(XwinDataP->display,False);}}static void xwinExpose(Widget w,XtPointer client,XEvent*xev,Boolean*cont){USED(w);USED(client);USED(cont);if(xev->xexpose.count==0){XwinDataP->putImage(XwinDataP->display,XtWindow(XwinDataP->emuWindow),XwinDataP->gc,XwinDataP->image,0,0,0,0,XwinDataP->hsize,XwinDataP->vsize,False);}}static void xwinConfigure(Widget w,XtPointer client,XEvent*xev,Boolean*cont){int newScale;USED(w);USED(client);USED(cont);if(xev->xconfigure.width==sizes[2].x&&xev->xconfigure.height==sizes[2].y){newScale=3;}else if(xev->xconfigure.width==sizes[1].x&&xev->xconfigure.height==sizes[1].y){newScale=2;}else if(xev->xconfigure.width==sizes[0].x&&xev->xconfigure.height==sizes[0].y){newScale=1;}else{newScale=CFGGET(scale);}(void)XwinSetScale(newScale);}
