/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char z80_c[]="$Id: z80.c,v 4.49 2006/12/22 14:29:19 erik Rel $";
#endif
#include <string.h>
#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif
#include "z80/z80.h"
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "monitor/breakpoint.h"
#ifdef XZX_IF1
#include "machine/if1.h"
#endif
#include "monitor/monitor.h"
#include "snapshot.h"
#include "sound/sound.h"
#include "tape.h"
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_INTERRUPT) != 0) { x } }
#else
#define DEB(x)
#endif
#ifndef HAVE_ENOUGH_MEMORY
static void z80DecodeCB(void);static void z80DecodeED(void);static void z80DecodeDD(void);static void z80DecodeFD(void);
#endif
struct z80 Z80;static uns8*parityTbl,*sz53pTbl,*sz53Tbl;void Z80Init(void){int i,j,k,p;parityTbl=Malloc(sizeof(uns8)*(size_t)(3*256),"Z80Init");sz53pTbl=&parityTbl[256];sz53Tbl=&parityTbl[512];for(i=0;i<256;i++){for(j=i,k=8,p=0;k;k--){p^=j&1;j>>=1;}parityTbl[i]=(uns8)(p?0:FLAG_P);sz53Tbl[i]=(uns8)(i&(FLAG_S|FLAG_5|FLAG_3));sz53pTbl[i]=sz53Tbl[i]|parityTbl[i];}sz53Tbl[0]|=FLAG_Z;sz53pTbl[0]|=FLAG_Z;(void)memset(&Z80,0,sizeof(Z80));}void Z80Reset(void){Z80.state&=~(CPU_HALTED|CPU_NMI_PENDING|CPU_INT_PENDING|CPU_EI_DELAYED);AF=BC=DE=HL=0;AF1=BC1=DE1=HL1=0;IX=IY=0xffff;SP=PC=0;F=FLAG_Z;R=R7=0;I=IM=IFF1=IFF2=0;HIDDEN=0;(void)BreakpointDestroyAll();}void Z80Int(uns8 ivec,unsigned int itime){Z80.state|=CPU_INT_PENDING;Z80.intVector=ivec;Z80.intTstates=itime;}void Z80Nmi(NmiCallbackFunc func){Z80.state|=CPU_NMI_PENDING;Z80.nmiCallback=func;}void Z80Debug(int state){if(state==0){Z80.state&=~CPU_DEBUGGED;}else{Z80.state|=CPU_DEBUGGED;}}int Z80Execute(volatile unsigned int*cycles){R7=R&0x80;if((Z80.state&(CPU_HALTED|CPU_NMI_PENDING|CPU_INT_PENDING))==CPU_HALTED){unsigned int inc=((*cycles-TSTATES-1)>>2)+1;TSTATES+=inc<<2;R+=inc;if((Z80.state&CPU_EI_DELAYED)!=0){Z80.state&=~CPU_EI_DELAYED;}}else{do{if(Z80.state!=CPU_NORMAL){if((Z80.state&CPU_NMI_PENDING)!=0){Z80.state&=~CPU_NMI_PENDING;DEB(Msg(M_DEBUG,"NMI: IM=%d IFF1=%d IFF2=%d",IM,IFF1,IFF2););if((Z80.state&CPU_HALTED)!=0){Z80.state&=~CPU_HALTED;PC++;}IFF1=0;R++;TSTATES+=11;PUSH(HPC,LPC);PC=0x0066;if(Z80.nmiCallback!=(NmiCallbackFunc)0){Z80.nmiCallback();}}if((Z80.state&CPU_INT_PENDING)!=0&&TSTATES>=Z80.intTstates+32){Z80.state&=~CPU_INT_PENDING;}if((Z80.state&(CPU_INT_PENDING|CPU_EI_DELAYED))==CPU_INT_PENDING){DEB(Msg(M_DEBUG,"INT: IM=%d IFF1=%d IFF2=%d",IM,IFF1,IFF2););if(IFF1!=0){if((Z80.state&CPU_HALTED)!=0){Z80.state&=~CPU_HALTED;PC++;}IFF1=IFF2=0;R++;switch(IM){case 0:TSTATES+=13;PUSH(HPC,LPC);PC=0x0038;break;case 1:TSTATES+=13;PUSH(HPC,LPC);PC=0x0038;break;case 2:TSTATES+=19;PUSH(HPC,LPC);{uns16 t16=(I<<8)|Z80.intVector;LPC=Z80_RDMEM(t16++);HPC=Z80_RDMEM(t16);}break;}}
#ifdef DEBUG_HALT
else if((Z80.state&CPU_HALTED)!=0){Z80.state&=~CPU_HALTED;Msg(M_WARN,"leaving HALT mode (forced)");}
#endif
}Z80.state&=~CPU_EI_DELAYED;if((Z80.state&CPU_DEBUGGED)!=0){MonitorEnter(MONITOR_SINGLESTEP);}}
#ifdef DEBUG
if((CFGGET(debug)&(D_CPU|D_SYSVAR|D_MNEMO))!=0){DebugZ80();}
#endif
{uns8 op=Z80_RDOP(PC++);R++;switch(op){
#include "z80_ops.c"
}}}while(TSTATES<*cycles);}R=(R&0x7f)|R7;return(TSTATES-*cycles);}
#ifndef HAVE_ENOUGH_MEMORY
static void z80DecodeCB(void){uns8 op=Z80_RDOP2(PC++);R++;switch(op){
#include "z80_cbops.c"
}}static void z80DecodeED(void){uns8 op=Z80_RDOP2(PC++);R++;switch(op){
#include "z80_edops.c"
}}static void z80DecodeDD(void){uns8 op=Z80_RDOP2(PC++);R++;switch(op){
#define XX IX
#define LXX LIX
#define HXX HIX
#include "z80_xxops.c"
#undef XX
#undef LXX
#undef HXX
}}static void z80DecodeFD(void){uns8 op=Z80_RDOP2(PC++);R++;switch(op){
#define NO_RCSID
#define XX IY
#define LXX LIY
#define HXX HIY
#include "z80_xxops.c"
#undef XX
#undef LXX
#undef HXX
#undef NO_RCSID
}}
#endif 

