/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifndef _XZX_Z80_MACROS_H
#define _XZX_Z80_MACROS_H
#define PARITY(reg)	parityTbl[(reg)]
#define SZ53P(reg)	sz53pTbl[(reg)]
#define SZ53(reg)	sz53Tbl[(reg)]
#define INC_R() {	R = (R & 0x80) | (++R & 0x7f);	}
#define DEC_R() {	R = (R & 0x80) | (--R & 0x7f);	}
#ifdef ISOLATE_TESTS
#define SCF() {	F = (F & (FLAG_S | FLAG_Z | FLAG_5 | FLAG_V | FLAG_3)) | FLAG_C;	}
#else
#define SCF() {	F = (F & (FLAG_S | FLAG_Z | FLAG_V)) | (A & (FLAG_5 | FLAG_3)) | FLAG_C;	}
#endif
#define CCF() {	F = (F & (FLAG_S | FLAG_Z | FLAG_V)) | (A & (FLAG_5 | FLAG_3)) | (F & FLAG_C ? FLAG_H : FLAG_C);	}
#define INC(reg) {	++(reg);	F = (F & FLAG_C) | SZ53((reg)) | ((reg) & 0xf ? 0 : FLAG_H) | ((reg) == 0x80 ? FLAG_V : 0);	}
#undef DEC
#define DEC(reg) {	F = (F & FLAG_C) | ((reg) & 0xf ? 0 : FLAG_H) | ((reg) == 0x80 ? FLAG_V : 0) | FLAG_N;	--(reg);	F |= SZ53((reg));	}
#define ADD(val) {	uns16 temp16 = A + (val);	F = ((A ^ (val) ^ temp16) & FLAG_H) | ((A ^ (val) ^ 0x80) & ((val) ^ temp16) & 0x80 ? FLAG_V : 0) |	(temp16 & 0x100 ? FLAG_C : 0);	A = (uns8)temp16;	F |= SZ53(A);	}
#define ADC(val) {	uns16 temp16 = A + (val) + (F & FLAG_C);	F = ((A ^ (val) ^ temp16) & FLAG_H) | ((A ^ (val) ^ 0x80) & ((val) ^ temp16) & 0x80 ? FLAG_V : 0) |	(temp16 & 0x100 ? FLAG_C : 0);	A = (uns8)temp16;	F |= SZ53(A);	}
#define SUB(val) {	uns16 temp16 = A - (val);	F = ((A ^ (val) ^ temp16) & FLAG_H) | ((A ^ (val)) & (A ^ temp16) & 0x80 ? FLAG_V : 0) | FLAG_N |	(temp16 & 0x100 ? FLAG_C : 0);	A = (uns8)temp16;	F |= SZ53(A);	}
#define SBC(val) {	uns16 temp16 = A - (val) - (F & FLAG_C);	F = ((A ^ (val) ^ temp16) & FLAG_H) | ((A ^ (val)) & (A ^ temp16) & 0x80 ? FLAG_V : 0) | FLAG_N |	(temp16 & 0x100 ? FLAG_C : 0);	A = (uns8)temp16;	F |= SZ53(A);	}
#define OR(val) {	A |= (val);	F = SZ53P(A);	}
#define AND(val) {	A &= (val);	F = SZ53P(A) | FLAG_H;	}
#define XOR(val) {	A ^= (val);	F = SZ53P(A);	}
#define CMP(val) {	uns16 temp16 = A - (val);	F = (temp16 & FLAG_S) | (temp16 & 0xff ? 0 : FLAG_Z) | (val & (FLAG_5 | FLAG_3)) | ((A ^ (val) ^ temp16) & FLAG_H) |	((A ^ (val)) & (A ^ temp16) & 0x80 ? FLAG_V : 0) | FLAG_N | (temp16 & 0x100 ? FLAG_C : 0);	}
#define RLC(reg) {	(reg) = ((reg) << 1) | ((reg) >> 7);	F = SZ53P((reg)) | ((reg) & FLAG_C);	}
#define RRC(reg) {	F = (reg) & FLAG_C;	(reg) = ((reg) >> 1) | ((reg) << 7);	F |= SZ53P((reg));	}
#define RL(reg) {	uns8 temp8 = (reg) >> 7;	(reg) = ((reg) << 1) | (F & FLAG_C);	F = SZ53P((reg)) | temp8;	}
#define RR(reg) {	uns8 temp8 = (reg) & FLAG_C;	(reg) = ((reg) >> 1) | (F << 7);	F = SZ53P((reg)) | temp8;	}
#define SLA(reg) {	F = (reg) >> 7;	(reg) <<= 1;	F |= SZ53P((reg));	}
#define SRA(reg) {	F = (reg) & FLAG_C;	(reg) = ((reg) & 0x80) | ((reg) >> 1);	F |= SZ53P((reg));	}
#define SLI(reg) {	F = (reg) >> 7;	(reg) = ((reg) << 1) | 0x01;	F |= SZ53P((reg));	}
#define SRL(reg) {	F = (reg) & FLAG_C;	(reg) >>= 1;;	F |= SZ53P((reg));	}
#define PUSH(regh, regl) {	Z80_WRMEM(--SP, (regh));	Z80_WRMEM(--SP, (regl));	}
#define POP(regh, regl) {	(regl) = Z80_RDMEM(SP++);	(regh) = Z80_RDMEM(SP++);	}
#define RET() {	POP(HPC, LPC);	}
#define RETN() {	RET();	IFF1 = IFF2;	}
#endif 

