/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: tzx.c,v 4.83 2006/12/22 14:28:58 erik Rel $";
#endif
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "ui/poke.h"
#include "tape.h"
#include "fileio/tzx.h"
enum{TZX_BLOCK_EOT=0x00,TZX_BLOCK_ROM=0x10,TZX_BLOCK_TURBO,TZX_BLOCK_PURE_TONE,TZX_BLOCK_PULSES,TZX_BLOCK_PURE_DATA,TZX_BLOCK_RAW_DATA,TZX_BLOCK_PAUSE=0x20,TZX_BLOCK_GROUP_START,TZX_BLOCK_GROUP_END,TZX_BLOCK_JUMP,TZX_BLOCK_LOOP_START,TZX_BLOCK_LOOP_END,TZX_BLOCK_CALL,TZX_BLOCK_RETURN,TZX_BLOCK_SELECT,TZX_BLOCK_STOP48=0x2a,TZX_BLOCK_COMMENT=0x30,TZX_BLOCK_MESSAGE,TZX_BLOCK_ARCHIVE_INFO,TZX_BLOCK_HARDWARE,TZX_BLOCK_EMULATION_INFO,TZX_BLOCK_CUSTOM,TZX_BLOCK_SNAPSHOT=0x40,TZX_BLOCK_CONCAT=0x5a };struct tzxHeader{const char ident[7];const char marker;unsigned char major;unsigned char minor;};
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_TAPE) != 0) { x } }
#else
#define DEB(x)
#endif
static void tzxReadBlockEot(TapeRecord*,uns8**);static void tzxReadBlockRom(TapeRecord*,uns8**);static void tzxReadBlockTurbo(TapeRecord*,uns8**);static void tzxReadBlockPureTone(TapeRecord*,uns8**);static void tzxReadBlockPulses(TapeRecord*,uns8**);static void tzxReadBlockPureData(TapeRecord*,uns8**);static void tzxReadBlockRawData(TapeRecord*,uns8**);static void tzxReadBlockPause(TapeRecord*,uns8**);static void tzxReadBlockGroupStart(TapeRecord*,uns8**);static void tzxReadBlockGroupEnd(TapeRecord*,uns8**);static void tzxReadBlockJump(TapeRecord*,uns8**);static void tzxReadBlockLoopStart(TapeRecord*,uns8**);static void tzxReadBlockLoopEnd(TapeRecord*,uns8**);static void tzxReadBlockCall(TapeRecord*,uns8**);static void tzxReadBlockReturn(TapeRecord*,uns8**);static void tzxReadBlockSelect(TapeRecord*,uns8**);static void tzxReadBlockStop48(TapeRecord*,uns8**);static void tzxReadBlockComment(TapeRecord*,uns8**);static void tzxReadBlockMessage(TapeRecord*,uns8**);static void tzxReadBlockArchiveInfo(TapeRecord*,uns8**);static void tzxReadBlockHardware(TapeRecord*,uns8**);static void tzxReadBlockEmulationInfo(TapeRecord*,uns8**);static void tzxReadBlockCustom(TapeRecord*,uns8**);
#ifndef EVALUATION
static void tzxReadBlockCustomPoke(TapeRecord*,unsigned int,uns8*);static void tzxReadBlockCustomInstructions(TapeRecord*,unsigned int,uns8*);
#endif
static void tzxReadBlockCustomUnknown(TapeRecord*,unsigned int,uns8*);static void tzxReadBlockSnapshot(TapeRecord*,uns8**);static void tzxReadBlockConcat(TapeRecord*,uns8**);static void tzxReadBlockUnknown(TapeRecord*,uns8**);static int tzxWriteBlockRom(TapeRecord*,FILE*);static int tzxWriteBlockComment(TapeRecord*,FILE*);int TzxRead(const char*filename){int readonly;size_t size;void*mmp;int ret=-1;readonly=1;if((mmp=Mmap(filename,1,&readonly,&size))!=(void*)-1){struct tzxHeader*header=(struct tzxHeader*)mmp;if(size>sizeof(struct tzxHeader)&&memcmp(header->ident,"ZXTape!",sizeof(header->ident))==0){uns8*scanBegin,*scanEnd,*scanPtr;unsigned int bnr;DEB(Msg(M_DEBUG,"TZX: version %d.%d",header->major,header->minor););if(header->major==10&&header->minor==1){Msg(M_WARN,"input file has invalid TZX version 10.1");}TRY();scanBegin=(uns8*)mmp+sizeof(struct tzxHeader);scanEnd=(uns8*)mmp+size;scanPtr=scanBegin;bnr=0;for(;;){TapeRecord*tr=Malloc(sizeof(TapeRecord),"TzxRead");(void)TapeInSetRecord(tr);tr->bnr=bnr++;if(scanPtr>=scanEnd){tzxReadBlockEot(tr,&scanPtr);break;}DEB(Msg(M_DEBUG,"TZX: scaning, id = %02x, pos = %6d",*scanPtr,scanPtr-scanBegin););switch(*scanPtr){case TZX_BLOCK_ROM:tzxReadBlockRom(tr,&scanPtr);break;case TZX_BLOCK_TURBO:tzxReadBlockTurbo(tr,&scanPtr);break;case TZX_BLOCK_PURE_TONE:tzxReadBlockPureTone(tr,&scanPtr);break;case TZX_BLOCK_PULSES:tzxReadBlockPulses(tr,&scanPtr);break;case TZX_BLOCK_PURE_DATA:tzxReadBlockPureData(tr,&scanPtr);break;case TZX_BLOCK_RAW_DATA:tzxReadBlockRawData(tr,&scanPtr);break;case TZX_BLOCK_PAUSE:tzxReadBlockPause(tr,&scanPtr);break;case TZX_BLOCK_GROUP_START:tzxReadBlockGroupStart(tr,&scanPtr);break;case TZX_BLOCK_GROUP_END:tzxReadBlockGroupEnd(tr,&scanPtr);break;case TZX_BLOCK_JUMP:tzxReadBlockJump(tr,&scanPtr);break;case TZX_BLOCK_LOOP_START:tzxReadBlockLoopStart(tr,&scanPtr);break;case TZX_BLOCK_LOOP_END:tzxReadBlockLoopEnd(tr,&scanPtr);break;case TZX_BLOCK_CALL:tzxReadBlockCall(tr,&scanPtr);break;case TZX_BLOCK_RETURN:tzxReadBlockReturn(tr,&scanPtr);break;case TZX_BLOCK_SELECT:tzxReadBlockSelect(tr,&scanPtr);break;case TZX_BLOCK_STOP48:tzxReadBlockStop48(tr,&scanPtr);break;case TZX_BLOCK_COMMENT:tzxReadBlockComment(tr,&scanPtr);break;case TZX_BLOCK_MESSAGE:tzxReadBlockMessage(tr,&scanPtr);break;case TZX_BLOCK_ARCHIVE_INFO:tzxReadBlockArchiveInfo(tr,&scanPtr);break;case TZX_BLOCK_HARDWARE:tzxReadBlockHardware(tr,&scanPtr);break;case TZX_BLOCK_EMULATION_INFO:tzxReadBlockEmulationInfo(tr,&scanPtr);break;case TZX_BLOCK_CUSTOM:tzxReadBlockCustom(tr,&scanPtr);break;case TZX_BLOCK_SNAPSHOT:tzxReadBlockSnapshot(tr,&scanPtr);break;case TZX_BLOCK_CONCAT:tzxReadBlockConcat(tr,&scanPtr);break;default:tzxReadBlockUnknown(tr,&scanPtr);break;}}ret=0;CATCH(Msg(M_ERR,"corrupted input file"););}else{Msg(M_ERR,"input file is not in TZX format");}Munmap(mmp,size);}return ret;}static void tzxReadBlockEot(TapeRecord*tr,uns8**scanPtr){USED(scanPtr);tr->type=TAPE_BLOCK_EOT;tr->shortDesc="***      End Of Tape      ***";tr->longDesc="End of Tape";}static void tzxReadBlockRom(TapeRecord*tr,uns8**scanPtr){TapeBlockRom*block=&(tr->block.rom);char buf[512];int pos;tr->type=TAPE_BLOCK_ROM;block->length=Le2Cpu(*scanPtr+0x03,2);block->data=Malloc(sizeof(uns8)*(size_t)block->length,"tzxReadBlockRom");(void)memcpy(block->data,*scanPtr+0x05,(size_t)block->length);block->pause=Le2Cpu(*scanPtr+0x01,2);tr->shortDesc=TapeIdentifyBlock(block->length,block->data,1);pos=sprintf(buf,"Standard\n""Length           : %6u bytes\n",block->length>=2?block->length-2:0);if(block->length!=0){pos+=sprintf(&buf[pos],"Flag             :   0x%02X\n""CheckSum         :   0x%02X\n",(unsigned int)block->data[0],(unsigned int)block->data[block->length-1]);}(void)sprintf(&buf[pos],"Pause after block: %6u ms",block->pause);tr->longDesc=Strdup(buf,"tzxReadBlockRom");*scanPtr+=0x05 +block->length;}static void tzxReadBlockTurbo(TapeRecord*tr,uns8**scanPtr){TapeBlockTurbo*block=&(tr->block.turbo);char buf[1024];int pos;tr->type=TAPE_BLOCK_TURBO;block->length=Le2Cpu(*scanPtr+0x10,3);block->lastByte=Le2Cpu(*scanPtr+0x0d,1);block->data=Malloc(sizeof(uns8)*(size_t)block->length,"tzxReadBlockTurbo");(void)memcpy(block->data,*scanPtr+0x13,(size_t)block->length);block->pause=Le2Cpu(*scanPtr+0x0e,2);block->pilotLength=Le2Cpu(*scanPtr+0x01,2);block->pilotPulses=Le2Cpu(*scanPtr+0x0b,2);block->sync1Length=Le2Cpu(*scanPtr+0x03,2);block->sync2Length=Le2Cpu(*scanPtr+0x05,2);block->bit0Length=Le2Cpu(*scanPtr+0x07,2);block->bit1Length=Le2Cpu(*scanPtr+0x09,2);tr->shortDesc=TapeIdentifyBlock(block->length,block->data,0);pos=sprintf(buf,"Turbo\n""Length           : %6u bytes\n",block->length>=2?block->length-2:0);if(block->length!=0){pos+=sprintf(&buf[pos],"Flag             :   0x%02X\n""CheckSum         :   0x%02X\n",(unsigned int)block->data[0],(unsigned int)block->data[block->length-1]);(void)sprintf(&buf[pos],"Pilot pulse      : %6u T-states\n""Pilot length     : %6u pulses\n""Sync first pulse : %6u T-states\n""Sync second pulse: %6u T-states\n""Bit-0 pulse      : %6u T-states\n""Bit-1 pulse      : %6u T-states\n""Last byte used   : %6u bits\n",block->pilotPulses,block->pilotLength,block->sync1Length,block->sync2Length,block->bit0Length,block->bit1Length,block->lastByte);}(void)sprintf(&buf[pos],"Pause after block: %6u ms",block->pause);tr->longDesc=Strdup(buf,"tzxReadBlockTurbo");*scanPtr+=0x13 +block->length;}static void tzxReadBlockPureTone(TapeRecord*tr,uns8**scanPtr){TapeBlockPureTone*block=&(tr->block.puretone);char buf[256];tr->type=TAPE_BLOCK_PURE_TONE;block->length=Le2Cpu(*scanPtr+0x01,2);block->pulses=Le2Cpu(*scanPtr+0x03,2);tr->shortDesc="Pure tone";(void)sprintf(buf,"Pure tone\n""Pulse length: %6u T-states\n""Tone length : %6u pulses",block->length,block->pulses);tr->longDesc=Strdup(buf,"tzxReadBlockPureTone");*scanPtr+=0x05;}static void tzxReadBlockPulses(TapeRecord*tr,uns8**scanPtr){TapeBlockPulses*block=&(tr->block.pulses);char buf[128];unsigned int i,j;tr->type=TAPE_BLOCK_PULSES;block->count=Le2Cpu(*scanPtr+0x01,1);block->lengths=Malloc(sizeof(unsigned int)*(size_t)block->count,"tzxReadBlockPulses");for(i=0,j=0x02;i<block->count;i++,j+=2){block->lengths[i]=Le2Cpu(*scanPtr+j,2);}tr->shortDesc="Sequence of pulses";(void)sprintf(buf,"%s\n""Number of pulses: %6u",tr->shortDesc,block->count);tr->longDesc=Strdup(buf,"tzxReadBlockPulses");*scanPtr+=0x02 +2*block->count;}static void tzxReadBlockPureData(TapeRecord*tr,uns8**scanPtr){TapeBlockPureData*block=&(tr->block.puredata);char buf[512];int pos;tr->type=TAPE_BLOCK_PURE_DATA;block->length=Le2Cpu(*scanPtr+0x08,3);block->lastByte=Le2Cpu(*scanPtr+0x05,1);block->data=Malloc(sizeof(uns8)*(size_t)block->length,"tzxReadBlockPureData");(void)memcpy(block->data,*scanPtr+0x0b,(size_t)block->length);block->pause=Le2Cpu(*scanPtr+0x06,2);block->bit0Length=Le2Cpu(*scanPtr+0x01,2);block->bit1Length=Le2Cpu(*scanPtr+0x03,2);(void)sprintf(buf,"Pure data%20d",block->length);tr->shortDesc=Strdup(buf,"tzxReadBlockPureData");pos=sprintf(buf,"Pure Data\n""Length           : %6u bytes\n",block->length>=2?block->length-2:0);if(block->length!=0){pos+=sprintf(&buf[pos],"Flag             :   0x%02X\n""CheckSum         :   0x%02X\n",(unsigned int)block->data[0],(unsigned int)block->data[block->length-1]);(void)sprintf(&buf[pos],"Bit-0 pulse      : %6u T-states\n""Bit-1 pulse      : %6u T-states\n""Last byte used   : %6u bits\n",block->bit0Length,block->bit1Length,block->lastByte);}(void)sprintf(&buf[pos],"Pause after block: %6u ms",block->pause);tr->longDesc=Strdup(buf,"tzxReadBlockPureData");*scanPtr+=0x0b +block->length;}static void tzxReadBlockRawData(TapeRecord*tr,uns8**scanPtr){TapeBlockRawData*block=&(tr->block.rawdata);char buf[512];tr->type=TAPE_BLOCK_RAW_DATA;block->length=Le2Cpu(*scanPtr+0x06,3);block->lastByte=Le2Cpu(*scanPtr+0x05,1);block->data=Malloc(sizeof(uns8)*(size_t)block->length,"tzxReadBlockRawData");(void)memcpy(block->data,*scanPtr+0x09,(size_t)block->length);block->pause=Le2Cpu(*scanPtr+0x03,2);block->bitLength=Le2Cpu(*scanPtr+0x01,2);if(block->bitLength==0){block->bitLength=1;}tr->shortDesc="Direct";(void)sprintf(buf,"%s\n""Length           : %6u bytes\n""Orig. frequency  : %6u T-states/sample (%5u Hz)\n""Last byte used   : %6u samples\n""Pause after block: %6u ms",tr->shortDesc,block->length,block->bitLength,(unsigned int)(0.5+(3500000.0/(double)block->bitLength)),block->lastByte,block->pause);tr->longDesc=Strdup(buf,"tzxReadBlockRawData");*scanPtr+=0x09 +block->length;}static void tzxReadBlockPause(TapeRecord*tr,uns8**scanPtr){TapeBlockPause*block=&(tr->block.pause);char buf[128];tr->type=TAPE_BLOCK_PAUSE;block->length=Le2Cpu(*scanPtr+0x01,2);tr->shortDesc=block->length==0?"Stop the tape":"Pause";if(block->length>0){(void)sprintf(buf,"%s\n""Duration: %6u ms",tr->shortDesc,block->length);tr->longDesc=Strdup(buf,"tzxReadBlockPause");}else{tr->longDesc=tr->shortDesc;}*scanPtr+=0x03;}static void tzxReadBlockGroupStart(TapeRecord*tr,uns8**scanPtr){char buf[128];unsigned int length;tr->type=TAPE_BLOCK_GROUP_START;length=Le2Cpu(*scanPtr+0x01,1);(void)sprintf(buf,"Group: %.*s",length>23?23:length,(char*)(*scanPtr+0x02));tr->shortDesc=Strdup(buf,"tzxReadGroupStart");(void)sprintf(buf,"Group start\n""Name: %.*s",length,*scanPtr+2);tr->longDesc=Strdup(buf,"tzxReadGroupStart");*scanPtr+=0x02 +length;}static void tzxReadBlockGroupEnd(TapeRecord*tr,uns8**scanPtr){tr->type=TAPE_BLOCK_GROUP_END;tr->shortDesc="Group end";tr->longDesc=tr->shortDesc;*scanPtr+=0x01;}static void tzxReadBlockJump(TapeRecord*tr,uns8**scanPtr){TapeBlockJump*block=&(tr->block.jump);char buf[128];tr->type=TAPE_BLOCK_JUMP;block->offset=(short)Le2Cpu(*scanPtr+0x01,2);(void)sprintf(buf,"Jump %d",tr->bnr+block->offset);tr->shortDesc=Strdup(buf,"tzxReadBlockJump");(void)sprintf(buf,"Jump\n""Destination block: %6d",(unsigned int)(tr->bnr+block->offset));tr->longDesc=Strdup(buf,"tzxReadBlockJump");*scanPtr+=0x03;}static void tzxReadBlockLoopStart(TapeRecord*tr,uns8**scanPtr){TapeBlockLoopStart*block=&(tr->block.loopstart);char buf[128];tr->type=TAPE_BLOCK_LOOP_START;block->count=Le2Cpu(*scanPtr+0x01,2);tr->shortDesc="Loop start";(void)sprintf(buf,"%s\n""Counter: %6u",tr->shortDesc,block->count);tr->longDesc=Strdup(buf,"tzxReadLoopStart");*scanPtr+=0x03;}static void tzxReadBlockLoopEnd(TapeRecord*tr,uns8**scanPtr){tr->type=TAPE_BLOCK_LOOP_END;tr->shortDesc="Loop end";tr->longDesc=tr->shortDesc;*scanPtr+=0x01;}static void tzxReadBlockCall(TapeRecord*tr,uns8**scanPtr){TapeBlockCall*block=&(tr->block.call);char buf[1024];int pos;unsigned int i,j;tr->type=TAPE_BLOCK_CALL;block->count=Le2Cpu(*scanPtr+0x01,2);block->offsets=Malloc(sizeof(int)*(size_t)block->count,"tzxReadBlockCall");for(i=0,j=0x03;i<block->count;i++,j+=2){block->offsets[i]=(short)Le2Cpu(*scanPtr+j,2);}tr->shortDesc="Call";pos=sprintf(buf,"Call");for(i=0;i<block->count;i++){pos+=sprintf(&buf[pos],"\nSubblock: %6u",(unsigned int)(tr->bnr+block->offsets[i]));}tr->longDesc=Strdup(buf,"tzxReadBlockCall");*scanPtr+=0x03 +2*block->count;}static void tzxReadBlockReturn(TapeRecord*tr,uns8**scanPtr){tr->type=TAPE_BLOCK_RETURN;tr->shortDesc="Return";tr->longDesc=tr->shortDesc;*scanPtr+=0x01;}static void tzxReadBlockSelect(TapeRecord*tr,uns8**scanPtr){TapeBlockSelect*block=&(tr->block.select);uns8*p;char buf[1024];int pos;unsigned int i;tr->type=TAPE_BLOCK_SELECT;block->count=Le2Cpu(*scanPtr+0x03,1);block->selections=Malloc(sizeof(*block->selections)*(size_t)block->count,"tzxReadBlockSelect");for(i=0,p=*scanPtr+0x04;i<block->count;i++){unsigned int length=Le2Cpu(p+0x02,1);block->selections[i].description=Malloc(sizeof(char)*(size_t)(length+1),"tzxReadBlockSelect");(void)memcpy((void*)block->selections[i].description,p+3,(size_t)length);block->selections[i].offset=(short)Le2Cpu(p,2);p+=3+length;}tr->shortDesc="Select";pos=sprintf(buf,tr->shortDesc);for(i=0;i<block->count;i++){pos+=sprintf(&buf[pos],"\nChoice           : %s""\nDestination block: %6u",block->selections[i].description,(unsigned int)(tr->bnr+block->selections[i].offset));}tr->longDesc=Strdup(buf,"tzxReadBlockSelect");*scanPtr+=0x03 +Le2Cpu(*scanPtr+0x01,2);}static void tzxReadBlockStop48(TapeRecord*tr,uns8**scanPtr){tr->type=TAPE_BLOCK_STOP48;tr->shortDesc="Stop the tape if in 48k mode";tr->longDesc=tr->shortDesc;*scanPtr+=0x05 +Le2Cpu(*scanPtr+0x01,4);}static void tzxReadBlockComment(TapeRecord*tr,uns8**scanPtr){TapeBlockComment*block=&(tr->block.comment);unsigned int length;char buf[512];int pos;tr->type=TAPE_BLOCK_COMMENT;length=Le2Cpu(*scanPtr+0x01,1);block->text=Malloc(sizeof(char)*(size_t)(length+1),"tzxReadBlockComment");(void)memcpy((void*)block->text,*scanPtr+2,(size_t)length);(void)sprintf(buf,"%.*s",length>30?30:length,(char*)block->text);tr->shortDesc=Strdup(buf,"tzxReadBlockComment");pos=sprintf(buf,"Description\n""Text: ");FormatLine(block->text,length,&buf[pos],6);tr->longDesc=Strdup(buf,"tzxReadBlockComment");*scanPtr+=0x02 +length;}static void tzxReadBlockMessage(TapeRecord*tr,uns8**scanPtr){TapeBlockMessage*block=&(tr->block.message);unsigned int length;char buf[1024];int pos;tr->type=TAPE_BLOCK_RAW_DATA;block->time=Le2Cpu(*scanPtr+0x01,1);;length=Le2Cpu(*scanPtr+0x02,1);block->text=Malloc(sizeof(char)*(size_t)(length+1),"tzxReadBlockMessage");(void)memcpy((void*)block->text,*scanPtr+3,(size_t)length);tr->shortDesc="Message";pos=sprintf(buf,"%s\n""Display time: %6u s\n""Text        : ",tr->shortDesc,block->time);FormatLine(block->text,length,&buf[pos],16);tr->longDesc=Strdup(buf,"tzxReadBlockMessage");*scanPtr+=0x03 +Le2Cpu(*scanPtr+length,1);}static void tzxReadBlockArchiveInfo(TapeRecord*tr,uns8**scanPtr){uns8*p;char buf[512];int pos;unsigned int i;tr->type=TAPE_BLOCK_ARCHIVE_INFO;tr->shortDesc="Archive info";pos=sprintf(buf,tr->shortDesc);for(i=Le2Cpu(*scanPtr+0x03,1),p=*scanPtr+4;i>0;i--){const char*t;switch(*p++){case 0x00:t="Title";break;case 0x01:t="Publisher";break;case 0x02:t="Author(s)";break;case 0x03:t="Release Date";break;case 0x04:t="Language";break;case 0x05:t="Program Type";break;case 0x06:t="Price";break;case 0x07:t="Loader";break;case 0x08:t="Origin";break;default:t="Comments";break;}pos+=sprintf(&buf[pos],"\n%-12s: ",t);pos+=FormatLine((char*)(p+1),*p,&buf[pos],14);pos=strlen(buf);p+=*p+1;}tr->longDesc=Strdup(buf,"tzxReadBlockArchiveInfo");*scanPtr+=0x03 +Le2Cpu(*scanPtr+0x01,2);}static void tzxReadBlockHardware(TapeRecord*tr,uns8**scanPtr){static const char*hwType[][256]={{"ZX Spectrum 16k","ZX Spectrum 48k, Plus","ZX Spectrum 48k Issue 1","ZX Spectrum 128k","ZX Spectrum 128k +2","ZX Spectrum 128k +2A, +3","Timex Sinclair TC-2048","Timex Sinclair TS-2068","Pentagon 128","Sam Coupe","Didaktik M","Didaktik Gama","ZX-81 with 1k RAM","ZX-81 with 16k RAM or more","ZX Spectrum 128k, Spanish version","ZX Spectrum, Arabic version","TK 90-X","TK 95","Byte","Elwro","ZS Scorpion","Amstrad CPC 464","Amstrad CPC 664","Amstrad CPC 6128","Amstrad CPC 464+","Amstrad CPC 6128+","Jupiter ACE""Enterprise","Commodore 64","Commodore 128"},{"Microdrive","Opus Discovery","Disciple","Plus-D","Rotronics Wafadrive","Beta 128","Byte Drive","Watsford","FIZ","Radofin","Didaktik disk drives","MB-02","ZX Spectrum +3 disk drive","JLO disk interface","FDD3000","Zebra disk drive","Ramex Millenia","Larken"},{"Sam Ram","Multiface","Multiface 128k","Multiface +3","MultiPrint","MB-02 ROM/RAM expansion"},{"Classic AY hardware","Fuller Box AY sound hardware","Currah microSpeech","SpecDrum","AY ACB stereo","AY ABC stereo"},{"Kempston","Cursor, Protek, AGF","Sinclair 2 Left","Sinclair 1 Right","Fuller"},{"AMX Mouse","Kempston mouse"},{"Trickstick","ZX Light Gun","Zebra Graphics Tablet"},{"ZX Interface 1","ZX Spectrum 128k"},{"Kempston S","Kempston E","ZX Spectrum +3","Tasman","DK'Tronics","Hilderbay","INES Printerface","ZX LPrint Interface 3","MultiPrint","Opus Discovery","Standard 8255 chip"},{"ZX Printer, Alphacom 32 & Compatibles","Generic Printer","EPSON Compatible"},{"VTX 5000","T/S 2050 or Westridge 2050"},{"RD Digital Tracer","DK'Tronics Light Pen","British MicroGraph Pad"},{"ZX interface 1"},{"Keypad for ZX Spectrum 128k"},{"Harley Systems ADC 8.2","Blackboard Electronics"},{"Orme Electronics"}};unsigned int count;uns8*p;char buf[512];int pos;unsigned int i;tr->type=TAPE_BLOCK_HARDWARE;count=Le2Cpu(*scanPtr+0x01,1);tr->shortDesc="Hardware";pos=sprintf(buf,"Hardware");for(i=0,p=*scanPtr+2;i<count;i++,p+=3){const char*t;switch(p[2]){default:case 0:t="Runs on";break;case 1:t="Uses";break;case 2:t="Does not use";break;case 3:t="Does not run on";break;}pos+=sprintf(&buf[pos],"\n%-15s: %s",t,hwType[p[0]][p[1]]);}tr->longDesc=Strdup(buf,"tzxReadBlockHardware");*scanPtr+=0x02 +3*count;}static void tzxReadBlockEmulationInfo(TapeRecord*tr,uns8**scanPtr){tr->type=TAPE_BLOCK_UNSUPPORTED;tr->shortDesc="Emulation info";tr->longDesc="No info yet :-(";*scanPtr+=0x08;}static void tzxReadBlockCustom(TapeRecord*tr,uns8**scanPtr){static struct{const char identification[0x10];TapeBlockCustomType id;void(*func)(TapeRecord*,unsigned int,uns8*);}types[]={
#ifndef EVALUATION
{"POKEs           ",TAPE_BLOCK_CUSTOM_POKE,tzxReadBlockCustomPoke},{"Instructions    ",TAPE_BLOCK_CUSTOM_INSTRUCTIONS,tzxReadBlockCustomInstructions},
#endif
{"Spectrum Screen ",TAPE_BLOCK_CUSTOM_SCREEN,tzxReadBlockCustomUnknown},{"ZX-Edit document",TAPE_BLOCK_CUSTOM_ZXEDIT,tzxReadBlockCustomUnknown},{"Picture         ",TAPE_BLOCK_CUSTOM_PICTURE,tzxReadBlockCustomUnknown},{"",TAPE_BLOCK_CUSTOM_UNSUPPORTED,tzxReadBlockCustomUnknown},};TapeBlockCustom*block=&(tr->block.custom);char buf[128];unsigned int length,i;tr->type=TAPE_BLOCK_CUSTOM;length=Le2Cpu(*scanPtr+0x11,4);for(i=0;i<NELEM(types)-1;i++){if(strncmp((char*)(*scanPtr+1),types[i].identification,16)==0){break;}}block->type=types[i].id;(*types[i].func)(tr,length,*scanPtr+0x15);(void)sprintf(buf,"%.16s",(char*)(*scanPtr+1));tr->shortDesc=Strdup(buf,"tzxReadBlockCustom");(void)sprintf(buf,"Custom\n""Type: %s",tr->shortDesc);tr->longDesc=Strdup(buf,"tzxReadBlockCustom");*scanPtr+=0x15 +length;}
#ifndef EVALUATION
static void tzxReadBlockCustomPoke(TapeRecord*tr,unsigned int length,uns8*data){TapeBlockCustomPokes*block=&(tr->block.custom.subblock.pokes);PokeRecord*pr;int i,j,k;USED(length);data+=1+Le2Cpu(data,1);block->count=Le2Cpu(data,1);data++;pr=Malloc(sizeof(PokeRecord)*(size_t)block->count,"tzxReadBlockCustomPoke");block->pokes=pr;for(i=block->count;i!=0;i--){j=(int)Le2Cpu(data,1);data++;k=j<MAX_POKE_NAME-1?j:MAX_POKE_NAME-1;pr->name[k]='\0';while(--k>=0){pr->name[k]=data[k];}data+=j;pr->state=POKE_STATE_UNPOKED;pr->original=1;pr->cnt=(int)Le2Cpu(data,1);assert(pr->cnt<MAX_POKE_VALUES);data++;for(j=pr->cnt;j>0;j--){pr->pokes[pr->cnt].page=Le2Cpu(data,1);pr->pokes[pr->cnt].addr=(uns16)Le2Cpu(data+0x01,2);pr->pokes[pr->cnt].pval=(uns8)Le2Cpu(data+0x03,1);pr->pokes[pr->cnt].oval=(uns8)Le2Cpu(data+0x04,1);if(pr->pokes[pr->cnt].oval==0){pr->original=0;}data+=5;}pr++;}}static void tzxReadBlockCustomInstructions(TapeRecord*tr,unsigned int length,uns8*data){TapeBlockCustomInstructions*block=&(tr->block.custom.subblock.instructions);char*text,*p;text=Malloc(sizeof(uns8)*(size_t)(length+1),"tzxReadBlockCustomInstructions");(void)memcpy(text,(char*)data,(size_t)length);for(p=text;*p!='\0';p++){if(*p==0x0d){*p=0x0a;}}block->text=text;}
#endif 
static void tzxReadBlockCustomUnknown(TapeRecord*tr,unsigned int length,uns8*data){TapeBlockCustomUnknown*block=&(tr->block.custom.subblock.unknown);block->data=Malloc(sizeof(uns8)*(size_t)length,"tzxReadBlockCustomUnknown");(void)memcpy(block->data,data,(size_t)length);}static void tzxReadBlockSnapshot(TapeRecord*tr,uns8**scanPtr){tr->type=TAPE_BLOCK_UNSUPPORTED;Msg(M_WARN,"unsupported TZX block %02x",tr->type);tr->shortDesc="Snapshot";tr->longDesc="No info yet :-(";*scanPtr+=0x05 +Le2Cpu(*scanPtr+0x02,3);}static void tzxReadBlockConcat(TapeRecord*tr,uns8**scanPtr){tr->type=TAPE_BLOCK_UNSUPPORTED;tr->shortDesc="Merged files";tr->longDesc=tr->shortDesc;*scanPtr+=0x10;}static void tzxReadBlockUnknown(TapeRecord*tr,uns8**scanPtr){tr->type=TAPE_BLOCK_UNSUPPORTED;Msg(M_WARN,"unknown TZX block %02x",tr->type);tr->shortDesc="Unknown";tr->longDesc="No info yet :-(";*scanPtr+=0x05 +Le2Cpu(*scanPtr+0x01,4);}int TzxWrite(const char*filename){FILE*fp;int ret=-1;if((fp=Fopen(filename,"wb"))!=(FILE*)0){static struct tzxHeader header={"ZXTape!",0x1a,1,12};if(fwrite(&header,sizeof(header),1,fp)==1){unsigned int bnr,blocks;blocks=TapeOutGetBlocks();for(bnr=0;bnr<blocks;bnr++){TapeRecord*tr=TapeOutGetRecord(bnr);assert(tr!=(TapeRecord*)0);switch(tr->type){case TAPE_BLOCK_ROM:ret=tzxWriteBlockRom(tr,fp);break;case TAPE_BLOCK_COMMENT:ret=tzxWriteBlockComment(tr,fp);break;default:assert(0);break;}if(ret!=0){break;}}}if(ret!=0){Msg(M_PERR,"couldn't write file <%s>",filename);}(void)fclose(fp);}return ret;}static int tzxWriteBlockRom(TapeRecord*tr,FILE*fp){TapeBlockRom*block=&(tr->block.rom);size_t length;(void)fputc(TZX_BLOCK_ROM,fp);(void)fputc((int)(block->pause&0x00ff),fp);(void)fputc((int)((block->pause&0xff00)>>8),fp);length=(size_t)block->length;(void)fputc((int)(length&0x00ff),fp);(void)fputc((int)((length&0xff00)>>8),fp);return(fwrite(block->data,length,1,fp)==1?0:-1);}static int tzxWriteBlockComment(TapeRecord*tr,FILE*fp){TapeBlockComment*block=&(tr->block.comment);size_t length;(void)fputc(TZX_BLOCK_COMMENT,fp);length=strlen(block->text)&0xff;(void)fputc((int)length,fp);return(fwrite(block->text,length,1,fp)==1?0:-1);}
