/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2007
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: monitor.c,v 4.82 2006/12/22 14:29:04 erik Rel $";
#endif
#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/SelectioB.h>
#include <Xm/SeparatoG.h>
#include <Xm/SSpinB.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>
#include "z80/z80.h"
#include "ui/resource.h"
#include "machine/machine.h"
#include "util.h"
#include "ui/ui.h"
#include "monitor/breakpoint.h"
#include "monitor/dis.h"
#include "sound/sound.h"
#include "ui/toolhelp.h"
#include "ui/xwin.h"
#include "monitor/monitor.h"
enum{S_RUN,S_RUN_TRACE,S_HALT,S_STEP,S_STEP_HALT,S_STATES_MAX};enum{PAN_CTRL,PAN_CPU,PAN_ASM,PAN_MEM,PAN_HW,PAN_MAX};enum{W_CPU=PAN_CPU,W_ASM,W_MEM,W_HW,W_TRACE,W_STEP,W_STOP};enum{W_ASMSELECT,W_BPSELECT,W_BPADD,W_BPADDTEMP,W_BPEDIT,W_BPREMOVE};enum{L_64B,L_128B,L_256B,L_512B,L_1K,L_16K,L_64K};enum{R_AF,R_BC,R_DE,R_HL,R_IX,R_IY,R_PC,R_SP,R_AF1,R_BC1,R_DE1,R_HL1,R_I,R_R,R_IM,R_IFF1,R_IFF2};enum{HW_VLINE,HW_TSTATES,HW_BANK0,HW_BANK1,HW_BANK2,HW_BANK3,HW_AY0,HW_AY1,HW_AY2,HW_AY3,HW_AY4,HW_AY5,HW_AY6,HW_AY7,HW_AY8,HW_AY9,HW_AY10,HW_AY11,HW_AY12,HW_AY13,HW_AY14,HW_AY15};typedef struct _panel{Widget shell;Widget button;Boolean active;void(*show)(void);void(*update)(void);void(*stateChanged)(void);}panel;typedef struct _field{const char*name;void*value;int top;int bottom;int left;int right;Widget widget;const char*format;int digits;}field;
#define ASM_VISIBLE_LINES	10
#define ASM_LINES	30
#define POPUP(panel)	{ (panel)->show();	(panel)->stateChanged();	}
#define POPDOWN(panel)	{ XtUnmanageChild((panel)->shell);	XtPopdown(XtParent((panel)->shell));	}
#define POPUP_PANELS()	{ panel *pp = &monitorPanels[PAN_CTRL];	for (; pp < &monitorPanels[PAN_MAX]; pp++)	{	if (pp->active == True)	{	POPUP(pp);	}	}	}
#define POPDOWN_PANELS()	{ panel *pp = &monitorPanels[PAN_CTRL];	for (; pp < &monitorPanels[PAN_MAX]; pp++)	{	if (pp->active == True)	{	POPDOWN(pp);	}	}	}
#define UPDATE_PANELS()	{ panel *pp = &monitorPanels[PAN_CTRL];	for (; pp < &monitorPanels[PAN_MAX]; pp++)	{	if (pp->active == True)	{	pp->update();	}	}	}
#define STATE_CHANGED()	{ panel *pp = &monitorPanels[PAN_CTRL];	for (; pp < &monitorPanels[PAN_MAX]; pp++)	{	if (pp->active == True)	{	pp->stateChanged();	}	}	}
static void monitorControlShow(void);static void monitorControlCallback(Widget,XtPointer,XtPointer);static void monitorControlUpdate(void);static void monitorControlStateChanged(void);static void monitorCpuShow(void);static void monitorCpuCallback(Widget,XtPointer,XtPointer);static void monitorCpuUpdate(void);static void monitorCpuUpdateFlags(int);static void monitorCpuStateChanged(void);static void monitorAsmShow(void);static void monitorAsmCallback(Widget,XtPointer,XtPointer);static void monitorAsmEventHandler(Widget,XtPointer,XEvent*,Boolean*);static void monitorAsmUpdate(void);static void monitorAsmMnemonicUpdate(void);static void monitorAsmStateChanged(void);static void monitorAsmAddBreakpoint(BreakpointType);static void monitorAsmEditBreakpoint(void);static Boolean monitorAsmRemoveBreakpoint(void);static void monitorMemoryShow(void);static void monitorMemoryCallback(Widget,XtPointer,XtPointer);static void monitorMemoryUpdate(void);static void monitorMemoryStateChanged(void);static void monitorHardwareShow(void);static void monitorHardwareCallback(Widget,XtPointer,XtPointer);static void monitorHardwareUpdate(void);static void monitorHardwareStateChanged(void);static int monitorState=S_RUN;static int monitorLastState=S_RUN;static Boolean monitorTracing=False;static Boolean monitorStepping=False;static panel monitorPanels[PAN_MAX]={{(Widget)0,(Widget)0,False,monitorControlShow,monitorControlUpdate,monitorControlStateChanged},{(Widget)0,(Widget)0,False,monitorCpuShow,monitorCpuUpdate,monitorCpuStateChanged},{(Widget)0,(Widget)0,False,monitorAsmShow,monitorAsmUpdate,monitorAsmStateChanged},{(Widget)0,(Widget)0,False,monitorMemoryShow,monitorMemoryUpdate,monitorMemoryStateChanged},{(Widget)0,(Widget)0,False,monitorHardwareShow,monitorHardwareUpdate,monitorHardwareStateChanged}};static Widget monitorControlTracing;static Widget monitorControlState;static Widget monitorControlButton;static XmString monitorStateLabels[S_STATES_MAX],monitorActionLabels[S_STATES_MAX];static field monitorCpuRegs[]={{"AF",(void*)&AF,0,1,0,1,(Widget)0,"%04X",4},{"BC",(void*)&BC,1,2,0,1,(Widget)0,"%04X",4},{"DE",(void*)&DE,2,3,0,1,(Widget)0,"%04X",4},{"HL",(void*)&HL,3,4,0,1,(Widget)0,"%04X",4},{"IX",(void*)&IX,0,1,2,3,(Widget)0,"%04X",4},{"IY",(void*)&IY,1,2,2,3,(Widget)0,"%04X",4},{"PC",(void*)&PC,3,4,2,3,(Widget)0,"%04X",4},{"SP",(void*)&SP,3,4,3,4,(Widget)0,"%04X",4},{"AF'",(void*)&AF1,0,1,1,2,(Widget)0,"%04X",4},{"BC'",(void*)&BC1,1,2,1,2,(Widget)0,"%04X",4},{"DE'",(void*)&DE1,2,3,1,2,(Widget)0,"%04X",4},{"HL'",(void*)&HL1,3,4,1,2,(Widget)0,"%04X",4},{"I",(void*)&I,0,1,3,4,(Widget)0,"%02X",2},{"R",(void*)&R,1,2,3,4,(Widget)0,"%02X",2},{"IM",(void*)&IM,0,0,0,0,(Widget)0,"%01d",1},{"IFF1",(void*)&IFF1,0,0,0,0,(Widget)0,"%01d",1},{"IFF2",(void*)&IFF2,0,0,0,0,(Widget)0,"%01d",1}};static Widget monitorCpuFlags;static Widget monitorAsmMnemonicList,monitorBpList;static Widget monitorMemoryAddress,monitorMemoryText;static int monitorMemoryLengths[]={64,128,256,512,1*1024,16*1024,64*1024},monitorMemoryLength;static field monitorHw[]={{"Vline",(void*)0,0,1,0,1,(Widget)0,"%6u",6},{"Tstates",(void*)&TSTATES,1,2,0,1,(Widget)0,"%6u",6},{"Bank0",(void*)&MachinePageNumbers[0],0,1,0,2,(Widget)0,"%s",0},{"Bank1",(void*)&MachinePageNumbers[2],1,2,0,2,(Widget)0,"%s",0},{"Bank2",(void*)&MachinePageNumbers[4],2,3,0,2,(Widget)0,"%s",0},{"Bank3",(void*)&MachinePageNumbers[6],3,4,0,2,(Widget)0,"%s",0},{"R0",(void*)OFFSETOF(MachineSettings,ay.registers[0]),0,1,0,1,(Widget)0,"%02X",2},{"R1",(void*)OFFSETOF(MachineSettings,ay.registers[1]),1,2,0,1,(Widget)0,"%02X",2},{"R2",(void*)OFFSETOF(MachineSettings,ay.registers[2]),2,3,0,1,(Widget)0,"%02X",2},{"R3",(void*)OFFSETOF(MachineSettings,ay.registers[3]),3,4,0,1,(Widget)0,"%02X",2},{"R4",(void*)OFFSETOF(MachineSettings,ay.registers[4]),0,1,1,2,(Widget)0,"%02X",2},{"R5",(void*)OFFSETOF(MachineSettings,ay.registers[5]),1,2,1,2,(Widget)0,"%02X",2},{"R6",(void*)OFFSETOF(MachineSettings,ay.registers[6]),2,3,1,2,(Widget)0,"%02X",2},{"R7",(void*)OFFSETOF(MachineSettings,ay.registers[7]),3,4,1,2,(Widget)0,"%02X",2},{"R8",(void*)OFFSETOF(MachineSettings,ay.registers[8]),0,1,2,3,(Widget)0,"%02X",2},{"R9",(void*)OFFSETOF(MachineSettings,ay.registers[9]),1,2,2,3,(Widget)0,"%02X",2},{"R10",(void*)OFFSETOF(MachineSettings,ay.registers[10]),2,3,2,3,(Widget)0,"%02X",2},{"R11",(void*)OFFSETOF(MachineSettings,ay.registers[11]),3,4,2,3,(Widget)0,"%02X",2},{"R12",(void*)OFFSETOF(MachineSettings,ay.registers[12]),0,1,3,4,(Widget)0,"%02X",2},{"R13",(void*)OFFSETOF(MachineSettings,ay.registers[13]),1,2,3,4,(Widget)0,"%02X",2},{"R14",(void*)OFFSETOF(MachineSettings,ay.registers[14]),2,3,3,4,(Widget)0,"%02X",2},{"R15",(void*)OFFSETOF(MachineSettings,ay.registers[15]),3,4,3,4,(Widget)0,"%02X",2}};static XmString monitorHwRomLabels[NPAGES];static const char hex[]="0123456789ABCDEF";void MonitorEnter(int why){if(why==MONITOR_SINGLESTEP){if(monitorState==S_STEP){monitorState=S_STEP_HALT;Z80Debug(1);UPDATE_PANELS();STATE_CHANGED();}else if(monitorTracing==True){UPDATE_PANELS();}}else{uns16 pc=PC-1;unsigned int bpno;if((bpno=BreakpointLookup(RPAGE(pc),pc))==BREAKPOINT_NOTFOUND){TSTATES+=11;PUSH(HPC,LPC);PC=0x0000;return;}BreakpointUnset(bpno);PC--;R--;monitorState=S_HALT;Z80Debug(1);if(monitorPanels[PAN_CTRL].active==False){monitorLastState=monitorState;XmToggleButtonSetState(XtNameToWidget(XwinDataP->topLevel,"*Monitor"),True,True);}STATE_CHANGED();}do{XwinProcessEvents();}while(monitorState==S_HALT||monitorState==S_STEP_HALT);}void MonitorShow(void){if(monitorStateLabels[S_RUN]==(XmString)0){static const char*states[]={"Running","Tracing","Halted","Stepping","Halted"};static const char*actions[]={"Stop","Stop","Continue","Stop","Step"};int i;for(i=S_RUN;i<S_STATES_MAX;i++){monitorStateLabels[i]=XmStringCreateLtoR((char*)states[i],XmFONTLIST_DEFAULT_TAG);monitorActionLabels[i]=XmStringCreateLtoR((char*)actions[i],XmFONTLIST_DEFAULT_TAG);}}monitorState=monitorLastState;Z80Debug(monitorState==S_RUN?0:1);monitorPanels[PAN_CTRL].active=True;POPUP_PANELS();UPDATE_PANELS();}void MonitorHide(void){POPDOWN_PANELS();monitorPanels[PAN_CTRL].active=False;Z80Debug(0);monitorLastState=monitorState;monitorState=S_RUN;}static void monitorControlShow(void){if(monitorPanels[PAN_CTRL].shell==(Widget)0){Widget form,frame,frame2,frame3,w;XmString title;Arg arg[6];Cardinal i;title=XmStringCreateLtoR((char*)"Monitor - Control",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNmarginWidth,10);i++;XtSetArg(arg[i],XmNmarginHeight,10);i++;XtSetArg(arg[i],XmNnoResize,True);i++;XtSetArg(arg[i],XmNautoUnmanage,False);i++;XtSetArg(arg[i],XmNdeleteResponse,XmUNMAP);i++;XtSetArg(arg[i],XmNdialogTitle,title);i++;assert(i<=XtNumber(arg));w=XmCreateFormDialog(XwinDataP->topLevel,(char*)"dialog",arg,i);XtAddCallback(w,XmNunmapCallback,monitorControlCallback,XtNull);monitorPanels[PAN_CTRL].shell=w;XmStringFree(title);frame=UiCreateFrame(monitorPanels[PAN_CTRL].shell,"Panels");XtVaSetValues(frame,XmNleftAttachment,XmATTACH_FORM,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_FORM,XtNull);i=0;XtSetArg(arg[i],XmNfractionBase,2);i++;assert(i<=XtNumber(arg));form=XmCreateForm(frame,(char*)"form",arg,i);XtManageChild(form);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNbottomPosition,1);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButton(form,(char*)"CPU",arg,i);XtAddCallback(w,XmNvalueChangedCallback,monitorControlCallback,W_CPU);XtManageChild(w);monitorPanels[PAN_CPU].button=w;i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNtopPosition,1);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButton(form,(char*)"Assembler",arg,i);XtAddCallback(w,XmNvalueChangedCallback,monitorControlCallback,W_ASM);XtManageChild(w);monitorPanels[PAN_ASM].button=w;i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,1);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNbottomPosition,1);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButton(form,(char*)"Memory",arg,i);XtAddCallback(w,XmNvalueChangedCallback,monitorControlCallback,W_MEM);XtManageChild(w);monitorPanels[PAN_MEM].button=w;i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,1);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNtopPosition,1);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButton(form,(char*)"Hardware",arg,i);XtAddCallback(w,XmNvalueChangedCallback,monitorControlCallback,W_HW);XtManageChild(w);monitorPanels[PAN_HW].button=w;frame2=UiCreateFrame(monitorPanels[PAN_CTRL].shell,"Trace");XtVaSetValues(frame2,XmNleftAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_WIDGET,XmNtopWidget,frame,XmNbottomAttachment,XmATTACH_FORM,XtNull);form=XmCreateForm(frame2,(char*)"form",(ArgList)0,0);XtManageChild(form);i=0;XtSetArg(arg[i],XmNsensitive,monitorTracing);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));monitorControlTracing=XmCreateToggleButton(form,(char*)"Tracing",arg,i);XtAddCallback(monitorControlTracing,XmNvalueChangedCallback,monitorControlCallback,W_TRACE);XtManageChild(monitorControlTracing);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,monitorControlTracing);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateToggleButton(form,(char*)"Single step",arg,i);XtAddCallback(w,XmNvalueChangedCallback,monitorControlCallback,W_STEP);XtManageChild(w);frame3=UiCreateFrame(monitorPanels[PAN_CTRL].shell,"CPU State");XtVaSetValues(frame3,XmNleftOffset,3,XmNleftAttachment,XmATTACH_WIDGET,XmNleftWidget,frame2,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_WIDGET,XmNtopWidget,frame,XmNbottomAttachment,XmATTACH_FORM,XtNull);form=XmCreateForm(frame3,(char*)"form",(ArgList)0,0);XtManageChild(form);i=0;XtSetArg(arg[i],XmNlabelString,monitorStateLabels[monitorState]);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));monitorControlState=XmCreateLabel(form,(char*)"label",arg,i);XtManageChild(monitorControlState);i=0;XtSetArg(arg[i],XmNlabelString,monitorActionLabels[monitorState]);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));monitorControlButton=XmCreatePushButton(form,(char*)"button",arg,i);XtAddCallback(monitorControlButton,XmNactivateCallback,monitorControlCallback,W_STOP);XtManageChild(monitorControlButton);}XtManageChild(monitorPanels[PAN_CTRL].shell);XtPopup(XtParent(monitorPanels[PAN_CTRL].shell),XtGrabNone);}static void monitorControlCallback(Widget w,XtPointer client,XtPointer call){panel*p;int oldState;;USED(w);if(((XmAnyCallbackStruct*)call)->reason==XmCR_UNMAP){XmToggleButtonSetState(XtNameToWidget(XwinDataP->topLevel,"*Monitor"),False,True);return;}switch((int)client){case W_CPU:case W_ASM:case W_MEM:case W_HW:p=&monitorPanels[(int)client];if((p->active=(((XmToggleButtonCallbackStruct*)call)->set))==True){POPUP(p);if(monitorState==S_HALT||monitorState==S_STEP_HALT){p->update();}else{UPDATE_PANELS();}if(monitorTracing==False){XtSetSensitive(monitorControlTracing,True);}}else{POPDOWN(p);for(p=&monitorPanels[PAN_CPU];p<&monitorPanels[PAN_MAX];p++){if(p->active==True){break;}}if(p==&monitorPanels[PAN_MAX]){XmToggleButtonSetState(monitorControlTracing,False,True);XtSetSensitive(monitorControlTracing,False);}}break;case W_TRACE:monitorTracing=((XmToggleButtonCallbackStruct*)call)->set;if(monitorState==S_RUN||monitorState==S_RUN_TRACE){monitorState=monitorTracing==True?S_RUN_TRACE:S_RUN;monitorPanels[PAN_CTRL].stateChanged();}break;case W_STEP:monitorStepping=((XmToggleButtonCallbackStruct*)call)->set;oldState=monitorState;monitorState=monitorStepping==True?S_STEP_HALT:S_HALT;if(oldState!=S_HALT&&oldState!=S_STEP_HALT){UPDATE_PANELS();}STATE_CHANGED();break;case W_STOP:switch(monitorState){case S_RUN:monitorState=S_HALT;UPDATE_PANELS();break;case S_RUN_TRACE:monitorState=S_HALT;break;case S_HALT:monitorState=monitorTracing==True?S_RUN_TRACE:S_RUN;break;case S_STEP_HALT:monitorState=S_STEP;break;}STATE_CHANGED();break;}if(monitorState==S_RUN){Z80Debug(0);}else{Z80Debug(1);}}static void monitorControlUpdate(void){}static void monitorControlStateChanged(void){XtVaSetValues(monitorControlState,XmNlabelString,monitorStateLabels[monitorState],XtNull);XtVaSetValues(monitorControlButton,XmNlabelString,monitorActionLabels[monitorState],XtNull);}static void monitorCpuShow(void){if(monitorPanels[PAN_CPU].shell==(Widget)0){Widget form,form2,frame,frame2,frame3,rowcol,w;XmString title;Arg arg[10];Cardinal i;int j;field*reg;title=XmStringCreateLtoR((char*)"Monitor - CPU",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNmarginWidth,10);i++;XtSetArg(arg[i],XmNmarginHeight,10);i++;XtSetArg(arg[i],XmNnoResize,True);i++;XtSetArg(arg[i],XmNautoUnmanage,False);i++;XtSetArg(arg[i],XmNdeleteResponse,XmUNMAP);i++;XtSetArg(arg[i],XmNdialogTitle,title);i++;XtSetArg(arg[i],XtNkeyboardFocusPolicy,XmPOINTER);i++;assert(i<=XtNumber(arg));w=XmCreateFormDialog(monitorPanels[PAN_CTRL].shell,(char*)"fd",arg,i);XtAddCallback(w,XmNunmapCallback,monitorCpuCallback,XtNull);monitorPanels[PAN_CPU].shell=w;XmStringFree(title);frame=UiCreateFrame(monitorPanels[PAN_CPU].shell,"Registers");XtVaSetValues(frame,XmNleftAttachment,XmATTACH_FORM,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_FORM,XtNull);i=0;XtSetArg(arg[i],XmNfractionBase,4);i++;assert(i<=XtNumber(arg));form=XmCreateForm(frame,(char*)"form",arg,i);XtManageChild(form);for(j=R_AF,reg=&monitorCpuRegs[j];j<=R_R;j++,reg++){i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,reg->left);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,reg->right);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNtopPosition,reg->top);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNbottomPosition,reg->bottom);i++;XtSetArg(arg[i],XmNfractionBase,6);i++;assert(i<=XtNumber(arg));form2=XmCreateForm(form,(char*)"form",arg,i);XtManageChild(form2);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,2);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabel(form2,(char*)reg->name,arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNcolumns,reg->digits);i++;XtSetArg(arg[i],XmNmaxLength,reg->digits);i++;XtSetArg(arg[i],XmNcursorPositionVisible,False);i++;XtSetArg(arg[i],XmNeditable,False);i++;XtSetArg(arg[i],XmNtraversalOn,False);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNleftWidget,w);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));reg->widget=XmCreateTextField(form2,(char*)(reg->name),arg,i);XtAddCallback(reg->widget,XmNfocusCallback,monitorCpuCallback,j);XtAddCallback(reg->widget,XmNlosingFocusCallback,monitorCpuCallback,j);XtManageChild(reg->widget);}frame2=UiCreateFrame(monitorPanels[PAN_CPU].shell,"Flags");XtVaSetValues(frame2,XmNleftAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_WIDGET,XmNtopWidget,frame,XmNbottomAttachment,XmATTACH_FORM,XtNull);form=XmCreateForm(frame2,(char*)"form",(ArgList)0,0);XtManageChild(form);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabel(form,(char*)"S Z X H X V P C",arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNtraversalOn,False);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,w);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));monitorCpuFlags=XmCreateLabel(form,(char*)"0 0 0 0 0 0 0 0",arg,i);XtManageChild(monitorCpuFlags);frame3=UiCreateFrame(monitorPanels[PAN_CPU].shell,"Interrupt");XtVaSetValues(frame3,XmNleftOffset,3,XmNleftAttachment,XmATTACH_WIDGET,XmNleftWidget,frame2,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_WIDGET,XmNtopWidget,frame,XmNbottomAttachment,XmATTACH_FORM,XtNull);rowcol=XmCreateRowColumn(frame3,(char*)"rowcol",(ArgList)0,0);XtManageChild(rowcol);w=form=XmCreateForm(rowcol,(char*)"form",(ArgList)0,0);XtManageChild(form);for(j=R_IM,reg=&monitorCpuRegs[j];j<=R_IFF2;j++,reg++){i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNleftWidget,w);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabel(form,(char*)(reg->name),arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNcolumns,1);i++;XtSetArg(arg[i],XmNmaxLength,1);i++;XtSetArg(arg[i],XmNcursorPositionVisible,False);i++;XtSetArg(arg[i],XmNeditable,False);i++;XtSetArg(arg[i],XmNtraversalOn,False);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNleftWidget,w);i++;XtSetArg(arg[i],XmNleftOffset,3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=reg->widget=XmCreateTextField(form,(char*)(reg->name),arg,i);XtAddCallback(w,XmNfocusCallback,monitorCpuCallback,j);XtAddCallback(w,XmNlosingFocusCallback,monitorCpuCallback,j);XtManageChild(w);}}XtManageChild(monitorPanels[PAN_CPU].shell);XtPopup(XtParent(monitorPanels[PAN_CPU].shell),XtGrabNone);}static void monitorCpuCallback(Widget w,XtPointer client,XtPointer call){field*reg;char buf[5],*str;int val;if(((XmAnyCallbackStruct*)call)->reason==XmCR_UNMAP){XmToggleButtonSetState(monitorPanels[PAN_CPU].button,False,True);return;}if(XmTextFieldGetEditable(w)==True){if(((XmAnyCallbackStruct*)call)->reason==XmCR_FOCUS){XtVaSetValues(w,XmNcursorPositionVisible,True,XtNull);}else{XtVaSetValues(w,XmNcursorPositionVisible,False,XtNull);XmTextFieldClearSelection(w,CurrentTime);reg=&monitorCpuRegs[(int)client];str=XmTextFieldGetString(w);if(sscanf(str,"%x",(unsigned int*)&val)==1){switch((int)client){case R_AF:monitorCpuUpdateFlags(val);case R_BC:case R_DE:case R_HL:case R_IX:case R_IY:case R_PC:case R_SP:case R_AF1:case R_BC1:case R_DE1:case R_HL1:*((uns16*)(reg->value))=(uns16)val;(void)sprintf(buf,reg->format,val);break;case R_I:case R_R:*((uns8*)(reg->value))=(uns8)val;goto bit8;case R_IM:if(val>=0&&val<=2){*((uns8*)(reg->value))=(uns8)val;}goto bit8;case R_IFF1:case R_IFF2:if(val>=0&&val<=1){*((uns8*)(reg->value))=(uns8)val;}bit8:(void)sprintf(buf,reg->format,val);break;}}XmTextFieldSetString(w,buf);XtFree(str);}}}static void monitorCpuUpdate(void){char buf[5];field*reg;for(reg=&monitorCpuRegs[R_AF];reg<=&monitorCpuRegs[R_HL1];reg++){(void)sprintf(buf,reg->format,*((uns16*)(reg->value)));XmTextFieldSetString(reg->widget,buf);}for(;reg<=&monitorCpuRegs[R_IFF2];reg++){(void)sprintf(buf,reg->format,*((uns8*)(reg->value)));XmTextFieldSetString(reg->widget,buf);}monitorCpuUpdateFlags((int)F);}static void monitorCpuUpdateFlags(int flags){char buf[17];int i,mask;XmString str;for(i=0,mask=0x80;i<16;mask>>=1){buf[i++]=flags&mask?'1':'0';buf[i++]=' ';}buf[--i]='\0';str=XmStringCreateLtoR(buf,XmFONTLIST_DEFAULT_TAG);XtVaSetValues(monitorCpuFlags,XmNlabelString,str,XtNull);XmStringFree(str);}static void monitorCpuStateChanged(void){field*reg;Boolean editable;editable=(monitorState==S_HALT||monitorState==S_STEP_HALT);for(reg=&monitorCpuRegs[R_AF];reg<=&monitorCpuRegs[R_IFF2];reg++){XmTextFieldSetEditable(reg->widget,editable);}}static void monitorAsmShow(void){if(monitorPanels[PAN_ASM].shell==(Widget)0){Widget form,frame,frame2,w;XmString title;Arg arg[10];Cardinal i;title=XmStringCreateLtoR((char*)"Monitor - Assembler",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNmarginWidth,10);i++;XtSetArg(arg[i],XmNmarginHeight,10);i++;XtSetArg(arg[i],XmNnoResize,True);i++;XtSetArg(arg[i],XmNautoUnmanage,False);i++;XtSetArg(arg[i],XmNdeleteResponse,XmUNMAP);i++;XtSetArg(arg[i],XmNdialogTitle,title);i++;XtSetArg(arg[i],XtNkeyboardFocusPolicy,XmPOINTER);i++;assert(i<=XtNumber(arg));w=XmCreateFormDialog(monitorPanels[PAN_CTRL].shell,(char*)"fd",arg,i);XtAddCallback(w,XmNunmapCallback,monitorAsmCallback,XtNull);monitorPanels[PAN_ASM].shell=w;XmStringFree(title);frame=UiCreateFrame(monitorPanels[PAN_ASM].shell,"Mnemonics");XtVaSetValues(frame,XmNleftAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_FORM,XmNbottomAttachment,XmATTACH_FORM,XtNull);i=0;XtSetArg(arg[i],XmNmarginWidth,0);i++;XtSetArg(arg[i],XmNmarginHeight,0);i++;assert(i<=XtNumber(arg));form=XmCreateForm(frame,(char*)"form",arg,i);XtManageChild(form);i=0;XtSetArg(arg[i],XmNvisibleItemCount,ASM_VISIBLE_LINES);i++;XtSetArg(arg[i],XmNlistSizePolicy,XmVARIABLE);i++;XtSetArg(arg[i],XmNselectionPolicy,XmSINGLE_SELECT);i++;XtSetArg(arg[i],XmNscrollBarDisplayPolicy,XmSTATIC);i++;XtSetArg(arg[i],XmNscrollHorizontal,False);i++;XtSetArg(arg[i],XmNscrollVertical,True);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));monitorAsmMnemonicList=XmCreateScrolledList(form,(char*)"list",arg,i);XtAddCallback(monitorAsmMnemonicList,XmNsingleSelectionCallback,monitorAsmCallback,W_ASMSELECT);UiFixiateListWidth(monitorPanels[PAN_ASM].shell,monitorAsmMnemonicList,30);XtManageChild(monitorAsmMnemonicList);frame2=UiCreateFrame(monitorPanels[PAN_ASM].shell,"Breakpoints");XtVaSetValues(frame2,XmNleftOffset,3,XmNleftAttachment,XmATTACH_WIDGET,XmNleftWidget,frame,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_FORM,XmNbottomAttachment,XmATTACH_FORM,XtNull);i=0;XtSetArg(arg[i],XmNfractionBase,3);i++;assert(i<=XtNumber(arg));form=XmCreateForm(frame2,(char*)"form",arg,i);XtManageChild(form);i=0;XtSetArg(arg[i],XmNlistSizePolicy,XmVARIABLE);i++;XtSetArg(arg[i],XmNselectionPolicy,XmSINGLE_SELECT);i++;XtSetArg(arg[i],XmNscrollBarDisplayPolicy,XmSTATIC);i++;XtSetArg(arg[i],XmNscrollHorizontal,False);i++;XtSetArg(arg[i],XmNscrollVertical,True);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));monitorBpList=XmCreateScrolledList(form,(char*)"list",arg,i);XtAddEventHandler(monitorBpList,ButtonPressMask,False,monitorAsmEventHandler,XtNull);XtSetSensitive(XtParent(monitorBpList),False);ToolhelpAddWidget(monitorBpList,"Press right button for action.");UiFixiateListWidth(monitorPanels[PAN_ASM].shell,monitorBpList,25);XtManageChild(monitorBpList);}XtManageChild(monitorPanels[PAN_ASM].shell);XtPopup(XtParent(monitorPanels[PAN_ASM].shell),XtGrabNone);}static void monitorAsmCallback(Widget w,XtPointer client,XtPointer call){switch(((XmAnyCallbackStruct*)call)->reason){case XmCR_UNMAP:XmToggleButtonSetState(monitorPanels[PAN_ASM].button,False,True);break;case XmCR_SINGLE_SELECT:assert((int)client==W_ASMSELECT);XmListDeselectAllItems(monitorAsmMnemonicList);(void)XBell(XtDisplayOfObject(w),5);break;case XmCR_ACTIVATE:switch((int)client){case W_BPADD:monitorAsmAddBreakpoint(BREAKPOINT_TYPE_PERMANENT);break;case W_BPADDTEMP:monitorAsmAddBreakpoint(BREAKPOINT_TYPE_TEMPORARY);break;case W_BPEDIT:monitorAsmEditBreakpoint();break;case W_BPREMOVE:(void)monitorAsmRemoveBreakpoint();break;}XmListDeselectAllItems(monitorBpList);break;case XmCR_OK:*((int*)client)=DIALOG_OK;break;case XmCR_CANCEL:*((int*)client)=DIALOG_CANCEL;break;}}static void monitorAsmEventHandler(Widget w,XtPointer client,XEvent*xev,Boolean*cont){USED(client);if(xev->xbutton.button==Button3){static Widget popup=(Widget)0;Widget addbp,addtempbp;static Widget editbp,removebp;int pos,item;Breakpoint*bpp;if(popup==(Widget)0){popup=XmCreatePopupMenu(monitorBpList,(char*)"pm",(ArgList)0,0);addbp=XmCreatePushButton(popup,(char*)"Add...",(ArgList)0,0);XtAddCallback(addbp,XmNactivateCallback,monitorAsmCallback,W_BPADD);XtManageChild(addbp);addtempbp=XmCreatePushButton(popup,(char*)"Add temporary...",(ArgList)0,0);XtAddCallback(addtempbp,XmNactivateCallback,monitorAsmCallback,W_BPADDTEMP);XtManageChild(addtempbp);editbp=XmCreatePushButton(popup,(char*)"Edit...",(ArgList)0,0);XtAddCallback(editbp,XmNactivateCallback,monitorAsmCallback,W_BPEDIT);XtManageChild(editbp);XtSetSensitive(editbp,False);removebp=XmCreatePushButton(popup,(char*)"Remove",(ArgList)0,0);XtAddCallback(removebp,XmNactivateCallback,monitorAsmCallback,W_BPREMOVE);XtManageChild(removebp);}pos=XmListYToPos(w,xev->xbutton.y);XtVaGetValues(w,XmNitemCount,&item,XtNull);if(pos>item){pos=item;}XmListSelectPos(w,pos,True);bpp=BreakpointRetrieve((unsigned int)--pos);XtSetSensitive(removebp,bpp!=(Breakpoint*)0);XmMenuPosition(popup,(XButtonPressedEvent*)xev);XtManageChild(popup);*cont=False;}}static void monitorAsmUpdate(void){monitorAsmMnemonicUpdate();}static void monitorAsmStateChanged(void){Boolean editable;editable=(monitorState==S_HALT||monitorState==S_STEP_HALT);XtSetSensitive(XtParent(monitorBpList),editable);if(BreakpointRetrieve(0)==(Breakpoint*)0){XmListDeleteAllItems(monitorBpList);}}static void monitorAsmMnemonicUpdate(void){XmString*strlist;char buf[32],*p;uns16 i,j;int k,lines;unsigned int addr;lines=ASM_LINES;addr=PC;strlist=(XmString*)XtMalloc((size_t)lines*sizeof(XmString*));for(k=0;k<lines;k++){p=buf;*p++=hex[(addr>>12)&0x0f];*p++=hex[(addr>>8)&0x0f];*p++=hex[(addr>>4)&0x0f];*p++=hex[addr&0x0f];*p++=' ';*p++=' ';j=Disassemble(buf+16,(uns16)addr)-addr;for(i=0;i<j;i++){int val=MEM((uns16)(addr+i));*p++=hex[(val>>4)&0x0f];*p++=hex[val&0x0f];}addr+=j;while(p<&buf[16]){*p++=' ';}strlist[k]=XmStringCreateLtoR(buf,XmFONTLIST_DEFAULT_TAG);}XtVaSetValues(monitorAsmMnemonicList,XmNitemCount,k,XmNitems,strlist,XtNull);while(--k>=0){XmStringFree(strlist[k]);}XtFree((char*)strlist);}static void monitorAsmAddBreakpoint(BreakpointType type){static Widget dialog=(Widget)0;static int answer;static struct field{const char*label;Widget spinbox;}fields[]={{"Memory page",(Widget)0},{"Memory address",(Widget)0}};Widget w;UiSpinbox*sbp;int page,addr;unsigned int bpno;char buf[24];if(dialog==(Widget)0){Widget form,form2;XmString title;Arg arg[13];Cardinal i;int j;title=XmStringCreateLtoR((char*)"Dialog",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNnoResize,True);i++;XtSetArg(arg[i],XmNautoUnmanage,False);i++;XtSetArg(arg[i],XmNdeleteResponse,XmDO_NOTHING);i++;XtSetArg(arg[i],XmNdialogStyle,XmDIALOG_FULL_APPLICATION_MODAL);i++;XtSetArg(arg[i],XmNdialogTitle,title);i++;assert(i<=XtNumber(arg));dialog=XmCreatePromptDialog(XwinDataP->topLevel,(char*)"dialog",arg,i);XmStringFree(title);XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_COMMAND_TEXT));XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_PROMPT_LABEL));XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_HELP_BUTTON));XtAddCallback(dialog,XmNokCallback,monitorAsmCallback,&answer);XtAddCallback(dialog,XmNcancelCallback,monitorAsmCallback,&answer);form=XmCreateForm(dialog,(char*)"form",(ArgList)0,0);XtManageChild(form);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabel(form,(char*)"Add breakpoint:",arg,i);XtManageChild(w);for(j=0;form2=w,j<2;j++){i=0;XtSetArg(arg[i],XmNfractionBase,100);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,form2);i++;assert(i<=XtNumber(arg));form2=XmCreateForm(form,(char*)"form",arg,i);i=0;XtSetArg(arg[i],XmNalignment,XmALIGNMENT_BEGINNING);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,55);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabel(form2,(char*)(fields[j].label),arg,i);XtManageChild(w);w=UiCreateSpinbox(form2,j==0?"%4d":"%04X",4,0,0,1,(XtCallbackProc)0);XtVaSetValues(w,XmNleftAttachment,XmATTACH_POSITION,XmNleftPosition,60,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_FORM,XmNbottomAttachment,XmATTACH_FORM,XtNull);fields[j].spinbox=w;XtManageChild(form2);}}if(CFGGET(machine)<SP_128){w=fields[0].spinbox;XtSetSensitive(XtParent(w),False);UiSpinboxSetValue(w,0);w=fields[1].spinbox;XtVaGetValues(w,XmNuserData,&sbp,XtNull);sbp->max=0xffff;sbp->min=0x4000;UiSpinboxSetValue(fields[1].spinbox,sbp->min);}else{w=fields[0].spinbox;XtVaGetValues(w,XmNuserData,&sbp,XtNull);XtSetSensitive(XtParent(w),True);sbp->max=7;
#ifdef XZX_SCORPION
sbp->max+=CFGGET(machine)==SP_SCORPION?8:0;
#endif
UiSpinboxSetValue(w,sbp->min);w=fields[1].spinbox;XtVaGetValues(w,XmNuserData,&sbp,XtNull);sbp->max=0x3fff;sbp->min=0x0000;UiSpinboxSetValue(fields[1].spinbox,sbp->min);}UiShowDialog(dialog,&answer);if(answer==DIALOG_OK){page=UiSpinboxGetValue(fields[0].spinbox);addr=UiSpinboxGetValue(fields[1].spinbox);if(CFGGET(machine)<SP_128){page=RPAGE(addr)-RAM0;addr&=0x3fff;}if((bpno=BreakpointCreate(type,RAM0+page,(uns16)addr))==BREAKPOINT_INVALID){UiShowMessage(DIALOG_ERROR,"Breakpoint already set!",(const char*)0,(const char*)0,DIALOG_OK);}else{Breakpoint*bpp;XmString bpstr;BreakpointSet(bpno);bpp=BreakpointRetrieve(bpno);assert(bpp!=(Breakpoint*)0);assert(strlen(bpp->opstr)<=15);(void)sprintf(buf,"%X:%04X %c %s",bpp->page-RAM0,bpp->addr,bpp->temporary==0?'P':'T',bpp->opstr);bpstr=XmStringCreateLtoR(buf,XmFONTLIST_DEFAULT_TAG);XmListAddItemUnselected(monitorBpList,bpstr,(int)bpno+2);XmStringFree(bpstr);}}}static void monitorAsmEditBreakpoint(void){}static Boolean monitorAsmRemoveBreakpoint(void){int*list,count;if(XmListGetSelectedPos(monitorBpList,&list,&count)){assert(count==1);if(UiShowMessage(DIALOG_ASK,"Remove breakpoint?",(const char*)0,(const char*)0,DIALOG_CANCEL)==DIALOG_OK){unsigned int bpno=(unsigned int)(*list-1);BreakpointUnset(bpno);BreakpointDestroy(bpno);XmListDeletePos(monitorBpList,*list);XtFree((char*)list);return True;}}return False;}static void monitorMemoryShow(void){if(monitorPanels[PAN_MEM].shell==(Widget)0){Widget form,form2,frame,sep,w;XmString title,label,l64b,l128b,l256b,l512b,l1k,l16k,l64k;Arg arg[9];Cardinal i;monitorMemoryLength=monitorMemoryLengths[L_128B];title=XmStringCreateLtoR((char*)"Monitor - Memory",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNmarginWidth,10);i++;XtSetArg(arg[i],XmNmarginHeight,10);i++;XtSetArg(arg[i],XmNnoResize,True);i++;XtSetArg(arg[i],XmNautoUnmanage,False);i++;XtSetArg(arg[i],XmNdeleteResponse,XmUNMAP);i++;XtSetArg(arg[i],XmNdialogTitle,title);i++;XtSetArg(arg[i],XtNkeyboardFocusPolicy,XmPOINTER);i++;assert(i<=XtNumber(arg));w=XmCreateFormDialog(monitorPanels[PAN_CTRL].shell,(char*)"fd",arg,i);XtAddCallback(w,XmNunmapCallback,monitorMemoryCallback,XtNull);monitorPanels[PAN_MEM].shell=w;XmStringFree(title);frame=UiCreateFrame(monitorPanels[PAN_MEM].shell,"Memory");XtVaSetValues(frame,XmNleftAttachment,XmATTACH_FORM,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_FORM,XmNbottomAttachment,XmATTACH_FORM,XtNull);i=0;XtSetArg(arg[i],XmNmarginWidth,0);i++;XtSetArg(arg[i],XmNmarginHeight,0);i++;assert(i<=XtNumber(arg));form=XmCreateForm(frame,(char*)"form",arg,i);XtManageChild(form);i=0;XtSetArg(arg[i],XmNfractionBase,100);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));form2=XmCreateForm(form,(char*)"form",arg,i);i=0;XtSetArg(arg[i],XmNalignment,XmALIGNMENT_BEGINNING);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabel(form2,(char*)"Address",arg,i);XtManageChild(w);monitorMemoryAddress=UiCreateSpinbox(form2,"%04X",4,0,0xffff,monitorMemoryLength,(XtCallbackProc)monitorMemoryUpdate);XtVaSetValues(monitorMemoryAddress,XmNleftAttachment,XmATTACH_WIDGET,XmNleftWidget,w,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_FORM,XmNbottomAttachment,XmATTACH_FORM,XtNull);XtManageChild(form2);i=0;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNtopWidget,monitorMemoryAddress);i++;XtSetArg(arg[i],XmNtopOffset,3);i++;assert(i<=XtNumber(arg));sep=XmCreateSeparatorGadget(form,(char*)"sep",arg,i);XtManageChild(sep);label=XmStringCreateLtoR((char*)"Length",XmFONTLIST_DEFAULT_TAG);l64b=XmStringCreateLtoR((char*)"64b",XmFONTLIST_DEFAULT_TAG);l128b=XmStringCreateLtoR((char*)"128b",XmFONTLIST_DEFAULT_TAG);l256b=XmStringCreateLtoR((char*)"256b",XmFONTLIST_DEFAULT_TAG);l512b=XmStringCreateLtoR((char*)"512b",XmFONTLIST_DEFAULT_TAG);l1k=XmStringCreateLtoR((char*)"1k",XmFONTLIST_DEFAULT_TAG);l16k=XmStringCreateLtoR((char*)"16k",XmFONTLIST_DEFAULT_TAG);l64k=XmStringCreateLtoR((char*)"64k",XmFONTLIST_DEFAULT_TAG);w=XmVaCreateSimpleOptionMenu(form,(char*)"om",label,0,L_128B,monitorMemoryCallback,XmVaPUSHBUTTON,l64b,(KeySym)0,(String)0,(XmString)0,XmVaPUSHBUTTON,l128b,(KeySym)0,(String)0,(XmString)0,XmVaPUSHBUTTON,l256b,(KeySym)0,(String)0,(XmString)0,XmVaPUSHBUTTON,l512b,(KeySym)0,(String)0,(XmString)0,XmVaPUSHBUTTON,l1k,(KeySym)0,(String)0,(XmString)0,XmVaPUSHBUTTON,l16k,(KeySym)0,(String)0,(XmString)0,XmVaPUSHBUTTON,l64k,(KeySym)0,(String)0,(XmString)0,XmNrightAttachment,XmATTACH_FORM,XmNrightOffset,-6,XmNtopAttachment,XmATTACH_FORM,XmNbottomAttachment,XmATTACH_WIDGET,XmNbottomWidget,sep,XtNull);XtManageChild(w);XmStringFree(label);XmStringFree(l64b);XmStringFree(l128b);XmStringFree(l256b);XmStringFree(l512b);XmStringFree(l1k);XmStringFree(l16k);XmStringFree(l64k);i=0;XtSetArg(arg[i],XmNcolumns,69);i++;XtSetArg(arg[i],XmNrows,8);i++;XtSetArg(arg[i],XmNeditMode,XmMULTI_LINE_EDIT);i++;XtSetArg(arg[i],XmNeditable,False);i++;XtSetArg(arg[i],XmNcursorPositionVisible,False);i++;XtSetArg(arg[i],XmNhighlightThickness,0);i++;XtSetArg(arg[i],XmNscrollBarDisplayPolicy,XmSTATIC);i++;XtSetArg(arg[i],XmNscrollHorizontal,False);i++;XtSetArg(arg[i],XmNscrollVertical,True);i++;assert(i<=XtNumber(arg));monitorMemoryText=XmCreateScrolledText(form,(char*)"text",arg,i);XtVaSetValues(XtParent(monitorMemoryText),XmNleftAttachment,XmATTACH_FORM,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_WIDGET,XmNtopWidget,sep,XmNbottomAttachment,XmATTACH_FORM,XtNull);XtManageChild(monitorMemoryText);}XtManageChild(monitorPanels[PAN_MEM].shell);XtPopup(XtParent(monitorPanels[PAN_MEM].shell),XtGrabNone);}static void monitorMemoryCallback(Widget w,XtPointer client,XtPointer call){UiSpinbox*sbp;int val;USED(w);switch(((XmAnyCallbackStruct*)call)->reason){case XmCR_UNMAP:XmToggleButtonSetState(monitorPanels[PAN_MEM].button,False,True);break;case XmCR_ACTIVATE:monitorMemoryLength=monitorMemoryLengths[(int)client];XtVaGetValues(monitorMemoryAddress,XmNuserData,&sbp,XtNull);sbp->offset=monitorMemoryLength;val=UiSpinboxGetValue(monitorMemoryAddress);val&=~(monitorMemoryLength-1);UiSpinboxSetValue(monitorMemoryAddress,val);monitorMemoryUpdate();break;case XmCR_MOVING_INSERT_CURSOR:case XmCR_MODIFYING_TEXT_VALUE:break;}}static void monitorMemoryUpdate(void){char*buf=XtMalloc((size_t)(monitorMemoryLength/16*70)),*p,*q;int addr,val,i,j;addr=UiSpinboxGetValue(monitorMemoryAddress);for(p=buf,i=monitorMemoryLength/16;i>0;i--){*p++=hex[(addr>>12)&0x0f];*p++=hex[(addr>>8)&0x0f];*p++=hex[(addr>>4)&0x0f];*p++=hex[addr&0x0f];q=p+48;*q++=' ';for(j=16;j>0;j--){*p++=' ';val=MEM(addr);*p++=hex[(val>>4)&0x0f];*p++=hex[val&0x0f];*q++=isprint(val)!=0?val:'.';addr++;}*q++='\n';p=q;}*(--p)='\0';assert((int)(p-buf)<(monitorMemoryLength/16*70));XmTextSetString(monitorMemoryText,buf);XtFree(buf);}static void monitorMemoryStateChanged(void){}static void monitorHardwareShow(void){if(monitorPanels[PAN_HW].shell==(Widget)0){static const char*roms[NPAGES]={"ROM","ROM0","ROM1","ROM0","ROM1","ROM2","ROM3",
#ifdef XZX_PENTAGON
"ROM0","ROM1","ROM2",
#endif
#ifdef XZX_SCORPION
"ROM0","ROM1","ROM2","ROM3",
#endif
#ifdef XZX_DIDAKTIK
"ROM","D80",
#endif
#ifdef XZX_IF1
"IF1",
#endif
#ifdef XZX_MF
"MF1ROM","MF128","MF3",
#endif
#ifdef XZX_TRDOS
"TRDOS",
#endif
#ifdef XZX_PLUSD
"PLUSD",
#endif
"RAM0","RAM1","RAM2","RAM3","RAM4","RAM5","RAM6","RAM7",
#ifdef XZX_SCORPION
"RAM8","RAM9","RAMA","RAMB","RAMC","RAMD","RAME","RAMF",
#endif
};Widget form,form2,frame,frame2,frame3,rowcol,w;XmString title;Arg arg[10];Cardinal i;int j;field*hwp;title=XmStringCreateLtoR((char*)"Monitor - Hardware",XmFONTLIST_DEFAULT_TAG);i=0;XtSetArg(arg[i],XmNmarginWidth,10);i++;XtSetArg(arg[i],XmNmarginHeight,10);i++;XtSetArg(arg[i],XmNnoResize,True);i++;XtSetArg(arg[i],XmNautoUnmanage,False);i++;XtSetArg(arg[i],XmNdeleteResponse,XmUNMAP);i++;XtSetArg(arg[i],XmNdialogTitle,title);i++;XtSetArg(arg[i],XtNkeyboardFocusPolicy,XmPOINTER);i++;assert(i<=XtNumber(arg));w=XmCreateFormDialog(monitorPanels[PAN_CTRL].shell,(char*)"fd",arg,i);XtAddCallback(w,XmNunmapCallback,monitorHardwareCallback,XtNull);monitorPanels[PAN_HW].shell=w;XmStringFree(title);frame=UiCreateFrame(monitorPanels[PAN_HW].shell,"ULA");XtVaSetValues(frame,XmNleftAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_FORM,XtNull);i=0;XtSetArg(arg[i],XmNfractionBase,2);i++;assert(i<=XtNumber(arg));form=XmCreateForm(frame,(char*)"form",arg,i);XtManageChild(form);for(hwp=&monitorHw[HW_VLINE];hwp<=&monitorHw[HW_TSTATES];hwp++){i=0;XtSetArg(arg[i],XmNalignment,XmALIGNMENT_BEGINNING);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,hwp->right);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNtopPosition,hwp->top);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNbottomPosition,hwp->bottom);i++;assert(i<=XtNumber(arg));w=XmCreateLabel(form,(char*)(hwp->name),arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNtraversalOn,False);i++;XtSetArg(arg[i],XmNalignment,XmALIGNMENT_BEGINNING);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,hwp->right);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNtopPosition,hwp->top);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNbottomPosition,hwp->bottom);i++;assert(i<=XtNumber(arg));hwp->widget=XmCreateLabel(form,(char*)(hwp->name),arg,i);XtManageChild(hwp->widget);}frame2=UiCreateFrame(monitorPanels[PAN_HW].shell,"AY8912");XtVaSetValues(frame2,XmNleftOffset,3,XmNleftAttachment,XmATTACH_WIDGET,XmNleftWidget,frame,XmNrightAttachment,XmATTACH_FORM,XmNtopAttachment,XmATTACH_FORM,XmNbottomAttachment,XmATTACH_FORM,XtNull);rowcol=XmCreateRowColumn(frame2,(char*)"rowcol",(ArgList)0,0);XtManageChild(rowcol);i=0;XtSetArg(arg[i],XmNfractionBase,4);i++;assert(i<=XtNumber(arg));form=XmCreateForm(rowcol,(char*)"form",arg,i);XtManageChild(form);for(j=HW_AY0,hwp=&monitorHw[j];j<=HW_AY15;j++,hwp++){i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,hwp->left);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,hwp->right);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNtopPosition,hwp->top);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNbottomPosition,hwp->bottom);i++;XtSetArg(arg[i],XmNfractionBase,7);i++;assert(i<=XtNumber(arg));form2=XmCreateForm(form,(char*)"form",arg,i);XtManageChild(form2);i=0;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,3);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));w=XmCreateLabel(form2,(char*)(hwp->name),arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNcolumns,hwp->digits);i++;XtSetArg(arg[i],XmNmaxLength,hwp->digits);i++;XtSetArg(arg[i],XmNcursorPositionVisible,False);i++;XtSetArg(arg[i],XmNeditable,False);i++;XtSetArg(arg[i],XmNtraversalOn,False);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_WIDGET);i++;XtSetArg(arg[i],XmNleftWidget,w);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_FORM);i++;assert(i<=XtNumber(arg));hwp->widget=XmCreateTextField(form2,(char*)(hwp->name),arg,i);XtAddCallback(hwp->widget,XmNfocusCallback,monitorHardwareCallback,j);XtAddCallback(hwp->widget,XmNlosingFocusCallback,monitorHardwareCallback,j);XtManageChild(hwp->widget);}for(j=0;j<NPAGES;j++){monitorHwRomLabels[j]=XmStringCreateLtoR((char*)roms[j],XmFONTLIST_DEFAULT_TAG);}frame3=UiCreateFrame(monitorPanels[PAN_HW].shell,"Memory pages");XtVaSetValues(frame3,XmNrightOffset,3,XmNleftAttachment,XmATTACH_FORM,XmNrightAttachment,XmATTACH_WIDGET,XmNrightWidget,frame2,XmNtopAttachment,XmATTACH_WIDGET,XmNtopWidget,frame,XmNbottomAttachment,XmATTACH_FORM,XtNull);i=0;XtSetArg(arg[i],XmNfractionBase,4);i++;assert(i<=XtNumber(arg));form=XmCreateForm(frame3,(char*)"form",arg,i);XtManageChild(form);for(hwp=&monitorHw[HW_BANK0];hwp<=&monitorHw[HW_BANK3];hwp++){i=0;XtSetArg(arg[i],XmNalignment,XmALIGNMENT_BEGINNING);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNrightPosition,hwp->right);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNtopPosition,hwp->top);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNbottomPosition,hwp->bottom);i++;assert(i<=XtNumber(arg));w=XmCreateLabel(form,(char*)(hwp->name),arg,i);XtManageChild(w);i=0;XtSetArg(arg[i],XmNalignment,XmALIGNMENT_BEGINNING);i++;XtSetArg(arg[i],XmNtraversalOn,False);i++;XtSetArg(arg[i],XmNleftAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNleftPosition,hwp->right);i++;XtSetArg(arg[i],XmNrightAttachment,XmATTACH_FORM);i++;XtSetArg(arg[i],XmNtopAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNtopPosition,hwp->top);i++;XtSetArg(arg[i],XmNbottomAttachment,XmATTACH_POSITION);i++;XtSetArg(arg[i],XmNbottomPosition,hwp->bottom);i++;assert(i<=XtNumber(arg));hwp->widget=XmCreateLabel(form,(char*)(hwp->name),arg,i);XtManageChild(hwp->widget);}}XtManageChild(monitorPanels[PAN_HW].shell);XtPopup(XtParent(monitorPanels[PAN_HW].shell),XtGrabNone);}static void monitorHardwareCallback(Widget w,XtPointer client,XtPointer call){field*hwp;char buf[3],*str;int val;if(((XmAnyCallbackStruct*)call)->reason==XmCR_UNMAP){XmToggleButtonSetState(monitorPanels[PAN_HW].button,False,True);return;}if(XmTextFieldGetEditable(w)==True){if(((XmAnyCallbackStruct*)call)->reason==XmCR_FOCUS){XtVaSetValues(w,XmNcursorPositionVisible,True,XtNull);}else{XtVaSetValues(w,XmNcursorPositionVisible,False,XtNull);XmTextFieldClearSelection(w,CurrentTime);hwp=&monitorHw[(int)client];str=XmTextFieldGetString(w);if(sscanf(str,"%x",(unsigned int*)&val)==1){*((uns8*)(hwp->value))=(uns8)val;(void)sprintf(buf,hwp->format,val);Ay8912OutByte((uns8)((unsigned long)client)-(uns8)HW_AY0,(uns8)val);}XmTextFieldSetString(w,buf);XtFree(str);}}}static void monitorHardwareUpdate(void){char buf[7];field*hwp;XmString str;hwp=&monitorHw[HW_VLINE];(void)sprintf(buf,hwp->format,TSTATES/Machine->tstatesPerLine);str=XmStringCreateLtoR(buf,XmFONTLIST_DEFAULT_TAG);XtVaSetValues(hwp->widget,XmNlabelString,str,XtNull);XmStringFree(str);hwp++;(void)sprintf(buf,hwp->format,*((unsigned int*)(hwp->value)));str=XmStringCreateLtoR(buf,XmFONTLIST_DEFAULT_TAG);XtVaSetValues(hwp->widget,XmNlabelString,str,XtNull);XmStringFree(str);for(hwp++;hwp<=&monitorHw[HW_BANK3];hwp++){XtVaSetValues(hwp->widget,XmNlabelString,monitorHwRomLabels[*((int*)(hwp->value))],XtNull);}for(;hwp<=&monitorHw[HW_AY15];hwp++){(void)sprintf(buf,hwp->format,*((uns8*)Machine+(unsigned long)(hwp->value)));XmTextFieldSetString(hwp->widget,buf);}}static void monitorHardwareStateChanged(void){field*hwp;Boolean editable;editable=(monitorState==S_HALT||monitorState==S_STEP_HALT);for(hwp=&monitorHw[HW_AY0];hwp<=&monitorHw[HW_AY15];hwp++){XmTextFieldSetEditable(hwp->widget,editable);}}
